#!/usr/bin/perl

package BaseTest::GlobusGatekeeper;

use POSIX qw(:signal_h :errno_h :sys_wait_h);
use strict;
use vars qw(@ISA);
use EDGUtils;
use IO::Handle;

@ISA=qw(BaseTest);

=head1 NAME

GlobusGatekeeper.pm 

=head1 SYNOPSIS

GlobusGatekeeper.pm - Test that the globus gatekeeper works.
# $Id: GlobusGatekeeper.pm,v 1.1.1.2 2005/02/08 16:09:42 mbarroso Exp $


=head1 DESCRIPTION 

B<GlobusGatekeeper.pm> This test checks that a globus-job-run, a
globus-job-submit, and a globusrun with two-phase commit all work.
The two-phase commit test is extremely important as edg-job-submits
will fail unless this works correctly.  (Note this involves an (as
yet) unidentified port on the Gatekeeper.)


=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

our $genTimeout = 600;  ##  was  360;
our $genSleep   = 10;   ##  was  5;

## ***************************************************************************************
sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "GlobusGatekeeper: setContext\n";

    # Number of identical jobs to send and number of parallel streams. 
    $self->addOption('njobs=i',
		     'Number of jobs in each stream.',
                     '1');
    $self->addOption('nstreams=i',
		     'Number of streams.',
                     '4');

    # Commands required to run this test.
    $self->{commands} = ["globus-job-run", 
			 "globus-job-submit", 
			 "globus-job-status",
			 "globus-job-get-output", 
			 "globus-job-clean", 
			 "grid-proxy-info"];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test GlobusGatekeeper [options] <resource> [resource...]

END
    ;

    $self->{DescText} = <<'END'

 This test checks that a globus-job-run, a
globus-job-submit, and a globusrun with two-phase commit all work.
The two-phase commit test is extremely important as edg-job-submit
commands will fail unless this works correctly.

 Checks that a globus gatekeeper is minimally functioning.  For
the two-phase test to be useful, it should be run from a
machine outside of the gatekeeper\'s local firewall.

 The exact sequence of operations is:

  ======> Step Number and Name: 0 = Trying globus authentication... = 
  Description: tryAuthentication 
  Command: globusrun -a -r lxshare0286.cern.ch/jobmanager 
  ======> Step Number and Name: 1 = Trying globus-job-run... = 
  Description: sendRandom 
  Command: globus-job-run lxshare0286.cern.ch/jobmanager /bin/echo 
  ======> Step Number and Name: 2 = Trying globus-job-run with staging... = 
  Description: stageRandom 
  Command: globus-job-run lxshare0286.cern.ch/jobmanager -stage 
  ======> Step Number and Name: 3 = Trying globus-job-submit... = 
  Description: submitRandom 
  Command: globus-job-submit lxshare0286.cern.ch/jobmanager /bin/echo 0.178552493115678 
  ======> Step Number and Name: 4 = Testing job status... = 
  Description: submitStatus 
  Command: globus-job-status https://lxshare0286.cern.ch:20005/8673/1083261635/ 
  ======> Step Number and Name: 5 = Check job output... = 
  Description: checkOutput 
  Command: globus-job-get-output https://lxshare0286.cern.ch:20005/8673/1083261635/ 
  ======> Step Number and Name: 6 = Clean up job... = 
  Description: clearJob 
  Command: globus-job-clean -force https://lxshare0286.cern.ch:20005/8673/1083261635/ 
  ======> Step Number and Name: 7 = Trying globusrun (two_phase)... = 
  Description: submitRandomGlobusrun 
  Command: globusrun -r lxshare0286.cern.ch/jobmanager -batch -file 
  ======> Step Number and Name: 8 = Testing job status... = 
  Description: submitStatus 
  Command: globus-job-status https://lxshare0286.cern.ch:20005/13167/1083261695/ 
  ======> Step Number and Name: 9 = Check job output... = 
  Description: checkOutput 
  Command: globus-job-get-output https://lxshare0286.cern.ch:20005/13167/1083261695/ 
  ======> Step Number and Name: 10 = Clean up job... = 
  Description: clearJob 
  Command: globus-job-clean -force https://lxshare0286.cern.ch:20005/13167/1083261695/

 (the values are provided as an example)

 Currently, inside of a MainScript regression test, this sequence of tests is run over
each available CE, at least 4 times (--nstreams), and all the sequences are run in parallel.
The --njobs option is controlling the number of loops, i.e. the number of times
this is repeated.

 An example of command in the latter case (MainScript) is:
opt/edg/bin/MainScript --forcingTB=CertTB --TList=CEGate --verbose --addOptionList="--xml --njobs=2 --nstreams=4" \
  --forceMachineList="lxshare0241.cern.ch lxshare0290.cern.ch lxshare0235.cern.ch lxshare0277.cern.ch lxshare0286.cern.ch"

END
    ;
}

## ***************************************************************************************
sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

    print "GlobusGatekeeper: checkPrerequisites\n";

    my $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    return $rc;
}

## ***************************************************************************************
sub cleanup {
    my $self = shift;

    print "GlobusGatekeeper: cleanup\n";

    unlink "/tmp/$$";

    my $rc = $self->SUPER::cleanup();
    return $rc;
}

sub runTest {
## ***************************************************************************************
    my $self = shift;

    STDOUT->autoflush(1);
    STDERR->autoflush(1);

## gg
    my $saveOut;
#prevGOOD    if ( $ENV{MAINSCRIPT} ) {
    if ( $self->{opts}->{xml} ) {
	my $base = "/tmp/$ENV{USER}-$$";
#	$self->{conffile} = "$base-GridFTPconf";
	$self->{outfile}  = "$base-GatekeeperOut";
	$self->{xmlfile}  = "$base-GatekeeperXML";
	$self->{xmlxist}  = __PACKAGE__;
#	$self->{xmlxist}  = "print";

	$self->{reqs} = {};    ## Hash
	$self->{steps} = {};    ## Hash
	$self->{mach} = {};    ## Hash
	$self->{case} = {};    ## Hash
##	$self->{title}  = __PACKAGE__;


	my $html_dir = ( defined $ENV{HTML_DIR} ) ? "$ENV{HTML_DIR}" : "$ENV{HOME}/TSTG/HTML";
	$html_dir = "${html_dir}/cal_gatekeeper";
	my $resmk = `mkdir -p $html_dir`;
	$self->{htmldir}  = "$html_dir";

#	$saveOut = switchStdouterr("$self->{outfile}");
    }

    print "GlobusGatekeeper: runTest\n";

    # Keep a list of all of the resources which fail.
    my @failed;

    # Loop over each argument doing a full test for each.
#     foreach my $host (@{$self->{args}}) {
#         print "Checking... $host\n";

# 	my $error = $self->testGatekeeper($host);
# 	push @failed, $host if ($error);
#     }

    @{$self->{logs}} = ();
    my $jOffset = 0;

    foreach my $jloop ( 1 .. $self->{opts}->{njobs}) {

	if ( $BaseTest::sigC ) { 
	    print STDOUT " Signal Value: $BaseTest::sigC \n";
	    last;
	}

    my ($pidref, $phref) = $self->parallelSubmit($jOffset, @{$self->{args}});
    my @pids = @$pidref;
    my @phs = @$phref;
    my %pha = {};
    foreach my $pid ( @pids ) {
	my $ph = shift @phs;
	$pha{$pid} = $ph;
    }

    $jOffset += scalar(@pids);

#    my @{$self->{logs}} = ();
    my @stack = ();
    my $stpid;
    my $proc = 0;
#    while ( ($stpid = waitpid(-1, &WNOHANG)) > 0 ) {
    while ( ($stpid = waitpid(-1, 0)) > 0 ) {
	# Put $stpid on a stack ...
	push @stack, $stpid;
	my $ph = $pha{$stpid};
	my @lines = ();
	print " Proc: $proc \n";
	while ( my $line = readline($ph) ) {
	    #p print " P: $line ";
	    push @lines, $line;
	}
	close ($ph);
	$proc++;
	push @{$self->{logs}}, \@lines;
    }

    print " Pids: @pids \n";
    print " Stack: @stack \n";

}

#     my $proc = 0;
#     foreach my $ph (@$phref) {
# 	print "\n Proc: $proc \n";
# 	while ( my $line = readline($ph) ) {
# 	    print " P: $line ";
# 	}
# 	$proc++;
#     }


## gg
#     if ( $ENV{MAINSCRIPT} ) {
# 	restoreStdouterr($saveOut);
#     }

    # If there are any host names in the list, then the test has failed.
    if ($#failed>=0) {
        my $names = join(",",@failed);
        $self->setErrorMessage("gatekeeper test failed for $names");
        return 1;
    }

    return 0;
}

## 
## The actual code for the test is here.   It takes a resource ID and 
## runs a complete test on the gatekeeper. 
##
## ***************************************************************************************
sub testGatekeeper {

    my ($self, $gk) = @_;
    my $mace = $gk;

    # Add jobmanager to end of hostname if not provided.
    $gk = "$gk/jobmanager" unless ($gk =~ /.*jobmanager.*/);

    if (0) { ## test hook
	$self->fillResHash('[OK]', "message 0", $mace, "Trying globus-job-run...", 0, "sendRandom", "globus-job-run $gk /bin/echo") unless ( $mace =~ "408" );
	$self->fillResHash('[FAIL]', "message 1", $mace, "Trying globus-job-run with staging...", 1, "stageRandom", "globus-job-run $gk -stage");
	return 0 if ( $mace =~ "adc0010" );
	$self->fillResHash('[OK]', "message 2", $mace, "Trying globus-job-submit...", 2, "submitRandom", "globus-job-submit $gk /bin/echo") unless ( $mace =~ "adc0006" );
	$self->fillResHash('[OK]', "message 3", $mace, "Testing job status...", 3, "submitStatus", "globus-job-status jobid");
	return 0 if (1);
    }

    if (0) {			## 2D test hook
	foreach my $c ( 0..1 ) {
	    my $loop = "case$c";
	    next if ( $mace =~ "405" && $c == 0 );
	    next if ( $mace =~ "adc0006" && $c == 1 );
	    $self->fillResHash2D($loop, '[OK]', "message 0", $mace, "Trying globus-job-run...", 0, "sendRandom", "globus-job-run $gk /bin/echo") unless ( $mace =~ "408" );
	    $self->fillResHash2D($loop, '[FAIL]', "message 1", $mace, "Trying globus-job-run with staging...", 1, "stageRandom", "globus-job-run $gk -stage");
	    next if ( $mace =~ "adc0010" );
	    $self->fillResHash2D($loop, '[OK]', "message 2", $mace, "Trying globus-job-submit...", 2, "submitRandom", "globus-job-submit $gk /bin/echo") unless ( $mace =~ "adc0006" );
	    $self->fillResHash2D($loop, '[OK]', "message 3", $mace, "Testing job status...", 3, "submitStatus", "globus-job-status jobid");
	}
	return 0 if (1);
    }

    my $tStart = time();
    my $detTimer = 1;
    my $tSpent;
    my $dateST;
##$dateST = `date '+%y%m%d-%H%M%S'`

    print "Trying globus authentication...";
    my ($result, $out) = tryAuthentication($gk);
    print $result . "\n";
    if ( $detTimer ) { $tSpent = time() - $tStart; $dateST = `date '+%y%m%d-%H%M%S'`; print " >>>>0< Time SPENT so far: $tSpent @ $dateST \n"; }
#p    print STDERR " sendRandom: $out \n";
    $self->fillResHash($result, $out, $mace, "Trying globus authentication...", 0, "tryAuthentication", "globusrun -a -r $gk");
    return 1  if ("$result" eq '[FAIL]') ;

    print "Trying globus-job-run...";
    my ($result, $out) = sendRandom($gk);
    print $result . "\n";
    if ( $detTimer ) { $tSpent = time() - $tStart; $dateST = `date '+%y%m%d-%H%M%S'`; print " >>>>1< Time SPENT so far: $tSpent @ $dateST \n"; }
#p    print STDERR " sendRandom: $out \n";
    $self->fillResHash($result, $out, $mace, "Trying globus-job-run...", 1, "sendRandom", "globus-job-run $gk /bin/echo");
    return 1  if ("$result" eq '[FAIL]') ;

    print "Trying globus-job-run with staging...";
    my ($result, $out) = stageRandom($gk);
    print $result . "\n";
    if ( $detTimer ) { $tSpent = time() - $tStart; $dateST = `date '+%y%m%d-%H%M%S'`; print " >>>>2< Time SPENT so far: $tSpent @ $dateST \n"; }
    $self->fillResHash($result, $out, $mace, "Trying globus-job-run with staging...", 2, "stageRandom", "globus-job-run $gk -stage");
    return 1  if ("$result" eq '[FAIL]') ;

    return 0 if (0);

    print "Trying globus-job-submit...";
    my ($result,$r,$jobid, $out) = submitRandom($gk);
    print $result . "\n";
    if ( $detTimer ) { $tSpent = time() - $tStart; $dateST = `date '+%y%m%d-%H%M%S'`; print " >>>>3< Time SPENT so far: $tSpent @ $dateST \n"; }
    $self->fillResHash($result, $out, $mace, "Trying globus-job-submit...", 3, "submitRandom", "globus-job-submit $gk /bin/echo $r");
    return 1  if ("$result" eq '[FAIL]') ;
    print "JobId: $jobid\n";
    print "Random: $r\n";

    print "Testing job status...\n";
    my ($result, $out) = submitStatus($jobid,40);  ##  was  10
    print "Status...$result" . "\n";
    if ( $detTimer ) { $tSpent = time() - $tStart; $dateST = `date '+%y%m%d-%H%M%S'`; print " >>>>4< Time SPENT so far: $tSpent @ $dateST \n"; }
    $self->fillResHash($result, $out, $mace, "Testing job status...", 4, "submitStatus", "globus-job-status $jobid");
    return 1  if ("$result" eq '[FAIL]') ;

    print "Check job output...";
    my ($result, $out) = checkOutput($jobid,$r);
    print $result . "\n";
    if ( $detTimer ) { $tSpent = time() - $tStart; $dateST = `date '+%y%m%d-%H%M%S'`; print " >>>>5< Time SPENT so far: $tSpent @ $dateST \n"; }
    $self->fillResHash($result, $out, $mace, "Check job output...", 5, "checkOutput", "globus-job-get-output $jobid");
    return 1  if ("$result" eq '[FAIL]') ;

    print "Clean up job...";
    my ($result, $out) = clearJob($jobid);
    print $result . "\n";
    if ( $detTimer ) { $tSpent = time() - $tStart; $dateST = `date '+%y%m%d-%H%M%S'`; print " >>>>6< Time SPENT so far: $tSpent @ $dateST \n"; }
    $self->fillResHash($result, $out, $mace, "Clean up job...", 6, "clearJob", "globus-job-clean -force $jobid");
    return 1  if ("$result" eq '[FAIL]');

    print "Trying globusrun (two_phase)...";
    my ($result,$r,$jobid,$out) = submitRandomGlobusrun($gk);
    print $result . "\n";
    if ( $detTimer ) { $tSpent = time() - $tStart; $dateST = `date '+%y%m%d-%H%M%S'`; print " >>>>7< Time SPENT so far: $tSpent @ $dateST \n"; }
    $self->fillResHash($result, $out, $mace, "Trying globusrun (two_phase)...", 7, "submitRandomGlobusrun", "globusrun -r $gk -batch -file");
    return 1  if ("$result" eq '[FAIL]') ;
    print "JobId: $jobid\n";
    print "Random: $r\n";

    print "Testing job status...\n";
    my ($result, $out) = submitStatus($jobid,40);  ##  was  10);
    print "Status...$result\n";
    if ( $detTimer ) { $tSpent = time() - $tStart; $dateST = `date '+%y%m%d-%H%M%S'`; print " >>>>8< Time SPENT so far: $tSpent @ $dateST \n"; }
    $self->fillResHash($result, $out, $mace, "Testing job status...", 8, "submitStatus", "globus-job-status $jobid");
    return 1  if ("$result" eq '[FAIL]') ;

    print "Check job output...";
    my ($result, $out) = checkOutput($jobid,$r);
    print $result . "\n";
    if ( $detTimer ) { $tSpent = time() - $tStart; $dateST = `date '+%y%m%d-%H%M%S'`; print " >>>>9< Time SPENT so far: $tSpent @ $dateST \n"; }
    $self->fillResHash($result, $out, $mace, "Check job output...", 9, "checkOutput", "globus-job-get-output $jobid");
    return 1  if ("$result" eq '[FAIL]') ;

    print "Clean up job...";
    my ($result, $out) = clearJob($jobid);
    print $result . "\n";
    if ( $detTimer ) { $tSpent = time() - $tStart; $dateST = `date '+%y%m%d-%H%M%S'`; print " >>>10< Time SPENT so far: $tSpent @ $dateST \n"; }
    $self->fillResHash($result, $out, $mace, "Clean up job...", 10, "clearJob", "globus-job-clean -force $jobid");
    return 1  if ("$result" eq '[FAIL]');
}

# Send an authentication request.
## ***************************************************************************************
sub tryAuthentication {

    my ($gk) = @_;

    my $out = `globusrun -a -r $gk`;
    return (($?) ? '[FAIL]' : '[OK]', $out);
#    return ($out =~ m/.*$r.*/m) ? '[OK]' : $out;
}

# Generate a random number, send it to the gatekeeper, and ensure that
# the number is present in the output.
## ***************************************************************************************
sub sendRandom {

    my ($gk) = @_;

    my $r = rand();

    sleep($genSleep); ##  10;  ##  Tentative ...

#    my $out = `globus-job-run $gk /bin/echo $r 2>&1`;
    my $command = "globus-job-run $gk /bin/echo $r 2>&1";
    my $out = timeout($command, $genTimeout);
    print "TIMEOUT " if ($$out[0] eq 2);
#    return (($out =~ m/.*$r.*/m) ? '[OK]' : '[FAIL]', $out);
    return (($$out[1] =~ m/.*$r.*/m) ? '[OK]' : '[FAIL]', $$out[1]);

}

# Generate a random number, send it to the gatekeeper, and ensure that
# the number is present in the output.
## ***************************************************************************************
sub stageRandom {

    my ($gk) = @_;

    my $r = rand();

    # Create a temporary file to hold the script.
    my $fn = "/tmp/script-$$.sh";
    open TMP, ">$fn";
    print TMP << "END"
#!/bin/sh
/bin/echo $r
exit 0
END
    ;
    close TMP;
    chmod 0740, $fn;

    # Run the script.
#    my $command = "globus-job-run $gk -stage $fn";
#    my $out = `$command 2>&1`;

    sleep($genSleep); ##  10;  ##  Tentative ...

    my $command = "globus-job-run '$gk' -stage '$fn' 2>&1";
    my $out = timeout($command, $genTimeout);
    print "TIMEOUT " if ($$out[0] eq 2);
    
    # Clean up the script.
    unlink $fn;

    # Now return the result.
#    return (($out =~ m/.*$r.*/m) ? '[OK]' : '[FAIL]', $out);
    return (($$out[1] =~ m/.*$r.*/m) ? '[OK]' : '[FAIL]', $$out[1]);
}


# This submits a job via globus.  It returns a list with a success
# flag, the random number which should be returned, and the job
# contact ID.
## ***************************************************************************************
sub submitRandom {

    my ($gk) = @_;
    my $r = rand();

#    my $out = `globus-job-submit $gk /bin/echo $r 2>&1`;

    sleep($genSleep); ##  10;  ##  Tentative ...

    my $command = "globus-job-submit '$gk' /bin/echo '$r' 2>&1";
    my $out = timeout($command, $genTimeout);
    print "TIMEOUT " if ($$out[0] eq 2);

#    chomp(my $jobid=$out);
    chomp(my $jobid=$$out[1]);
    my $flag = ($!) ? '[OK]' : '[FAIL]';
#    return ($flag, $r, $jobid, $out);
    return ($flag, $r, $jobid, $$out[1]);
}

# This submits a job via globus using globusrun.  It returns a list
# with a success flag, the random number which should be returned, and
# the job contact ID.  
## ***************************************************************************************
sub submitRandomGlobusrun {

    my ($gk) = @_;

    my $r = rand();

# Create the RSL file.
    open RSL, ">/tmp/$$";
    print RSL << "END"
  &(executable="/bin/echo")
   (arguments= "$r")
   (two_phase=300)
   (stdout=x-gass-cache://\$(GLOBUS_GRAM_JOB_CONTACT)stdout anExtraTag)
   (stderr=x-gass-cache://\$(GLOBUS_GRAM_JOB_CONTACT)stderr anExtraTag)
END
    ;
    close RSL;

    sleep($genSleep); ##  10;  ##  Tentative ...

#    my $out = `globusrun -r $gk -batch -file /tmp/$$ 2>&1`;
    my $command = "globusrun -r $gk -batch -file /tmp/$$ 2>&1";
    my $out = timeout($command, $genTimeout);
    print "TIMEOUT " if ($$out[0] eq 2);

#    chomp(my $jobid=$out);
    chomp(my $jobid=$$out[1]);

    $jobid =~ m/.*(https\S*)\s*/;
    $jobid = $1;
    my $flag = ($!) ? '[OK]' : '[FAIL]';

#    return ($flag, $r, $jobid, $out);
    return ($flag, $r, $jobid, $$out[1]);
}

# This waits in a loop until the timeout is reached or the "DONE"
# status is obtained.
## ***************************************************************************************
sub submitStatus {

    my ($jobid, $limit) = @_;

    my $out;
    my $command;
    my $status = "INITIALIZE";
    CHECK: while ($limit>0) {
	sleep($genSleep);
	$command = "globus-job-status $jobid 2>&1";
	$out = timeout($command, $genTimeout);
#	print "TIMEOUT " if ($$out[0] eq 2);

	$status = $$out[1];
	if ($$out[0] eq 2) {
	    $status .= " TIMEOUT ";
	}
	$status .= "SYSTEM ERROR" unless ($!);

	chomp($status);
	print "$status\n";

	last CHECK if ("$status" eq "DONE" || ("$status" eq "SYSTEM ERROR"));
	$limit--;
    }

#    return (("$status" eq "DONE") ? '[OK]' : '[FAIL]', $out);
##    return (("$status" eq "DONE") ? '[OK]' : '[FAIL]', $$out[1]);
    return (("$status" eq "DONE") ? '[OK]' : '[FAIL]', $status);
}

# Retrieve the job output and check that the random number was
# correctly echoed.
## ***************************************************************************************
sub checkOutput {

    my ($jobid, $r) = @_;

    sleep($genSleep); ##  10;  ##  Tentative ...

#    my $out = `globus-job-get-output $jobid 2>&1`;
    my $command="globus-job-get-output $jobid 2>&1";
    my $out = timeout($command, $genTimeout);
    print "TIMEOUT " if ($$out[0] eq 2);
    print "$$out[1]\n";
#    return (($out =~ m/.*$r.*/m) ? '[OK]' : '[FAIL]', $out);
    return (($$out[1] =~ m/.*$r.*/m) ? '[OK]' : '[FAIL]', $$out[1]);
}

# This submits a job via globus.  It returns a list with a success
# flag, the random number which should be returned, and the job
# contact ID.
## ***************************************************************************************
sub clearJob {

#     if ( $ENV{MAINSCRIPT} ) {
# 	return ('[OK]', ""); ## TEMP gg
#     }

    my ($jobid) = @_;
    chomp($jobid);
#    my $out = `globus-job-clean -force $jobid 2>&1`;
#    return (($!) ? '[OK]' : '[FAIL]', $out);

    sleep($genSleep); ##  10;  ##  Tentative ...

    my $command = "globus-job-clean -force $jobid 2>&1";
    my $out = timeout($command, $genTimeout);
    print "TIMEOUT " if ($$out[0] eq 2);
    return (($!) ? '[OK]' : '[FAIL]', $$out[1]);
}

## ***************************************************************************************
sub buildXMLFileNOTVERYGOODandquiteold {

    my $self = shift;

    print "GlobusGatekeeper: buildXMLFile\n";

    print STDERR " Called buildXMLFile() to process $self->{outfile} \n";

##test    my $resBXML = `build-XML.pl $self->{outfile} $self->{xmlfile}`;
    my $resBXML = `build-XML-col-wise.pl $self->{outfile} $self->{xmlfile}`;
    print STDERR " After buildXMLFile: \n$resBXML \n";

    return 0;
}

## ***************************************************************************************
sub buildXMLFile {
    my $self = shift;
#    my $ret = $self->buildXMLFile1D();
    my $ret = $self->buildXMLFile2D();
    return $ret;
}

## Type with 1 row only, several steps and several columns 
## ***************************************************************************************
sub buildXMLFile1D {
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_steps;
    my @lof_rows = ("");
#notbad    my $Title = (caller(0))[3];
    my $Title = __PACKAGE__;
    chomp(my $date = `date`);
#     my $brf_info = "Rather Simple Storm";
#     my $Jdl = `cat $self->{jdlFile}`;
#     my $Scr = `cat  $self->{shFile}`;
#     my $det_info = "Job Storm with a simple JDL script, and minimal sandbox transfers.\n\nThe JDL file is:\n\n".$Jdl."\n\nWhile the Script file is:\n\n".$Scr;
#     $det_info =~ s/\n/<br>/g;
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile() to write $self->{xmlfile} \n";

    my $cols = {};
    ## Col names
    foreach my $mace ( sort keys %{$self->{reqs}} ) {
	push @lof_cols, $mace;
    }

    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	push @lof_steps, "Step_$stepn";
    }

    my $nsteps = scalar(@lof_steps)-1;
#p    print STDERR " Called buildXMLFile() to write @lof_cols \n";
    my $rowM = "";
    foreach my $mace ( sort keys %{$self->{reqs}} ) {
	my $stepM = "";
	for my $stepn (0..$nsteps) {
	    my $retv = $ret{$self->{reqs}->{$mace}->{$stepn}->{stat1}};
	    my $logf = $self->{reqs}->{$mace}->{$stepn}->{olout};
		$logf =~ s/&/&amp;/g; # This must be changed first.
		$logf =~ s/\"/&quot;/g;
		$logf =~ s/</&lt;/g;
		$logf =~ s/>/&gt;/g;
	    $stepM .= makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step");
	}
	$rowM .= makeX($stepM, "col");
    }
    my $colM = makeX($rowM, "row");

    ## building the step info list ...
    my $stepI = "";
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	my $brf_info = "Step_$stepn";
#old	my $brf_info = $self->{steps}->{$stepn}->{sname};
	$brf_info =~ s/ /_/g;
#old	my $det_info = $self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	my $det_info = $self->{steps}->{$stepn}->{sname}." : \n".$self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	$stepI .= makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
    }

    my $body = $colM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
@lof_rows = map { makeX($_, "row"); } @lof_rows;
$info .= makeX("@lof_rows", "rows_list");
@lof_cols = map { makeX($_, "col"); } @lof_cols;
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepI;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDERR makeX($info." \n ".$body, "log");

    return 0;
}

## ***************************************************************************************
sub fillResHash {

    my ($self, $stat1, $olout, $mace, $sname, $stepn, $sdesc, $scomm) = @_;
    $self->{globalresult} .= " $stat1" if $self->{globalresult};

#prevGOOD    return unless $ENV{MAINSCRIPT};
    return unless $self->{opts}->{xml};

    $self->{reqs}->{$mace}->{$stepn}->{stat1} = $stat1;
    $self->{reqs}->{$mace}->{$stepn}->{olout} = $olout;
    $self->{steps}->{$stepn}->{sname} = $sname;
    $self->{steps}->{$stepn}->{sdesc} = $sdesc;
    $self->{steps}->{$stepn}->{scomm} = $scomm;

    print STDOUT " STEPSTEP: ======> Step Number and Name: $stepn = $sname = \n";
    print STDOUT " STEPSTEP: Description: $sdesc \n"; 
    print STDOUT " STEPSTEP: Command: $scomm \n";

    return;

}

## Type with several rows, several steps and several columns 
## ***************************************************************************************
sub buildXMLFile2D {
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_steps;
    my @lof_rows;
#notbad    my $Title = (caller(0))[3];
    my $Title = __PACKAGE__;
    chomp(my $date = `date`);
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile() to write $self->{xmlfile} \n";

#    my $cols = {};
    ## Col names
    foreach my $mace ( sort keys %{$self->{mach}} ) {
	push @lof_cols, $mace;
    }

#    my $cols = {};
    ## Row names
    foreach my $case ( sort keys %{$self->{case}} ) {
	push @lof_rows, $case;
    }

    ## List of steps ... (steps are numbers)
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	push @lof_steps, "Step_$stepn";
    }

    my $nsteps = scalar(@lof_steps)-1;
    print STDERR " Called buildXMLFile() to cols @lof_cols \n";
    print STDERR " Called buildXMLFile() to rows @lof_rows \n";
    my $rowM = "";
    foreach my $row ( @lof_rows ) {
	my $colM = "";
	foreach my $mace ( @lof_cols ) {
	    my $stepM = "";
	    for my $stepn (0..$nsteps) {
		my $retv = $ret{$self->{reqs}->{$row}->{$mace}->{$stepn}->{stat1}};
		my $subs = $self->{reqs}->{$row}->{$mace}->{$stepn}->{stat2};
		my $logf = $self->{reqs}->{$row}->{$mace}->{$stepn}->{olout};
		$logf =~ s/&/&amp;/g; # This must be changed first.
		$logf =~ s/\"/&quot;/g;
		$logf =~ s/</&lt;/g;
		$logf =~ s/>/&gt;/g;
#good		$stepM .= makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step");
		$stepM .= makeX(makeX("$logf", "output").makeX("$retv", "retval").makeX("$subs", "substatus"), "step");
	    }
	    $colM .= makeX($stepM, "col");
	}
	$rowM .= makeX($colM, "row");
    }

    ## building the step info list ...
    my $stepI = "";
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	my $brf_info = "Step_$stepn";
#old	my $brf_info = $self->{steps}->{$stepn}->{sname};
	$brf_info =~ s/ /_/g;
#old	my $det_info = $self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	my $det_info = $self->{steps}->{$stepn}->{sname}." : \n".$self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	$stepI .= makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
    }

    my $body = $rowM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
@lof_rows = map { makeX($_, "row"); } @lof_rows;
$info .= makeX("@lof_rows", "rows_list");
@lof_cols = map { makeX($_, "col"); } @lof_cols;
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepI;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDERR makeX($info." \n ".$body, "log");

    return 0;
}

## ***************************************************************************************
sub fillResHash2D {
#prevGOOD    return unless $ENV{MAINSCRIPT};
    my $self = shift;
    return unless $self->{opts}->{xml};
    my ($case, $stat1, $olout, $mace, $sname, $stepn, $sdesc, $scomm) = @_;
    $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1} = $stat1;
    $self->{reqs}->{$case}->{$mace}->{$stepn}->{olout} = $olout;
    $self->{steps}->{$stepn}->{sname} = $sname;
    $self->{steps}->{$stepn}->{sdesc} = $sdesc;
    $self->{steps}->{$stepn}->{scomm} = $scomm;
    $self->{mach}->{$mace}++;
    $self->{case}->{$case}++;
    return;

}

# sub makeX {
## ***************************************************************************************
#   my $str  = shift;
#   my $flag = shift;
#   return "<$flag>\n$str\n</$flag>\n";
# }

# Run job submissions in parallel.  This takes all of the parameters
# from the state of the object. 
## ***************************************************************************************
sub parallelSubmit {

    my ($self, $jOffset, @hosts) = @_;
    my @pids;
    my @phs;
    $jOffset /= scalar(@hosts);

    foreach my $i (1 .. $self->{opts}->{nstreams}) {
	foreach my $host (@hosts) {

	    local *PH;
      FORK: { 
	    my $pid;
  die "cannot fork: $!" unless defined ( $pid = open ( PH, "-|" ) );
#          if ($pid = fork) {
          if ($pid) {
              # parent here
              # child process pid is available in $pid
              push @pids, $pid;
	      push @phs, *PH;
          } elsif (defined $pid) {
              # $pid is zero here if defined
              # child here
              # parent process available with getppid

	      STDOUT->autoflush(1);
	      STDERR->autoflush(1);

	      # Do the actual submission.  The error return is ignored
	      # because there is no way to feed back this information
	      # easily.  
# 	      my $rc = $self->submit(["$self->{prefix}.$i",
# 				      "$self->{opts}->{njobs}"]);
	      my $jO = $jOffset+$i;
	      print STDOUT " Starting testGatekeeper with $host (job# $jO) \n";
	      my $tStart = time();
	      my $rc = $self->testGatekeeper($host);
	      my $tAlmost = time() - $tStart;
	      print STDOUT " TimeCEG: $tAlmost \n";
              # This is important otherwise child may execute parent code!
              exit;
          } elsif ($! =~ /No more process/) {
              # Recoverable error (EAGAIN); try again. 
              sleep 5;
              redo FORK;
          } else {
              # Strange error! Print error message. 
              print STDERR "Fork failure.\n";
          }
      }
#	$self->{job}->{submit}->{file} = "$self->{prefix}.$i" if ( $self->{opts}->{nstreams} == 1 );
    }
    }

    return (\@pids, \@phs);
}

# Evaluate the results of the test.
sub evaluateTest {
    my $self = shift;

    print "GlobusGatekeeper: evaluateTest\n";

    my $stepn = 0;

    $self->{steps}->{$stepn}->{sname} = "GlobusGatekeeper";
    $self->{steps}->{$stepn}->{sdesc} = "11 steps";
#    $self->{steps}->{$stepn}->{scomm} = "Several commands";

    my @logs = @{$self->{logs}};
    my @ls = grep { s/ STEPSTEP:// } @{$logs[0]};
    $self->{steps}->{$stepn}->{scomm} = "@ls";

    foreach my $logref ( @logs ) {
	my @lines = grep { ! /STEPSTEP:/ } @$logref;
	if ( ! $self->{opts}->{xml} ){
	    print "@$logref\n";
	}    
	my @m = split /\s+/, $lines[0];
	my $failed = grep { /\[FAIL\]/ } @lines;
	my $oked   = grep { /\[OK\]/ }   @lines;
	chop $m[6];
	print " Eval: $m[4] $m[6] $failed $oked \n";
	my $case = "Job_$m[6]";
	my $mace = $m[4];
	$mace =~ s/\./<br> @ /;
	$self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1} = ( $oked == 11 && $failed == 0 ) ? "[OK]" : "[FAIL]";
	$self->{reqs}->{$case}->{$mace}->{$stepn}->{olout} = "@lines";
	@m = split /\s+/, $lines[-1];
	$self->{reqs}->{$case}->{$mace}->{$stepn}->{stat2} = "$m[2]"."sec";
	$self->{mach}->{$mace}++;
	$self->{case}->{$case}++;
	$self->{globalresult} .= " $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1}" if $self->{globalresult};
    }

    return 0;
}

## ***************************************************************************************
sub buildXMLFileNew {
 
    my $self = shift;
    my ($moptc, $moptr) = @_;
    if ( ! $moptc ) {
	$moptc = "mace";
	$moptr = "mawn";
    } elsif ( ! $moptr ) {
	$moptr = "jind";
    }

#    print "Stack: buildXMLFile\n";
#    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_rows;
    my $Title = "$self->{type}";
    chomp(my $date = `date`);
    my $brf_info = "Rather Simple Storm";
    $brf_info = $self->{brfinfo} if ( $self->{brfinfo} );
    my $Jdl = `cat $self->{jdlFile}`;
    my $Scr = `cat  $self->{shFile}`;
#old    my $det_info = "Job Storm with a simple JDL script, and minimal sandbox transfers.\n\nThe JDL file is:\n\n".$Jdl."\n\nWhile the Script file is:\n\n".$Scr;
    my $det_info = "$brf_info.\n\nThe JDL file is:\n\n".$Jdl."\n\nWhile the Script file is:\n\n".$Scr;
##notgood?    $det_info =~ s/\n/<br>/g;
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile() to write $self->{xmlfile} \n";

    my $cols = {};
    my $order = 0;
#prevGOOD    my $machType = 'mace';
#    my $machType = 'mawn';
    my $machType = "$moptc";
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	if ( $machType eq "mawn" ) {
	    ## Some fixing ...
	    if ( ! $self->{reqs}->{$index}->{mawn} ) { $self->{reqs}->{$index}->{mawn} = "unknown"; }
	}
        $cols->{$self->{reqs}->{$index}->{$machType}}->{freq}++;
        if ( $cols->{$self->{reqs}->{$index}->{$machType}}->{freq} == 1 ) { $cols->{$self->{reqs}->{$index}->{$machType}}->{order} = $order++; }

#prevGOOD	push @lof_rows, "Job_".$index;
#good	push @lof_rows, ( $self->{reqs}->{$index}->{mawn} || "Job_".$index );
	push @lof_rows, ( $self->{reqs}->{$index}->{$moptr} || "Job_".$index );
    }

    foreach my $col ( sort { $cols->{$a}->{order} <=> $cols->{$b}->{order} } keys %{$cols} ) {
#    foreach my $col ( sort keys %{$cols} ) {
#p	print " $col $cols->{$col}->{order} $cols->{$col}->{freq} \n";
#	$col =~ s/\./ /g;
	push @lof_cols, "$col";
#p	print "  $col \n";
    }

    my $ncols = scalar(@lof_cols)-1;
    my $colM = "";
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	my $retv = $ret{$self->{reqs}->{$index}->{stat1}};
	my $logfn;
	my $logf;
	if ( $retv == 0 ) {
	    $logfn = $self->{reqs}->{$index}->{olout};
	    $logf = `cat $logfn` if $logfn;
	} elsif ( $retv == 5 ) {
##edg2.0	    $logf = $self->{reqs}->{$index}->{mess};
	    $logfn = $self->{reqs}->{$index}->{olog};
	    $logf = `cat $logfn` if $logfn;
	} else {
##edg2.0	    if ( $self->{reqs}->{$index}->{stat2} eq "MISSING_FILES" ) { $logf = $self->{reqs}->{$index}->{mess}; }
##edg2.0	    else {
	    $logfn = $self->{reqs}->{$index}->{olog};
	    $logf = `cat $logfn` if $logfn;
##edg2.0	}
	}
	unlink $logfn if $logfn;
	my $coln   = $cols->{$self->{reqs}->{$index}->{$machType}}->{order};
	my $rowM = "";
	for my $c (0..$ncols) {
	    if ( $c == $coln ) {
		$logf =~ s/&/&amp;/g; # This must be changed first.
		$logf =~ s/\"/&quot;/g;
		$logf =~ s/</&lt;/g;
		$logf =~ s/>/&gt;/g;
		$rowM .= makeX(makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step"), "col");
	    } else {
		$rowM .= makeX(makeX(makeX(" ", "output").makeX(" ", "retval"), "step"), "col");
	    }
	}
	$colM .= makeX($rowM, "row");
    }

my $stepM = makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
my $body = $colM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
@lof_rows = map { makeX($_, "row"); } @lof_rows;
$info .= makeX("@lof_rows", "rows_list");
@lof_cols = map { makeX($_, "col"); } @lof_cols;
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepM;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDOUT makeX($info." \n ".$body, "log");

    return 0;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
