#!/usr/bin/perl

package BaseTest::GridFTP;

use strict;
use vars qw(@ISA);
use EDGUtils;
use IO::Handle;

use TestUtils;

@ISA=qw(BaseTest);

=head1 NAME

GridFTP.pm 

=head1 SYNOPSIS

GridFTP.pm - Verify that transfers to and between GridFTP servers works.
# $Id: GridFTP.pm,v 1.1.1.2 2005/02/08 16:09:42 mbarroso Exp $

=head1 DESCRIPTION 

B<GridFTP.pm> This test checks that transfers between the user
interface (UI) machine and all of the given hosts work.  It checks
both absolute URLs and URLs relative to the home area of the user.  

Additionally, third party transfers between all pairs of hosts are
checked.  The number of checks performed goes as n^2; so be careful on
the number of hosts specified.

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

our $nodcau1;
our $nodcau2;

sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "GridFTP: setContext\n";

    # The command needed to run this test.
    $self->{commands} = ["globus-url-copy", 
			 "grid-proxy-info",
			 "edg-gridftp-rm"];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::GridFTP [options] <server node> [server node...] 

END
    ;

    $self->{DescText} = <<'END'

Check that a GridFTP server works by transfering files to and from the
local machine.  Also checks third-party transfers by transfering files
between each pair of remote servers.

For local->machine the three checks are:
 Step  1) local->machine
 Step  2) machine->local
 Step  3) check that contents are correct

For machine1->machine2 the three checks are:
 Step  1) machine1->machine2
 Step  2) machine2->local
 Step  3) check that contents are correct

END
    ;

}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

    print "GridFTP: checkPrerequisites\n";

    my $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if ($rc);

    return $rc;
}


sub runTest {
    my $self = shift;

    my $saveOut;
    if ( $ENV{MAINSCRIPT} ) {
	my $base = "/tmp/$ENV{USER}-$$";
	$self->{conffile} = "$base-GridFTPconf";
	$self->{outfile}  = "$base-GridFTPout";
	$self->{stepfile} = "$base-GridFTPstep";

# 	$self->{xmlfile}  = "$base-GridFTP.XML";
# 	$self->{xmlxist}  = __PACKAGE__;
# 	$self->{htmldim}  = "2D";

	$self->{reqs} = {};    ## Hash
	$self->{steps} = {};    ## Hash
	$self->{mach} = {};    ## Hash
	$self->{case} = {};    ## Hash
#	$self->{debug}  = 1;

	my $package = __PACKAGE__;
	my @m = split /::/, $package;
	$self->{type} = "$m[$#m]";
	$self->{xmlfile}  = "$base-".$self->{type}.".XML";
	## To trigger the processing thru buildXMLParser
	$self->{xmlxist}  = "$self->{type}";

	my $html_dir = ( defined $ENV{HTML_DIR} ) ? "$ENV{HTML_DIR}" : "$ENV{HOME}/TSTG/HTML";
#old	$self->{htmldir} = "$html_dir/cal_gridftp";;
	$html_dir = "${html_dir}/$self->{type}";
	my $resmk = `mkdir -p $html_dir`;
	$self->{htmldir}  = "$html_dir";

#nomore	$saveOut = switchStdouterr("$self->{outfile}");
    }

    print "GridFTP: runTest\n";

    # Keep a list of all directories which fail.
    my @failed;

    # Loop over each argument doing a full test for each.
    foreach my $dir (("/tmp", "~")) {

	if ( $BaseTest::sigC ) { 
	    print STDOUT " Signal Value: $BaseTest::sigC \n";
	    last;
	}

        print "Checking transfers to $dir\n";
	
##PAR	my $error = $self->testGridFTP($dir, \@{$self->{args}});

	@{$self->{hBlocks}} = ();
	do {
	    my @tHosts = @{$self->{args}};
	    while ( scalar(@tHosts) > 8 ) {
		my @hblock = splice (@tHosts, 0, 8);
		push @{$self->{hBlocks}}, \@hblock;
	    }
	    if ( scalar(@tHosts) > 0 ) {
		push @{$self->{hBlocks}}, \@tHosts;
	    }
	    print " Number of Host Blocks: ", scalar(@{$self->{hBlocks}})," \n";
	};

##OLDPERL56    my @{$self->{logs}} = ();
#badnoneed??    @{$self->{logs}} = ();
    my $jOffset = 0;

##PAR    foreach my $jloop ( 1 .. $self->{opts}->{njobs}) {
	foreach my $hblockref ( @{$self->{hBlocks}} ) {

	if ( $BaseTest::sigC ) { 
	    print STDOUT " Signal Value: $BaseTest::sigC \n";
	    last;
	}

    my ($pidref, $phref) = $self->parallelSubmit($jOffset, $dir, $hblockref, $self->{args});
    my @pids = @$pidref;
    my @phs = @$phref;
    my %pha = {};
    foreach my $pid ( @pids ) {
	my $ph = shift @phs;
	$pha{$pid} = $ph;
    }

    $jOffset += scalar(@pids);

#    my @{$self->{logs}} = ();
    my @stack = ();
    my $stpid;
    my $proc = 0;
#    while ( ($stpid = waitpid(-1, &WNOHANG)) > 0 ) {
    while ( ($stpid = waitpid(-1, 0)) > 0 ) {
	# Put $stpid on a stack ...
	push @stack, $stpid;
	my $ph = $pha{$stpid};
	my @lines = ();
	print " Proc: $proc \n";
	while ( my $line = readline($ph) ) {
	    #p print " P: $line ";
	    push @lines, $line;
	}
	close ($ph);
	$proc++;

	print " Whilines: $proc ",scalar @lines," \n";

	if ( scalar(@lines) != 0 ) {
	    push @{$self->{logs}}, \@lines;
	}
    }

    print " Pids: @pids \n";
    print " Stack: @stack \n";

##PAR}
}

##PAR	push @failed, $dir if ($error);

    }

    if ( $ENV{MAINSCRIPT} ) {
#nomore	restoreStdouterr($saveOut);
    }

    # If there are any host names in the list, then the test has failed.
    if ($#failed>=0) {
        my $names = join(",",@failed);
        $self->setErrorMessage("GridFTP test failed for $names");
        return 1;
    }

    return 0;
}


## 
## The actual code for the test is here.   It takes a resource ID and 
## runs a complete test on the gatekeeper. 
##

sub testGridFTP {

    my ($self, $prefix, $hostls, $arrayref) = @_;
    my @hosts = @{$arrayref};

    if ( "$prefix" eq "/tmp" ) { $self->{store}->{stpoff} = -1; }
    else { $self->{store}->{stpoff} = 2; }

    # Create a file name which is always in the temp area for the 
    # local machine as well as one in the location specified by
    # the prefix.
    my $local = "/tmp/$ENV{USER}-$$";

    # Choose a host and check connections to all of the other hosts.
##PAR    foreach my $host1 (@hosts) {
    foreach my $host1 ($hostls) {

	my $prefix1 = $prefix;

#pFTP    print STDERR " testGridFTP1: $prefix1 $host1 $local \n";

	undef $nodcau1;
	if ( $ENV{RBhostname_} =~ /$host1/ ) { next; }
	elsif ( $ENV{dCacheSEhostname_} =~ /$host1/ ) {
	    if ( "$prefix" eq "/tmp" ) {
		$prefix1 = "/pnfs/cern.ch/data/dteam";
	    } else {
		next;
	    }
	    $nodcau1  = " -nodcau";
	}
	elsif ( $host1 =~ /castorgrid/ ) {
	    if ( "$prefix" eq "/tmp" ) {
		$prefix1 = "/castor/cern.ch/grid/dteam";
	    } else {
		next;
	    }
	}
	
	# Fill up the file with a random number.
	my $value = fillFile($local);
	print "\nUsing prefix/value: $prefix1, $value\n";
	
	my $remote = "$ENV{USER}-$$" . "_local-$host1";
	my $check = "$local--$remote";
	
	# Check that transfers from local machine to remote machine works.
	if ( $ENV{MAINSCRIPT} ) { 
      if ( $BaseTest::sigC ) { last; }  ##  GG
	    $self->{store}->{label} = "Check local --> host1"; 
	    $self->{store}->{src} = "local";
	    $self->{store}->{tgt} = "$host1";
	    $self->{store}->{stepn} = 0;
	}
    
        # Make sure home dir is mapped at the destination host - not mandatatory on LCG SE(s) 
	my $result1 = "[UNTESTED]";
	my $result2 = "[UNTESTED]";
	my $result3 = "[UNTESTED]";
        my $arrayref1 = timeoutWithRC2("edg-gridftp-ls gsiftp://$host1/$prefix1 | grep -e root", 20);
	my ($rct1, $crc, $out1) = @$arrayref1;
        if ( "$prefix" eq "~" && ($rct1 || $out1) ) {
	     print "No $prefix dir mapped on $host1\n";
	     print STDERR "No $prefix dir mapped on $host1\n";
	     print "Check local --> $host1:  ";
	     print "$result1 $result2 $result3\n";
	}	
	else {
	    $result1 = $self->doTransfer("file://$local",
				 "gsiftp://$host1/$prefix1/$remote");
	    $result2 = $self->doTransfer("gsiftp://$host1/$prefix1/$remote",
				 "file://$check");
	    $result3 = $self->checkResult($value,$check);
	    print "Check local --> $host1:  ";
	    print "$result1 $result2 $result3\n";
	    $self->{globalresult} .= " $result1 $result2 $result3" if $self->{globalresult};
	    if ( "$result1 $result2 $result3" ne "[OK] [OK] [OK]" ) { next; }
	}

    
	# Clean up the check file.
	unlink $check;
	
	# These are then third party transfers.
	foreach my $host2 (@hosts) {

	    undef $nodcau2;
	    my $prefix2 = $prefix;

	    if ( $ENV{dCacheSEhostname_} =~ /$host2/ ) {
		if ( "$prefix" eq "/tmp" ) {
		    $prefix2 = "/pnfs/cern.ch/data/dteam";
		} else {
		    next;
		}
		$nodcau2  = " -nodcau";
	    }
	    elsif ( $host2 =~ /castorgrid/ ) {
		if ( "$prefix" eq "/tmp" ) {
		    $prefix2 = "/castor/cern.ch/grid/dteam";
		} else {
		    next;
		}
	    }

	    if ("$host1" ne "$host2") {
		if ( $ENV{MAINSCRIPT} ) { 
      if ( $BaseTest::sigC ) { last; }  ##  GG
		    $self->{store}->{label} = "Check host1 --> host2"; 
		    $self->{store}->{src} = $host1;
		    $self->{store}->{tgt} = $host2;
		    $self->{store}->{stepn} = 0;
		}
		my $third = "$ENV{USER}-$$" . "_$host1-$host2";
		my $check = "$local--$third";
		# Make sure home dir is mapped at the destination host - not mandatatory on LCG SE(s) 
		my $result1 = "[UNTESTED]";
		my $result2 = "[UNTESTED]";
		my $result3 = "[UNTESTED]";
##OUT1		my $out1 = `edg-gridftp-ls gsiftp://$host1/$prefix1 | grep -e root 2>&1`;
##OLD		my $out2 = `edg-gridftp-ls gsiftp://$host2/$prefix2 | grep -e root 2>&1`;
        my $arrayref2 = timeoutWithRC2("edg-gridftp-ls gsiftp://$host2/$prefix2 | grep -e root", 20);
	my ($rct2, $crc, $out2) = @$arrayref2;
		if ( "$prefix" eq "~" && ($rct1 || $out1 || $rct2 || $out2) ){
		    print "No $prefix dir mapped on $host2\n";
		    print STDERR "No $prefix dir mapped on $host2\n";
		    print "Check $host1 --> $host2:  ";
		    print STDERR "Check $host1 --> $host2:  ";
		    print "$result1 $result2 $result3\n";
		}	
		else {	
		    $result1 = $self->doTransfer("gsiftp://$host1/$prefix1/$remote",
				      "gsiftp://$host2/$prefix2/$third");
		    $result2 = $self->doTransfer("gsiftp://$host2/$prefix2/$third",
				      "file://$check");
		    $result3 = $self->checkResult($value,$check);
		    print "Check $host1 --> $host2:  ";
		    print STDERR "Check $host1 --> $host2:  ";
		    print "$result1 $result2 $result3\n";
		    print STDERR "$result1 $result2 $result3\n";
		    $self->{globalresult} .= " $result1 $result2 $result3" if $self->{globalresult};
		    if ( "$result1 $result2 $result3" ne "[OK] [OK] [OK]" ) { next; }
	        }


		# Delete files related to the "third" host. 
		unlink $check;
		my $out = `edg-gridftp-ls gsiftp://$host2/$prefix2/$remote 2>&1`;
		if ( $out) {               
		    my $rmes = `edg-gridftp-rm gsiftp://$host2/$prefix2/$third 2>&1 `;
		    if ($rmes) {
			print "Remove failed: gsiftp://$host2/$prefix2/$third.\n";
			print "$rmes\n";
		    }
		}    

	    }
	}

       # Make sure the remote file has been transferred
        my $out = `edg-gridftp-ls gsiftp://$host1/$prefix1/$remote 2>&1`;
	if ( $out) {
	    # Clean up the first file that was transferred. 
	    my $rmes = `edg-gridftp-rm gsiftp://$host1/$prefix1/$remote 2>&1 `;
	    if ($rmes) {
		print "Remove failed: gsiftp://$host1/$prefix1/$remote.\n";
		print "$rmes\n";
	    }
        }

	# Remove the local file and the check file. 
	unlink $local;
	unlink $check;

	print "\n";
##PAR    }
    }

    if ( $self->{globalresult} =~ "[FAIL]" ) { return 1; }
    else { return 0; }

}


# Transfer a file from one url to the other.  Check the return
# status.
sub doTransfer {

    my ($self, $src, $dest) = @_;
    $self->{store}->{stepn}++;
    my $nodcau = $nodcau1 || $nodcau2 || " ";
    my $command = "globus-url-copy $nodcau $src $dest";
##Humm    print STDOUT " CMD:: doTransfer: $command \n";
##OLD    my $output = `$command 2>&1`;
    my $arrayref = timeoutWithRC2("$command 2>&1", 20);
##NotTooGood    my $arrayref = timeoutWithRC3("$command", 20);
    my ($rct, $crc, $output) = @$arrayref;
##OLD    my $flag = ($!) ? '[OK]' : '[FAIL]';
#p        print STDERR "doTransfer $command $rct $crc $output\n";
    my $flag = ( ! $crc ) ? '[OK]' : '[FAIL]';
#NO    $flag = '[FAIL]' if $rct;
    chomp(my $dateST = `date '+%y%m%d-%H%M%S'`);
    if ($output ne "") {
        $output = "doTransfer $command FAILED ==> \n$output\n";
	print STDOUT $output;
	print STDERR "++++ $dateST $$ $output";
    }
    if ( $rct ) {
	$flag = '[FAIL]';
	$output = "doTransfer $command TIMEOUTed \n";
	print STDOUT $output;
	print STDERR "++++ $dateST $$ $output";
    }
    my $stepn = $self->{store}->{stepn} + $self->{store}->{stpoff};
    $self->fillResHash2D($stepn, $flag, $output, "doTransfer", "$command");
    return ($flag);
}


# Transfer a file from one url to the other.  Check the return
# status.
sub doTransferWK {

    my ($self, $src, $dest) = @_;
    $self->{store}->{stepn}++;
    my $nodcau = $nodcau1 || $nodcau2 || " ";
    my $command = "globus-url-copy $nodcau $src $dest";
##Humm    print STDOUT " CMD:: doTransfer: $command \n";
##OLD    my $output = `$command 2>&1`;
####    my $arrayref = timeoutWithRC2("$command 2>&1", 20);
    my $href = mytimeout("$command", 20);
    my %hh = %{$href};
    my @hhh = %hh;
####    my ($rct, $crc, $output) = @$arrayref;
    my ($rct, $output) = ($hh{'rc'}, $hh{'out'});
##OLD    my $flag = ($!) ? '[OK]' : '[FAIL]';
#p        print STDERR "doTransfer $command $rct $crc $output\n";
        print STDERR "doTransfer $command @hhh\n";
####    my $flag = ( ! $crc ) ? '[OK]' : '[FAIL]';
    my $flag = ( ! $rct ) ? '[OK]' : '[FAIL]';
#NO    $flag = '[FAIL]' if $rct;
    if ($output ne "") {
        $output = "doTransfer $command FAILED ==> \n$output\n";
	print STDOUT $output;
    }
####    if ( $rct ) {
    if ( $rct == -1 ) {
	$flag = '[FAIL]';
	$output = "doTransfer $command TIMEOUTed \n";
	print STDOUT $output;
    }
    my $stepn = $self->{store}->{stepn} + $self->{store}->{stpoff};
    $self->fillResHash2D($stepn, $flag, $output, "doTransfer", "$command");
    return ($flag);
}


sub doTransferOLD {

    my ($self, $src, $dest) = @_;
    $self->{store}->{stepn}++;
    my $nodcau = $nodcau1 || $nodcau2 || " ";
    my $output = `globus-url-copy $nodcau $src $dest 2>&1`;
    my $flag = ($!) ? '[OK]' : '[FAIL]';
    if ($output ne "") {
        $output = "doTransfer $src TO $dest FAILED ==> \n$output\n";
	print STDOUT $output;
    }
    my $stepn = $self->{store}->{stepn} + $self->{store}->{stpoff};
    $self->fillResHash2D($stepn, $flag, $output, "doTransfer", "globus-url-copy $nodcau $src $dest");
    return ($flag);
}

# Fill the given file with a new random number.
sub fillFile {
    my $fname = shift;
    my $value = rand();
    open RAN, ">$fname";
    print RAN "$value\n";
    close RAN;
    return $value;
}

sub checkResult {
    my ($self, $value, $fname) = @_;
    $self->{store}->{stepn}++;
    my $val = `cat $fname`;
    chomp($val);
    my $result = ("$value" eq "$val") ? '[OK]' : '[FAIL]';
    my $output = "";
    chomp(my $dateST = `date '+%y%m%d-%H%M%S'`);
    if ( "$result" ne "[OK]" ) {
	$output = " checkResult $fname $value DIDN'T COMPARE TO $val \n";
	print STDOUT $output;
	print STDERR "++++ $dateST $$ $output";
    }
    my $stepn = $self->{store}->{stepn} + $self->{store}->{stpoff};
    $self->fillResHash2D($stepn, $result, $output, "checkResult", "compare");
    return ($result);
}

sub fillResHash2D {
    return unless $ENV{MAINSCRIPT};
    my ($self, $stepn, $stat1, $olout, $sdesc, $scomm) = @_;
    my $case = $self->{store}->{src};
    my $mace = $self->{store}->{tgt};
    my $sname = $self->{store}->{label};
    my @m = split /\s+/, $sname;
#     if ( "$scomm" eq "globus-url-copy" ) {
# 	if ( $stepn%3 == 0 ) { $scomm .= " $m[1] $m[3]"; }
# 	else { $scomm .= " $m[3] $m[1]"; }
#     }
    if ( $stepn < 3 ) { $sname = "Steps 0 to 2 are for Transfers to/from \"/tmp\" directory on remote node"; }
    else { $sname = "Steps 3 to 5 are for Transfers to/from USER's HOME directory on remote node"; }
    $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1} = $stat1;
    $self->{reqs}->{$case}->{$mace}->{$stepn}->{olout} = $olout;
    $self->{steps}->{$stepn}->{sname} = $sname;
    $self->{steps}->{$stepn}->{sdesc} = $sdesc;
    $self->{steps}->{$stepn}->{scomm} = "Ex: $scomm";
    $self->{mach}->{$mace}++;
    $self->{case}->{$case}++;
#p    print " fillResHash2D: $stepn $case $mace \n";
    return;
}

sub fillResHash2DNew {
    return unless $ENV{MAINSCRIPT};
    my ($self, $stepn, $stat1, $olout, $sdesc, $scomm, $case, $mace) = @_;
#    my $case = $self->{store}->{src};
#    my $mace = $self->{store}->{tgt};
#    my $sname = $self->{store}->{label};
    my $sname = "Check $case --> $mace";
#    my @m = split /\s+/, $sname;
#     if ( "$scomm" eq "globus-url-copy" ) {
# 	if ( $stepn%3 == 0 ) { $scomm .= " $m[1] $m[3]"; }
# 	else { $scomm .= " $m[3] $m[1]"; }
#     }
    if ( $stepn < 3 ) { $sname = "Steps 0 to 2 are for Transfers to/from \"/tmp\" directory on remote node"; }
    else { $sname = "Steps 3 to 5 are for Transfers to/from USER's HOME directory on remote node"; }
    $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1} = $stat1;
    $self->{reqs}->{$case}->{$mace}->{$stepn}->{olout} = $olout;
    $self->{steps}->{$stepn}->{sname} = $sname;
    $self->{steps}->{$stepn}->{sdesc} = $sdesc;
    $self->{steps}->{$stepn}->{scomm} = "Ex: $scomm";
    $self->{mach}->{$mace}++;
    $self->{case}->{$case}++;
    print " fillResHash2DNew: $stepn $case $mace \n";
    return;
}

sub buildXMLFile {
    my $self = shift;
#p    $self->printStack(3);
    my $ret = $self->buildXMLFile2D();
    return $ret;
}

## Type with several rows, several steps and several columns 
sub buildXMLFile2Dold {  ##  See EDGUtils from now on
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
#    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_steps;
    my @lof_rows;
#notbad    my $Title = (caller(0))[3];
    my $Title = __PACKAGE__;
    chomp(my $date = `date`);
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile() to write $self->{xmlfile} \n";

#    my $cols = {};
    ## Col names
    foreach my $mace ( sort keys %{$self->{mach}} ) {
	push @lof_cols, $mace;
    }

#    my $cols = {};
    ## Row names
    foreach my $case ( sort keys %{$self->{case}} ) {
	push @lof_rows, $case;
    }

    ## List of steps ... (steps are numbers)
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	push @lof_steps, "Step_$stepn";
    }

    my $nsteps = scalar(@lof_steps)-1;
    print STDERR " Called buildXMLFile() to cols @lof_cols \n";
    print STDERR " Called buildXMLFile() to rows @lof_rows \n";
    my $rowM = "";
    foreach my $row ( @lof_rows ) {
	my $colM = "";
	foreach my $mace ( @lof_cols ) {
	    my $stepM = "";
	    for my $stepn (0..$nsteps) {
		my $retv = $ret{$self->{reqs}->{$row}->{$mace}->{$stepn}->{stat1}};
		my $logf = $self->{reqs}->{$row}->{$mace}->{$stepn}->{olout};
		$stepM .= makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step");
	    }
	    $colM .= makeX($stepM, "col");
	}
	$rowM .= makeX($colM, "row");
    }

    ## building the step info list ...
    my $stepI = "";
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	my $brf_info = "Step_$stepn";
#old	my $brf_info = $self->{steps}->{$stepn}->{sname};
	$brf_info =~ s/ /_/g;
#old	my $det_info = $self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	my $det_info = $self->{steps}->{$stepn}->{sname}." : \n".$self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	$stepI .= makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
    }

    my $body = $rowM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
$info .= makeX("@lof_rows", "rows_list");
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepI;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDERR makeX($info." \n ".$body, "log");

    return 0;
}

# Run job submissions in parallel.  This takes all of the parameters
# from the state of the object. 
## ***************************************************************************************
sub parallelSubmit {

    my ($self, $jOffset, $dir, $hblockref, $hostsref) = @_;
    my @pids;
    my @phs;
#    $jOffset /= scalar(@hosts);

##PAR    foreach my $i (1 .. $self->{opts}->{nstreams}) {
    my $i = 1;
	foreach my $host (@$hblockref) {

	      if ( $ENV{RBhostname_} =~ /$host/ ) { next; }

	    local *PH;
      FORK: { 
	    my $pid;
  die "cannot fork: $!" unless defined ( $pid = open ( PH, "-|" ) );
#          if ($pid = fork) {
          if ($pid) {
              # parent here
              # child process pid is available in $pid
              push @pids, $pid;
	      push @phs, *PH;
          } elsif (defined $pid) {
              # $pid is zero here if defined
              # child here
              # parent process available with getppid

	      STDOUT->autoflush(1);
	      STDERR->autoflush(1);

	      # Do the actual submission.  The error return is ignored
	      # because there is no way to feed back this information
	      # easily.  
# 	      my $rc = $self->submit(["$self->{prefix}.$i",
# 				      "$self->{opts}->{njobs}"]);
	      my $jO = $jOffset+$i;

	      if ( $ENV{RBhostname_} =~ /$host/ ) { next; }

	      chomp(my $dateST = `date '+%y%m%d-%H%M%S'`);
#p	      print STDOUT "++++ $dateST $$/n Starting testGridFTP with $host (job# $jO) $dir \n";
	      print STDOUT "++++ $dateST $$\n Starting testGridFTP with $host (job# $jO) $dir \n";
	      my $tStart = time();
	      my $rc = $self->testGridFTP($dir, $host, $hostsref);
	      my $tAlmost = time() - $tStart;
	      print STDOUT " TimeFTP: $tAlmost \n";
              # This is important otherwise child may execute parent code!
              exit;
          } elsif ($! =~ /No more process/) {
              # Recoverable error (EAGAIN); try again. 
              sleep 5;
              redo FORK;
          } else {
              # Strange error! Print error message. 
              print STDERR "Fork failure.\n";
          }
      }
#	$self->{job}->{submit}->{file} = "$self->{prefix}.$i" if ( $self->{opts}->{nstreams} == 1 );
    }
##PAR    }

    return (\@pids, \@phs);
}

# Evaluate the results of the test.
sub evaluateTest {
    my $self = shift;

    print "GridFTP: evaluateTest\n";

    my $stepn = 0;

    if ( 1 ) {
    $self->{steps}->{$stepn}->{sname} = "GridFTP";
    $self->{steps}->{$stepn}->{sdesc} = "9 steps";
    $self->{steps}->{$stepn}->{scomm} = "Several commands";
}

    my @logs = @{$self->{logs}};
    print " GridFTP: LOGS ", scalar(@logs), " \n";
    foreach my $logref ( @logs ) {
	my @lines = grep { /Check |testGridFTP/ } @$logref;
	$stepn++;
#p	print " Log: $stepn\n@lines\n";
	print " Log= $stepn\n@$logref\n";
    }


    if ( 1 ) {
#     my @ls = grep { s/ STEPSTEP:// } @{$logs[0]};
#     $self->{steps}->{$stepn}->{scomm} = "@ls";

    foreach my $logref ( @logs ) {
	my @lines = grep { /Check |testGridFTP/ } @$logref;
	my @m = split /\s+/, $lines[0];
	my $failed = grep { /\[FAIL\]/ } @lines;
	my $oked   = grep { /\[OK\]/ }   @lines;
	chop $m[6];
	print " Eval: $m[4] $m[6] $failed $oked \n";
	my $case = "Job_$m[6]";
	my $mace = $m[4];
	my $prefix = $m[7];

    if ( "$prefix" eq "/tmp" ) { $self->{store}->{stpoff} = -1; }
    else { $self->{store}->{stpoff} = 2; }

#	foreach my $lg ( @lines[1,2,3] ) {
	shift @lines;
	foreach my $lg ( @lines ) {
	    my @m = split /\s+/, $lg;
#	    $m[1] =~ s/\./<br> @ /;
	    chop($m[3]);
	    $stepn = $self->{store}->{stpoff};
	    foreach my $fg ( @m[4,5,6] ) {
		$stepn++;
		$self->fillResHash2DNew($stepn, $fg, "@$logref", "doTransfer", "command", $m[1], $m[3]);
		$self->{globalresult} .= " $fg" if $self->{globalresult};
	    }
	}

# 	$mace =~ s/\./<br> @ /;
# 	$self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1} = ( $oked == 11 && $failed == 0 ) ? "[OK]" : "[FAIL]";
# 	$self->{reqs}->{$case}->{$mace}->{$stepn}->{olout} = "@lines";
# 	@m = split /\s+/, $lines[-1];
# 	$self->{reqs}->{$case}->{$mace}->{$stepn}->{stat2} = "$m[2]"."sec";
# 	$self->{mach}->{$mace}++;
# 	$self->{case}->{$case}++;
# 	$self->{globalresult} .= " $self->{reqs}->{$case}->{$mace}->{$stepn}->{stat1}" if $self->{globalresult};

    }

}

    return 0;
}

## ***************************************************************************************
sub buildXMLFileNEWBAD {
    my $self = shift;
#    my $ret = $self->buildXMLFile1D();
    my $ret = $self->buildXMLFile2D();
    return $ret;
}

## Type with several rows, several steps and several columns 
## ***************************************************************************************
sub buildXMLFile2DNEWBAD {
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_steps;
    my @lof_rows;
#notbad    my $Title = (caller(0))[3];
    my $Title = __PACKAGE__;
    chomp(my $date = `date`);
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile() to write $self->{xmlfile} \n";

#    my $cols = {};
    ## Col names
    foreach my $mace ( sort keys %{$self->{mach}} ) {
	push @lof_cols, $mace;
    }

#    my $cols = {};
    ## Row names
    foreach my $case ( sort keys %{$self->{case}} ) {
	push @lof_rows, $case;
    }

    ## List of steps ... (steps are numbers)
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	push @lof_steps, "Step_$stepn";
    }

    my $nsteps = scalar(@lof_steps)-1;
    print STDERR " Called buildXMLFile() to cols @lof_cols \n";
    print STDERR " Called buildXMLFile() to rows @lof_rows \n";
    my $rowM = "";
    foreach my $row ( @lof_rows ) {
	my $colM = "";
	foreach my $mace ( @lof_cols ) {
	    my $stepM = "";
	    for my $stepn (0..$nsteps) {
		my $retv = $ret{$self->{reqs}->{$row}->{$mace}->{$stepn}->{stat1}};
		my $subs = $self->{reqs}->{$row}->{$mace}->{$stepn}->{stat2};
		my $logf = $self->{reqs}->{$row}->{$mace}->{$stepn}->{olout};
		$logf =~ s/&/&amp;/g; # This must be changed first.
		$logf =~ s/\"/&quot;/g;
		$logf =~ s/</&lt;/g;
		$logf =~ s/>/&gt;/g;
#good		$stepM .= makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step");
		$stepM .= makeX(makeX("$logf", "output").makeX("$retv", "retval").makeX("$subs", "substatus"), "step");
	    }
	    $colM .= makeX($stepM, "col");
	}
	$rowM .= makeX($colM, "row");
    }

    ## building the step info list ...
    my $stepI = "";
    foreach my $stepn ( sort { $a <=> $b } keys %{$self->{steps}} ) {
	my $brf_info = "Step_$stepn";
#old	my $brf_info = $self->{steps}->{$stepn}->{sname};
	$brf_info =~ s/ /_/g;
#old	my $det_info = $self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	my $det_info = $self->{steps}->{$stepn}->{sname}." : \n".$self->{steps}->{$stepn}->{sdesc}." : \n".$self->{steps}->{$stepn}->{scomm};
	$stepI .= makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
    }

    my $body = $rowM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
@lof_rows = map { makeX($_, "row"); } @lof_rows;
$info .= makeX("@lof_rows", "rows_list");
@lof_cols = map { makeX($_, "col"); } @lof_cols;
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepI;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDERR makeX($info." \n ".$body, "log");

    return 0;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
