#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack;

use File::Basename;
use EDGUtils;

use POSIX qw(:signal_h :errno_h :sys_wait_h);
use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle);

           use IPC::SysV qw(IPC_PRIVATE S_IRWXU IPC_CREAT IPC_NOWAIT);
#          use IPC::Semaphore;
           use IPC::Msg;

=head1 NAME

Stack.pm - Check that globusrun authentication works on the hosts
# $Id: Stack.pm,v 1.1.1.2 2005/02/08 16:09:42 mbarroso Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<Stack.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

## ***************************************************************************************
sub setContext {
    my $self = shift;
    my $stack = shift || 1;
    $self->SUPER::setContext( $stack );

#    print "Stack: setContext\n";
    $self->{debug} = 1;  ## auxiliary to new printLocation method.
    $self->printLocation( (caller(0))[3] );

    # Add some storm size options.

    $self->addOption('zombSlep=i',
		     'Monitoring idling time (in sec).',
                     '10');
    $self->addOption('childTimeout=i',
		     'Elapse time after which the child is killed (in sec).',
                     '1800');
    $self->addOption('pollingPeriod=i',
		     'Polling period for edg-job-status (in sec).',
                     '30');
    $self->addOption('batchSleep:i',
		     'Sleep time for the batch job.');
    $self->addOption('jobCancel=i',
		     "To run a edg-job-cancel --all before starting the submission.\n\tIt specifies the waiting time in sec after this command is run.");
    $self->addOption('singleSubmit!',
		     "When set, DOES NOT Allow resubmitting a request which failed during edg-job-submit phase.\n\tPlease note this is DIFFERENT from the retrycount option");
    $self->addOption('useCEList',
		     'Use CE list build from a pre-matching test to submit the requests (disables RB matchmaking).');
    $self->addOption('circular',
                     'To scan circularly the CEList built with useCEList');
    $self->addOption('multiRB:s',
		     'Comma Separated List of RBs to submit to (if different from default one).');
    $self->addOption('display:s',
		     "List of (col,row) settings to display the results via XML (default: \"mace,mawn\").\n\tPossible setups are: [1Col,[mawn|mace|jind]]|[mace,[mawn|jind]]|[mawn,[mace|jind]].");
    $self->addOption('CalStress',
		     'Use Cals method for stressing the submission to the NS/RB.');
    $self->addOption('selectNodes:s',
		     "Space separated list of CE nodes where the matching check will be run against.\n\tThese nodes must be part of the gang matching list, as returned by the buildCEList routine.\n\tRequires option useCEList");
    $self->addOption('useSemaphores=i',
		     'Use IPC::Msg and monitorJobs to inform SubmitUtils about job completion.');
    $self->addOption('site:s',
		     'One CE to target (no default)');

    # Commands necessary to run this test.
    $self->{commands} = [dgjob("submit"), 
			 dgjob("get-output"), 
			 dgjob("status"),
			 dgjob("cancel"),
			 dgjob("list-match"),
			 dgjob("get-logging-info"),
			 "SubmitUtils.pm",
			 "grid-proxy-info"];

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack [options]

END
    ;

    $self->{DescText} = <<'END'

This submits a simple job which simply echos the [OK] string.  It does
this from multiple streams of multiple jobs, i.e. a job "storm".

END
    ;

	$self->{type} = "";
        $self->{debug} = 1;  ##  To print subroutine location headers

## Given thru options ...
	$self->{maxStack}  = "";
	$self->{maxSubs}   = "";
	$self->{reqLapse}  = "";
	$self->{zombSlep}  = "";
	$self->{serie}     = "";

## Working variables ...
	$self->{harvest} = "";
	$self->{zombies} = "";
	$self->{user1}   = "";

# Hashes 
	$self->{plist} = {};    ## Hash
	$self->{hlist} = {};    ## Hash
	$self->{reqs}  = {};    ## Hash
	$self->{procs}  = {};    ## Hash

	$self->{jdlFile} = "";
	$self->{shFile}  = "";

	$self->{outFile} = "";
	$self->{saveOut} = "";

## Internals ...
	$self->{date}     = "";
	$self->{hostname} = "";

}


# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.) 
## ***************************************************************************************
sub setupTest {
    my $self = shift;

#    print "Stack: setupTest\n";
    $self->printLocation( (caller(0))[3] );

    # Call the parent method to ensure that the temporary directory is
    # created.  
    my $error = $self->SUPER::setupTest();
    if ($error) {
        $self->setErrorMessage('Stack setup failed');
        return 1;
    }

    # Retrieve the temporary directory. 
    my $tmpdir = $self->{tempdir};

    STDOUT->autoflush(1);
    STDERR->autoflush(1);

    chomp($self->{hostname} = `hostname`);
    chomp(my $date = "@" . `date`);
    $date =~ s/\s+/_/g;
    $date .= "@";
    $self->{date} = $date;

    ## Specific to the Stack ...
    $self->{maxStack} = $self->{opts}->{maxStack};
    $self->{maxSubs} = $self->{opts}->{maxSubs};
    $self->{reqLapse} = $self->{opts}->{reqLapse};
    $self->{zombSlep} = $self->{opts}->{zombSlep};
    $ENV{STACK_SERIE} = $self->{opts}->{serie};
    $self->{outFile} = "/tmp/$ENV{USER}-$$-"."$self->{type}.log$ENV{STACK_SERIE}";
    $ENV{STACK_POLLING_PERIOD} = $self->{opts}->{pollingPeriod};
    $ENV{STACK_CHILD_TIMEOUT} = $self->{opts}->{childTimeout};
    if ( $self->{opts}->{useSemaphores} ) {
	if ( $self->{opts}->{useSemaphores} == 1 ) { $self->{opts}->{useSemaphores} = '123457'; }
	elsif ( $self->{opts}->{useSemaphores} == 2 ) { $self->{opts}->{useSemaphores} = 1000000+$self->{opts}->{serie}; }
	$ENV{STACK_SEMAPHORES} = $self->{opts}->{useSemaphores};
    }

    if ( $self->{opts}->{batchSleep} ) {
	if ( $self->{opts}->{childTimeout} < $self->{opts}->{batchSleep} ) {
	    print STDERR " When using --batchSleep option, take care to increase --childTimeout (default=1800 sec) \n by the same value\n";
	    exit 1;
	}
    }

    ## Redirect STDOUT and STDERR to outFile
    $self->{saveOut} = switchStdouterr($self->{outFile}) if (0) ;

    # Create the necessary setup used prior to the Storm itself (variables ...)
    $error = $self->setupEnvironment();
    if ($error) {
        $self->setErrorMessage('Specific Environment Setup failed');
        return 1;
    }

    ## Check and/or create the Storm Specific setup ...
    $error = $self->stormSetup();
    if ($error) {
        $self->setErrorMessage('Specific Storm Setup failed');
        return 1;
    }

#old    my $namJDL = "/tmp/$ENV{USER}-$$-"."$self->{type}.jdl";
#old    my $namSCR = "/tmp/$ENV{USER}-$$-"."$self->{type}.sh";
    my $namJDL = "$self->{tempdir}/"."$self->{type}.jdl";
    my $namSCR = "$self->{tempdir}/"."$self->{type}.sh";


    # Create the script for this job.
##    my $script = $self->createScript($tmpdir);
    my $script = $self->createScript($namSCR);
    if ($script) {
	$self->{shFile} = $script;
    } else {
        $self->setErrorMessage('could not create job script');
        return 1;
    }

    # Create the JDL file.
##    my $file = $self->createJDL($tmpdir, $script, $self->{opts}->{site});
    my $file = $self->createJDL($namJDL, $script, $self->{opts}->{site});
    if ($file) {
        $self->{jdlFile} = $file;
    } else {
        $self->setErrorMessage('could not create JDL file');
        return 1;
    }

    ## Setup the RB stack if required ...
    if ( $self->{opts}->{multiRB} ) {
	my @rbList = split /,/, $self->{opts}->{multiRB};
	@{$self->{rblist}} = ();
	foreach my $rb ( @rbList ) {
	    my @m = split /\./, $rb;
	    $self->{count}->{$m[0]} = 0;
	    push @{$self->{rblist}}, $m[0], $rb;
	}
    }

    ## Set the Child Reaper up ...
##    $self->{zombies} = 0;
##    $SIG{CHLD} = sub { $self->{zombies}++ };

## gg for XML creation and processing ...
#prevGOOD    if ( $ENV{MAINSCRIPT} ) {
    if ( $self->{opts}->{xml} ) {
#old	my $base = "/tmp/$ENV{USER}-$$";
	my $base = "$self->{tempdir}";
#	$self->{conffile} = "$base/GridFTPconf";
##	$self->{outfile}  = "$base/GatekeeperOut";
	$self->{xmlfile}  = "$base/".$self->{type}.".XML";
	## To trigger the processing thru buildXMLParser
	$self->{xmlxist}  = "$self->{type}";

	my $html_dir = ( defined $ENV{HTML_DIR} ) ? "$ENV{HTML_DIR}" : "$ENV{HOME}/TSTG/HTML";
	$html_dir = "${html_dir}/$self->{type}";
	my $resmk = `mkdir -p $html_dir`;
	$self->{htmldir}  = "$html_dir";
    }

    ## Creating the jobids file ...
    my $namJDB = "$self->{tempdir}/"."jobids";
    $self->{stackjobids} = $namJDB;
    $self->{stackjnotdone} = "${namJDB}.notgone";
    my $eRes = `echo \"#\" VO: $self->{opts}->{vo} > $namJDB `;

    ## Print a banner for the beginning of the test itself
    print " SUM Checking GG Type: $self->{type} Serie: $ENV{STACK_SERIE} Jobs: $self->{maxSubs} Date: $self->{date} Delay: $self->{reqLapse} and StackSize: $self->{maxStack} \n";

    ## IPC stuff (try) ...
    if (0) {
#            my $sem = new IPC::Semaphore('123456', 248, S_IRWXU | IPC_CREAT);
#            $sem->setall( (0) x 248);
#            $self->{sem} = $sem;
       } else {
	   if ( $self->{opts}->{useSemaphores} ) {
#	   my $msg = new IPC::Msg('123457', S_IRWXU | IPC_CREAT);
	       my $msg = new IPC::Msg($self->{opts}->{useSemaphores}, S_IRWXU | IPC_CREAT);
	       $self->{msg} = $msg;
	   }
       }

    ## Erase any left over job for this user ...
    if ( $self->{opts}->{jobCancel} ) { ## temporary disabled: does not work very well in edg-1.4.3 ...
	my $cmd = dgjob("cancel")." --all --noint";
	my $ca = `$cmd 2> /dev/null `;
	my $sleep = `sleep $self->{opts}->{jobCancel}`;
	print " CANCEL: $ca";
    }

    return 0;
}

# Build any necessary setup for the storm.
## ***************************************************************************************
sub buildCEList {
    my $self = shift;

#old    print "DataStorm: setupEnvironment\n";
    $self->printLocation( (caller(0))[3] );


     print STDERR " Ready ? $self->{jdlFile} $self->{shFile} \n";

    @{$self->{CEs}} = ();

	## Achieving the matching indeed ...
        my $cmdl = dgjob("list-match");
	my $command = "$cmdl $self->{jdlFile}";
# 	print "\n InputDatal = {$LFNs{$LFNl[$n0]}, $LFNs{$LFNl[$n1]}, $LFNs{$LFNl[$n2]}} \n";
# 	print " InputData  = {$LFNs{$LFNl[$n0]}, $self->{GUIDs}->{$LFNl[$n1]}, $LFNs{$LFNl[$n2]}} \n";

	my $resEJLM = `$command 2>&1`;
    print $resEJLM;  ##  ?? useless ??

	## Retrieving the matching CEs ...
	my %seen = ();
#old	my @machL = grep { ! $seen{$_}++ } map { /(\S+):2119\/jobmanager-/ } split /\s+/, $resEJLM;
	my @machL = grep { ! $seen{$_}++ } map { /(\S+):2119\/\w+?-/ } split /\s+/, $resEJLM;
	print " matchingCEs: @machL\n";

	## Retrieving the matching CE queues ...
     	%seen = ();
#old	my @machS = grep { ! $seen{$_}++ } map { /(\S+:2119)\/jobmanager-/ } split /\s+/, $resEJLM;
#old	   @machS = grep { /\S+:2119\/jobmanager-/ } split /\s+/, $resEJLM;
	my @machS = grep { ! $seen{$_}++ } map { /(\S+:2119)\/\w+?-/ } split /\s+/, $resEJLM;
	   @machS = grep { /\S+:2119\/\w+?-/ } split /\s+/, $resEJLM;
	my %CEq;
	foreach my $ma ( @machS ) {
#p	    print " ma: $ma \n";
#old	    $ma =~ /(\S+:2119)\/(jobmanager-\S+)/;
	    $ma =~ /(\S+:2119)\/(\w+?-\S+)/;
#p	    print " ma12: $1 $2 $seen{$1} \n";
	    if ( $seen{$1} ) {
		my $mb = $1;
 		my @m = split /:/, $mb;

 		## To filter out unwanted nodes  ...
 		if ( ! $self->{opts}->{selectNodes} ) {
 		    $CEq{$mb} = $ma;
 		} elsif ( $self->{opts}->{selectNodes} =~ /$m[0]/ ) {
		    $CEq{$mb} = $ma;
		}
	    }
	}

    @{$self->{CEs}} = values %CEq;
	print " MatchingCEqueues: @{$self->{CEs}} \n";
#p    print STDERR " CEs: @{$self->{CEs}} \n";

    if ( $self->{opts}->{useCEList} ) {
	if ( ! scalar(@{$self->{CEs}}) ) {
	    $self->setErrorMessage('Setup failed while building the CEList which returned empty');
	    return 1;
	}	
    }

    return 0;
}


# This takes the JDL file and submits it.  This overrides the original
# to allow the multiple streams and multiple jobs.
## ***************************************************************************************
sub runTest {
    my $self = shift;
    my $iter = shift || "noit";

#    print "Stack: runTest\n";
    $self->printLocation( (caller(0))[3] );

    ## To set the Child Reaper up ...
    $self->{zombies} = 0;
    $SIG{CHLD} = sub { $self->{zombies}++ };

    ## To be able to catch CTRL-C ... (USR1 => kill -10 under RH7.3)
    $self->{user1} = 0;
    $SIG{INT}  = sub { print STDOUT " Signal SIGINT received: $$ \n";  $self->{user1}++ };
    $SIG{USR1} = sub { print STDOUT " Signal SIGUSR1 received: $$ \n"; $self->{user1}++ };

#my $ph  = 0;
#my $pid = 0;
my $index = 0;
$self->{harvest} = 0;
$self->{plist} = {};
$self->{hlist} = {};
my $currTime = time();
my $reqLapse = $self->{reqLapse};
    my $forceload = 0;
    $self->{stacksize} = 0;

MAIN: while ( 1 ) {

  $forceload = $self->cforceload($index);

## To actually enforce the delay ...
#1012 if ( (time()-$currTime) < $reqLapse && $index > 0 ) {
  if ( ( (time()-$currTime) < $reqLapse && $index > 0 ) && ! $forceload ) {
#171203    my $sleep = `sleep $reqLapse`;
    sleep $reqLapse;
    next MAIN;
  } 

  if ( ! $self->{user1} ) {

## Submitting the request ...
#1012  if ( scalar(keys(%{$self->{plist}})) < $self->{maxStack} && $index < $self->{maxSubs} ) {
      if ( ( scalar(keys(%{$self->{plist}})) < $self->{maxStack} || $forceload ) && $index < $self->{maxSubs} ) {

    ## Building the argument list
#old    my $jdlTemp = "/tmp/$ENV{USER}-$$-tmparg-$index.jdl";
    my $jdlTemp = "$self->{tempdir}/tmparg-$index.jdl";
    my $bid = `cat $self->{jdlFile} > $jdlTemp`;

    my $rc = $self->addArguments($jdlTemp, $index);
##    exit 1 if ( ! $rc );
    return $rc if $rc;

    ## Selecting the RB if required ...
    if ( $self->{opts}->{multiRB} ) {
	my @rbs = sort { $self->{count}->{$a} <=> $self->{count}->{$b} } keys %{$self->{count}};
	my $rb = $rbs[0];

	## (Re)-building the hash ...
	my @rbL = @{$self->{rblist}};
	my %rbH = ();
	while ( my $rbc = shift @rbL ) {
	    my $rbl = shift @rbL;
	    $rbH{$rbc} = $rbl;
	}
	## And fetching the full RB name from the short one ...
	my $rbf = $rbH{$rb};

	## setenv ...
	## --config
	$ENV{EDG_WL_UI_CONFIG_VAR} = "$ENV{EDG_TESTROOT}/etc/broker_conf/edg_wl_ui_cmd_var-${rbf}.conf";
	## --config-vo
	$ENV{EDG_WL_UI_CONFIG_VO}  = "$ENV{EDG_TESTROOT}/etc/broker_conf/edg_wl_ui-${rbf}.conf";

	$self->{reqs}->{$index}->{rb} = $rb;
	$self->{count}->{$rb}++;
	print " RBS\t$index :\n", map { "\t$_\t=>\t$self->{count}->{$_} \n" } sort keys %{$self->{count}};
    }

#p    my @resJDL = `cat $jdlTemp`;
#p    print STDOUT " resJDL: \n@resJDL\n";

      ## Creating the log name (if required ...)
#      my $tmpdir = $self->{tempdir};
#old      my $logFile = "/tmp/$ENV{USER}-$$-"."$self->{type}-${index}.log";

      my $citer = "";
      if ( "$iter" ne "noit" ) { $citer = "${iter}-"; }

      my $logFile = "$self->{tempdir}/$citer"."$self->{type}-${index}.log";
      $self->{reqs}->{$index}->{olog} = $logFile;
      my $outFile = "$self->{tempdir}/$citer"."$self->{type}-${index}.out";

##    my ($pid, $ph) = submit($jdlTemp, $index, $logFile);
    my ($pid, $ph) = submitTimeout($jdlTemp, $index, $logFile, $outFile, $self->{opts}->{childTimeout});
    $self->{plist}->{$pid} = $index;
    $self->{hlist}->{$pid} = $ph;
    $self->{reqs}->{$index}->{pid} = $pid;
    my ($nprocs, $ndescr, $freemem, $freeswap) = monitorProcs();
#useless    $self->{reqs}->{$index}->{ph}  = $ph;
    print "PID $index / $self->{maxSubs} / stacksize: $self->{stacksize} / pid: $pid / procs: $nprocs / descr: $ndescr / freemem: $freemem / freeswap: $freeswap \n";
    $index++;
    $currTime = time();
#171203    my $sleep = `sleep $reqLapse`;
    sleep $reqLapse;

  }

}  ## End of if_user1

## We come here only when we're done ...
  if ( $self->{harvest} >= $self->{maxSubs} ) { 
#     print " We are done with $index $self->{harvest} ", scalar(keys(%{$self->{plist}})), " \n";
#     print " SUM Ending GG $self->{type}: serie $ENV{STACK_SERIE}: We are done with $index $self->{harvest} \n";
#     last; 
      my $rc = $self->submitOver($index);
      last MAIN; 
  }

## Harvesting the dead children ...
  my $nzomb = 0;
  while ( 1 ) {
      last if $self->{zombies};
#old      next MAIN if ( scalar(keys(%{$self->{plist}})) < $self->{maxStack} );
      if ( $self->{user1} ) {
	  print STDOUT " SelfUsr1: $self->{harvest} $self->{maxSubs} $index \n";
#bad?	  if ( $self->{harvest} >= $self->{maxSubs} ) {
	  if ( $self->{harvest} >= $index ) {
	      my $rc = $self->submitOver($index);
	      last MAIN;
	  }
      } else {

## Test for special submit stack ...
# 	  if (0) {
# #	  my $jobidfile = "$self->{tempdir}/jobids";
# #p	  print " SPECIAL: index: $index zombies: $self->{zombies} $nzomb \n";
# #	  my ($dum, $stacksize, $rest) = split /\s+/, `wc $jobidfile`;
# 	  my ($dum, $stacksize, $rest);
# 	  if (0) {
# 	  open SZ, "wc -l $jobidfile |";
# 	  while (<SZ>) {
# 	      ($dum, $stacksize, $rest) = split /\s+/;
# 	      last;
# 	  }
# 	  close SZ;
# 	  $self->{zombies}--;
#       } else {
# 	  open SZ, "$jobidfile";
# 	  1 while <SZ>;
# 	  $stacksize = $.;
# 	  close SZ;
#       }
# 	  $stacksize--;
# #p	  print " SPECIAL: index: $index stacksize: $stacksize $self->{zombies} $nzomb \n";
# 	  $forceload = ($index-$stacksize) < $self->{maxStack} ;
# #bad?	  next MAIN if ( ($index-$stacksize) < $self->{maxStack} );
#       }

	  $forceload = $self->cforceload($index);
	  next MAIN if ( $forceload && $index < $self->{maxSubs} );
## -- EndOF

	  next MAIN if ( scalar(keys(%{$self->{plist}})) < $self->{maxStack} );
      }
      $nzomb++;
      last if ( $nzomb*$self->{zombSlep} > 900 );
#child      my $sleep = `sleep $self->{zombSlep}`;
      sleep $self->{zombSlep};
  }
  continue {
      $self->checkSem();
    my ($nprocs, $ndescr, $freemem, $freeswap) = monitorProcs();
    print "PID1 procs: $nprocs / descr: $ndescr / freemem: $freemem / freeswap: $freeswap \n";
  }

  print "ZOMBIES $self->{zombies} $nzomb\n" unless ( $self->{zombies}*$nzomb == 1 || 
						     "$self->{zombies} $nzomb" eq "2 0" );
  $self->{zombies} = 0;

#   if ( $self->{sem} ) {
#       my $sem = $self->{sem};
#       my @sem = $sem->getall();
#       print " Semaphores: @sem \n";
#   }

  my $rc = $self->waitforChildren();

}  ## End of MAIN while
continue {

    ## Semaphore monitoring ...
      $self->checkSem();

      ## Resource monitoring ...
    my ($nprocs, $ndescr, $freememK, $freeswapK) = monitorProcs();
      my $freemem = $freememK;
      my $freeswap = $freeswapK;
      chop($freemem);
      chop($freeswap);

######################## To reenable
# #       if ( ($self->{procs}->{nprocs} != $nprocs) || ($self->{procs}->{ndescr} != $ndescr) || ($self->{procs}->{freeswap} != $freeswap) ) {
# #     print "PID2 procs: $nprocs / descr: $ndescr / freemem: $freememK / freeswap: $freeswapK \n";
# # }
######################## To reenable
      ($self->{procs}->{nprocs}, $self->{procs}->{ndescr}, $self->{procs}->{freemem}, $self->{procs}->{freeswap}) = ($nprocs, $ndescr, $freemem, $freeswap);

}

  return 0;

}  ## End of runTest routine


## ***************************************************************************************
sub submitOver {

    my $self = shift;
    my $index = shift;

    print " We are done with $index $self->{harvest} ", scalar(keys(%{$self->{plist}})), " \n";
    print " SUM Ending GG $self->{type}: serie $ENV{STACK_SERIE}: We are done with $index $self->{harvest} \n";

    return 0;

}


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
## ***************************************************************************************
sub createJDL {
    my ($self, $dir, $script, $site) = @_;

#    print "Stack: createJDL\n";
    $self->printLocation( (caller(0))[3] );

    my ($sname, $spath, $suffix) = fileparse($script, []);

    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable    = "$sname";
StdOutput     = "std.out";
StdError      = "std.err";
InputSandbox  = {"$script"};
OutputSandbox = {"std.out","std.err"};
END
    ; 

    print JDL "Requirements = RegExp(\".*$site.*\",other.CEId);\n" if $site;
    close JDL;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
## ***************************************************************************************
sub createScript {
    my ($self, $dir) = @_;

#    print "Stack: createScript\n";
    $self->printLocation( (caller(0))[3] );

    my $file = "$dir/JobScript.sh";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<"END"
#!/bin/sh
/bin/echo "Hello From Script"
#/bin/sleep 30
/bin/echo "Goodbye. [OK]"
END
    ; 
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# Build any necessary setup prior to the storm.
## ***************************************************************************************
sub setupEnvironment {
    my $self = shift;

#    print "Stack: setupEnvironment\n";
    $self->printLocation( (caller(0))[3] );

    return 0;
}


# Build any necessary setup for during the storm.
## ***************************************************************************************
sub stormSetup {
    my $self = shift;

#    print "Stack: stormSetup\n";
    $self->printLocation( (caller(0))[3] );

    return 0;
}


# Add any necessary arguments to the existing local JDL file before submission.
## ***************************************************************************************
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

#    print "Stack: addArguments\n";
    $self->printLocation( (caller(0))[3] );

    return 0;
}


# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
## ***************************************************************************************
sub evaluateTest {
    my $self = shift;

#    print "Stack: evaluateTest\n";
    $self->printLocation( (caller(0))[3] );

    return 0 if (0);

    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {

	## Fixing some weird values ...
	if ( $self->{reqs}->{$index}->{mace} eq "free" ) {
	    my @m = split /:/, $self->{reqs}->{$index}->{macef};
#GOOD	    my @m = split /\./, $self->{reqs}->{$index}->{macef};
	    my @m = split /\./, $m[0];
#test	    $self->{reqs}->{$index}->{mace} = "free:$m[0]";
	    my $head = shift @m;
	    my $tail = join ".", @m;
	    $self->{reqs}->{$index}->{mace} = "freeCE:<br>$head<br>@"."$tail";
	} else {
	    my @m = split /\./, $self->{reqs}->{$index}->{mace};
#test	    $self->{reqs}->{$index}->{mace} = "$m[0]";
	    my $head = shift @m;
	    my $tail = join ".", @m;
	    $self->{reqs}->{$index}->{mace} = "fixed$head<br>@"."$tail";
	    $self->{reqs}->{$index}->{mace} =~ s/fixedCE:/fixedCE:<br>/;
	}
	delete $self->{reqs}->{$index}->{macef};
	$self->{reqs}->{$index}->{jobid} =~ /https:\/\/(\w*?)\./;
	my $rb = $1;
	$self->{reqs}->{$index}->{mace} = "RB:$rb:<br>$self->{reqs}->{$index}->{mace}";

#	if ( ! $self->{reqs}->{$index}->{mawn} ) { $self->{reqs}->{$index}->{mawn} = "unknown"; }
	if ( $self->{reqs}->{$index}->{mawn} ) {
	    my @m = split /\./, $self->{reqs}->{$index}->{mawn};
	    $self->{reqs}->{$index}->{mawn} = "WN:$m[0]";
	}
	if ( $self->{reqs}->{$index}->{cse} ) {
	    my @m = split /\./, $self->{reqs}->{$index}->{cse};
	    $self->{reqs}->{$index}->{mawn} .= " SE:$m[0]";
	}

	## Evaluating the global result ...
	$self->{globalresult} .= " $self->{reqs}->{$index}->{stat1}" if $self->{globalresult};

	## Printing out the Summary values ...
	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};

    }

    return 0 if ( ! -e $self->{outFile} );

    restoreStdouterr($self->{saveOut}) if (1);

    my $resOutA = `cat $self->{outFile} `;
    print " All Output of GG---GG $self->{type} done: \n$resOutA \n";

    my $resOut = `grep SUM $self->{outFile} `;
    print " After GG $self->{type} done: \n$resOut \n";

    return 0;

}

## ***************************************************************************************
sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if ($rc);

#    print "Stack: checkPrerequisites\n";
    $self->printLocation( (caller(0))[3] );

       $rc = checkProxyTime(2);
    $self->setErrorMessage('user proxy is not valid long enough (at least 2 hours)') if $rc;

    return $rc;
}

# Do any necessary, post-test cleanup. 
## ***************************************************************************************
sub cleanup {
    my $self = shift;

#    print "Stack: cleanup\n";
    $self->printLocation( (caller(0))[3] );

#     unlink $self->{jdlFile};
#     unlink $self->{shFile};
#     unlink $self->{outFile};

    my $rc = 0;
    $rc = $self->SUPER::cleanup() if (1);

#     my $dir = $self->{tempdir};
#     rmtree($dir) if $dir;

#    if ( $self->{sem} ) { $self->{sem}->remove(); }
    if ( $self->{msg} ) { $self->{msg}->remove(); }

    return $rc;
}

## ***************************************************************************************
sub waitforChildren {
    my $self = shift;

  my @pidL = reaper();
  print " Reaper List: @pidL \n" if scalar(@pidL);

## Reading the stack of dead children ...
  my $pid = 0;
  foreach $pid ( @pidL ) {

      $self->{harvest}++;
      my $line = "";
      my $vue = 0;
      my $aborterror = 0;
      my @mstore = ();
#oldi      my $index;
      my $index = $self->{plist}->{$pid};
      my $ph    = $self->{hlist}->{$pid};

      while ( $line = readline($ph) ) {
	push @mstore, $line;
##prev	if ( $line =~ /Linux|Aborted|nothing|Error|Summary/ ) { 
	if ( $line =~ /Status:|Linux|Aborted|nothing|Error|Summary/ ) { 
	    if ( $line =~ /Aborted|Error/ ) { $aborterror = 1; }
	  if ( $line =~ s/^ Summary / / ) { 
	      print " SUM $line"; 
	      $line =~ s/^\s+//;
	      my @m = split/\s+/, $line;
#	      if ( $line =~ /=>/ ) { my @l = split/, /, $line; $self->fillResHash(@l); next; }
#oldi	      if ( $line =~ /=>/ ) { chomp(my $ll=$line); $index = $self->fillResHash($ll); }
	      if ( $line =~ /=>/ ) { chomp(my $ll=$line); $index = $self->fillResHashN($index, $ll); }
	      $vue++;
	      next;
	  }
##NO	  print " Read $self->{plist}->{$pid} $line"; 
#wrong	  if ( $line =~ /Aborted|Error/ ) { { $self->{maxSubs}++; print " Increment: $self->{maxSubs} \n" } unless ( $self->{opts}->{singleSubmit} ) ; }
##deprecated: replaced with $aborterror flag ...
##	  if ( $line =~ /Aborted|Error/ ) { $self->increment(); }
	  $vue++;
	}
      }
      close ($ph);

      if ( $vue == 0 ) {
	print "ERROR BIZarre \n";
	print @mstore;
#wrong	{ $self->{maxSubs}++; print " Increment: $self->{maxSubs} \n"; } unless $self->{opts}->{singleSubmit};
	$self->increment();
	if ( ! $self->{reqs}->{$index}->{stat1} ) {
	    $self->{reqs}->{$index}->{stat1} = "[FAIL]";
	    $self->{reqs}->{$index}->{stat2} = "BIZARRE";
	}
      } else {  ##  DEBUG
# 	  print "DEBUG BIZarre ------------------- $aborterror \n";
# 	  print @mstore;
# 	  print "DEBUG BIZarrf ------------------- \n";
	  $self->increment() if $aborterror ;
      }

      if ( ! -e $self->{reqs}->{$index}->{olog} && $self->{reqs}->{$index}->{stat2} eq "TIMEOUTSUB" ) {
	  my $res = `echo $self->{reqs}->{$index}->{stat2} > $self->{reqs}->{$index}->{olog}`;
      }

#prev      if ( ! -e $self->{reqs}->{$index}->{olog} || $self->{reqs}->{$index}->{stat2} eq "TIMEOUT" ) {
      if ( ! -e $self->{reqs}->{$index}->{olog} ) {
	  delete $self->{reqs}->{$index}->{olog};
      } else {
	  print "ERROR Ugly failure ! The command returned: \n";
	  system("cat $self->{reqs}->{$index}->{olog}");
#p	  print " Increment2: $self->{maxSubs} \n";
      }


      delete $self->{plist}->{$pid};
      delete $self->{hlist}->{$pid};

#p      print " STAK\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};

  } ## end of foreach loop

    return 0; # ($harvest, $maxSubs);

}


## Submit the JDL with a fork but keep reading from it thru PH filehandle
## ***************************************************************************************
# sub submit {

#   local *PH;
#   my $ljdl  = shift;
#   my $index = shift;

#   my $pid = 0;

#   die "cannot fork: $!" unless defined ( $pid = open ( PH, "-|" ) );

#   if ( $pid ) { # Parent

#     print "Submit $index $pid $? \n";
#     return ($pid, *PH);

#   } else { # Child

#     STDOUT->autoflush(1);
#     exec("SubmitUtils.pm", "$ljdl", "$index");
#     exit;    ## for safety ...

#   }

# }


## Submit the JDL with a fork but keep reading from it thru PH filehandle
## ***************************************************************************************
# sub submiTimeoutGood {

#   local *PH;
#   my $ljdl  = shift;
#   my $index = shift;
#   my $timeout = 500;

#   my $pid = 0;

#   die "cannot fork: $!" unless defined ( $pid = open ( PH, "-|" ) );

#   if ( $pid ) { # Parent

#     print "Submit $index $pid $? \n";
#     return ($pid, *PH);

#   } else { # Child

#     STDOUT->autoflush(1);

#         local $SIG{ALRM} = sub { die "time out" }; 
#         alarm $timeout;

#     exec("SubmitUtils.pm", "$ljdl", "$index");

#         alarm 0;

#     exit;    ## for safety ...

#   }

# }


## Submit the JDL with a fork but keep reading from it thru PH filehandle
## ***************************************************************************************
sub submitTimeout {

  local *PH;
  my $ljdl    = shift;
  my $index   = shift;
  my $logFile = shift;
  my $outFile = shift;
  my $timeout = shift;
##  my $timeout = $self->{opts}->{childTimeout};

  my $pid = 0;

  die "cannot fork: $!" unless defined ( $pid = open ( PH, "-|" ) );

  if ( $pid ) { # Parent

    print "Submit $index $pid $? \n";
    return ($pid, *PH);

  } else { # Child

    STDOUT->autoflush(1);
    STDERR->autoflush(1);

        local $SIG{ALRM} = sub { print STDERR " Signal USR1 sent: $$ \n"; kill 'USR1' => $$ }; 

    eval {
	print STDERR " Starting the CHILD process: $$ for timeout: $timeout sec. \n";
        alarm $timeout;

    exec("SubmitUtils.pm", "$ljdl", "$index", "$logFile", "$outFile");

        alarm 0;
    };

    exit;    ## for safety ...

  }

}

## Retrieves the dead children, and store their pid on the local stack
## ***************************************************************************************
sub reaper {

    my @stack = ();
    my $stpid;
    while ( ($stpid = waitpid(-1, &WNOHANG)) > 0 ) {
	# Put $stpid on a stack ...
	push @stack, $stpid;
    }

    return @stack;

}

## ***************************************************************************************
# sub fillResHash {
#     my ($self, $sel) = @_;
#     my $index;
#     my @l = split/, /, $sel;
#     foreach my $it ( @l ) {
# 	my ( $k, $v ) = split /=>/, $it;
# 	if ( $k eq "index" ) { $index = $v; next; }
# #good	if ( $k =~ /index|flag|none|mess/ ) { next; }
# 	elsif ( $k =~ /flag|none/ ) { next; }
# 	$self->{reqs}->{$index}->{$k} = $v;
#     }

#     return $index;

# }

## ***************************************************************************************
sub fillResHashN {
    my ($self, $index, $sel) = @_;
#    my $index;
    my @l = split/, /, $sel;
    foreach my $it ( @l ) {
	my ( $k, $v ) = split /=>/, $it;
#	if ( $k eq "index" ) { $index = $v; next; }
	if ( $k eq "index" ) { next; }
#good	if ( $k =~ /index|flag|none|mess/ ) { next; }
	elsif ( $k =~ /flag|none/ ) { next; }
	$self->{reqs}->{$index}->{$k} = $v;
    }

    return $index;

}

## ***************************************************************************************
sub buildXMLFile {
 
    my $self = shift;
    my ($moptc, $moptr) = @_;
    if ( ! $moptc ) {
	$moptc = "mace";
	$moptr = "mawn";
    } elsif ( ! $moptr ) {
	$moptr = "jind";
    }

    print "Stack: buildXMLFile\n";
#    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_rows;
    my $Title = "$self->{type}";
    chomp(my $date = `date`);
    my $brf_info = "Rather Simple Storm";
    $brf_info = $self->{brfinfo} if ( $self->{brfinfo} );
    my $Jdl = `cat $self->{jdlFile}`;
    $Jdl = `cat $self->{tempdir}/tmparg-0.jdl`;
    my $Scr = `cat  $self->{shFile}`;
#old    my $det_info = "Job Storm with a simple JDL script, and minimal sandbox transfers.\n\nThe JDL file is:\n\n".$Jdl."\n\nWhile the Script file is:\n\n".$Scr;
    my $det_info = "$brf_info.\n\nThe JDL file is:\n\n".$Jdl."\n\nWhile the Script file is:\n\n".$Scr;
##notgood?    $det_info =~ s/\n/<br>/g;
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile() to write $self->{xmlfile} \n";

    my $cols = {};
    my $order = 0;
#prevGOOD    my $machType = 'mace';
#    my $machType = 'mawn';
    my $machType = "$moptc";
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	if ( $machType eq "mawn" ) {
	    ## Some fixing ...
	    if ( ! $self->{reqs}->{$index}->{mawn} ) { $self->{reqs}->{$index}->{mawn} = "unknown"; }
	}
        $cols->{$self->{reqs}->{$index}->{$machType}}->{freq}++;
        if ( $cols->{$self->{reqs}->{$index}->{$machType}}->{freq} == 1 ) { $cols->{$self->{reqs}->{$index}->{$machType}}->{order} = $order++; }

#prevGOOD	push @lof_rows, "Job_".$index;
#good	push @lof_rows, ( $self->{reqs}->{$index}->{mawn} || "Job_".$index );
	push @lof_rows, ( $self->{reqs}->{$index}->{$moptr} || "Job_".$index );
    }

    foreach my $col ( sort { $cols->{$a}->{order} <=> $cols->{$b}->{order} } keys %{$cols} ) {
#    foreach my $col ( sort keys %{$cols} ) {
#p	print " $col $cols->{$col}->{order} $cols->{$col}->{freq} \n";
#	$col =~ s/\./ /g;
	push @lof_cols, "$col";
#p	print "  $col \n";
    }

    my $ncols = scalar(@lof_cols)-1;
    my $colM = "";
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	my $retv = $ret{$self->{reqs}->{$index}->{stat1}};
	my $subs = $self->{reqs}->{$index}->{stat2};
	my $logfn;
	my $logf;
	if ( $retv == 0 ) {
	    $logfn = $self->{reqs}->{$index}->{olout};
	    $logf = `cat $logfn` if $logfn;
	} elsif ( $retv == 5 ) {
##edg2.0	    $logf = $self->{reqs}->{$index}->{mess};
	    $logfn = $self->{reqs}->{$index}->{olog};
	    $logf = `cat $logfn` if $logfn;
	} else {
##edg2.0	    if ( $self->{reqs}->{$index}->{stat2} eq "MISSING_FILES" ) { $logf = $self->{reqs}->{$index}->{mess}; }
##edg2.0	    else {
	    $logfn = $self->{reqs}->{$index}->{olog};
	    $logf = `cat $logfn` if $logfn;
##edg2.0	}
	}
	unlink $logfn if $logfn;
	my $coln   = $cols->{$self->{reqs}->{$index}->{$machType}}->{order};
	my $rowM = "";
	for my $c (0..$ncols) {
	    if ( $c == $coln ) {
		$logf =~ s/&/&amp;/g; # This must be changed first.
		$logf =~ s/\"/&quot;/g;
		$logf =~ s/</&lt;/g;
		$logf =~ s/>/&gt;/g;
		if ( $self->{type} =~ /DataStorm|GfalStorm/ ) {
		    $rowM .= makeX(makeX(makeX("$logf", "output").makeX("$retv", "retval").makeX("$subs", "substatus"), "step"), "col");
		} else {
		    $rowM .= makeX(makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step"), "col");
		}
	    } else {
		$rowM .= makeX(makeX(makeX(" ", "output").makeX(" ", "retval"), "step"), "col");
	    }
	}
	$colM .= makeX($rowM, "row");
    }

my $stepM = makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
my $body = $colM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
@lof_rows = map { makeX($_, "row"); } @lof_rows;
$info .= makeX("@lof_rows", "rows_list");
@lof_cols = map { makeX($_, "col"); } @lof_cols;
$info .= makeX("@lof_cols", "cols_list");
    $info =~ s/:lxshare/:h/g;
    $info =~ s/<br>lxshare/<br>h/g;
#    $info .= $linfo;
$info .= $stepM;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDOUT makeX($info." \n ".$body, "log");

    return 0;
}

## Type with 1 column only
## ***************************************************************************************
sub buildXMLFile1Col {
 
    my $self = shift;
    my ($moptr) = @_;
    if ( ! $moptr ) {
	$moptr = "mawn";
    }

#    print "Stack: buildXMLFile\n";
    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_rows;
    my $Title = "$self->{type}";
    chomp(my $date = `date`);
    my $brf_info = "Rather Simple Storm";
    my $Jdl = `cat $self->{jdlFile}`;
    $Jdl = `cat $self->{tempdir}/tmparg-0.jdl`;
    my $Scr = `cat  $self->{shFile}`;
    my $det_info = "Job Storm with a simple JDL script, and minimal sandbox transfers.\n\nThe JDL file is:\n\n".$Jdl."\n\nWhile the Script file is:\n\n".$Scr;
##notgood?    $det_info =~ s/\n/<br>/g;
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile1Col() to write $self->{xmlfile} \n";

    my $cols = {};
    my $order = 0;
    ## $machType probably useless in this case
#    my $machType = 'mawn';
    my $machType = 'mace';
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
        $cols->{$self->{reqs}->{$index}->{$machType}}->{freq}++;
        if ( $cols->{$self->{reqs}->{$index}->{$machType}}->{freq} == 1 ) { $cols->{$self->{reqs}->{$index}->{$machType}}->{order} = $order++; }
#prevGOOD 	my $mac = " ";
#prevGOOD 	if ( $self->{reqs}->{$index}->{mawn} ) { $mac = $self->{reqs}->{$index}->{mawn}; }
#prevGOOD 	else { $mac = "CE:".$self->{reqs}->{$index}->{mace}; }
#prevGOOD 	push @lof_rows, $mac;
 	push @lof_rows, ( $self->{reqs}->{$index}->{$moptr} || "Job_".$index );
    }

#prev	 foreach my $col ( sort { $cols->{$a}->{order} <=> $cols->{$b}->{order} } keys %{$cols} ) {
#prev	     print " $col $cols->{$col}->{order} $cols->{$col}->{freq} \n";
#prev	     push @lof_cols, $col;
#prev	 }
    ## Only 1 column in this case ...
    push @lof_cols, "Status";
    
    my $ncols = scalar(@lof_cols)-1;
    my $colM = "";
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	my $retv = $ret{$self->{reqs}->{$index}->{stat1}};
	my $logfn;
	my $logf;
	if ( $retv == 0 ) {
	    $logfn = $self->{reqs}->{$index}->{olout};
	    $logf = `cat $logfn` if $logfn;
	} elsif ( $retv == 5 ) {  ## UNTESTED
##edg2.0	    $logf = $self->{reqs}->{$index}->{mess};
	    $logfn = $self->{reqs}->{$index}->{olog};
	    $logf = `cat $logfn` if $logfn;
	} else {
##edg2.0            if ( $self->{reqs}->{$index}->{stat2} eq "MISSING_FILES" ) { $logf = $self->{reqs}->{$index}->{mess}; }
##edg2.0            else {
	    $logfn = $self->{reqs}->{$index}->{olog};
	    $logf = `cat $logfn` if $logfn;
##edg2.0	}
	}
	unlink $logfn if $logfn;
	##useless my $coln   = $cols->{$self->{reqs}->{$index}->{$machType}}->{order};
	my $coln = 0;
	my $rowM = "";
	for my $c (0..$ncols) {
	    if ( $c == $coln ) {
		$logf =~ s/&/&amp;/g; # This must be changed first.
		$logf =~ s/\"/&quot;/g;
		$logf =~ s/</&lt;/g;
		$logf =~ s/>/&gt;/g;
		$rowM .= makeX(makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step"), "col");
	    } else {
		$rowM .= makeX(makeX(makeX(" ", "output").makeX(" ", "retval"), "step"), "col");
	    }
	}
	$colM .= makeX($rowM, "row");
    }

my $stepM = makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
my $body = $colM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
@lof_rows = map { makeX($_, "row"); } @lof_rows;
$info .= makeX("@lof_rows", "rows_list");
@lof_cols = map { makeX($_, "col"); } @lof_cols;
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepM;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDOUT makeX($info." \n ".$body, "log");

    return 0;
}

# Building the HTML report from the XML report (gg)
# Subclasses should not need to override this method.
sub runXMLParserScript {
    my $self = shift;

#    print "BaseTest: runXMLParserScript\n";
    $self->printLocation( (caller(0))[3] );

    my $ptrbsf = $self->can('buildXMLFile');
    return 0 if ( ! $ptrbsf );
    my $errbsf = $self->buildXMLFile();
    return 1 if ( $errbsf );

##    my $parser = "log2html-script.sh";
    if (0) {
    my $html_dir = ( defined $ENV{HTML_DIR} ) ? "$ENV{HTML_DIR}" : "$ENV{HOME}/TSTG/HTML";
    $html_dir = "${html_dir}/cal_gatekeeper";
    my $resmk = `mkdir -p $html_dir`;
  }
#    my $html_out = "index.html";
    my $html_out = "$self->{htmlout}";
#    my $parser = "log2html-script.sh --styledir $EDG_TESTROOT/etc/parser_conf/html_style ";
#    my $parser = "log2html-script.pl --outdir $self->{htmldir} --outfile $html_out --styledir $ENV{EDG_TEST_HTML_STYLE} ";
    my $parser = "log2html-script.pl -j -f $self->{htmldir}/$html_out-L0 ";
    print STDERR " Running XMLParsing: $parser $self->{xmlfile} \n";
## Call this with "bash -x" or "bash -v" when needed ...
    my $resPars = `$parser $self->{xmlfile}`;
    print STDERR " After XMLParsing: \n$resPars \n" if ( $resPars ne "" ) ;

    if (1) {
	    my $parser = "wmt -f $self->{prefix} -s -g -m -H $self->{htmldir}/$html_out-W1 -C 1";
	    print STDERR " Second XMLParsing: $parser \n";
	    ## Call this with "bash -x" or "bash -v" when needed ...
	    my $resPars = `$parser`;
	    print STDERR " After XMLParsing: \n$resPars \n" if ( $resPars ne "" ) ;

	       $parser = "wmt -f $self->{prefix} -q -s -g -m -H $self->{htmldir}/$html_out-W2 -C 1";
	    print STDERR " Third  XMLParsing: $parser \n";
	    ## Call this with "bash -x" or "bash -v" when needed ...
	       $resPars = `$parser`;
	    print STDERR " After XMLParsing: \n$resPars \n" if ( $resPars ne "" ) ;
    }

    if ( -e "$self->{htmldir}/$html_out-W1" ) {
##old	my $rescat = `cat "$self->{htmldir}/$html_out-W1" "$self->{htmldir}/$html_out-W2" "$self->{htmldir}/$html_out-L0" > $self->{htmldir}/$html_out`;
	my $rescat = `echo -e "<table><tr><td valign=\"top\" align=\"center\" width=\"60%\">" > $self->{htmldir}/$html_out`;
	$rescat .= `cat "$self->{htmldir}/$html_out-W1" >> $self->{htmldir}/$html_out`;
	$rescat .= `echo -e " " >> $self->{htmldir}/$html_out`;  ##  Insert CR
	$rescat .= `echo -e "</td><td valign=\"top\" align=\"center\" width=\"60%\"> " >> $self->{htmldir}/$html_out`;
	$rescat .= `cat "$self->{htmldir}/$html_out-W2" >> $self->{htmldir}/$html_out`;
	$rescat .= `echo -e " " >> $self->{htmldir}/$html_out`;  ##  Insert CR
	$rescat .= `echo -e "</td></tr></table> " >> $self->{htmldir}/$html_out`;
	$rescat .= `cat "$self->{htmldir}/$html_out-L0" >> $self->{htmldir}/$html_out`;
    } else {
	my $rescat = `cat "$self->{htmldir}/$html_out-L0" > $self->{htmldir}/$html_out`;
    }

    return 0;
}

## ***************************************************************************************
sub dgjob {
    my $cmdn = shift;
#     my $root = "dg-job-";
#     if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
# 	$root = "e".$root;
#     }
    my $root = "-job-";
    if ( $ENV{EDG_FLAVOR} eq 'edg2' ) {
	$root = "edg".$root;
    } elsif ( $ENV{EDG_FLAVOR} eq 'glite1' ) {
 	$root = "glite".$root;
	$cmdn =~ s/get-//;
    }
    return "$root"."$cmdn";
}

## ***************************************************************************************
sub increment { 
    my $self = shift;
    return if $self->{opts}->{singleSubmit};

    $self->{maxSubs}++; 
#p    print " Increment: $self->{maxSubs} \n"; 
    return;
} 

## ***************************************************************************************
sub cforceload {
    my $self = shift;
    return 0 unless $self->{opts}->{CalStress};

    my $index = shift;

	  open SZ, "$self->{stackjobids}";
	  1 while <SZ>;
	  my $stacksize = $.;
	  close SZ;

	  $stacksize--;
    $self->{stacksize} = $stacksize;
#p	  print " SPECIALC: index: $index stacksize: $stacksize $self->{zombies} \n";

	  return (($index-$stacksize) < $self->{maxStack}) ;
}

## ***************************************************************************************
sub checkSem {
    my $self = shift;
    return 0 unless $self->{opts}->{useSemaphores};

    my @stacknew = ();

  if ( $self->{msg} ) {
      my $msg = $self->{msg};
#	  while ( my $type = $msg->rcv($buf, 1024, 1234567) ) {
	  while ( 1 ) {
	      my $buf;
	      my $type = $msg->rcv($buf, 1024, 1234567, IPC_NOWAIT);
#	      print " Type: $type \n";
	      if ( ! $buf ) { last; }
#p	      print " Message: $buf \n";
	      my ($index, $jobid, $p) = split /\s+/, $buf;
# 	      my $sem = $self->{sem};
# 	      $sem->setval($i, 1);
#prev	      $msg->snd(1+$index, "$index VAZY $p");
#	      undef $buf;
#prev	      print " Messend: $index VAZY $p \n";
	      if ( $self->{reqs}->{$index}->{pid} != $p ) {
		  print STDERR " ERROR Identification: $self->{reqs}->{$index}->{pid} != $p \n";
	      }
	      $self->{reqs}->{$index}->{jobid} = $jobid;
	      $self->{lookup}->{$jobid} = 1+$index;
	      push @stacknew, $jobid;
	  }

      open JND, ">>$self->{stackjnotdone}";
      foreach ( @stacknew ) {
#p	  print " JND: $_\n";
	  print JND "$_\n";
      }
      close JND;

      unless ( $self->{checkTime} ) { 
	  $self->{checkTime} = time();
	  return;
      } else {
	  if ( time()-$self->{checkTime} < $self->{opts}->{pollingPeriod} ) { return; }
      }

      $self->{opts}->{aliveJobids} = 1;

      if ( -s $self->{stackjnotdone} ) {

      my ($notdfile, @refjdone) = $self->monitorJobs($self->{stackjnotdone});
# 	  my $notdfile = "uu";
# 	  my @refjdone = ();
#       $self->monitorJobs($self->{stackjnotdone});

# #       open JND, ">$self->{stackjnotdone}";
# #       foreach ( @$refjnotdone ) {
# # 	  print JND "$_\n";
# #       }
# #       close JND;

#pls      print " BEFO ls -l $self->{tempdir}\n" . `ls -l $self->{tempdir}`;
      if ( -e $notdfile ) { rename $notdfile, $self->{stackjnotdone}; }
#pls      print " AFTE ls -l $self->{tempdir}\n" . `ls -l $self->{tempdir}`;

      foreach ( @refjdone ) {
##	  print " JDone: $_\n";
	  print " JDone: $_ MsgSend: $self->{lookup}->{$_} VAZY \n";
	  $msg->snd($self->{lookup}->{$_}, "$self->{lookup}->{$_} VAZY");
	  $self->{lookup}->{$_} = -$self->{lookup}->{$_};
      }
  }

      $self->{checkTime} = time();

  }

}

## ***************************************************************************************
sub checkSemOLD {
    my $self = shift;

  if ( $self->{msg} ) {
      my $msg = $self->{msg};
      my $ds = $msg->stat();
      my $qnum = $ds->qnum;
      if ( $qnum ) {
	  print " Msg: $qnum \n";
	  foreach (1..$qnum) {
	      my $buf;
	      my $type = $msg->rcv($buf, 1024);
	      print " Message: $buf \n";
	      my ($i, $h, $p) = split /\s+/, $buf;
	      my $sem = $self->{sem};
	      $sem->setval($i, 1);
	  }
      }
  }

}

## ***************************************************************************************
sub monitorJobsNONEED {

    my ($self, $onetime) = @_;

    my ($notdfile, @refjdone) = $self->SUPER::monitorJobs($onetime);

    return ($notdfile, @refjdone);

}


# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
