#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::UserStorm;

use FindBin qw($Bin);
use lib $Bin;

use File::Basename;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

UserStorm.pm - Check that globusrun authentication works on the hosts
# $Id: UserStorm.pm,v 1.1.1.2 2005/02/08 16:09:42 mbarroso Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<UserStorm.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;

    $self->SUPER::setContext();

    $self->printLocation( (caller(0))[3] );

    $self->addOption('maxSubs=i',
                     'Number of jobs.',
                     '6');
    $self->addOption('maxStack=i',
                     'Number of simultaneous jobs in the stack.',
                     '2');
    $self->addOption('reqLapse=i',
                     'Delay between 2 submissions (in sec).',
                     '15');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '811');
    $self->addOption('customJDL=s',
                     'File name of the custom JDL file.');
    $self->addOption('customScript=s',
                     'File name of the custom SCRIPT file.');
#     $self->addOption('circular',
#                      'To scan circularly the CEList built with useCEList');

    # Additionnal Commands necessary to run this test.
    my @commands = (
#			  "edg-vo-env",
#			  "edg-gridftp-rm",
#			  "edg-gridftp-ls",
#			  "edg-gridftp-mkdir",
#			  "gdmp_host_subscribe",
#			  "gdmp_job_status",
#			  "globus-replica-catalog",
#			  "ldapsearch",
#			  "edg-replica-manager-replicateFile",
#			  "edg-replica-manager-copyAndRegisterFile",
#			  "edg-replica-manager-deleteFile",
#			  "edg-replica-manager-listReplicas"
		   );
    push @{$self->{commands}}, @commands;

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack::UserStorm [options]

END
    ;

    $self->{DescText} = <<'END'

 This Storm is mainly a JobStorm (see test JStorm) where the user can
provide either the JDL file or the Script 
to send and run on the WNs (not both).
 The wrapper takes care of all oddities in this special mode,
and allows to very flexibly design a new test this way.

 This is done thru 2 special options of the Stack family:
--customJDL & --customScript.

 Apart thelatter, there are no other specific options to this storm yet. 
All available options are described in the JobStorm itself.

 It is part of the Functionality or Stress test category.

 The test returns OK if all submitted jobs have been running,
whatever the results of the User Script can be. To be eventually improved.

END
    ;

	$self->{type} = "UserStorm";
#	 $self->{subDir} = "";
#	 $self->{location} = "";
#	 $self->{VO} = "";
#	 $self->{dataFL} = [];      ## List
#	 $self->{SPs} = {};         ## Hash
	 $self->{SEs} = [];         ## List
#	 $self->{SEpairs} = {};     ## Hash
	$self->{CEs} = [];         ## List

    return 0;

}

# Build any necessary setup for the storm.
sub setupEnvironment {
    my $self = shift;

    $self->printLocation( (caller(0))[3] );

    return 0;
}

sub dgjob {
    my $cmdn = shift;
    my $root = "dg-job-";
    if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
	$root = "e".$root;
    }
    return "$root"."$cmdn";
}

# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

    $self->printLocation( (caller(0))[3] );

    my $bid;
#    $bid = `echo 'Arguments = "S$ENV{STACK_SERIE} I${index} rien"; ' >> $jdlTemp`;

    my $nCE = scalar(@{$self->{CEs}});
    my $rnd = int(rand($nCE));
    my $ce;
    if ( $self->{opts}->{circular} ) {
	$ce = ${$self->{CEs}}[($index%$nCE)];
    } else {
	$ce = ${$self->{CEs}}[$rnd];
    }
    if ( $self->{opts}->{useCEList} ) {
#glue    my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
#norm    my $reqs = 'Requirements = other.CEId=="'.$ce.'"';
	my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
	print STDERR " Requirements: $reqs \n";
	$bid = `echo '$reqs; ' >> $jdlTemp`;
	my @m = split /:/, $ce;
	$self->{reqs}->{$index}->{mace} = "CE:$m[0]";
    } else {
	$self->{reqs}->{$index}->{mace} = "free";
    }


    return 0;
}

# Prepare everything for running the test itself ...
sub setupTest {
    my $self = shift;

    $self->printLocation( (caller(0))[3] );

    @{$self->{SEs}} = @{$self->{args}};

    if ( ! ( $self->{opts}->{customJDL} || $self->{opts}->{customScript} ) ) {
	$self->setErrorMessage('you must specify one of customJDL or customScript options');
	return 1;
    }
    if ( $self->{opts}->{customJDL} && $self->{opts}->{customScript} ) {
	$self->setErrorMessage('options customJDL and customScript are mutually exclusive');
	return 1;
    }
    if ( $self->{opts}->{customJDL} && ! -s $self->{opts}->{customJDL} ) {
	$self->setErrorMessage('custom JDL file does not exist');
	return 1;
    }
    if ( $self->{opts}->{customScript} && ! -s $self->{opts}->{customScript} ) {
	$self->setErrorMessage('custom Script file does not exist');
	return 1;
    }

#    print " Defaults: @{$self->{optionDefaults}} \n";
#    print " Linkage:  @{$self->{optionLinkage}}  \n";
#    exit 0 if (1);

    my $rc = $self->SUPER::setupTest();
    return $rc if $rc;

    ## Possible only after jdlFile and shFile get created
    $self->buildCEList();  ##  ?? do we need this here as well ?
    if ( $self->{opts}->{useCEList} ) {
	if ( ! scalar(@{$self->{CEs}}) ) {
	    $self->setErrorMessage('Setup failed while building the CEList which returned empty');
	    return 1;
	}	
    }

    $self->{zombSlep} = ( $self->{zombSlep} > $self->{reqLapse} ) ? $self->{reqLapse} : $self->{zombSlep};
    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "\n";

    return $rc;
}

sub createJDL {
    my ($self, $jdlFile, $scriptFile) = @_;

    $self->printLocation( (caller(0))[3] );
    print " jdlFile scriptFile: $jdlFile $scriptFile \n";

    my ($mname, $mpath, $msuffix) = fileparse($scriptFile, []);

    my $uScript;
    my $uJDL;
    my $sname;
##    my $script;
    my $sist;
    my $isb;
    my $storeUJDL;

    ## Case for customScript provided

    if ( $self->{opts}->{customScript} ) {
	$uScript = $self->{opts}->{customScript};
	my ($fname, $fpath, $suffix) = fileparse($uScript, []);
	$sname = $fname;
##	$script = $uScript;

    # Write out the job JDL description.
    open JDL, ">$jdlFile";
    print JDL <<"END"
Executable = "$mname";
##Arguments = "a 2 c";
InputSandbox = {"$uScript", "$scriptFile"};
StdOutput = "result.out";
StdError = "result.err";
OutputSandbox = {"result.out","result.err"};
    Requirements = other.GlueCEPolicyMaxCPUTime<6000000;
    Rank = other.GlueCEPolicyMaxCPUTime;
END
    ;

    if ( $self->{opts}->{site} ) {
	my $site = $self->{opts}->{site};
	print JDL "Requirements = RegExp(\".*$site.*\",other.CEId);\n" if $site;
    }
    close JDL;

    ## Case for customJDL provided

    } elsif ( $self->{opts}->{customJDL} ) {
	$uJDL = $self->{opts}->{customJDL};
	open JDL, "<$uJDL";
	while (<JDL>) {
	    $storeUJDL .= $_;
	    # Executable = "sname";
	    if ( /Executable/ ) {
#p		print " EE: $_ \n";
		$_ =~ /"(.*)"/ ;
		$sname = $1;
	    }
	}
	close JDL;
	open JDL, "<$uJDL";
	while (<JDL>) {
	    # InputSandbox = {"script","scrap"};
	    if ( /InputSandbox/ ) {
		chomp($isb = $_);
		$_ =~ /\{(.*)\}/ ;
		$sist = $1;
#p    	        print " II: $sist\n";
	        my @list = split /,/, $sist;
	    foreach my $it ( @list ) {
		$it =~ /"(.*)"/ ;
		$sist = $1;
		my ($fname, $fpath, $suffix) = fileparse($sist, []);
		if ( $fname eq $sname ) { $uScript = $sist; }
	    }
	    }
	}
	close JDL;
	if ( $uScript && ! -s $uScript ) { 
	    $self->setErrorMessage('custom Script taken out from custom JDL does not exist');
	    return;
	}

	my @isbs = split /}/, $isb;
        $isb = $isbs[0] . ', "' . "$scriptFile" . '"};';
        chomp($storeUJDL);

    # Write out the job JDL description, 
    # while adding the NEW Executable and InputSandbox clauses
    open JDL, ">$jdlFile";
    print JDL <<"END"
$storeUJDL
Executable = "$mname";
$isb
END
    ;

    if ( $self->{opts}->{site} ) {
	my $site = $self->{opts}->{site};
	print JDL "Requirements = RegExp(\".*$site.*\",other.CEId);\n" if $site;
    }
    close JDL;

    }

    print STDOUT "\n Custom: E=$sname S=$uScript \n\n";

    if ( ! ( $sname && $uScript ) ) { 
	$self->setErrorMessage('settings extracted from customScript/customJDL are wrong');
	return;
    }
    $self->{sname} = "$sname";
    $self->{uScript} = "$uScript";
    $self->{uJDL} = "$uJDL" if $uJDL;

    # Write out the job description.
    open SCRIPT, ">$scriptFile";
    print SCRIPT <<"END"
#!/usr/bin/perl -w

print STDERR "userStorm: ARGS: \@ARGV\\n";
print STDOUT "userStorm: ARGS: \@ARGV\\n";
system("ls -l; host \$ENV{HOSTNAME}; echo; echo '======> Begin execution user script'; echo");
chmod 0755, "./$self->{sname}";

system("./$self->{sname} \@ARGV");
my \$stat = \$?;

if ( ! \$stat ) {
    my \$res = `echo; echo '======> End execution user script'; echo; date; pwd; host \$ENV{HOSTNAME}; echo EDG_WL_JOBID: \$ENV{EDG_WL_JOBID}; uname -a; ls -l`;
    print STDOUT \$res;
}

exit 0;
END
    ;

    return "$jdlFile";

}

sub createScript {
    my ($self, $file) = @_;

    $self->printLocation( (caller(0))[3] );

    ## Quite dummy now, in this UserStorm subclass only
    ## since everything is built in the createJDL method 

    return "$file";

}

# This will run the evaluateTest method on all jobs.
sub evaluateTest {
    my $self = shift;

    $self->printLocation( (caller(0))[3] );

    my $rc = $self->SUPER::evaluateTest();
    return $rc if ($rc);
    $rc = 7 ;
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
#p	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
	my $retval = $self->{reqs}->{$index}->{stat1};
	if ( $retval != 0 &&  $retval < $rc ) {
	    $rc = $retval ;
	}
          
    }
    $rc=0 if ( $rc == 7 );   
    return $rc;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
