#!/bin/bash
######################################################################
#
# Authors: Miquel Barcelo  -  barcelo@ifae.es
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
#######################################################################
#
# Requires :
#       - The EDG_TESTROOT environment variable being defined
#       - getopt (installed by default on most systems)
#
##################################################################
# SCRIPT GDMP_func_main.sh
#
# USAGE: GDMP_func_main.sh [-h] -n
#        GDMP_func_main.sh [-h] -i [-s int]
#        GDMP_func_main.sh [-h] [-s int] [-d int] [-S int] [-p int] [-r string] [-v string] [-l string] test_host reference_host 
#
# ARGUMENTS: 
#        -h/--help           Shows help.
#        -n/--numstep        Prints the number of selectable steps on the test.
#        -s/--step           Run only the specified step, not the entire test.
#        -i/--info           Prints brief info about the selected step using "-s".
#                            Prints detailed info about the entire test if no step selected.
#        -d/--debug          Selects verbosity (debug) level. 0=none 1=brief 2=full. (Default: 0)
#        -S/--filesize       Size in kB of the file used for GDMP replication. (Default: 256)
#        -p/--gdmpport       Port to use for GDMP. (Default: 2000)
#        -r/--remotedir      Sub-directory (on the GDMP server) to put the file. (Default: edg_testing)
#        -v/--vo             VO to be used. (Default: wpsix)
#        -l/--logfile        Log test results to this file. By default is dumped to stdout.
#
# DESCRIPTION:  Simple GDMP replication tests. Creates a file on test_host, replicates it to reference_host
#               and finally delete it on both hosts.  
#
# RETURN VALUE: 0 PASSED
#	 	1 FAIL
#		2 WARNING
#		3 XFAIL
#		4 UNRESOLVED
#		5 UNTESTED
#		6 UNSUPPORTED
#
##################################################################

# We should take a look for an /etc/edg-test.conf or 
# $EDG_LOCATION/etc/edg-test.conf for configuration file

if [ -z "$EDG_TESTROOT" ]
then
      echo "EDG_TESTROOT environment variable must be defined!"
      exit 4
fi    

source $EDG_TESTROOT/sbin/common.sh 
source $EDG_TESTROOT/sbin/message.sh
source $EDG_TESTROOT/sbin/runCommand.sh
source $EDG_TESTROOT/sbin/getLogID_GDMP.sh
source $EDG_TESTROOT/sbin/getLogEnd_GDMP.sh
source $EDG_TESTROOT/sbin/runCommand_GDMP.sh
source $EDG_TESTROOT/sbin/saveVars.sh
source $EDG_TESTROOT/sbin/loadVars.sh
source $EDG_TESTROOT/sbin/echoFunc.sh
source $EDG_TESTROOT/sbin/testRetCodes.sh

# Take a look to the bottom for the real body of script

# Prints out some help about usage of this script

message_help ()
{
    message 0 "Usage: GDMP_func_main.sh [-h] -n"
    message 0 "       GDMP_func_main.sh [-h] -i [-s int]"
    message 0 "       GDMP_func_main.sh [-h] [-s int] [-d int] [-S int] [-p int] [-r string] [-v string] [-l string] test_host reference_host"
    message 0 " "
    message 0 "Options/Arguments:" 
    message 0 "       -h/--help           Shows help."
    message 0 "       -n/--numstep        Prints the number of selectable steps on the test."
    message 0 "       -s/--step           Run only the specified step, not the entire test."
    message 0 "       -i/--info           Prints brief info about the selected step using ""-s""."
    message 0 "                           Prints detailed info about the entire test if no step selected."
    message 0 "       -d/--debug          Selects verbosity (debug) level. 0=none 1=brief 2=full. (Default: 0)"
    message 0 "       -S/--filesize       Size in kB of the file used for GDMP replication. (Default: 256)"
    message 0 "       -p/--gdmpport       Port to use for GDMP. (Default: 2000)"
    message 0 "       -r/--remotedir      Sub-directory (on the GDMP server) to put the file. (Default: edg_testing)"
    message 0 "       -v/--vo             VO to be used. (Default: wpsix)"
    message 0 "       -l/--logfile        Log test results to this file. By default is dumped to stdout."
}

checkSumTest() {
# Arguments:
#  $1 - Remote Host where the replicated file lives
#  $2 - Test file name
#  $3 - File dir path on the remote host (without the file name)
#  $4 - File dir path locally (original file)
#  $5 - LogFile name where to append logmessages

local L_HOST=$1
local L_TEST_FILE=$2
local L_PATH_HOST=$3
local L_PATH_LOCAL=$4
local L_MAIN_LOG_FILE=$5

local L_RETVALUE=$RET_PASSED
local L_TMP_LOG_FILE1=/tmp/checkSumTest.$$.tmp1
local L_TMP_LOG_FILE2=/tmp/checkSumTest.$$.tmp2

runCommand "globus-job-run $L_HOST /usr/bin/cksum $L_PATH_HOST/$L_TEST_FILE" $L_TMP_LOG_FILE1
if [ $? -ne 0 ] ; then
    L_RETVALUE=$RET_FAIL
else
    runCommand "/usr/bin/cksum $L_PATH_LOCAL/$L_TEST_FILE" $L_TMP_LOG_FILE2

    local L_REMOTE_CKSUM=`grep $L_TEST_FILE $L_TMP_LOG_FILE1 | tail -1 | awk '{print $1 $2}'`
    local L_LOCAL_CKSUM=`grep $L_TEST_FILE $L_TMP_LOG_FILE2 | tail -1 | awk '{print $1 $2}'`

    if [ $L_REMOTE_CKSUM -ne $L_LOCAL_CKSUM ] ;  then
        echo "FAIL. Files checksum or size appear to be different!" >> $L_TMP_LOG_FILE1
        echo "Remote file ChecksumSize $L_HOST$L_PATH_HOST/$L_TEST_FILE = <$L_REMOTE_CKSUM>" >> $L_TMP_LOG_FILE1
        echo "Local file ChecksumSize $L_PATH_LOCAL/$TEST_FILE = <$L_LOCAL_CKSUM>" >> $L_TMP_LOG_FILE1
        L_RETVALUE=$RET_FAIL
    fi
    cat $L_TMP_LOG_FILE1 >> $L_MAIN_LOG_FILE
    cat $L_TMP_LOG_FILE2 >> $L_MAIN_LOG_FILE
    rm -f $L_TMP_LOG_FILE1 $L_TMP_LOG_FILE2
fi

return $L_RETVALUE
}

dotest ()
{
    if [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ] 
    then
	message 1 "$0: wrong number of arguments."
	exit $RET_UNRESOLVED
    fi

    local SELECTION=$1
    local HOST_A=$2
    local HOST_B=$3
    local NEWRETVALUE
    if [ -z "$4" ] ; then
	local LOGFILE=.tmplog_$PPID
    else
	local LOGFILE=$4.new
    fi

    #initialize to "test ok"

    local RETVALUE=$RET_PASSED

    case "$SELECTION" in
    0)  runCommand "gdmp_job_status -S $HOST_B -c gdmp.conf" $LOGFILE
	if [ $? -ne 0 ] ; then RETVALUE=$RET_WARNING ; fi

        GDMP_PATH_B_HOST=`grep GDMP_STORAGE_DIR $LOGFILE | awk -F= '{print $2}'`
	if [ -z $GDMP_PATH_B_HOST ] ; then 
	    echo "ERROR: $HOST_B does not support $GDMP_VO vo" >> $LOGFILE
	    RETVALUE=$RET_UNSUPPORTED
	fi
	GDMP_PATH_B_HOST=$GDMP_PATH_B_HOST/$GDMP_TEST_DIR

	runCommand "gdmp_job_status -S $HOST_A -c gdmp.conf" $LOGFILE.B
	if [ $? -ne 0 ] ; then RETVALUE=$RET_FAIL ; fi

    	GDMP_PATH_A_HOST=`grep GDMP_STORAGE_DIR $LOGFILE.B | awk -F= '{print $2}'`
	if [ -z $GDMP_PATH_A_HOST ] ; then 
	    echo "ERROR: $HOST_A does not support $GDMP_VO vo" >> $LOGFILE.B
	    RETVALUE=$RET_UNSUPPORTED
	fi
	GDMP_PATH_A_HOST=$GDMP_PATH_A_HOST/$GDMP_TEST_DIR

	cat $LOGFILE.B >> $LOGFILE ; rm -f $LOGFILE.B
	;;
    1)	runCommand "gdmp_ping -S $HOST_A -P $GDMP_PORT -V $GDMP_VO" $LOGFILE
	if [ $? -ne 0 ] ; then RETVALUE=$RET_WARNING ; fi

	runCommand "gdmp_ping -S $HOST_B -P $GDMP_PORT -r $HOST_A -p $GDMP_PORT -V $GDMP_VO" $LOGFILE.B
	if [ $? -ne 0 ] ; then RETVALUE=$RET_FAIL ; fi

	cat $LOGFILE.B >> $LOGFILE ; rm -f $LOGFILE.B	
	;;
    2) 	runCommand "gdmp_host_subscribe -S $HOST_B -P $GDMP_PORT -r $HOST_A -p $GDMP_PORT -V $GDMP_VO" $LOGFILE
	if [ $? -ne 0 ] ; then RETVALUE=$RET_FAIL ; fi
	;;
    3)  FILE_SUFX=`date +%y%m%d%H%M%S`
   	GDMP_FILE=dummy_file_GDMP.$FILE_SUFX
   	runCommand "dd if=/dev/urandom of=/tmp/$GDMP_FILE bs=1024 count=$FILE_SIZE" &> /dev/null
	runCommand "globus-job-run $HOST_A /bin/mkdir -m 777 -p $GDMP_PATH_A_HOST" $LOGFILE
	if [ $? -ne 0 ] ; then RETVALUE=$RET_FAIL ; fi

	runCommand "globus-url-copy file:///tmp/$GDMP_FILE gsiftp://$HOST_A$GDMP_PATH_A_HOST/$GDMP_FILE" $LOGFILE.B
        if [ $? -ne 0 ] ; then RETVALUE=$RET_FAIL ; fi

	runCommand "globus-job-run $HOST_A /bin/chmod a+w $GDMP_PATH_A_HOST/$GDMP_FILE" $LOGFILE.C
	if [ $? -ne 0 ] && [ $RETVALUE -eq $RET_PASSED  ] ; then RETVALUE=$RET_WARNING ; fi

	runCommand "globus-job-run $HOST_B /bin/mkdir -m 777 -p $GDMP_PATH_B_HOST" $LOGFILE.D
	if [ $? -ne 0 ] && [ $RETVALUE -eq $RET_PASSED  ] ; then RETVALUE=$RET_WARNING ; fi
 
        cat $LOGFILE.B >> $LOGFILE ; rm -f $LOGFILE.B
        cat $LOGFILE.C >> $LOGFILE ; rm -f $LOGFILE.C
	cat $LOGFILE.D >> $LOGFILE ; rm -f $LOGFILE.D
 	;;

    4)	runCommand_GDMP "gdmp_register_local_file -R -p $GDMP_PATH_A_HOST/$GDMP_FILE -S $HOST_A -P $GDMP_PORT -V $GDMP_VO" $HOST_A $GDMP_PORT $GDMP_VO $LOGFILE
        if [ $? -ne 0 ] ; then RETVALUE=$RET_FAIL ; fi

        local CHECKOUT=`grep "Registered file" $LOGFILE`
        if [ -z "$CHECKOUT" ] ; then RETVALUE=$RET_FAIL ; fi
	;;
    5)  runCommand "gdmp_publish_catalogue -S $HOST_A -P $GDMP_PORT -V $GDMP_VO $RC_FLAG" $LOGFILE
	if [ $? -ne 0 ] ; then RETVALUE=$RET_FAIL ; fi
	;;
    6) 	runCommand "gdmp_job_status -S $HOST_B -c import_catalogue | grep $GDMP_FILE" $LOGFILE
	if [ -z "`grep $GDMP_TEST_DIR/$GDMP_FILE $LOGFILE`" ] ; then RETVALUE=$RET_WARNING ; fi
	runCommand "gdmp_job_status -S $HOST_A -c local_file_catalogue | grep $GDMP_FILE" $LOGFILE.B
	if [ -z "`grep $GDMP_TEST_DIR/$GDMP_FILE $LOGFILE.B`" ] ; then RETVALUE=$RET_FAIL ; fi
	runCommand "gdmp_job_status -S $HOST_A -c export_catalogue | grep $GDMP_FILE" $LOGFILE.C
	if [ -z "`grep $GDMP_TEST_DIR/$GDMP_FILE $LOGFILE.C`" ] ; then RETVALUE=$RET_FAIL ; fi
	cat $LOGFILE.B >> $LOGFILE ; rm -f $LOGFILE.B
        cat $LOGFILE.C >> $LOGFILE ; rm -f $LOGFILE.C
	;;
    7)  runCommand_GDMP "gdmp_replicate_put -f $FILE_SUFX -S $HOST_A -P $GDMP_PORT -r $HOST_B -p $GDMP_PORT -V $GDMP_VO $RC_FLAG" $HOST_B $GDMP_PORT $GDMP_VO $LOGFILE
	if [ $? -ne 0 ] ; then RETVALUE=$RET_FAIL ; fi
	
	local CHECKOUT=`grep "successfully replicated=1" $LOGFILE`
	if [ -z "$CHECKOUT" ] ; then RETVALUE=$RET_FAIL ; fi
	;;
    8)  checkSumTest $HOST_A $GDMP_FILE $GDMP_PATH_A_HOST /tmp $LOGFILE
	RETVALUE=$?
	checkSumTest $HOST_B $GDMP_FILE $GDMP_PATH_B_HOST /tmp $LOGFILE
	NEWRETVALUE=$?
	if [ $RETVALUE -eq $RET_PASSED ] ; then RETVALUE=$NEWRETVALUE ; fi
	;;
    9)  runCommand "gdmp_remove_local_file -p $GDMP_PATH_B_HOST/$GDMP_FILE -S $HOST_B -P $GDMP_PORT -V $GDMP_VO $RC_FLAG" $LOGFILE
	if [ $? -ne 0 ] ; then RETVALUE=$RET_WARNING ; fi

	runCommand "gdmp_remove_local_file -p $GDMP_PATH_A_HOST/$GDMP_FILE -S $HOST_A -P $GDMP_PORT -V $GDMP_VO $RC_FLAG" $LOGFILE.B
	if [ $? -ne 0 ] ; then RETVALUE=$RET_FAIL ; fi	
	
	runCommand "rm -f /tmp/$GDMP_FILE" &> /dev/null

	cat $LOGFILE.B >> $LOGFILE ; rm -f $LOGFILE.B
	;;
# if other test requested, return UNSUPPORTED
    *)  return $RET_UNSUPPORTED
    esac

    if [ -z $4 ] ; then 
	cat $LOGFILE
    else
	cat $LOGFILE >> $4
    fi
    rm -f $LOGFILE

    return $RETVALUE
}

helptest ()
{
    local SELECTION=$1
    case "$SELECTION" in
    1)	echo "gdmp ping"
	;;
    0)  echo "get storage dir"
	;;
    2)  echo "gdmp subscribe"
	;;
    3)  echo "globus-url-cpy"
	;;
    4)  echo "register file"
	;;
    5)  echo "pub. catalogue"
	;;
    6)  echo "catalogue check"
	;;
    7)  echo "replicate put"
	;;
    8)  echo "check files"
	;;
    9)  echo "remove files"
	;;
    *)  echo "<br>"
	echo "<b>Tests performed</b><br>"
	echo "Critical tests on <i>italic</i><br><br>"
	echo "<u>GDMP_ping</u><br>"
	echo "<i>gdmp_ping -S HOST_A -P $GDMP_PORT -V $GDMP_VO</i><br>"
	echo "<i>gdmp_ping -S HOST_B -P $GDMP_PORT -r HOST_B -p $GDMP_PORT -V $GDMP_VO</i><br><br>"
	echo "<u>grid-info-search</u><br>"
	echo "gdmp_job_status -S HOST_B -c gdmp.conf<br>"
	echo "<i>gdmp_job_status -S HOST_A -c gdmp.conf</i><br><br>"
	echo "<u>subscribe</u><br>"
	echo "<i>gdmp_host_subscribe -S HOST_B -P $GDMP_PORT -r HOST_A -p $GDMP_PORT -V $GDMP_VO</i><br><br>"
	echo "<u>globus url copy</u><br>"
	echo "<i>globus-job-run HOST_A /bin/mkdir -m 777 -p GDMP_PATH_A_HOST</i><br>"
	echo "<i>globus-url-copy file:///tmp/GDMP_FILE gsiftp://HOST_A/GDMP_PATH_A_HOST/GDMP_FILE</i><br>"
	echo "<i>globus-job-run HOST_A /bin/chmod a+w GDMP_PATH_A_HOST/GDMP_FILE</i><br>"
	echo "globus-job-run HOST_B /bin/mkdir -m 777 -p GDMP_PATH_B_HOST<br><br>"
	echo "<u>register file</u><br>"
	echo "<i>gdmp_register_local_file -R -p GDMP_PATH_A_HOST/GDMP_PATH_A_HOST -S HOST_A -P $GDMP_PORT -V $GDMP_VO</i><br><br>"
	echo "<u>publish catalogue</u><br>"
	echo "<i>gdmp_publish_catalogue -S HOST_A -P $GDMP_PORT -V $GDMP_VO $RC_FLAG</i><br><br>"
	echo "<u>catalogue check</u><br>"
	echo "gdmp_job_status -S $HOST_B -c import_catalogue | grep $GDMP_FILE<br>"
	echo "<i>gdmp_job_status -S $HOST_A -c local_file_catalogue | grep $GDMP_FILE</i><br>"
	echo "<i>gdmp_job_status -S $HOST_A -c export_catalogue | grep $GDMP_FILE</i><br><br>"
	echo "<u>replicate put</u><br>"
	echo "<i>gdmp_replicate_put -f GDMP_FILE -S HOST_A -P $GDMP_PORT -r HOST_B -p $GDMP_PORT -V $GDMP_VO $RC_FLAG</i><br><br>"
	echo "<u>check files</u><br><br>"
	echo "<u>remove files</u><br>"
	echo "gdmp_remove_local_file -p GDMP_PATH_B_HOST/GDMP_FILE -S HOST_B -P $GDMP_PORT -V $GDMP_VO $RC_FLAG<br>"
	echo "<i>gdmp_remove_local_file -p GDMP_PATH_A_HOST/GDMP_FILE -S HOST_A -P $GDMP_PORT -V $GDMP_VO $RC_FLAG</i><br><br>"
	;;
    esac
}

#Verbosity level
  VERBOSE_LEVEL=0
#Size of file used to test replication
  FILE_SIZE=256
#Replica Catalogue flag: -n do not write to RC ; -C write to RC
  RC_FLAG=-C
#tests number
  TEST_NUM=10
#VO to test
  GDMP_VO=wpsix
#sub-directory to place the files
  GDMP_TEST_DIR=edg_testing
# default GDMP port
  GDMP_PORT=2000
  
  NAME="`/bin/basename $0`"

  ARGS_SHORT="hins:l:d:S:r:v:p:"
  ARGS_LONG="help,info,numstep,step:,logfile:,debug:,filesize:,remotedir:,vo:,gdmpport:"

  TEMP=`getopt -o $ARGS_SHORT --long $ARGS_LONG -n $NAME -- "$@"`

  if [ $? != 0 ] ; then 
    message 0 "Syntax error"
    message_help
    exit $RET_UNRESOLVED
  fi
  eval set -- "$TEMP"

  while true ; do
    case "$1" in
	-h|--help)      HELP=1                   ;;
	-i|--info)      NFO=1                    ;;
	-n|--numstep)   NSTEP=1                  ;;
	-s|--step)      SELSTEP=$2       ; shift ;;
	-l|--logfile)   LOGOUT=$2        ; shift ;;
	-d|--debug)     VERBOSE_LEVEL=$2 ; shift ;;
	-S|--filesize)  FILE_SIZE=$2     ; shift ;;
	-r|--remotedir)	GDMP_TEST_DIR=$2 ; shift ;;
	-v|--vo)        GDMP_VO=$2       ; shift ;;
	-p|--gdmpport)  GDMP_PORT=$2     ; shift ;;
	--) shift ; break ;;
	*) message 0 "Internal error!" ; exit $RET_UNRESOLVED ;;
    esac
    shift
  done

# parse the positional parameters

  HOST_A=$1
  HOST_B=$2

  if [ -n "$HELP" ]
  then
      message_help
      exit $RET_PASSED
  fi
  
  if [ -n "$NSTEP" ]
  then 
    message 0 "$TEST_NUM"
    exit $RET_PASSED
  fi

  if [ -n "$NFO" ]
  then
    helptest "$SELSTEP"
    exit $RET_PASSED
  fi     

  if [ -z "$HOST_A" ] || [ -z "$HOST_B" ]
  then
    message 0 "Must specify a target host and a reference host!"
    message_help
    exit $RET_UNRESOLVED
  fi

# The host to test and the reference host can't be the same !
  
  if [ "$HOST_A" = "$HOST_B" ] ; then
    message 0 "Target host and reference host must be different"
    exit $RET_UNRESOLVED
  fi

  rm -f $LOGOUT

  RETVAL=0
  RETVALNEW=0

  if [ -z "$SELSTEP" ]
  then
    k=0
    while [ $k -lt $TEST_NUM ] 
    do
	dotest $k $HOST_A $HOST_B $LOGOUT
        RETVALNEW=$?
        if [ $RETVAL -ne $RET_FAIL ] ; then RETVAL=$RETVALNEW ; fi 
	$ECHO -n "`helptest $k`"
 	case "$RETVALNEW" in
	0) echo_success ;;
	1) echo_failure ;;
	2) echo_passed  ;;
	esac
        k=`expr $k + 1`
    done
  else
	loadVars /tmp/$NAME.state_$UID
	dotest $SELSTEP $HOST_A $HOST_B $LOGOUT
        RETVAL=$?
        $ECHO -n "`helptest $SELSTEP`"
 	case "$RETVAL" in
	0) echo_success ;;
	1) echo_failure ;;
	2) echo_passed  ;;
	esac
	saveVars /tmp/$NAME.state_$UID "GDMP_PATH_A_HOST GDMP_PATH_B_HOST FILE_SUFX GDMP_FILE"
  fi    

exit $RETVAL









