#! /usr/bin/env bash
######################################################################
#
# Authors: Rene Metery  -  rene.metery@c-s.fr 
#          based on script written by:
#             - Miquel Barcelo  - barcelo@ifae.es
#             - Gonzalo Merino  - merino@ifae.es
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
########################################################################
# Last Modification : 28/01/03
#                     by F.Chollet chollet@lapp.in2p3.fr
#                     Allow remote testing from local UI to remote nodes  
########################################################################
# SCRIPT HTML_0D_parser <conffile>
#
# USAGE: HTML_0D_parser-script.sh [-h] -c file
#        HTML_0D_parser-script.sh [-h] -t file [-a string] [-f string] [-d string] [-s string] [-T string] [-R string] [-C string]
#
# ARGUMENTS: 
#        -h/--help           Shows help.
#        -c/--conf           Uses configuration from file.
#        -e/--errorstop      Stop test on error (FAIL).
#        -t/--testfile       Main test file.
#        -a/--testargs       Options/Arguments to be passed to the test.
#        -f/--outfile        Main HTML output file. (Default: index.html).
#        -d/--outdir         Output directory for HTML.
#        -s/--style          Directory with the style files for HTML formatting.
#        -T/--title          Title for the HTML page.
#        -R/--rows           Elements to be put on the rows.
#
# DESCRIPTION: Runs specified test on multiple hosts, generating an HTML report
#                      
# RETURN VALUE: 0 - if all run correct
#		1 - if some error produced
#	        4 - no mandatory parameter is passed or $EDG_TESTROOT not defined
#
# REQUIREMENTS: The test script must support the following arguments:
#
#        -l file             Logs output to specified file, creating it if not exists.
#        -n                  Prints to stdout the number of steps on the test.
#                            If the test is single-step should print 1.
#
#               The test script should take the first positional parameter as the host to test.
#
#               Following arguments are optional. They can be ignored but the test shouldn't fail if found.
#
#        -s number           Selects specific step on the test to be run, not the entire test.
#                            If don't want to support it, should run the entire test.
#        -i                  Prints to stdout info about the test, to be put on the HTML page but doesn't runs the test.
#                            If "-s" is used to select an specific step, should print brief information about that step.
#                            If no step is selected, it should print more detailed information about the entire test.
#                            If don't want to support it, should exit immediately without printing anything.
#
#               Test script may support other arguments options. They can be passed to it using the "-a" argument (see above).
#
##################################################################

# Prints out some help about usage of this script

message_help ()
{
    message 1 "Usage: HTML_0D_parser-script.sh [-h] -c file"
    message 1 "       HTML_0D_parser-script.sh [-h] -t file [-a string] [-f string] [-d string] [-s string] [-T string] [-R string] [-C string]"
    message 1 " "
    message 1 "Options/Arguments:"
    message 1 "        -h/--help           Shows help."
    message 1 "        -c/--conf           Uses configuration from file."
    message 1 "        -e/--errorstop      Stop test on error (FAIL)."
    message 1 "        -t/--testfile       Main test file."
    message 1 "        -a/--testargs       Options/Arguments for the test."
    message 1 "        -f/--outfile        Main HTML output file. (Default: index.html)."
    message 1 "        -d/--outdir         Output directory for HTML."
    message 1 "        -s/--style          Directory with the style files for HTML formatting."
    message 1 "        -T/--title          Title for the HTML page."
    message 1 "        -R/--rows           Elements to be put on the rows."
}


# We should take a look for an /etc/edg-test.conf or 

# $EDG_LOCATION/etc/edg-test.conf for configuration file

if [ -z "$EDG_TESTROOT" ]
then
      echo "EDG_TESTROOT environment variable must be defined!"
      exit 4
fi    

source $EDG_TEST_FUNCTIONS/common.sh
source $EDG_TEST_FUNCTIONS/sourcer.sh
sourcetestsiteconffile

NAME="`/bin/basename $0`"

ARGS_SHORT="hec:t:a:f:d:s:T:R:"
ARGS_LONG="help,errorstop,conf:,testfile:,testargs:,outfile:,outdir:,styledir:,title:,rows:"

TEMP=`getopt -o $ARGS_SHORT --long $ARGS_LONG -n $NAME -- "$@"`

if [ $? != 0 ] ; then 
    message_help
    exit 1
fi

eval set -- "$TEMP"

while true ; do
    case "$1" in
	-h|--help)      HELP=1                    ;;
	-c|--conf)      CONFFILE=$2       ; shift ;;
	-e|--errorstop) ERRORSTOP=1               ;;
	-t|--testfile)  TEST_FILE=$2      ; shift ;;
	-a|--testargs)  TEST_ARGS=$2      ; shift ;;
	-f|--outfile)   HTML_OUT=$2       ; shift ;;
	-d|--outdir)	HTML_DIR=$2       ; shift ;;
	-s|--styledir)  HTML_STYLE=$2     ; shift ;;
	-T|--title)     TEST_TITLE=$2     ; shift ;;
	-R|--rows)      TEST_ROWS_LIST=$2 ; shift ;;
	--) shift ; break ;;
	*) echo "Internal error!" ; exit 7 ;;
    esac
    shift
done

if [ -n "$HELP" ] ; then
    message_help
    exit 0
fi

if [ -n "$CONFFILE" ] ; then
    if [ -e "$CONFFILE" ] ; then 
	source "$CONFFILE"
    else
	message 1 "$CONFFILE not found"
	exit 1
    fi
fi

if [ -z "$TEST_FILE" ] ; then
    message_help
    exit 1
fi

if [ -x "$TEST_FILE" ] ; then
    TEST_STEPS=`$TEST_FILE -n`
else
    message 1 "$TEST_FILE not found"
    exit 1
fi

if [ -d "$HTML_STYLE" ] ; then
    HTML_create
else
    message 1 "$HTML_STYLE does not exist"
    exit 1
fi

HTML_dump page_start.html
HTML_echo "$TEST_TITLE"
HTML_dump title_start.html
HTML_echo "$TEST_TITLE"
HTML_dump title_end.html
HTML_echo "Test started on `date`<br><br>" 

HTML_dump table_start.html
HTML_dump first_row_start.html

HTML_dump first_row_col_start.html
HTML_echo "Tests on $TEST_ROWS_LIST"
HTML_dump first_row_col_end.html

HTML_dump first_row_col_start.html
HTML_echo "Results"
HTML_dump first_row_col_end.html

HTML_dump first_row_end.html

i=1
while [ "$i" -le "`$TEST_FILE -n`" ]
do
    HTML_dump row_start.html
    HTML_dump first_col_start.html
  
    HTML_echo "`$TEST_FILE -s $i -i`"    
    HTML_dump first_col_end.html

    HTML_dump col_start.html
    HTML_LOG=log-$i.txt
#   if the test is done remotely from UI 
    if  [ "$TEST_RSH" != "" ] && [ "`hostname`" != "$RNODE" ]
    then 
	$TEST_RSH $RNODE $RCOMMAND $TEST_ARGS -s $i &> $HTML_DIR/$HTML_LOG 
#   or done localy
    else 
	$TEST_FILE $TEST_ARGS -s $i -l $HTML_DIR/$HTML_LOG &> /tmp/${USER}_$$_testinfo.log
    fi
    HTML_validate $?
    VAL_RES=$?
    case "$VAL_RES" in
	0) i=`expr $i + 1` ;;
	1) i=`expr $i + 1`
	   if [ ! -z "$ERRORSTOP" ] ; then
	   while [ $i -le "`$TEST_FILE -n`" ] ; do

                  HTML_dump col_end.html
                  HTML_dump row_end.html

                  HTML_dump row_start.html
                  HTML_dump first_col_start.html
                  HTML_echo "`$TEST_FILE -s $i -i`"
                  HTML_dump first_col_end.html

                  HTML_dump col_start.html
		  HTML_dump null_test.html
		  i=`expr $i + 1`
	   done
	   fi
	   ;;
	2) test -e /tmp/${USER}_$$_testinfo.log && cat /tmp/${USER}_$$_testinfo.log >> $HTML_DIR/$HTML_LOG
           i=$((`$TEST_FILE -n` +1))
	   ;;
    esac
    rm -f /tmp/${USER}_$$_testinfo.log
    HTML_dump col_end.html

    HTML_dump row_end.html
done


HTML_dump table_end.html
$TEST_FILE -i &> /tmp/${USER}_$$_testinfo.log
HTML_cat /tmp/${USER}_$$_testinfo.log
rm -f /tmp/${USER}_$$_testinfo.log
HTML_echo "Test ended on `date`"
HTML_dump page_end.html
