#!/bin/bash
######################################################################
#
# classify_rpms.sh  
#
# Author:                      Andrea Formica, Irakli Mandjavidze
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
######################################################################
# Modified : 29/01/03 Frederique Chollet chollet@lapp.in2p3.fr
#                     names of temporary file refer to $$
##################################################################

##################################################################
# FUNCTION: classify_rpms <list name> <output file>
#
# ARGUMENTS: <list name>    {string} (mandatory)
#            <output file>  {string}
#
# Input:  <list name>                  Input file or directory with the rpms list
# Output: <output file>=RES_FILE       Output files, with the classification of rpms based
#                                      on the --test option of rpm command
#
#*** List of RPMs that are already present ***      > $RES_FILE.installed
#*** List of RPMs that can be upgraded ***          > $RES_FILE.canbeupgraded
#*** List of RPMs that can be installed ***         > $RES_FILE.canbeinstalled
#*** List of RPMs that failed the NAME test ***     > $RES_FILE.failed
#*** List of RPMs that failed upgrade ***           > $RES_FILE.failedupgrade
#*** List of RPMs that failed installation ***      > $RES_FILE.failedinstall
#
# RETURN VALUE: 0 - list found and analysed
#		1 - list not found
#	        4 - no mandatory parameter is passed
##################################################################

classify_rpms()
{
    PRESTRING="${PRESTRING}+"
    local THIS_SCRIPT="classify_rpms.sh"
    local HOUR=`date | awk '{print $4}'`
    local PRETXT="[${THIS_SCRIPT} ${HOUR}]:"
    
    if [ $LEVEL -gt 1 ]
    then
        PRETXT="$PRESTRING [${THIS_SCRIPT} ${HOUR}]:"
    fi

    local PKGDIR=
    local RES_FILE=classify_rpms_$$.out
#
# Get argument
    PKGDIR=$1
    [ $2 != "" ] && RES_FILE=$2

    if [ -z $1 ]
    then
        [ $DEBUG ] && writeOutput "classify_rpms(): no mandatory parameter is passed"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: input  $1={directory | file}"
        [ $DEBUG ] && writeOutput "$THIS_SCRIPT: output $2={output file string}"
        echo
        PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
	return 4
    fi

    [ -e  $RES_FILE.installed ]       && (rm $RES_FILE.installed)
    [ -e  $RES_FILE.canbeupgraded ]   && (rm $RES_FILE.canbeupgraded)
    [ -e  $RES_FILE.canbeinstalled ]  && (rm $RES_FILE.canbeinstalled)
    [ -e  $RES_FILE.failed ]          && (rm $RES_FILE.failed)
    [ -e  $RES_FILE.failedupgrade ]   && (rm $RES_FILE.failedupgrade)
    [ -e  $RES_FILE.failedinstall ]   && (rm $RES_FILE.failedinstall)

    touch $RES_FILE.installed
    touch $RES_FILE.canbeupgraded
    touch $RES_FILE.canbeinstalled
    touch $RES_FILE.failed
    touch $RES_FILE.failedupgrade
    touch $RES_FILE.failedinstall


#"*** List of RPMs that are already present ***"      >> $RES_FILE.installed
#"*** List of RPMs that can be upgraded ***"          >> $RES_FILE.canbeupgraded
#"*** List of RPMs that can be installed ***"         >> $RES_FILE.canbeinstalled
#"*** List of RPMs that failed the NAME test ***"     >> $RES_FILE.failed
#"*** List of RPMs that failed upgrade ***"           >> $RES_FILE.failedupgrade
#"*** List of RPMs that failed installation ***"      >> $RES_FILE.failedinstall
#
#
# Check if input name is a directory
    if [ -d $PKGDIR ]; then
	if [ -f /tmp/rpmlist_$$ ]; then
	    rm /tmp/rpmlist_$$
	fi
	for rpm in $PKGDIR/*.rpm; do
		RPMFILE=`basename $rpm`
		echo $RPMFILE >> /tmp/rpmlist_$$
	done
	RPMFILENAME=/tmp/rpmlist_$$
    else
	RPMFILENAME=$PKGDIR
    fi

# Analyse RPMS
    if [ -f $RPMFILENAME ]
    then
        nlines=`wc -l $RPMFILENAME |  awk '{print $1}'`
        errflg=1
        k=1
        until [ $k -gt $nlines ] ; do
            rpm=`sed -n ${k}p $RPMFILENAME`
            INSTOK=0

            RPMFILE=$EDG_DISTRIB_RPMS/$rpm
            [ $DEBUG ] && writeOutput "${PRETXT} TESTING $RPMFILE ..."
            NAME=`rpm --queryformat "%{NAME}" -qp $RPMFILE 2>&1`
            if [ $? != 0 ]; then
                echo $NAME >> $RES_FILE.failed
            fi
            DIST_RPM=`rpm -qp $RPMFILE 2>&1`

            INSTALL_FILE=`echo $NAME | awk -F'-' '{print $1}'`
            INSTALL_SCRIPT=$EDG_DISTRIB_RPMS/${INSTALL_FILE}.install

            [ $DEBUG ] &&  writeOutput "${PRETXT} fileexist  $INSTALL_SCRIPT" 
            fileexist $INSTALL_SCRIPT
#            print_status            
            if [ $? = 0 ]; then
                [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} Special installation for $RPMFILE...skip checks"
                echo $NAME >> $RES_FILE.canbeinstalled
#                echo $NAME >> $RES_FILE.failedinstall
                k=$[ k + 1 ]
                continue
            else
                INST_RPM=`rpm -q $NAME 2>&1`
                [ $? = 0 ] && INSTOK=1
            fi
            
#Start classification
            if [ $INSTOK -eq 1 ]; then
	        if [ "$DIST_RPM" = "$INST_RPM" ]; then
                    [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} file $DIST_RPM is installed!"
	            echo $rpm >> $RES_FILE.installed
                else
	            oldrpm=`ls ${RH62_DIS_RPMS_DIR}/${INST_RPM}* |  awk -F'/' '{print $NF}'`
                    [ $DEBUG ] && writeOutput "${PRETXT} fileexist  $RH62_DIS_RPMS_DIR/$oldrpm" 
                    fileexist $RH62_DIS_RPMS_DIR/$oldrpm
	            if [ $? != 0 ]; then 
                        oldrpm=`ls ${EDG_DISTRIB_RPMS}/${INST_RPM}* |  awk -F'/' '{print $NF}'`
                        [ $DEBUG ] && writeOutput "${PRETXT} fileexist  ${EDG_DISTRIB_RPMS}/$oldrpm" 
                        fileexist ${EDG_DISTRIB_RPMS}/$oldrpm
                    fi                    
                    [ $DEBUG ] && print_status 
                   
                    [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} testrpm $RPMFILE UPGRADE"
                    testrpm  $RPMFILE $RES_FILE "Uvh"
                    print_status
                fi
	    else                   
                [ $LEVEL -gt 0 ] && writeOutput "${PRETXT} testrpm $RPMFILE INSTALL"
                testrpm  $RPMFILE $RES_FILE "ivh"
                print_status
            fi
            k=$[ k + 1 ]
        done
    fi
    
    cat $RES_FILE.installed > $RES_FILE
    echo "***" >> $RES_FILE
#rm -f $RES_FILE.installed

    cat $RES_FILE.canbeupgraded >> $RES_FILE
    echo "***" >> $RES_FILE
#rm -f $RES_FILE.canbeupgraded

    cat $RES_FILE.canbeinstalled >> $RES_FILE
    echo "***" >> $RES_FILE
#rm -f $RES_FILE.canbeinstalled

    cat $RES_FILE.failed >> $RES_FILE
    echo "***" >> $RES_FILE
#rm -f $RES_FILE.failed

    cat $RES_FILE.failedinstall >> $RES_FILE
    echo "***" >> $RES_FILE
#rm -f $RES_FILE.failedinstall

    cat $RES_FILE.failedupgrade >> $RES_FILE
    echo "***" >> $RES_FILE
#rm -f $RES_FILE.failedupgrade


    if [ -f /tmp/rpmlist_$$ ]; then
        rm /tmp/rpmlist_$$
    fi
    [ $DEBUG ] && writeOutput "${PRETXT} ### END"
    PRESTRING=`echo ${PRESTRING} | $CUT -c 2-`
    
    return 0
}

