#!/bin/bash

######################################################################
#
# Authors: Alessandro Cavalli  - cavalli@cnaf.infn.it
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
######################################################################

######################################################################
# FUNCTION: grisqueryce <host>
#
# ARGUMENTS: <host> {string} (optional)
#
# DESCRIPTION: Queries the local GRIS or the one specified with <host>
#              of a ComputingElement and checks if it's responding and 
#              if it returns complete data.
#
# RETURN VALUE: 0 - if everything is OK.
#               1 - if required data is missing
#               4 - if the GRIS doesn't respond
######################################################################

grisqueryce()
{

    OPTION=$1

    HOST=${OPTION:-`/bin/hostname`}
    PORT=2135
    DN="mds-vo-name=local,o=grid"

    CEIDOK=no
    CLOSESEOK=no
    MISSING=no
    MISSING_MAY=no

    CE_ATTR_LIST=`echo "CEId GlobusResourceContactString GRAMVersion Architecture OpSys \
   MinPhysicalMemory MinLocalDiskSpace TotalCPUs FreeCPUs NumSMPs MinSPUProcessors \
   MaxSPUProcessors TotalJobs RunningJobs IdleJobs MaxTotalJobs MaxRunningJobs \
   WorstTraversalTime EstimatedTraversalTime Active Priority MaxCPUTime \
   MaxWallClockTime AverageSI00 MinSI00 MaxSI00 AuthorizedUser \
   RunTimeEnvironment AFSAvailable OutboundIP InboundIP" | tr 'A-Z' 'a-z'`

    CE_ATTR_LIST_MAY=`echo "QueueName LRMSType LRMSVersion" | tr 'A-Z' 'a-z'`

    CLOSESE_ATTR_LIST=`echo "CloseSE CEId" | tr 'A-Z' 'a-z'`

    CLOSESE_ATTR_LIST_MAY=`echo "MountPoint" | tr 'A-Z' 'a-z'`

    echo

    grid-info-search -LLL -h ${HOST} -p ${PORT} -b "${DN}" -x  | tr 'A-Z' 'a-z' > /tmp/$$_all 2>&1

    ERR=`grep -i "ldap_bind: Can't contact LDAP server" /tmp/$$_all`

    if [ "$ERR" = "ldap_bind: Can't contact LDAP server" ]
	then
	echo "GRIS isn't responding at ${HOST}:${PORT}/${DN} "
	echo
	rm -f /tmp/$$_all
	return 4
    fi


    while read attribute value
      do
      cat /dev/null > /tmp/$$_entry

      if [ "$attribute" = "dn:" ] ; then
      	
	  if [ "`echo $value | cut -c1-5`" = "ceid=" ] ; then
	      echo "Analizyng ceId: " `echo ${value} |sed -e "s/,.*//g" `
	      CEIDOK=yes

	      while read attribute value
		do
		echo ${attribute} ${value} >> /tmp/$$_entry

		if [ -z "${attribute}" -a -z "${value}" ] ; then
               
		    for attr in ${CE_ATTR_LIST} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Attribute '${attr}' not defined"
			    MISSING=yes
			fi
		    done

		    for attr in ${CE_ATTR_LIST_MAY} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Optional attribute '${attr}' not defined"
			    MISSING_MAY=yes
			fi
		    done
               
		    echo
		    continue 2
		fi
	      done
	  fi

	  if [ "`echo $value | cut -c1-5`" = "close" ] ; then
	      echo "Analizyng closeSE: " `echo ${value} |sed -e "s/,.*//g" `
	      CLOSESEOK=yes

	      while read attribute value
		do
		echo ${attribute} ${value} >> /tmp/$$_entry

		if [ -z "${attribute}" -a -z "${value}" ] ; then

		    for attr in ${CLOSESE_ATTR_LIST} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Attribute '${attr}' not defined"
			    MISSING=yes
			fi
		    done

		    for attr in ${CLOSESE_ATTR_LIST_MAY} ; do
			if [ -z  "`grep -i ${attr}: /tmp/$$_entry | sed -n -e 1p | sed -e \"s/.*: //g\"`" ] ; then
			    echo "Optional attribute '${attr}' not defined"
			    MISSING_MAY=yes
			fi
		    done
               
		    echo
		    continue 2
		fi
	      done
	  fi

      fi
    done < /tmp/$$_all

    rm -f /tmp/$$_entry
    rm -f /tmp/$$_all

    if [ "${CEIDOK}${CLOSESEOK}${MISSING}${MISSING_MAY}" = "yesyesnono" ] ; then
	echo "The GRIS for the Computing Element is up"
        echo "and data returned is complete"
	echo
	return 0
    fi

    if [ "${MISSING}${MISSING_MAY}" = "noyes" ] ; then
	echo "The GRIS for the Computing Element is up"
        echo "and all mandatory attributes are defined"
	echo "But one or more OPTIONAL attributes are not defined"
	echo
	return 0
    fi

    if [ "${CEIDOK}" = "no" ] ; then
	echo "ERROR: no CEID data returned from GRIS."
    fi

    if [ "${CLOSESEOK}" = "no" ] ; then
	echo "ERROR: no CloseSE data returned from GRIS."
    fi

    if [ "${MISSING}" = "yes" ] ; then
	echo "ERROR: missing attribute(s)"
    fi

    echo
    return 1
}
