#! /usr/bin/env bash
######################################################################
#
# Authors: Rene Metery  -  rene.metery@c-s.fr 
#          based on scripts written by :
#             - Miquel Barcelo     - barcelo@ifae.es
#             - Frederique Chollet - chollet@lapp.in2p3.fr
#             - Rene Metery        - rene.metery@c-s.fr
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
#######################################################################
#
# Requires :
#       - The EDG_TESTROOT environment variable being defined
#       - getopt (installed by default on most systems)
#
##################################################################
#
usage() {
    message 0 "Usage: $PROG_NAME [-h]" 
    message 0 "       $PROG_NAME [-n]"
    message 0 "       $PROG_NAME [-i] [-s int]"
    message 0 "       $PROG_NAME [-d string] [-l string] [-x] [-L string]" 
    message 0 "       $PROG_NAME [-c string]"
    message 0 " "
    message 0 "By default the script runs the entire test (all steps)."    
    message 0 "Informative Options/Arguments:"
    message 0 "       -h/--help           Shows help."
    message 0 "       -n/--numstep        Prints the number of selectable steps on the test."
    message 0 "       -i/--info           Prints brief info about the selected step using ""-s""."
    message 0 "                           Prints detailed info about the entire test if no"
    message 0 "                           step selected."
    message 0 "       -s/--step           Run only the specified step, not the entire test."
    message 0 " "
    message 0 "Executive Options/Arguments:"
    message 0 "       -d/--def            Indicates the path to the directory containing all"
    message 0 "                           the .def files."
    message 0 "       -l/--log            Log test results to this file. (default: it is dumped"
    message 0 "                           to stdout)."
    message 0 "       -x/--xml            Print the informations in the log file in a xml way,"
    message 0 "                           so that they can be transform to an HTML page."
    message 0 "                           (default: it is printed without xml tags)."
    message 0 "       -L/--Lang           Indicates the language in which the tests are written"
    message 0 "                           (default: bash)."
    message 0 " "
    message 0 "       -c/--conf           Indicates the path to a configuration file containing"
    message 0 "                           all executives options (.def file location, log file,"
    message 0 "                           xml or not, language)."
}

plaintext() {
    sed -e 's/&/\&amp;/g;s/</\&lt;/g;s/>/\&gt;/g;s/"/\&quot;/g'
}
#
# DESCRIPTION:  This script takes a .def file in entry describing the tests
#               and runs the tests.
#               This script is compatible with the HTML_1D_parser-script
#               and can provide log files compatible with the log2html-script.sh
#               script.
#               
#
# RETURN VALUE: 0 PASSED
#	 	1 FAIL
#		2 WARNING
#		3 XFAIL
#		4 UNRESOLVED
#		5 UNTESTED
#		6 UNSUPPORTED
#               
##################################################################


# We should take a look for an $EDG_TESROOT/etc/edg-test.conf or 
# ~/.edg-tsft/edg-test.conf for configuration file


##################### Pre-requested tests #####################
if test -z "$EDG_TESTROOT"
then
      echo "EDG_TESTROOT environment variable must be defined!" > /dev/stderr
      exit 4
else
      if ! test -d $EDG_TESTROOT
      then
            echo "ERROR: $EDG_TESTROOT directory does not exist." > /dev/stderr
	    exit 4
      fi
fi    

################ Function initialization ######################
. $EDG_TESTROOT/sbin/sourcer.sh || exit 4
################ Variable initialization ######################

sourceedgtestconffile
sourcetestsiteconffile

export DEBUG=1
export XML=""
export SEL_STEP=""
export CONF_FILE="no"
export LANG_NAME="bash"
export HOST="$(hostname --long)"
export RSH=""
export RCOMMAND="export EDG_TESTROOT=$EDG_TESTROOT;source $EDG_TESTROOT/etc/edg-test.conf;source $EDG_TEST_FUNCTIONS/sourcer.sh;$TMP_LOG_CMD_FILE"

export TMP_LOG_STEP_FILE=$EDG_TEST_TMP/step$$.tmp
export TMP_LOG_INFO_FILE=$EDG_TEST_TMP/info$$.tmp
export TMP_LOG_CMD_FILE=$EDG_TEST_TMP/cmd$$.tmp
export TMP_LOG_INFO_DET_FILE=$EDG_TEST_TMP/inf_det$$.tmp
export TMP_LOG=$EDG_TEST_TMP/log$$.tmp


####################### do_test() #############################
do_one_step()
{
    ### Check function argument
    if test -z "$1" 
    then
	message 1 "$0: wrong number of arguments." > /dev/stderr
	exit 4
    fi
 
    ### Log file initialization
    if test -z "$2"
    then
          export TMP_LOG_FILE=$EDG_TEST_TMP/tmplog_$PPID
    else
          export TMP_LOG_FILE=$2.new
    fi			    			    
    $TOUCH $TMP_LOG_FILE
    if  [ $? != 0 ]
    then
	message 1 "$0: fail to create log file" > /dev/stderr
	exit 4
    fi
   
    $TAIL --follow=name $TMP_LOG_FILE 2>/dev/null | $GREP -e Executing -e Testing& 
    follow_pid=$!
    export follow_pid=$(($follow_pid - 1))
    trap '$RM -f $TMP_LOG_FILE; kill -4 $follow_pid 2>/dev/null; exit 130' TERM INT

    ### Exit code initialization
    export RET_OK=0
    export RET_ERROR=1
    export RET_WARNING=2
    export RET_XFAIL=3
    export RET_UNRESOLVED=4
    export RET_UNTESTED=5
    export RET_UNSUPPORTED=6
    export RETVALUE=$RET_DEFVAL

    grep_selected_step $1
    grep_selected_pre_command
    chmod 700 $TMP_LOG_CMD_FILE    
    $TMP_LOG_CMD_FILE   
    grep_selected_command
    chmod 700 $TMP_LOG_CMD_FILE
    
    if test -z "$2" && ! test "$XML" = "1" ;then
      xterm -T "$TITLE"  -e $TAIL -f $TMP_LOG_FILE 2>/dev/null &
    fi
 

    ### A step
    ### Print beginning infos in XML ?
    if ! test "$XML" = "1"
    then
	  grep_selected_title
	  RET_TITLE=$?
          
	  ### Test performed locally ?
	  if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
	  then
	        scp $TMP_LOG_CMD_FILE $selected_host:$TMP_LOG_CMD_FILE
	        $RSH $selected_host $RCOMMAND >> $TMP_LOG_FILE 2>&1		
	  else	  
	        $TMP_LOG_CMD_FILE >> $TMP_LOG_FILE 2>&1
	  fi	  
	  RET=$?
	  $ECHO -n "$(grep_selected_brf_info)"
	  trace_status $RET
	  $ECHO -n "$(grep_selected_brf_info)" >> $TMP_LOG_FILE
	  trace_status $RET >> $TMP_LOG_FILE 2>&1
    else
	  $ECHO -e "\t\t\t<step>" >> $TMP_LOG_FILE
	  $ECHO -e "\t\t\t\t<output>" >> $TMP_LOG_FILE
	  
	  ### Test performed locally ?
	  plaintext < $XML_FIFO >> $TMP_LOG_FILE &
	  XMLPID=$!
	  if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
	  then
	        scp $TMP_LOG_CMD_FILE $selected_host:$TMP_LOG_CMD_FILE
		$RSH $selected_host $RCOMMAND > $XML_FIFO 2>&1
	  else	  
	        $TMP_LOG_CMD_FILE > $XML_FIFO 2>&1	
	  fi
	  RET=$?
	  wait $XMLPID
	  trace_status $RET >> $TMP_LOG_FILE 2>&1
	  $ECHO -e "\t\t\t\t</output>" >> $TMP_LOG_FILE	  
	  $ECHO -en "\t\t\t\t<retval>" >> $TMP_LOG_FILE
	  $ECHO -n $RET >> $TMP_LOG_FILE
          $ECHO "</retval>" >> $TMP_LOG_FILE
	  if [ -e $EDG_TEST_TMP/$$substatus_$1.tmp ] ; then
	      $ECHO -e "\t\t\t\t<substatus>" >> $TMP_LOG_FILE
	      $CAT $EDG_TEST_TMP/$$substatus_$1.tmp  >> $TMP_LOG_FILE 
	      $RM $EDG_TEST_TMP/$$substatus_$1.tmp
	      $ECHO -e "\t\t\t\t</substatus>" >> $TMP_LOG_FILE
	  fi    
	  $ECHO -e "\t\t\t</step>" >> $TMP_LOG_FILE
    fi
    	      
    ### Print in the log file or in stdout

    if test -z "$2"
    then
	 if test "$XML" = "1" 
	 then
	     $CAT $TMP_LOG_FILE | column -s % -t
	 fi    
    else
	 $CAT $TMP_LOG_FILE | column -s % -t >> $2
    fi
    $RM -f $TMP_LOG_FILE

    wait $follow_pid
    return $RET
}

################## do_multiple_test() #########################
do_multiple_test()
{
 ### Log file initialization
 if test -z "$1"
 then
       export TMP_LOG_FILE=$EDG_TEST_TMP/tmplog_$PPID
 else
       export TMP_LOG_FILE=$1.new
 fi
 $TOUCH $TMP_LOG_FILE
 if  [ $? != 0 ]
    then
	message 1 "$0: fail to create log file" > /dev/stderr
	exit 4
 fi

 $TAIL --follow=name $TMP_LOG_FILE 2>/dev/null | $GREP -e Executing -e Testing & 
 follow_pid=$!
 export follow_pid=$(($follow_pid - 1))
 trap '$RM -f $TMP_LOG_FILE; kill -4 $follow_pid 2>/dev/null; exit 130' TERM INT 

 ### Exit code initialization
 export RET_OK=0
 export RET_ERROR=1
 export RET_WARNING=2
 export RET_XFAIL=3
 export RET_UNRESOLVED=4
 export RET_UNTESTED=5
 export RET_UNSUPPORTED=6
 export RET_DEFVAL=7
 export RETVALUE=$RET_DEFVAL

 i=1
 while test $i -le $NB_TEST;
 do
     grep_selected_step $i
     grep_selected_pre_command   
     chmod 700 $TMP_LOG_CMD_FILE    
     $TMP_LOG_CMD_FILE   

     grep_selected_command
     chmod 700 $TMP_LOG_CMD_FILE
    
     ### Show one xterm for several tests
    if test $i = 1 && test -z "$1" && ! test "$XML" = "1" 
    then
	xterm -T "$TITLE" -geometry 100x25+300+0 -e $TAIL -f  $TMP_LOG_FILE 2>/dev/null &
    fi

    grep_selected_title
    RET_TITLE=$?
     
    ### Is there a title in the step ?
    if test $RET_TITLE = 1
    then
           ### No
	   if test $i = $NB_TEST
	   then
	         ### LAST step without a title, execute the command and print the result
		 ### Print beginning infos in XML ?
		 if ! test "$XML" = "1"
		 then
		      ### Test performed locally ?
		      if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
		      then
		            scp $TMP_LOG_CMD_FILE $selected_host:$TMP_LOG_CMD_FILE
			    $RSH $selected_host $RCOMMAND >> $TMP_LOG_FILE 2>&1
		      else		         
		            $TMP_LOG_CMD_FILE >> $TMP_LOG_FILE 2>&1
		      fi
		      RET=$? 
		      $ECHO -n "$(grep_selected_brf_info)"
		      trace_status $RET
		      $ECHO -n "$(grep_selected_brf_info)" >> $TMP_LOG_FILE 2>&1
		      trace_status $RET >> $TMP_LOG_FILE 2>&1
		      return_status
		      RET=$?
		 else
		      ### Test performed locally ?
  		      plaintext < $XML_FIFO >> $TMP_LOG_FILE &
		      XMLPID=$!
		      if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
		      then
		            scp $TMP_LOG_CMD_FILE $selected_host:$TMP_LOG_CMD_FILE
			    $RSH $selected_host $RCOMMAND > $XML_FIFO 2>&1
		      else
		            $TMP_LOG_CMD_FILE > $XML_FIFO 2>&1
		      fi
		      RET=$?
		      wait $XMLPID
		      trace_status $RET >> $TMP_LOG_FILE 2>&1
		      $ECHO -e "\t\t\t\t</output>" >> $TMP_LOG_FILE
		      $ECHO -en "\t\t\t\t<retval>" >> $TMP_LOG_FILE
		      return_status
		      $ECHO -n $? >> $TMP_LOG_FILE
		      $ECHO "</retval>" >> $TMP_LOG_FILE
		      if [ -e $EDG_TEST_TMP/$$substatus_$i.tmp ] ; then
			  $ECHO -e "\t\t\t\t<substatus>" >> $TMP_LOG_FILE
			  $CAT $EDG_TEST_TMP/$$substatus_$i.tmp  >> $TMP_LOG_FILE 
			  $RM $EDG_TEST_TMP/$$substatus_$i.tmp
			  $ECHO -e "\t\t\t\t</substatus>" >> $TMP_LOG_FILE
		      fi    
		      $ECHO -e "\t\t\t</step>" >> $TMP_LOG_FILE	
           	      
		 fi
	   else
	         ### A step without a title, execute the command and trace the return code
		 ### Print beginning infos in XML ?
		 if ! test "$XML" = "1"
		 then
		       ### Test performed locally ?
		       if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
		       then
		             scp $TMP_LOG_CMD_FILE $selected_host:$TMP_LOG_CMD_FILE
			     $RSH $selected_host $RCOMMAND >> $TMP_LOG_FILE 2>&1
		       else
		             $TMP_LOG_CMD_FILE >> $TMP_LOG_FILE 2>&1
		       fi
		       RET=$? 
		       $ECHO -n "$(grep_selected_brf_info)"
		       trace_status $RET
		       $ECHO -n "$(grep_selected_brf_info)" >> $TMP_LOG_FILE 2>&1
		       trace_status $RET >> $TMP_LOG_FILE 2>&1
		 else
		       ### Test performed locally ?
		       plaintext < $XML_FIFO >> $TMP_LOG_FILE &
		       XMLPID=$!
		       if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
		       then
		             scp $TMP_LOG_CMD_FILE $selected_host:$TMP_LOG_CMD_FILE
			     $RSH $selected_host $RCOMMAND > $XML_FIFO 2>&1
		       else
		             $TMP_LOG_CMD_FILE > $XML_FIFO 2>&1
		       fi
		       RET=$?
		       wait $XMLPID
		       trace_status $RET >> $TMP_LOG_FILE 2>&1
		 fi
	   fi
    else
           ### Yes
           case $i in
               1)        ### First step with a title
	                 ### Only step ?
			 if test $i = $NB_TEST
			 then
			       ### Yes
			       ### Print beginning infos in XML ?
			       if ! test "$XML" = "1"
			       then
				     ### Test performed locally ?
				     if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
				     then
				           scp $TMP_LOG_CMD_FILE $selected_host:$TMP_LOG_CMD_FILE
					   $RSH $selected_host $RCOMMAND >> $TMP_LOG_FILE 2>&1
				     else
				           $TMP_LOG_CMD_FILE >> $TMP_LOG_FILE 2>&1
				     fi
				     RET=$? 
				     $ECHO -n "$(grep_selected_brf_info)"
				     trace_status $RET
				     $ECHO -n "$(grep_selected_brf_info)" >> $TMP_LOG_FILE 2>&1
				     trace_status $RET >> $TMP_LOG_FILE 2>&1
				     return_status
				     RET=$?
			       else
			             $ECHO -e "\t\t\t<step>" >> $TMP_LOG_FILE
				     $ECHO -e "\t\t\t\t<output>" >> $TMP_LOG_FILE

				     ### Test performed locally ?
				     plaintext < $XML_FIFO >> $TMP_LOG_FILE &
				     XMLPID=$!
				     if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
				     then
				           scp $TMP_LOG_CMD_FILE $selected_host:$TMP_LOG_CMD_FILE
					   $RSH $selected_host $RCOMMAND > $XML_FIFO 2>&1
				     else
				           $TMP_LOG_CMD_FILE > $XML_FIFO 2>&1
				     fi
				     RET=$?
				     wait $XMLPID
				     trace_status $RET >> $TMP_LOG_FILE 2>&1
				     $ECHO -e "\t\t\t\t</output>" >> $TMP_LOG_FILE
				     $ECHO -en "\t\t\t\t<retval>" >> $TMP_LOG_FILE
				     return_status
				     $ECHO -n $? >> $TMP_LOG_FILE
				     $ECHO "</retval>" >> $TMP_LOG_FILE
				     if [ -e $EDG_TEST_TMP/$$substatus_$i.tmp ] ; then
					 $ECHO -e "\t\t\t\t<substatus>" >> $TMP_LOG_FILE
					 $CAT $EDG_TEST_TMP/$$substatus_$i.tmp  >> $TMP_LOG_FILE 
					 $RM $EDG_TEST_TMP/$$substatus_$i.tmp
					 $ECHO -e "\t\t\t\t</substatus>" >> $TMP_LOG_FILE
				     fi    
				     $ECHO -e "\t\t\t</step>" >> $TMP_LOG_FILE

                               fi			     
			 else
			       ### No
	                       ### Print beginning infos in XML ?
			       if ! test "$XML" = "1"
			       then			     
			             ### Test performed locally ?
				     if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
				     then
			                   scp $TMP_LOG_CMD_FILE $selected_host:$TMP_LOG_CMD_FILE
					   $RSH $selected_host $RCOMMAND >> $TMP_LOG_FILE 2>&1
			             else
			                   $TMP_LOG_CMD_FILE >> $TMP_LOG_FILE 2>&1
			             fi
				     RET=$?
			             $ECHO -n "$(grep_selected_brf_info)"
				     trace_status $RET
			             $ECHO -n "$(grep_selected_brf_info)" >> $TMP_LOG_FILE 2>&1
				     trace_status $RET >> $TMP_LOG_FILE 2>&1
			       else
			             $ECHO -e "\t\t\t<step>" >> $TMP_LOG_FILE
			             $ECHO -e "\t\t\t\t<output>" >> $TMP_LOG_FILE
			       
			             ### Test performed locally ?
				     plaintext < $XML_FIFO >> $TMP_LOG_FILE &
				     XMLPID=$!
				     if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
				     then
			                   scp $TMP_LOG_CMD_FILE $selected_host:$TMP_LOG_CMD_FILE
					   $RSH $selected_host $RCOMMAND > $XML_FIFO 2>&1
			             else
			                   $TMP_LOG_CMD_FILE > $XML_FIFO 2>&1
				     fi	     
				     RET=$?
				     wait $XMLPID
			             trace_status $RET >> $TMP_LOG_FILE 2>&1
			       fi
			 fi	             
			 ;;
	       $NB_TEST) ### Last step with a title
	                 ### Print beginning infos in XML ?
			 if ! test "$XML" = "1"
			 then
			       ### Test performed locally ?
			       if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
			       then
			             scp $TMP_LOG_CMD_FILE $selected_host:$TMP_LOG_CMD_FILE
				     $RSH $selected_host $RCOMMAND >> $TMP_LOG_FILE 2>&1
			       else
			             $TMP_LOG_CMD_FILE >> $TMP_LOG_FILE 2>&1
			       fi
			       RET=$? 
			       $ECHO -n "$(grep_selected_brf_info)"
			       trace_status $RET
			       $ECHO -n "$(grep_selected_brf_info)" >> $TMP_LOG_FILE 2>&1
			       trace_status $RET >> $TMP_LOG_FILE 2>&1
			       return_status
			       RET=$? 
			 else
			       $ECHO -e "\t\t\t\t</output>" >> $TMP_LOG_FILE
			       $ECHO -en "\t\t\t\t<retval>" >> $TMP_LOG_FILE
			       return_status
			       $ECHO -n $? >> $TMP_LOG_FILE
			       $ECHO "</retval>" >> $TMP_LOG_FILE
			       n=`expr $i - 1`
			       if [ -e $EDG_TEST_TMP/$$substatus_$n.tmp ] ; then
					 $ECHO -e "\t\t\t\t<substatus>" >> $TMP_LOG_FILE
					 $CAT $EDG_TEST_TMP/$$substatus_$n.tmp  >> $TMP_LOG_FILE 
					 $RM $EDG_TEST_TMP/$$substatus_$n.tmp
					 $ECHO -e "\t\t\t\t</substatus>" >> $TMP_LOG_FILE
			       fi    				
			       $ECHO -e "\t\t\t</step>" >> $TMP_LOG_FILE
			       $ECHO -e "\t\t\t<step>" >> $TMP_LOG_FILE
			       $ECHO -e "\t\t\t\t<output>" >> $TMP_LOG_FILE
			       
			       ### Test performed locally ?
			       plaintext < $XML_FIFO >> $TMP_LOG_FILE &
			       XMLPID=$!
			       if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
			       then
			             scp $TMP_LOG_CMD_FILE $selected_host:$TMP_LOG_CMD_FILE
				     $RSH $selected_host $RCOMMAND > $XML_FIFO 2>&1
			       else			    
			             $TMP_LOG_CMD_FILE > $XML_FIFO 2>&1
			       fi			       
			       RET=$?
			       wait $XMLPID
			       trace_status $RET >> $TMP_LOG_FILE 2>&1
			       $ECHO -e "\t\t\t\t</output>" >> $TMP_LOG_FILE
			       $ECHO -en "\t\t\t\t<retval>" >> $TMP_LOG_FILE
			       return_status
			       $ECHO -n $? >> $TMP_LOG_FILE
			       $ECHO "</retval>" >> $TMP_LOG_FILE
			       if [ -e $EDG_TEST_TMP/$$substatus_$i.tmp ] ; then
					 $ECHO -e "\t\t\t\t<substatus>" >> $TMP_LOG_FILE
					 $CAT $EDG_TEST_TMP/$$substatus_$i.tmp  >> $TMP_LOG_FILE 
					 $RM $EDG_TEST_TMP/$$substatus_$i.tmp
					 $ECHO -e "\t\t\t\t</substatus>" >> $TMP_LOG_FILE
			       fi    
			       $ECHO -e "\t\t\t</step>" >> $TMP_LOG_FILE			       	
		   
			 fi	                
			 ;;
	       *)        ### A step with a title
	                 ### Print beginning infos in XML ?
			 if ! test "$XML" = "1"
			 then
			       ### Test performed locally ?
			       if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
			       then
			             scp $TMP_LOG_CMD_FILE $selected_host:$TMP_LOG_CMD_FILE
				     $RSH $selected_host $RCOMMAND >> $TMP_LOG_FILE 2>&1
			       else
			             $TMP_LOG_CMD_FILE >> $TMP_LOG_FILE 2>&1
			       fi
			       RET=$?   
			       $ECHO -n "$(grep_selected_brf_info)"
			       trace_status $RET
			       $ECHO -n "$(grep_selected_brf_info)" >> $TMP_LOG_FILE 2>&1
			       trace_status $RET >> $TMP_LOG_FILE 2>&1
			 else
			       $ECHO -e "\t\t\t\t</output>" >> $TMP_LOG_FILE
			       $ECHO -en "\t\t\t\t<retval>" >> $TMP_LOG_FILE
			       return_status
			       $ECHO -n $? >> $TMP_LOG_FILE
			       $ECHO "</retval>" >> $TMP_LOG_FILE
			       $ECHO -e "\t\t\t\t</retval>" >> $TMP_LOG_FILE
			       n=`expr $i - 1`
			       if [ -e $EDG_TEST_TMP/$$substatus_$n.tmp ] ; then
					 $ECHO -e "\t\t\t\t<substatus>" >> $TMP_LOG_FILE
					 $CAT $EDG_TEST_TMP/$$substatus_$n.tmp  >> $TMP_LOG_FILE 
					 $RM $EDG_TEST_TMP/$$substatus_$n.tmp
					 $ECHO -e "\t\t\t\t</substatus>" >> $TMP_LOG_FILE
			       fi    
			       $ECHO -e "\t\t\t</step>" >> $TMP_LOG_FILE
			       $ECHO -e "\t\t\t<step>" >> $TMP_LOG_FILE
			       $ECHO -e "\t\t\t\t<output>" >> $TMP_LOG_FILE
			       
			       ### Test performed locally ?
			       plaintext < $XML_FIFO >> $TMP_LOG_FILE &
			       XMLPID=$!
			       if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
			       then
			             scp $TMP_LOG_CMD_FILE $selected_host:$TMP_LOG_CMD_FILE
				     $RSH $selected_host $RCOMMAND > $XML_FIFO 2>&1
			       else
			             $TMP_LOG_CMD_FILE > $XML_FIFO 2>&1
			       fi
			       RET=$?
			       wait $XMLPID			       
			       trace_status $RET >> $TMP_LOG_FILE 2>&1
			 fi
			 ;;
	   esac
    fi
    i=$(($i + 1))
 done

 ### Print in the log file or in stdout
 if test -z "$1"
 then
    if test "$XML" = "1" 
    then
        $CAT $TMP_LOG_FILE | column -s % -t
    fi    
 else
    $CAT $TMP_LOG_FILE | column -s % -t >> $1
 fi

 $RM -f $TMP_LOG_FILE
 wait $follow_pid
 return $RET
}



################# Parse the command line ######################
PROG_NAME="`/bin/basename $0`"

ARGS_SHORT="hinxs:d:l:L:c:"
ARGS_LONG="help,info,numstep,xml,step:,def:,log:,Lang:,conf:"

GETOPT_OUT=`$GETOPT -o $ARGS_SHORT --long $ARGS_LONG -n $PROG_NAME -- "$@"`

if ! test $? = 0 || ! test $1
then 
      $ECHO "Try \`$PROG_NAME -h' for more informations." > /dev/stderr
      exit 4
fi

eval set -- "$GETOPT_OUT"
while true ; do
    case "$1" in
	-h|--help)      HELP=1                   ;;
	-i|--info)      INFO=1                   ;;
	-n|--numstep)   NB_STEP=1                ;;
	-x|--xml)       XML=1                    ;;
	-s|--step)      SEL_STEP=$2      ; shift ;;
	-d|--def)       DEF_FILE=$2      ; shift ;;
	-l|--log)       LOGOUT=$2        ; shift ;;
	-L|--Lang)      LANG_NAME=$2     ; shift ;;
	-c|--conf)      CONF_FILE=$2     ; shift ;;
	--)             shift            ; break ;;
	*)              $ECHO "Internal error!"  > /dev/stderr ; exit 4 ;;
    esac
    shift
done


### Show the help
if test -n "$HELP"
then
      usage
      exit 0
fi

### Read values from .conf file if it exists
if ! test "$CONF_FILE" = "no"
then
      if ! test -e $CONF_FILE
      then
            $ECHO "ERROR: .conf file does not exist." > /dev/stderr
	    exit 4
      else
            . $CONF_FILE
	    if test "$DEF_FILE" = ""
	    then
	          DEF_FILE=$TEST_DEF
	    fi
	    
	    if test "$LOGOUT" = ""
	    then 
	          LOGOUT=$TEST_LOG
	    fi
	    if test "$XML" = ""
	    then
	          XML=$TEST_XML
	    fi
	    
	    if test "$LANG_NAME" = ""
	    then
	          LANG_NAME=$TEST_LANGUAGE
	    fi
	    HOST=$TEST_HOST
	    RSH=$TEST_RSH
	    RCOMMAND=$TEST_RCOMMAND
      fi
fi

### Test the existence of the .def file
if test -z "$DEF_FILE"
then
      $ECHO "ERROR: You MUST pass in argument a .def file in order to run the test." > /dev/stderr
      exit 4
else 
      if ! test -e $DEF_FILE
      then
            $ECHO "ERROR: .def file does not exist." > /dev/stderr
            exit 4
      fi
fi

### Test the existence of the ssh command
if ! test "$RSH" = ""
then 
      $RSH -V >/dev/null 2>&1
      if ! test "$?" = "0"
      then
            $ECHO "ERROR: ssh command not found." > /dev/stderr
	    exit 4
      fi
fi

### Show the total number of steps
if test -n "$NB_STEP"
then   
    NB_TEST=$(check_step)
    $ECHO "$NB_TEST"
    exit 0
fi

### Show infos about steps
if test -n "$INFO"
then  
    print_info "$SEL_STEP"
    $RM -f $TMP_LOG_INFO_FILE
    $RM -f $TMP_LOG_STEP_FILE
    exit 0
fi     

### Process the langage
case "$LANG_NAME" in
     bash|sh)             
              LANG_PATH="#! $(which bash 2>&1)"
	      ;;
     csh|tcsh)
              LANG_PATH="#! $(which csh 2>&1)"	      
              ;;
     perl)    
              LANG_PATH="#! $(which perl 2>&1)"
              ;;
     python)  
              LANG_PATH="#! $(which python 2>&1)"
              ;;
     ruby)    
              LANG_PATH="#! $(which ruby 2>&1)"
              ;;
     *)       
              LANG_PATH="#! $(which bash 2>&1)"
              ;;
esac
if ! test $? = 0
then
      $ECHO "ERROR: $LANG_NAME is not installed in this computer." > /dev/stderr
      exit 4
fi


######################### Main ################################
### Variables initialization
RETVAL=0
RETVALNEW=0
NB_TEST=$(check_step)
NB_TITLE=$(check_title)


### Test of SEL_STEP
if ! test -z "$SEL_STEP"
then
      if test $SEL_STEP -gt $NB_TEST
      then
            $ECHO "ERROR: the number of step $SEL_STEP does not exist."
	    $ECHO "This must be a value between 1 and $NB_TEST."
	    exit 4
      fi
fi


### Grep informations about the tests
TITLE=$(grep_info_title)
DATE=$(date +%c)
RUNHOST=$(hostname --long)
#ROWS_LIST=$HOST
ROWS_LIST=$SITE_NAME
COLS_LIST="Results"
BRF_INFO=$(grep_info_brf)
grep_info_det

### Print beginning infos in XML ?
if test "$XML" = "1"
then

      # remove the use of ascii escape sequence in echoFunc.sh (not compatible with HTML browsing)
      unset SETCOLOR_SUCCESS
      unset SETCOLOR_FAILURE
      unset SETCOLOR_WARNING
      unset SETCOLOR_NORMAL
      export MOVE_TO_COL="echo -n %"
      XML_FIFO=$EDG_TEST_TMP/xml-fifo-$$
      mkfifo $XML_FIFO
      if ! test $? = 0
      then
            $ECHO "ERROR: cannot create fifo $XML_FIFO" > /dev/stderr
            exit 4
      fi

      $ECHO "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" > $TMP_LOG
      $ECHO "<?xml-stylesheet href=\"$EDG_TESTROOT/etc/log.xsl\" type=\"text/xsl\"?>" >> $TMP_LOG
      $ECHO "<log>" >> $TMP_LOG
      $ECHO -e "\t<info>" >> $TMP_LOG
      $ECHO -e "\t<title>" >> $TMP_LOG
      $ECHO "$TITLE" | plaintext >> $TMP_LOG
      $ECHO -e "\t</title>" >> $TMP_LOG
      $ECHO -e "\t<date>" >> $TMP_LOG
      $ECHO "$DATE" | plaintext >> $TMP_LOG
      $ECHO -e "\t</date>" >> $TMP_LOG
      $ECHO -e "\t<host>" >> $TMP_LOG
      $ECHO "$RUNHOST" | plaintext >> $TMP_LOG
      $ECHO -e "\t</host>" >> $TMP_LOG
      $ECHO -e "\t<rows_list>" >> $TMP_LOG
      for row in $ROWS_LIST;
      do
      	$ECHO -ne "\t\t<row>" >> $TMP_LOG
      	$ECHO -n "$row" | plaintext >> $TMP_LOG
      	$ECHO "</row>" >> $TMP_LOG
      done
      $ECHO -e "\t</rows_list>" >> $TMP_LOG
      $ECHO -e "\t<cols_list>" >> $TMP_LOG
      for col in $COLS_LIST;
      do
      	$ECHO -ne "\t\t<col>" >> $TMP_LOG
      	$ECHO -n "$col" | plaintext >> $TMP_LOG
      	$ECHO "</col>" >> $TMP_LOG
      done
      $ECHO -e "\t</cols_list>" >> $TMP_LOG
      
      
      ### Take all brief infos or just brief infos about a selected step
      i=1
      while ! test "$($ECHO $BRF_INFO | $CUT -d# -f$i)" = "";
      do          
	  i=$(($i + 1))
      done
      
      
      ### Take all brief infos or just brief infos about a selected step
      if ! test -z "$SEL_STEP"
      then
            k=$SEL_STEP
            i=$(($SEL_STEP + 1))
      else
            k=1
      fi

      ### NB_TITLE = NB_TEST ?
      while ! test $k = $i;
      do          
          if ! test "$NB_TITLE" = "$NB_TEST" && ! test "$SEL_STEP" = ""
	  then	
	        $ECHO -e "\t\t<step>" >> $TMP_LOG
		$ECHO -e "\t\t\t<brf_info>" >> $TMP_LOG
		grep_selected_step $k
		$ECHO $(grep_selected_brf_info) | plaintext >> $TMP_LOG
		$ECHO -e "\t\t\t</brf_info>" >> $TMP_LOG
		$ECHO -e "\t\t\t<det_info>" >> $TMP_LOG
		grep_selected_det_info $k
		$CAT $TMP_LOG_INFO_FILE | plaintext >> $TMP_LOG
		$ECHO -e "\t\t\t</det_info>" >> $TMP_LOG
		$ECHO -e "\t\t\t<substatus_info>" >> $TMP_LOG
		grep_selected_step $k
		grep_selected_substatus_info | plaintext >> $TMP_LOG
		$ECHO -e "\t\t\t</substatus_info>" >> $TMP_LOG
		$ECHO -e "\t\t</step>" >> $TMP_LOG		
	  else	  
                $ECHO -e "\t\t<step>" >> $TMP_LOG
		$ECHO -e "\t\t\t<brf_info>" >> $TMP_LOG
		$ECHO $($ECHO $BRF_INFO | $CUT -d# -f$k) | plaintext >> $TMP_LOG
		$ECHO -e "\t\t\t</brf_info>" >> $TMP_LOG
		$ECHO -e "\t\t\t<det_info>" >> $TMP_LOG
		grep_selected_info_det $k | plaintext >> $TMP_LOG
		$ECHO -e "\t\t\t</det_info>" >> $TMP_LOG
		$ECHO -e "\t\t\t<substatus_info>" >> $TMP_LOG
		grep_selected_step $k
		grep_selected_substatus_info | plaintext >> $TMP_LOG
		$ECHO -e "\t\t\t</substatus_info>" >> $TMP_LOG
		$ECHO -e "\t\t</step>" >> $TMP_LOG	     
	  fi
	  k=$(($k + 1))
      done
      $ECHO -e "\t</info>" >> $TMP_LOG
      
      ### Print in the log file or in stdout
      if test "$LOGOUT" = ""
      then
            $CAT $TMP_LOG
      else 
            $CAT $TMP_LOG > $LOGOUT
      fi
fi

for selected_host in "$HOST" ;
do
    ### Print beginning tests infos in XML ?              
    if test "$XML" = "1"
    then
          $ECHO -e "\t<row>" > $TMP_LOG
	  $ECHO -e "\t\t<col>" >> $TMP_LOG
	  if test "$LOGOUT" = ""
	  then
	        $CAT $TMP_LOG
	  else
                $CAT $TMP_LOG >> $LOGOUT
	  fi
    fi


    ### Launch Tests    
    if  test -z "$SEL_STEP" 
    then      
          ### Do all tests
	  do_multiple_test $LOGOUT
	  RETVAL=$?                  
    else
          ### Launch selected test
	  do_one_step $SEL_STEP $LOGOUT
	  RETVAL=$?
    fi    


    ### Print ending testsinfos in XML ?              
    if test "$XML" = "1"
    then
          $ECHO -e "\t\t</col>" > $TMP_LOG
	  $ECHO -e "\t</row>" >> $TMP_LOG	

	  if test "$LOGOUT" = ""
	  then
                $CAT $TMP_LOG
          else
                $CAT $TMP_LOG >> $LOGOUT
          fi
    fi
done
NOW=$(date +%H:%M:%S) 
### Print ending testsinfos in XML ?              
if test "$XML" = "1"
then      
      $ECHO "</log>" > $TMP_LOG
      if test "$LOGOUT" = ""
      then
            $CAT $TMP_LOG
      else
           $CAT $TMP_LOG >> $LOGOUT
	   sed "s/$DATE/$DATE - $NOW/" $LOGOUT > $TMP_LOG
	   $CAT $TMP_LOG > $LOGOUT 	    
      fi
      $RM -f $XML_FIFO
fi

### Test performed locally ?
if test "$RSH" = "" && ! test "$selected_host" = "$(hostname --long)"
then
      $RSH $selected_host "$RM -f $TMP_LOG_CMD_FILE"
else
      $RM -f $TMP_LOG_CMD_FILE
fi
$RM -f $TMP_LOG_INFO_FILE
$RM -f $TMP_LOG_STEP_FILE
$RM -f $TMP_LOG_INFO_DET_FILE
$RM -f $TMP_LOG

exit $RETVAL
