#!/usr/bin/perl -w
###############################################################################
#
# Script to convert XML log file to HTML page
# Author: Andrey Kiryanov <Andrey.Kiryanov@cern.ch>
# Written to replace the log2html-script.sh
#
# $Id: log2html-script.pl,v 1.1.1.2 2005/02/08 16:09:43 mbarroso Exp $
#
###############################################################################
use strict;
use File::Basename;
use Getopt::Std;

BEGIN {
# delete $ENV{LANG};
# delete $ENV{LANGVAR};
# delete $ENV{SUPPORTED};
#$ENV{'LC_ALL'} = "C";
#$ENV{'LC_CTYPE'} = "C";
$ENV{'LANG'} = "en_US";
$ENV{'LANG'} = "C";
#$ENV{PERL_BADLANG} = 0;
}

sub chopsection($$) {
	if($_[0] =~ s/^((.|\n)*?)<\s*($_[1])\s*>\s*((.|\n)*?)\s*<\s*\/\3\s*>/$1/i) {
		return $4;
	} else {
		return undef;
	}
}
sub nextsection($) {
	if($_[0] =~ /^\s*<\s*([^<>\s]+)\s*>(.|\n)*<\s*\/\1\s*>/) {	
		return lc $1;
	} else {
		return undef;
	}
}
sub htmlhead($) {
	return "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\">\n<html><head><title>$_[0]</title></head><body style=\"background:white;\">";
}

# Step 0: Get command line options and read input file
my $ph = '&#8212;';
my ($out_desc, $out_log) = ("#", "#");
our ($opt_h, $opt_d, $opt_D, $opt_l, $opt_L, $opt_f, $opt_j);
my ($xml, $tmp, $step, @rows, @cols, $brfinfo, @detinfo, @out, @ret, @sub, @stit, @jtit);
getopts('hjLl:Dd:f:');
die("Options -l and -L are mutually exclusive\n") if $opt_l && $opt_L;
die("Options -d and -D are mutually exclusive\n") if $opt_d && $opt_D;
$out_log = undef if $opt_L;
$out_desc = undef if $opt_D;
my $in_file = shift;
if(! $in_file || $opt_h) {
	print <<EOF;
Usage: log2html-script.pl [-h] [-L|l <filename>] [-D|d <filename>] [-f <filename>] <XML filename>
Options:
	-h  Show help
        -j  To add Job number label into the step column for Stack family storms
	-L  Skip logging info
	-l  Write logging info to a separate file instead of main HTML page
	-D  Skip tests descriptions
	-d  Write tests descriptions to a separate file instead of main HTML page
	-f  Write HTML page to a file (stdout by default)
EOF
	exit 1;
}
open(IN, "<$in_file") || die("Cannot open input file: $!\n");
while(<IN>) {
	$xml .= $_;
}
close(IN);
# Step 1: Strip comments and determine whether input file is valid XML log
$xml =~ s/<!--\s(.|\n)*?\s-->//g;
$xml = chopsection($xml, "log");
die("Input file is not a valid XML log!\n") if ! defined $xml;
if($opt_f) {
	close(STDOUT);
	open(STDOUT, ">$opt_f") || die("Cannot open file $opt_f : $!\n");
	$tmp = dirname($opt_f);
	$tmp =~ s/\/+/\//g;
}
if($opt_l) {
	open(LOG, ">$opt_l") || die("Cannot open file $opt_l : $!\n");
	$out_log = dirname($opt_l) . "/";
	$out_log =~ s/\/+/\//g;
	$out_log =~ s/^$tmp\/// if $opt_f;
	$out_log .= basename($opt_l) . "#";
} else {
	*LOG = *STDOUT;
}
if($opt_d) {
	open(DESC, ">$opt_d") || die("Cannot open file $opt_d : $!\n");
	$out_desc = dirname($opt_d) . "/";
	$out_desc =~ s/\/+/\//g;
	$out_desc =~ s/^$tmp\/// if $opt_f;
	$out_desc .= basename($opt_d) . "#";
} else {
	*DESC = *STDOUT;
}
# Step 2: Process the INFO section
my ($title, $date, $host) = ("", "", "");
my $info = chopsection($xml, "info");
if(defined $info) {
	$title = chopsection($info, "title");
	if(! defined $title) {
		print STDERR "Warning: XML log has no TITLE section\n";
		$title = "";
	};
	$date = chopsection($info, "date");
	$host = chopsection($info, "host");
# Step 3: Get list of columns and rows
	$tmp = chopsection($info, "rows_list");
	if(defined $tmp) {
		while($_ = chopsection($tmp, "row")) {
			push @rows, $_;
		}
		if($#rows < 0) {
			@rows = split(/\s+/, $tmp);
			print STDERR "Warning: XML log uses deprecated syntax for row names\n" unless $#rows < 0;
		}
	} else {
		print STDERR "Warning: XML log has no ROWS_LIST section\n";
	}
	$tmp = chopsection($info, "cols_list");
	if(defined $tmp) {
		while($_ = chopsection($tmp, "col")) {
			push @cols, $_;
		}
		if($#cols < 0) {
			@cols = split(/\s+/, $tmp);
			print STDERR "Warning: XML log uses deprecated syntax for column names\n" unless $#cols < 0;
		}
	} else {
		print STDERR "Warning: XML log has no COLS_LIST section\n";
	}
# Step 4: Get descriptions of test steps
	$step = -1;
	while($tmp = chopsection($info, "step")) {
		$step++;
		$brfinfo .= "<br>" if defined $brfinfo;
		if($_ = chopsection($tmp, "brf_info")) {
			if($out_desc) {
				$brfinfo .= "<a href=\"${out_desc}desc$step\" style=\"text-decoration:none;color:black;\">$_</a>";
			} else {
				$brfinfo .= $_;
			}
		} else {
			$brfinfo .= $ph;
		}
		if($_ = chopsection($tmp, "det_info")) {
			push @detinfo, $_;
		} else {
			push @detinfo, $ph;
		}
	}
	print STDERR "Warning: XML log has no step descriptions defined\n" if $step == -1;
	$brfinfo = "" if $step == -1 || $brfinfo =~ /^$ph(<br>$ph)*$/;
} else {
	print STDERR "Warning: XML log has no INFO section\n";
}

# Step 5: Print the HTML head
print htmlhead($title) . "<div style=\"text-align:center;\">";
print "<h1 style=\"color:blue;\">$title</h1>" if $title;
print "<p>Test run on: <b>$host</b></p>" if $host;
print "<p>Test run date: <b>$date</b></p>" if $date;

# Step 6: Fill the output table
my ($t, $s, $q, $tag);
my @status = ($ph,  qw/OK         FAIL   WARNING XFAIL  UNRESOLVED UNTESTED UNSUPPORTED/);
my @fg =     (qw/black green      red    blue    red    black      black    black      /);
my @bg =     (qw/white lightgreen yellow yellow  yellow darkgrey   darkgrey darkgrey   /);

my ($r, $c) = (-1, -1);
while($tag = nextsection($xml)) {
	$tmp = chopsection($xml, $tag);
	if($tag eq "row") {
		$r++;
		$c = -1;
		$tag = "col";
	} else {
		$c++;
		$r = -1; 
		$tag = "row";
	}
	while($t = chopsection($tmp, $tag)) {
		if($tag eq "row") {
			$r++;
		} else {
			$c++;
		}
		$step = -1;
		while($s = chopsection($t, "step")) {
			$step++;
			$q = chopsection($s, "output");
			if(defined $q) {
				push @{$out[$r][$c]}, $q;
			} else {
				push @{$out[$r][$c]}, $ph;
			}
			$sub[$r][$c] .= "<br>" if defined $sub[$r][$c];
			$q = chopsection($s, "substatus");
			if(defined $q) {
				$sub[$r][$c] .= $q;
			} else {
				$sub[$r][$c] .= $ph;
			}
			$ret[$r][$c] .= "<br>" if defined $ret[$r][$c];
			$q = chopsection($s, "retval");
			if($q =~ /^\d$/) {
				$q++;
			} else {
				$q = 0;
			}
			if($out_log) {
				$ret[$r][$c] .= "<a href=\"${out_log}log$r-$c-$step\" style=\"text-decoration:none;color:$fg[$q];background:$bg[$q];\">$status[$q]</a>";
			} else {
				$ret[$r][$c] .= "<span style=\"color:$fg[$q];background:$bg[$q];\">$status[$q]</span>";
			}
		}
		$out[$r][$c] = () if $step == -1 || $out[$r][$c] =~ /^$ph(<br>$ph)*$/;
		$sub[$r][$c] = "" if $step == -1 || $sub[$r][$c] =~ /^$ph(<br>$ph)*$/
	}
}
# Step 7: Format and print the output table
print "\n<table style=\"margin:auto;border-collapse:collapse;\" cellpadding=5>";
for($t = -1 ; $t <= $r ; $t++) {
	print "\n<tr>";
	for($s = -1 ; $s <= $c; $s++) {
		if($t == -1) {
			if($s == -1) {
				print "<th>" if $brfinfo;
				print "<th>";
			} else {
				print "<th style=\"border:1px solid;\" colspan=2>";
				if($cols[$s]) {
					print $cols[$s];
				} else {
					print $ph;
				}
			}
			next;
		}
		if($s == -1) {
		        push @jtit, "Job_$t";
			unless ( $opt_j ) {
			    print "<td style=\"border:1px solid;text-align:right;white-space:nowrap;\">$brfinfo" if $brfinfo;
			} else {
			    print "<td style=\"border:1px solid;text-align:right;white-space:nowrap;\">Job_$t : $brfinfo" if $brfinfo;
			}
			print "<th style=\"border:1px solid;\">";
			if($rows[$t]) {
				print $rows[$t];
			} else {
				print $ph;
			}
		} else {
			if($sub[$t][$s]) {
				print "<td style=\"border:1px solid;white-space:nowrap;\">$ret[$t][$s]<td style=\"border:1px solid;text-align:left;white-space:nowrap;\">$sub[$t][$s]";
			} else {
				print "<td style=\"border:1px solid;white-space:nowrap;\" colspan=2>$ret[$t][$s]";
			}
		}
	}
}
print "</table>";

@stit = split("<br>", $brfinfo);

# Step 8: Print tests descriptions if requested
if($out_desc && $brfinfo) {
	print DESC htmlhead("$title - Test Description") if $opt_d;
	print DESC "\n<h1 style=\"color:blue;\">Test Description</h1><div style=\"text-align:left\">";
	$brfinfo =~ s/href="[^#"]*#([^#"]+)"[^>]+>/name="$1">/g;
	foreach(split("<br>", $brfinfo)) {
		print DESC "\n<p><b>$_:</b></p><pre>" . (shift @detinfo) . "</pre>";
	}
	print DESC "</div>";
	if($opt_d) {
		print DESC "</body></html>";
		close(DESC);
	}
}
# Step 9: Print logging info if requested
if($out_log) {
	$tmp = 0;
	print LOG htmlhead("$title - Logging info") if $opt_l;
	print LOG "\n<h1 style=\"color:blue;\">Logging info</h1><div style=\"text-align:left\">";
	for($t = 0 ; $t <= $#out ; $t++) {
		for($s = 0 ; $s <= $#{$out[$t]} ; $s++) {
			for($step = 0 ; $step <= $#{$out[$t][$s]} ; $step++) {
				print LOG "<hr style=\"border:none;height:1px;background:red;color:red;width:80%;\">" if $tmp;
				##GOOD print LOG "\n<p><b>$stit[$step]:$rows[$t]:$cols[$s]</b></p>";
				my $glog = "<p><b>";
				unless ( $opt_j ) {
				    $glog .= "$stit[$step] : " if $stit[$step];
				} else {
				    $glog .= "$jtit[$t] : " if $jtit[$t];
				}
				$glog .= "$rows[$t] : " if $rows[$t];
				$glog .= "$cols[$s] : " if $cols[$s];
				$glog .= "</b></p>";
				$glog =~ s/<br>/ /g;
				##BETTER print LOG "\n$glog";
				$glog = "" if ( "$out[$t][$s][$step]" eq "$ph" );
				$glog = "" if ( "$out[$t][$s][$step]" eq "" );
				##BETTER print LOG "\n<a name=\"log$t-$s-$step\"></a><pre>$out[$t][$s][$step]</pre>";
				print LOG "\n<a name=\"log$t-$s-$step\"></a>$glog\n<pre>$out[$t][$s][$step]</pre>";
				$tmp = 1;
			}
		}
	}
	print LOG "</div>";
	if($opt_l) {
		print LOG "</body></html>";
		close(LOG);
	}
}
print "</div></body></html>";
close(STDOUT);
# That's all, folks...
