#!/usr/bin/perl

package BaseTest;

use strict;

use Getopt::Long;
use EDGUtils;
require Exporter;

#####################################CTRL+C
## Cf 'cat /usr/include/asm/signal.h'
## Cf 'kill -l'
## To simulate CTRL+C, send 'kill -USR1 pid', or 'kill -10 pid' on the process "edg-testbed-test"
## OR simulate CTRL+C, send 'kill -USR2 pid', or 'kill -12 pid' on the process "edg-testbed-test"
## Here we choose USR1 for the time being ...
our $sigK = "USR1";
use POSIX qw(:signal_h);
use vars qw($sigC);
#####################################CTRL+C

use TestUtils;

our $globalSelf;

=head1 NAME

BaseTest.pm - A set of utilities for EDG site certification tests
# $Id: BaseTest.pm,v 1.1.1.2 2005/02/08 16:09:43 mbarroso Exp $

=head1 SYNOPSIS

BaseTest.pm 

=head1 DESCRIPTION 

B<BaseTest.pm> contains a set of utilities to support the EDG site
certification tests.  This is not intended to be called directly
but to be used through the edg-tb-test command. 

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


# This constructor creates an anonymous hash and blesses this into the
# correct class.  Subclasses can modify the context by subclassing the
# setContext method.
#
# The only argument is an array containing the test arguments. 
sub new {

    # Get the class and arguments. 
    my $class = shift;
    my $argref = shift;

    # Create the instance of this class.
    my $self = {};
    bless $self, $class;

    # Load the arguments into the context and fix context for subclasses.
    $self->{args} = $argref;
    $self->{opts} = {};
    $self->setContext($self);

    return $self;
}

# Subclasses should override this method if they wish to modify the
# context.  Typically can be used to set the HelpText and the DescText
# for the subclassed test.  Overridden methods should always call the
# parent's setContext method before any modifications are made. 
sub setContext {
    my $self = shift;

    print "BaseTest: setContext\n";

    # Create the empty arrays for the option linkage, descriptions,
    # and defaults.
    $self->{optionLinkage} = [];
    $self->{optionDesc} = [];
    $self->{optionDefaults} = [];

    # Add the help and desc options.  Subclasses may add more
    # options if desired.
    $self->addOption("help",
		     "Print short help text.");
    $self->addOption("desc",
		     "Print longer description of test.");
    if ( $ENV{MAINSCRIPT} ) {
	$self->addOption("optv",
			 "Print effective option values.",
			 1);
#     } else {
# 	$self->addOption("optv",
# 			 "Print effective option values.");
    }

## gg
    $self->{globalresult} = " ";
    if ( defined $ENV{MAINSCRIPT} ) {
	$self->{conffile} = "";
	$self->{outfile}  = "";
	$self->{stepfile} = "";

	$self->{htmldim}  = "";

	## XML parser case
	$self->{xmlfile}  = "";
	$self->{xmlxist}  = "";
	$self->{htmldir}  = "";
	$self->{htmlout}  = "index.html";

	$self->addOption("stackDebug=i",
			 "Print caller stack for debugging (minimum depth: 1).",
			 0);
	$self->addOption("xml",
			 "To prepare and produce an XML file (default: false).");
	$self->printStack(3);

    }

}


# Process the complete test. 
sub process {
    my $self = shift;

    # Process the command line arguments.
    my $error = $self->checkArguments();

    # Check to see if an error occurred.
    if ($error) {
        print "ERROR in processing options.\n";
        $self->printHelp();
        return 1;
    }

    # Set any defaults for parameters not set by user.
       $error = $self->setDefaults();

    # Check to see if an error occurred.
    if ($error) {
        print "ERROR in setting default option values.\n";
        $self->printHelp();
        return 1;
    }

    # If the help or desc was requested, print that then return.
    $self->printHelp() if ($self->{opts}->{help});
    $self->printDesc() if ($self->{opts}->{desc});
    $self->printOptv() if ($self->{opts}->{optv});
    return 0 if (($self->{opts}->{help}) || ($self->{opts}->{desc}));

    $error = $self->checkPrerequisites();
    $self->checkError($error, "checking prerequisites");

    $error = $self->setupTest();
    $self->checkError($error, "setting up test");

#####################################CTRL+C
    my $loopC;
    my $sigsetC;
    my $blocksetC;

    if ( defined $ENV{MAINSCRIPT} ) {
	##no	if ($self->{opts}->{timeoutLapse}) {
	if ( $ENV{EDG_TIMEOUT} ) {
	    $sigsetC   = POSIX::SigSet->new;
	    $blocksetC = POSIX::SigSet->new(SIGUSR1);

	    $SIG{$sigK} = \&catch_it;
	    print STDERR " Setting up catch_it with signal: $sigK \n";
	}
    }
    #####################################CTRL+C

    $error = $self->runTest();
    $self->checkError($error, "running test");

    return 0 if ( $self->{opts}->{NOXcute} );

    $error = $self->evaluateTest();
    $self->checkError($error, "evaluating test");

##gg
    $error = $self->parsing("noerror");

    $error = $self->cleanup();
    $self->checkError($error, "cleaning up test");

}

#####################################CTRL+C
sub catch_it {
	$SIG{$sigK} = \&catch_it;
	#	exit if $sig;
	$sigC = shift;
	    print STDOUT "\n #####################################CTRL+C\n";
	    print STDOUT " Received SIG$sigC. globalSelf: $globalSelf \n";
	    print STDOUT " #####################################CTRL+C\n\n";

	    print STDERR "\n #####################################CTRL+C\n";
	    print STDERR " Received SIG$sigC. globalSelf: $globalSelf \n";
	    print STDERR " #####################################CTRL+C\n\n";

	if ( 0 ) {
	my $self = $$globalSelf;
	bless ($self);
	print STDOUT " Received SIG$sigC. globalSelf: $globalSelf \n";

    my $error = $self->evaluateTest();
    $self->checkError($error, "evaluating test from catch_it");

    $error = $self->parsing("noerror");

    $error = $self->cleanup();
    $self->checkError($error, "cleaning up test from catch_it");

	exit;
}

  return 0;

    }
#####################################CTRL+C

# Process the command line arguments and check that they are
# sensible.  If not, raise an error. 
sub checkArguments {
    my $self = shift;

#old    print "BaseTest: checkArguments\n";
    if ( $ENV{MAINSCRIPT} ) { $self->printStack(3); }

    # Hand these off to the standard POSIX processor.
    @ARGV = @{$self->{args}};
    my $result = &GetOptions(\%{$self->{opts}}, @{$self->{optionLinkage}});

    # Return an error flag if necessary.  (GetOptions returns 0 on error.)
    return (! $result);
}

# Process the command line arguments and set the defaults.
sub setDefaults {
    my $self = shift;

#old    print "BaseTest: setDefaults\n";
    if ( $ENV{MAINSCRIPT} ) { $self->printStack(3); }

    # Loop through all of the options. 
    for my $i (0 .. $#{$self->{optionDefaults}}) {

        my $default = ${$self->{optionDefaults}}[$i];
        if ("$default" ne "") {
            my $linkage = ${$self->{optionLinkage}}[$i];
            $linkage =~ /([^=:!]+)([=:!].*)?/;
            my $option = $1;
            if (!defined($self->{opts}->{$option})) {
                print "Setting default $option = $default.\n";
                $self->{opts}->{$option} = $default;
            }
        }
    }

  if ( $ENV{MAINSCRIPT} ) { print STDOUT " ---EndSettingDefaults\n"; }

    return 0;
}

# Derived classes should override this if they wish to check
# prerequisites for the test.  Returning 0 indicates that all
# prerequisites are satisfied; returning any non-zero value indicates
# failure.  In this case the error text should be set via the
# setErrorText method. 
sub checkPrerequisites {
    my $self = shift;

    print "BaseTest: checkPrerequisites\n";
    $self->printStack();

    # Verify that needed job submission commands exist on the 
    # underlying system.  The test must set $self->{commands} to be a
    # reference to an array containing the necessary commands. 
    if ($self->{commands}) {
	my $arrayref = checkCommandsExist(\@{$self->{commands}});
	my @failed = @{$arrayref};
	if ($#failed>=0) {
	    my $names = join(",",@failed);
	    $self->setErrorMessage("command(s) '$names' unavailable");
	    return 1;
	}
    }

    return 0;
}

# Do any necessary preprocessing for the test.
sub setupTest {
    my $self = shift;

    print "BaseTest: setupTest\n";

    return 0;
}

# Run the test and collect results.
sub runTest {
    my $self = shift;

    print "BaseTest: runTest\n";

    return 0;
}

# Evaluate the results of the test.
sub evaluateTest {
    my $self = shift;

    print "BaseTest: evaluateTest\n";

    return 0;
}

# Do any necessary, post-test cleanup. 
sub cleanup {
    my $self = shift;

    print "BaseTest: cleanup\n";
    $self->printLocation();

    if ( defined $ENV{MAINSCRIPT} ) {
	return 0 if ( "$self->{outfile}" eq "" );
##NOTYET	unlink $self->{outfile}, $self->{stepfile}, $self->{conffile};
##NOTYET	unlink $self->{outfile}, $self->{xmlfile};
    }

    return 0;
}

# Building the HTML report from the Step file - old method(gg)
# Subclasses should not need to override this method.
sub runParserScript {
    my $self = shift;

#p    print "BaseTest: runParserScript\n";
    $self->printLocation( (caller(0))[3] );

    my $ptrbsf = $self->can('buildStepFile');
    return 0 if ( ! $ptrbsf );
    my $errbsf = $self->buildStepFile();
    return 1 if ( $errbsf );
    my $errbcf = $self->buildConfFile(\@{$self->{args}});
    return 1 if ( $errbcf );

##?? To be used with '-a $stepFile' below ??
##??    my $stepFile = '"'."-f /tmp/$ENV{USER}-$$-GridFTPstep".'"';

## To communicate with Extractor.pl
    $ENV{STEPFILE} = "$self->{stepfile}";

## Call this with "bash -x" or "bash -v" when needed ...
    my $parser = "$ENV{EDG_TESTROOT}/sbin/HTML_"."$self->{htmldim}"."_parser-script.sh";
    my $resPars = `/bin/bash  $parser -c $self->{conffile}`;
    print " After Parsing: \n$resPars \n";

    return 0;
}

# Building the HTML report from the XML report (gg)
# Subclasses should not need to override this method.
sub runXMLParserScript {
    my $self = shift;

    print "BaseTest: runXMLParserScript\n";

    my $ptrbsf = $self->can('buildXMLFile');
    return 0 if ( ! $ptrbsf );
    my $errbsf = $self->buildXMLFile();
    return 1 if ( $errbsf );

##    my $parser = "log2html-script.sh";
    if (0) {
    my $html_dir = ( defined $ENV{HTML_DIR} ) ? "$ENV{HTML_DIR}" : "$ENV{HOME}/TSTG/HTML";
    $html_dir = "${html_dir}/cal_gatekeeper";
    my $resmk = `mkdir -p $html_dir`;
  }
#    my $html_out = "index.html";
    my $html_out = "$self->{htmlout}";
#    my $parser = "log2html-script.sh --styledir $EDG_TESTROOT/etc/parser_conf/html_style ";
#    my $parser = "log2html-script.pl --outdir $self->{htmldir} --outfile $html_out --styledir $ENV{EDG_TEST_HTML_STYLE} ";
    my $parser = "log2html-script.pl -f $self->{htmldir}/$html_out ";
    print STDERR "$parser $self->{xmlfile} \n";
## Call this with "bash -x" or "bash -v" when needed ...
    my $resPars = `$parser $self->{xmlfile}`;
    print STDERR " After XMLParsing: \n$resPars \n" if ( $resPars ne "" ) ;

    return 0;
}


# A utility method to print the help text.  Subclasses should not need
# to override this method.
sub printHelp {
    my $self = shift;
    my $text = $self->{HelpText} || "Empty help.\n";
    print "\n$text\n";
    print "Available options (default): \n";
    for my $i (0 .. $#{$self->{optionLinkage}}) {
#old	print "$self->{optionLinkage}[$i] ($self->{optionDefaults}[$i])\n     $self->{optionDesc}[$i]\n\n";
	print "$self->{optionLinkage}[$i] ($self->{optionDefaults}[$i]):\t$self->{optionDesc}[$i]\n";
    }
    print "---\n";
}

# A utility to print the description text.  Subclasses should not need
# to override this method. 
sub printDesc {
    my $self = shift;
    my $text = $self->{DescText} || "Empty description.\n";
    print "\n$text\n";

    ## Not very well maintained below
    return 0 if (1);

    ## Added by GG
    print "\n The commands required/used in this test are:\n";
    foreach my $c ( @{$self->{commands}} ) {
	print "\t$c\n";
    }
}

# A utility method to print the option values.  Subclasses should not need
# to override this method.
sub printOptv {
    my $self = shift;
#     my $text = $self->{HelpText} || "Empty help.\n";
#     print "\n$text\n";
    open OPTV, ">&=3";
#good    print OPTV "Specific Test options >value< (default): \n\n";
    print OPTV "</pre>\n<h3>\n Specific Test options   >value<   (default): \n</h3><pre>\n";
    for my $i (0 .. $#{$self->{optionLinkage}}) {
#old	print "$self->{optionLinkage}[$i] ($self->{optionDefaults}[$i])\n     $self->{optionDesc}[$i]\n\n";
	my ($oname, $dum) = split /[!:=]/, $self->{optionLinkage}[$i];
	if ( $oname =~ /help|desc|optv/ ) { next; }
	print OPTV "--"."$oname"."=\t>$self->{opts}->{$oname}<"."\t($self->{optionDefaults}[$i])\n";
    }
    print OPTV "---\n";
    close OPTV;
}

# A utility method to add an option with description to the option
# list.  The default value is optional.  If specified and the user
# doesn't set a value the default will be used. The empty string is
# considered an undefined default. (See Getopt::Long description for
# proper syntax.)
sub addOption { 
    my ($self, $tag, $desc, $default) = @_;

    push @{$self->{optionLinkage}}, $tag;
    push @{$self->{optionDesc}}, $desc;
    if ($default) {
        push @{$self->{optionDefaults}}, $default;
    } else {
        push @{$self->{optionDefaults}}, '';
    }
}


# A utility to set the error message. 
sub setErrorMessage {
    my ($self, $message) = @_;
    $self->{message} = $message;
    $self->{globalresult} .= " [CRASHED]" if $self->{globalresult};
}


# A utility function to print an error message and exit if
# necessary.  This should not be called by subclasses. 
sub checkError {
    my ($self, $error, $stage) = @_;

    if ($error) {
        print "ERROR while $stage.\n";
        my $message = $self->{message};
        $message = "none" unless $message;
        print "Message: $message.\n";

	my $dummy = $self->parsing("error");

        exit 1;
    }
}


sub parsing {
    my $self = shift;
    my $flag = shift;
    $self->printLocation( (caller(0))[3] );
    my $error;
    #     $error = `cat $self->{outfile}`;
    #     print STDOUT "$error";

    if ( defined $ENV{MAINSCRIPT} ) {

#p	print " globalresult =$self->{globalresult}= \n";
	if ( $self->{globalresult} ) {
	    $error = $self->resultParser($flag);
	}

	if ( $self->{xmlxist} eq "" ) {

	    $error = $self->runParserScript();
	    $self->checkError($error, "HTMLparsing test");

	} elsif ( $self->{xmlxist} eq "print" ) {

	    $error = `cat $self->{outfile}`;
	    print STDOUT "$error";

	} else {

	    $error = $self->runXMLParserScript();
	    $self->checkError($error, "XMLparsing test");

	}

    }
    return 0;
}

sub resultParser {
    my $self = shift;
    my $flag = shift;
    $self->printLocation( (caller(0))[3] );
    my $error;
    my $global = $self->{globalresult};
    $global =~ s/\s+//;
#    my $result = "NOTEVALUATED";
    my $result = "CRASHED";
##    if ( $flag eq "noerror" ) {
	## Try to find out what is available in $global ...
	## And compute (cook ?) an overall value ...
	my @m = split /\s+/, $global;
	$result = 'OK';  ##  Default if nothing is "seen"
	foreach my $item ( @m ) {
	    if ( $item !~ /^\[[A-Z]*\]$/ ) { print STDERR " UGLY: =$item= \n"; next; }
	    $result = 'FAIL' unless ( $item eq '[OK]' );
	}
##    }
    print STDOUT " GLOBALRESULT: [$result] HTMLDIR: =$self->{htmldir}= HTMLOUT: =$self->{htmlout}= \n";
    return 0;
}


sub printLocationUU {
    my $self = shift;
    if ($self->{debug} > 0) {
	my $method = shift;
	my $class = ref $self;
	print " ==> ${class}::${method}()\n";
    }
    return 0;
}

sub parseTURLs {
  my $self = shift;

  my ($str) = @_;

  my $href = {};

  # Use a hash ref with array refs components to store the output:
    $href->{nt} = 0;
    $href->{turls} = [];
    $href->{protocols} = [];
    $href->{hostnames} = [];
    $href->{paths} = [];

    while ( $str =~ m/(([^:\s\n]+):\/\/([^\/]+)\/+(.+))/g ) {
#BAD?    while ( $str =~ m/(([^:\s\n]+):\/\/([^\/]+)(?::\d+)?\/+(.+))/g ) {
	++($href->{nt});
	push @{$href->{turls}}, $1;
	push @{$href->{protocols}}, $2;
##	push @{$href->{hostnames}}, $3;
	my $hp = $3;
	push @{$href->{paths}}, "/" . $4;
#p	print " parseTURLs: $href->{nt} $2 $3 $1 $4 \n";
#	$hp =~ /(.*?)(:\d+)?/;
	my @m = split /:/, $hp;
	push @{$href->{hostnames}}, $m[0];
    }

    return $href;
}

# # # A utility to temporarily switch the STDOUT to a file
# # sub switchStdout {
# #     local *FH;
# #     my ($self, $file) = @_;
# # #p    print " Sending STDOUT to $file \n";
# #     open FH, ">&STDOUT" or die "Can't redirect STDOUT \n";
# #     open STDOUT, ">$file";
# #     select STDOUT; $|=1;
# #     return *FH;
# # }

# # # A utility to restore the STDOUT to its initial state
# # sub restoreStdout {
# #     local *FH;
# #     ($self, *FH) = @_;
# #     close STDOUT;
# #     open STDOUT, ">&FH";
# # }

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
