#!/usr/bin/perl

package BaseTest::EDGLifecycle::MatchMakingScriptChecksum;

use strict;
use vars qw(@ISA);
use File::Basename;

@ISA=qw(BaseTest::EDGLifecycle);

=head1 NAME

MatchMakingScriptChecksum.pm - A job needed for the Matchmaking test with mdsum test

=head1 SYNOPSIS

MatchMakingScriptChecksum.pm 

=head1 DESCRIPTION 

B<MatchMakingScriptChecksum.pm> This module is based on the HelloScript.pm module done by Charles Loomis.
But here the job is done to test the Matchmaking on the EDG 2.0  

=head1 AUTHOR

Charles Loomis (charles.loomis.cern.ch) and Eric Fede (eric.fede@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Eric Fede and Charles Loomis. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut


sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "MatchMakingScriptChecksum: setContext\n";

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::MatchMakingScriptChecksum[options]

END
    ;

    $self->{DescText} = <<'END'


END
    ;
    # new option
    $self->addOption('SEhost=s',
                     'Name of the target SE.',
                     'lxshare0408.cern.ch');

    $self->addOption('VOname=s',
                     'Name of user VO.',
                     'wpsix');

    $self->addOption('RCConf=s',
                     'Path to the edg-replica-manager.conf.',
                     '');

    $self->addOption('nrecord=i',
                     'Number of record to do.',
                     '1');
 
    $self->addOption('filesize=i',
                     'Size (Ko) of the file to copy on SE.',
                     '1');

}

# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.) 
sub setupTest {
    my $self = shift;

    print "MatchMakingScriptChecksum: setupTest\n";

    # Call the parent method to ensure that the temporary directory is
    # created.  
    my $error = $self->SUPER::setupTest();
    if ($error) {
        $self->setErrorMessage('default setup failed');
        return 1;
    }

    # Retrieve the temporary directory. 
    my $tmpdir = $self->{tempdir};

    # Create the list of file to put on SE
    my @listofSEfile = createfilename($self->{opts}->{nrecord});

    # Create the file on tempory directory
  
    createphysicalfile (@listofSEfile,$tmpdir,$self->{opts}->{filesize});

    # Do the copy and registration

    my $copyregisterstatus = CopyAndRegister(@listofSEfile,$tmpdir,$self->{opts}->{VOname},$self->{opts}->{RCConf},$self->{opts}->{SEhost});

    # Create the script for this job.
    my $script = createScript($tmpdir);
    if (! $script) {
        $self->setErrorMessage('could not create job script');
        return 1;
    }

    # Create the JDL file.
    my $file = $self->createJDL($tmpdir, $script);
    if ($file) {
        $self->{jdl} = $file;
    } else {
        $self->setErrorMessage('could not create JDL file');
        return 1;
    }
 
    my $toto = AddInputData(@listofSEfile,$self->{opts}->{VOname},$file);

    return 0;
    #return 1;
}


##
## Below here are utility functions for supporting the methods above.
## These should not need to be called by subclasses.
##


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createJDL {
    my ($self, $dir, $script, $site) = @_;

    my ($sname, $spath, $suffix) = fileparse($script, []);

    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable    = "$sname";
StdOutput     = "std.out";
StdError      = "std.err";
InputSandbox  = {"$script"};
OutputSandbox = {"std.out","std.err"};
DataAccessProtocol  = {"file"};
END
    ; 
    
    print JDL "Requirements = $self->{opts}->{requirements};\n" 
	if $self->{opts}->{requirements};
    print JDL "Rank = $self->{opts}->{rank};\n" 
	if $self->{opts}->{rank};
    print JDL "RetryCount = $self->{opts}->{retrycount};\n" 
	if ("$self->{opts}->{retrycount}" ne "");
	#Modify By Fede
	#if $self->{opts}->{retrycount};
    print JDL "MyProxyServer = $self->{opts}->{myproxyhost};\n" 
	if $self->{opts}->{myproxyhost};
    close JDL;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createScript {
    my $dir = shift;

    my $file = "$dir/JobScript.sh";

    # Write out the job description.
    open SCRIPT, ">$file";


    print SCRIPT <<'END'

#!/bin/sh

# get the file list
`cat .BrokerInfo | grep srm:// >listsrm`
`tr -d '"' <listsrm > listsrm.ok`
flist=`cat listsrm.ok`

# To get the physical file name of the needed file

i=0
for srm in $flist; do
# Get TURL
  TURL=`edg-rm  --vo=iteam getTurl $srm gsiftp`
# Get file
  name="/$(echo "$TURL" | cut -d/ -f6-)"
# name
  theSum=`md5sum $name`
  echo "$theSum"
  i=$((i+1))
done

`rm -f listsrm.ok listsrm`
END
    ; 
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# create a list of filename with a name with a name depending of the input parameter.
sub createfilename {
	my ($nboffile) = @_;
        my @listofname;
        my @validchars = ('A' .. 'Z', '0' .. '9');

	for (1 .. $nboffile){
        my $filename = '';
        for (1 .. 12) {
             # take a random position
                my $randompos = int(rand(($#validchars+1)));
                $filename .= $validchars[$randompos];
        }
     push @listofname,$filename;
    # put the name on the lista
     }
    return @listofname;
}
sub createphysicalfile {
    my (@file) = @_;
    my $tmpfilepath=$file[$#file-1];
    my $size=$file[$#file];
    my $i;
    my $commande;
    my $command;	
    print "MatchMakingScript: Random file creation \n";
    for $i (0 .. $#file-2) {
	my $filename="$tmpfilepath/$file[$i]";
	# Create a single file
    	if ($size < 1000 ){
       	$commande="dd if=/dev/random of=$filename bs=1024 ibs=1024 count=$size  conv=sync";}
        elsif ($size > 1000000) {
        my $size=$size/1000000;
        $commande="dd if=/dev/random of=$filename bs=1024M ibs=1024M count=$size  conv=sync";}
        else {
        my $size=$size/1000;
        $commande="dd if=/dev/random of=$filename bs=1024k ibs=1024k count=$size  conv=sync";
        }
    open(IN,"$commande |") or die $!;
    close IN;
    $command = `md5sum $filename >>/$tmpfilepath/md5sumlocal`;
    }

}    


sub CopyAndRegister {
    my (@filelist) = @_;
    my $SEhost=$filelist[$#filelist];
    my $RCconf=$filelist[$#filelist-1];
    my $VO = $filelist[$#filelist-2];
    my $tmpfilepath=$filelist[$#filelist-3];
    my $returnstatus=0;
    my $i;
    my $useconfig='';
    if ($RCconf ne '') { $useconfig='--config='.$RCconf;}
    print "MatchMakingScript: Copy and register the file on SE \n";
    for $i (0 .. $#filelist-4) {
        print "edg-rm --vo=$VO -i $useconfig copyAndRegisterFile file:$tmpfilepath/$filelist[$i]  -d $SEhost -l lfn:$filelist[$i] >> $tmpfilepath/GUIDLIST";
        print "\n";
    	my $commandresult = `edg-rm --vo=$VO -i $useconfig copyAndRegisterFile file:$tmpfilepath/$filelist[$i]  -d $SEhost -l lfn:$filelist[$i] >> $tmpfilepath/GUIDLIST`;
    	my $flag = ($!) ? 0 : 1;
        if ($commandresult != 0 ){$returnstatus = 1;}
   }
   return $returnstatus;
}

sub AddInputData {
	my (@filelist) = @_;
	my $JDLfile=$filelist[$#filelist];
	my $VONAME=$filelist[$#filelist-1];
        my $i;
        open FILE, ">>$JDLfile";
        print FILE "InputData     = {";
        for $i (0 .. $#filelist-3) {
          # add the SE requieremnt on job description.
          print FILE "\"lfn:$filelist[$i]\",";
        }
        print FILE "\"lfn:$filelist[$#filelist-2]\"};\n";
        print FILE "DataAccessProtocol  = { \"file\",\"rfio\",\"gridftp\" };\n";
        print FILE "VirtualOrganisation = \"$VONAME\";";
	return 0;
}          
# Do any necessary, post-test cleanup.
sub cleanup {
    my $self = shift;

    print "MatchMakingScript: cleanup\n";
    #remove the replica
    my $RCconf = $self->{opts}->{RCConf};
    my $VOName = $self->{opts}->{VOname};
    my $i;
    my $returnstatus=0;
    my $GUIDID;
    my $commandresult;
    my $useconfig='';
    if ($RCconf ne '') { $useconfig='--config='.$RCconf;}
    # Open GUID file
    print "MatchMakingScript : Remove the logical file from the replica\n";
    my $GUIDpathfile="/tmp/$ENV{USER}-$$/GUIDLIST";
    open(GUIDD,$GUIDpathfile);
    while ($i = <GUIDD>){
    	chomp($i);
    	$commandresult = `edg-rm --vo=$VOName -i $useconfig deleteFile $i -a`;
    	my $flag = ($!) ? 0 : 1;
	if ($flag != 0 ){$returnstatus = 1;}
    }
    close (GUID);

    my $dir = $self->{tempdir};
#    rmtree($dir) if $dir;

    return $returnstatus;
} 
# the evaluation of the test
sub evaluateOutput {
    my ($self,$stdout,$stderr) = @_;
    my $command;
    my $tmpdir=$self->{tempdir};
    # Check that the std.out and std.err files exist.
    return 1 if (! (-e "$stdout"));
    return 1 if (! (-e "$stderr"));

    # Check that the standard error has a zero size.
    return 1 if (-s "$stderr");

    $command=`cat $stdout | cut -d" " -f1 | sort >/$tmpdir/stdoutformat`;
    $command=`cat /$tmpdir/md5sumlocal | cut -d" " -f1 | sort >/$tmpdir/md5sumlocalformat`;
    
    # Check the standard output contains the string [OK].
    
    my $value = `diff /$tmpdir/md5sumlocalformat /$tmpdir/stdoutformat`;
    if ($value eq "") {
	   return 0;
        }
    return 1;
}
# This ensures that the module returns a true value when loaded.
# If this isn't here, loading the module may incorrectly fail.
1;

