#!/usr/bin/perl

package BaseTest::EDGLifecycle::ProxyRenewal;

use strict;
use vars qw(@ISA);
use File::Basename;

@ISA=qw(BaseTest::EDGLifecycle);

=head1 NAME

ProxyRenewal.pm - A test to determine if the proxy renewal works.
# $Id: ProxyRenewal.pm,v 1.1.1.2 2005/02/08 16:09:43 mbarroso Exp $

=head1 SYNOPSIS

ProxyRenewal.pm

=head1 DESCRIPTION 

B<ProxyRenewal.pm> A test to determine if the proxy renewal mechanism
works.  The test just waits in a loop until checking to see if the
proxy is renewed or if it expires.  This job should be submitted with
a short-lived proxy. 

=head1 AUTHOR

Charles Loomis (charles.loomis@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2002 by Charles A. Loomis, Jr. and Le Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

    print "ProxyRenewal: setContext\n";

    # Add the time option.
    $self->addOption('time=i',
		     'Seconds to wait for renewal',
                     '900');

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::ProxyRenewal [options] 

END
    ;

    $self->{DescText} = <<'END'

 This is a test to determine if the proxy renewal mechanism
(using the RB and the MyProxy server) works.
 It submits a job which sleeps and loops, checking the number of
times the proxy is renewed or expires.  The test fails if the proxy expires.
works.  The test just waits in a loop until checking to see if the
proxy is renewed or if it expires. 

 In the (refurbished) MainScript version (PXRenew test), the test deals 
by itself with both short and long proxy creations, as well as with job
output retrieval for checking the results, which was not very handy before.
 In addition, several test jobs are sent simultaneously (saving test time)
to a given RB, allowing for a more accurate testing since the jobs are 
usually running on different CEs. The options which control the 
number of jobs are --nstreams & --njobs (see CalStorm description).
 There is no way as yet to choose a specific CE.
 Also, it can redirect the jobs to any specified RB, using the --forcingRB
general option of the MainScript wrapper.

 There are 2 pitfalls however:
- the test duration is about 35-40min, because of lack of flexibility in
short proxy creation. To be improved.
- the test often fails because there are still ghost jobs polluting
the RB database, which puzzle the proxy renewal mechanism.
To be improved as well.

 A typical submission used in a regression test is:
opt/edg/bin/MainScript --forcingTB=CertTB --TList=PXRenew \
  --verbose --forcingVO=dteam --forcingRB=lxshare0240.cern.ch\
--addOptionList=\"--keepTempDir --nstreams=3 --njobs=2\"

END
    ;

}


# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.) 
sub setupTest {
    my $self = shift;

    print "ProxyRenewal: setupTest\n";

    $self->{type} = "ProxyRenewal";

    # Call the parent method to ensure that the temporary directory is
    # created.  
    my $error = $self->SUPER::setupTest();
    if ($error) {
        $self->setErrorMessage('default setup failed');
        return 1;
    }

    # Retrieve the temporary directory. 
    my $tmpdir = $self->{tempdir};

    # Create the script for this job.
    my $script = createScript($tmpdir);
    if (! $script) {
        $self->setErrorMessage('could not create job script');
        return 1;
    }

    # Create the JDL file.
    my $file = $self->createJDL($tmpdir, $script, 
			 $self->{opts}->{time});
    if ($file) {
        $self->{jdl} = $file;
    } else {
        $self->setErrorMessage('could not create JDL file');
        return 1;
    }

    return 0;
}


##
## Below here are utility functions for supporting the methods above.
## These should not need to be called by subclasses.
##


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createJDL {
    my ($self, $dir, $script, $time) = @_;

    my ($sname, $spath, $suffix) = fileparse($script, []);

    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable    = "$sname";
Arguments     = "$time";
StdOutput     = "std.out";
StdError      = "std.err";
InputSandbox  = {"$script"};
OutputSandbox = {"std.out","std.err"};
##MyProxyServer = "$self->{opts}->{myproxyhost}";

END
    ; 

#     print JDL "Requirements = $self->{opts}->{requirements};\n" 
# 	if $self->{opts}->{requirements};
## --requirements=\"other.GlueCEPolicyMaxWallClockTime>2200\"
    if ( $ENV{MAINSCRIPT} ) {
	my $timeR = $self->{opts}->{time} + 120;
	print JDL "Requirements = other.GlueCEPolicyMaxWallClockTime>$timeR;\n";
    } elsif ( $self->{opts}->{requirements} ) {
	print JDL "Requirements = $self->{opts}->{requirements};\n" 
    }
    print JDL "Rank = $self->{opts}->{rank};\n" 
	if $self->{opts}->{rank};
    print JDL "RetryCount = $self->{opts}->{retrycount};\n" 
	if ($self->{opts}->{retrycount}>=0);
    print JDL "MyProxyServer = \"$self->{opts}->{myproxyhost}\";\n" 
	if $self->{opts}->{myproxyhost};
    close JDL;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# Submits a job and returns the job ID from the job submission.
# If a resource is specified then a direct submission is used,
# otherwise the matching is done with the job manager. 
sub createScript {
    my $dir = shift;

    my $file = "$dir/JobScript.sh";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/usr/bin/perl

	use strict;

    my $limit = shift or die "Must supply time limit in seconds.\n";

    my $cmd = 'grid-proxy-info -timeleft';
    my $start = time;

    # Get the current limit on the proxy. 
    my $remainder = `$cmd`;
    chomp($remainder);
    print " Initial time limit: $remainder $limit\n";

    if ( $limit <= $remainder ) {
	print "Proxy Renewal impossible. Wrong job setup\n";
	print "[FAIL]\n";
        exit 1;
    }

    # Count number of proxy renewals.  Loop until the time limit
    # or until the proxy expires. 
    my $nrenew = 0;
    while (((time()-$start)<$limit) && $remainder>=0) {

	my $last = $remainder;

	sleep(60);
	my $duration = time()-$start;

	# Determine the time remaining for the proxy.
	$remainder = `$cmd`;
	chomp($remainder);
	print " Current time limit: $remainder ($duration)\n";

	# Test if the proxy was renewed. 
	if ($remainder>$last) {
	    $nrenew++;
	    print "Proxy renewed (" . (scalar localtime()) . ")\n";
	}
    }

    # Check if the test was a success.
    if ($remainder<0) {
	print "Proxy expired.\n";
	print "[FAIL]\n";
    } elsif ($nrenew<=0) {
	print "No proxy renewals.\n";
	print "[FAIL]\n";
    } else {
	print "Renewals: $nrenew\n";
	print "[OK]\n";
    }

    exit;
END
    ; 
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size. 
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# Override the EDGLifecycle::evaluateOutput method
sub evaluateOutput {
    my ($self,$stdout,$stderr) = @_;
    my $value = $self->SUPER::evaluateOutput($stdout,$stderr);

    print "ProxyRenewal: evaluateOutput\n";

#     # Check that the std.out and std.err files exist.
#     return 1 if (! (-e "$stdout"));
#     return 1 if (! (-e "$stderr"));

#     # Check that the standard error has a zero size.
#     return 1 if (-s "$stderr");

#     # Check the standard output contains the string [OK]. 
#     open FILE, "<$stdout";
#     my $value = 1;
#     while (<FILE>) {
#         $value = 0 if m/.*\[OK\].*/;
#     }

    my @resC = `cat $stdout | grep -e renewed`;
    print " ProxyRenewal::evaluateOutput: @resC";

    return $value;
}

# Override the EDGLifecycle::submit method
sub submit { 

    my ($self, $argref) = @_;
#    $self->printStack(5);
    $self->printLocation( (caller(0))[3] );

    my $rc = `mkdir -p /tmp/$ENV{USER}/short`;
    return $rc if $rc;

    my $shortProxyFile = "/tmp/$ENV{USER}/short/myproxydeleg-$$";
    my $shortProxyFileSave = "/tmp/$ENV{USER}/short/myproxydeleg-$$"."_save";
    chomp(my $proxyFile = `grid-proxy-info -path`);

    if ( -e $shortProxyFile ) {
	$rc = `rm $shortProxyFile`;
    }

    ## Get the short proxy before submitting ...
#     my $shp = `myproxy-get-delegation -s lxshare0207.cern.ch -d -t 0:15 -o /tmp/grodid/short/x509up_u1110 -a /tmp/x509up_u1110`;
#     print STDERR " MPGD: $shp \n";
    my $timeM = int ( ($self->{opts}->{time} - 180)/60 );
    my $shp;
    if (0) {
	$shp = `myproxy-get-delegation -s $ENV{PXhostname} -d -t 1:00 -o $shortProxyFile -a $proxyFile 2>&1`;
    } else {
	$shp = `echo $ENV{GSI_PASSWORD} | grid-proxy-init -valid 0:$timeM -pwstdin -out $shortProxyFile 2>&1`;
    }
    $rc = $?;
    print STDERR " MPGD0: $rc\n$shp \n";
    return $rc if $rc;

    return 1 unless ( -e $shortProxyFile );

    my $choice = 1;

    ## Sets the alternate proxy area
    if ($choice==1) {
	$ENV{X509_USER_PROXY} = "$shortProxyFile";
    } else {
	rename $proxyFile, "$shortProxyFileSave";
	rename "${shortProxyFile}", $proxyFile;
#p	system("ls -l $proxyFile $shortProxyFile $shortProxyFileSave");
    }

    # Get the current limit on the proxy. 
    my $cmd = 'grid-proxy-info -timeleft';
    chomp(my $remainder = `$cmd`);
##    if ( $remainder > 1800 ) {  ## if 0:30 above
    if ( $remainder > $timeM*60 ) {
	print STDERR " MPGD1: $remainder \n";
	if ($choice==1) {
	    undef $ENV{X509_USER_PROXY};
	} else {
	    rename $proxyFile, "${shortProxyFile}";
	    rename "$shortProxyFileSave", $proxyFile;
#p	    system("ls -l $proxyFile $shortProxyFile $shortProxyFileSave");
	}
	return 1;
    }

    ## Then submit ...
    $rc = $self->SUPER::submit($argref);

    ## And destroy the short proxy ...
    sleep 10;
    if ($choice==1) {
	undef $ENV{X509_USER_PROXY};
    } else {
	rename $proxyFile, "${shortProxyFile}";
	rename "$shortProxyFileSave", $proxyFile;
#p	system("ls -l $proxyFile $shortProxyFile $shortProxyFileSave");
    }

    return $rc;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
