#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::CECycle;

use FindBin qw($Bin);
use lib $Bin;

use File::Basename;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

CECycle.pm - Check that the site specific CEs are indeed operational.
# $Id: CECycle.pm,v 1.1.1.2 2005/02/08 16:09:43 mbarroso Exp $

=head1 SYNOPSIS

Check that the site specific CEs are indeed operational.

=head1 DESCRIPTION 

B<CECycle.pm> Taking into account 
the CEs specified in the site.conf, the test is meant to check (thru a 
edg-job-list-match operation) that these CEs are indeed accepted by the RB.
 Then a job is sent to each CE in turn, to check they are fully
alive and operational. This will replace the simple HelloWorld test,
where the CE itself was not specified.
 The summary page displays the results of the former check within the MATCHING 
step, and the latter check within the RUNNING step.

 To be used from the MainScript.


=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;

    $self->SUPER::setContext();

#    print "CECycle: setContext\n";
    $self->printLocation( (caller(0))[3] );

    $self->addOption('maxSubs=i',
                     'Number of jobs.',
                     '6');
    $self->addOption('maxStack=i',
                     'Number of simultaneous jobs in the stack.',
                     '2');
    $self->addOption('reqLapse=i',
                     'Delay between 2 submissions (in sec).',
                     '15');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '111');

    # Additionnal Commands necessary to run this test.
    my @commands = (
#			  "edg-vo-env",
#			  "edg-gridftp-rm",
#			  "edg-gridftp-ls",
#			  "edg-gridftp-mkdir",
#			  "gdmp_host_subscribe",
#			  "gdmp_job_status",
#			  "globus-replica-catalog",
#			  "ldapsearch",
#			  "edg-replica-manager-replicateFile",
#			  "edg-replica-manager-copyAndRegisterFile",
#			  "edg-replica-manager-deleteFile",
#			  "edg-replica-manager-listReplicas"
		   );
    push @{$self->{commands}}, @commands;

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack::CECycle [options]

END
    ;

    $self->{DescText} = <<'END'

 Checks one by one that the site specific CEs are indeed operational.

 Taking into account 
the CEs specified in the site.conf, the test is meant to check (thru a 
edg-job-list-match operation) that these CEs are indeed accepted by the RB.
 Then a job is sent to each CE in turn, to check they are fully
alive and operational. This will replace the simple HelloWorld test,
where the CE itself was not specified.
 The summary page displays the results of the former check within the MATCHING 
step, and the latter check within the RUNNING step.

 To be used only from the MainScript.
 It is mainly a Functionality test, based on Stack family (see test JStorm).
 All available options are described in the JStorm test itself.

  Standard submission from the MainScript:
opt/edg/bin/MainScript --forcingTB=CertTB --TList=CECycle --verbose --forcingVO=dteam \
  --addOptionList="--keepTempDir --reqLapse=2 --maxStack=50 --batchSleep=10 --maxSubs=80 --serie=11001" \
  --forceMachineList="lxshare0235.cern.ch lxshare0241.cern.ch lxshare0290.cern.ch lxshare0277.cern.ch"

END
    ;

	$self->{type} = "CECycle";
#	 $self->{subDir} = "";
#	 $self->{location} = "";
#	 $self->{VO} = "";
#	 $self->{dataFL} = [];      ## List
#	 $self->{SPs} = {};         ## Hash
	 $self->{SEs} = [];         ## List
#	 $self->{SEpairs} = {};     ## Hash
	$self->{CEs} = [];         ## List

    return 0;

}

# Build any necessary setup for the storm.
sub setupEnvironment {
    my $self = shift;

    print "CECycle: setupEnvironment\n";

    return 0;
}

sub dgjob {
    my $cmdn = shift;
    my $root = "dg-job-";
    if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
	$root = "e".$root;
    }
    return "$root"."$cmdn";
}

# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

#    print "CECycle: addArguments\n";
    $self->printLocation( (caller(0))[3] );

    my $bid = `echo 'Arguments = "S$ENV{STACK_SERIE} I${index} rien"; ' >> $jdlTemp`;

if (0) {
    my $nCE = scalar(@{$self->{CEs}});
    my $rnd = int(rand($nCE));
#    my $ce = ${$self->{CEs}}[$rnd];
    my $ce = ${$self->{CEs}}[($index%$nCE)];
    if ( $self->{opts}->{useCEList} ) {
#glue    my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
#norm    my $reqs = 'Requirements = other.CEId=="'.$ce.'"';
	my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
	print STDERR " Requirements: $reqs \n";
	$bid = `echo '$reqs; ' >> $jdlTemp`;
	my @m = split /:/, $ce;
	$self->{reqs}->{$index}->{mace} = $m[0];
    } else {
	$self->{reqs}->{$index}->{mace} = "free";
    }
} else {
    $self->chooseCE($index, $jdlTemp);
}
    
    return 0;
}

# Prepare everything for running the test itself ...
sub setupTest {
    my $self = shift;

#    print "CECycle: setupTest\n";
    $self->printLocation( (caller(0))[3] );

    ## List of requested CEs
    @{$self->{CEr}} = @{$self->{args}};

#    print " Defaults: @{$self->{optionDefaults}} \n";
#    print " Linkage:  @{$self->{optionLinkage}}  \n";
#    exit 0 if (1);

    my $rc = $self->SUPER::setupTest();
    return $rc if $rc;

    ## Possible only when jdlFile and shFile get created
    $rc = $self->buildCEList();
    return $rc if $rc;

    ## Filtering out ...
    my @celist = @{$self->{CEs}};
    $self->{CEs} = [];
    ## The matching list
    $self->{CEm} = [];
    foreach my $ces ( @celist ) {
##	next if ( $ces =~ 'lcgce' );  ##  TEST
	my @m = split /:/, $ces;
	print " CE: $m[0] \n";
	push @{$self->{CEm}}, $ces;
	foreach my $cer ( @{$self->{CEr}} ) {
	    if ( $cer =~ $m[0] ) { push @{$self->{CEs}}, $ces; last; }
	}
    }
    ## Selected CE list
    foreach my $ces ( @{$self->{CEs}} ) {
	print " Selected CE: $ces \n";
    }

    $self->{zombSlep} = ( $self->{zombSlep} > $self->{reqLapse} ) ? $self->{reqLapse} : $self->{zombSlep};
    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "\n";

    return $rc;
}

sub buildXMLFileHUM {
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
#    $self->{debug} = 1;
#p    $self->printLocation( (caller(0))[3] );

    $self->buildXMLFile1Col();
    return 0;

}

sub createJDL {
    my ($self, $file, $script) = @_;

    print "CECycle: createJDL\n";
    print " file script: $file $script \n";

    my ($sname, $spath, $suffix) = fileparse($script, []);

#    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable = "$sname";
InputSandbox = {"$script"};
StdOutput = "result.out";
StdError = "result.err";
OutputSandbox = {"result.out","result.err"};
Requirements = other.GlueCEPolicyMaxCPUTime<6000000;
Rank = other.GlueCEPolicyMaxCPUTime;
END
    ;

    if ( $self->{opts}->{site} ) {
	my $site = $self->{opts}->{site};
	print JDL "Requirements = RegExp(\".*$site.*\",other.CEId);\n" if $site;
    }
    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


sub createScript {
#   my $dir = shift;
#   my $file = "$dir/JobScript.sh";
    my ($self, $file) = @_;

    print "CECycle: createScript\n";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<"END"
#!/bin/sh
set -xv
echo "Hello from $Bin "
echo "Arguments: " \$1 \$2 \$3
date
##sleep 2700
##date
pwd
echo "-- end of $self->{type}.sh -- "
##notyet /usr/afsws/bin/tokens
##printenv
ghostname=`hostname --long 2>&1`
gipname=`hostname --ip-address 2>&1`
echo \$ghostname "has address" \$gipname
host \$HOSTNAME
host \$HOST
echo EDG_WL_JOBID: \$EDG_WL_JOBID
uname -a
ls -l
env | grep DEFAULT_SE
theCE=`edg-brokerinfo getCE 2>&1 | cut -d":" -f 1`
echo DEFAULTCE: \$theCE
echo EDG_WL_LOG_DESTINATION: \$EDG_WL_LOG_DESTINATION
defSE=`edg-rm --vo dteam pi | grep 'default SE' | awk '{print \$4}' `
echo DEFAULTSE: \$defSE
firstSE=`edg-brokerinfo getCloseSEs 2>&1 | cut -d" " -f 1`
echo FIRSTSE: \$firstSE
SELIST=`edg-brokerinfo getCloseSEs 2>&1`
echo SELIST: \$SELIST
END
    ;
    close SCRIPT;
    if ( $self->{opts}->{batchSleep} ) {
	my $res = `echo 'sleep $self->{opts}->{batchSleep}' >> $file `;
    }

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
sub evaluateTest {
    my $self = shift;

#    print "CECycle: evaluateTest\n";
    $self->printLocation( (caller(0))[3] );

    ## Checking the matching ...
    my @ind = sort { $a <=> $b } keys %{$self->{reqs}};
    my $lind = $#ind;
    $lind++;
    my $lfile = "$self->{tempdir}/matching.out";
    my $stat1 = "[OK]";
    if ( scalar(@{$self->{CEr}}) != scalar(@{$self->{CEs}}) ) { $stat1 = "[FAIL]"; }
    $self->{reqs}->{$lind}->{stat1} = "$stat1";
    $self->{reqs}->{$lind}->{mawn} = "MATCHING";
    if ( $stat1 eq "[FAIL]" ) { $self->{reqs}->{$lind}->{olog} = "$lfile"; }
    else { $self->{reqs}->{$lind}->{olout} = "$lfile"; }
    open LF, ">$lfile";
    print LF "\n REQUESTED: \n\n";
    foreach my $ces ( @{$self->{CEr}} ) {
	$self->{CEu}->{$ces} = "[OK]";
	print LF " Requested CE: $ces \n";
    }
    print LF "\n SELECTED: \n\n";
    foreach my $ces ( @{$self->{CEs}} ) {
	print LF " selected CE: $ces \n";
    }
    close LF;

    ## Checking the running ...
    $lind++;
    $lfile = "$self->{tempdir}/running.out";
    $stat1 = "[OK]";
    open LF, ">$lfile";
    print LF "\n SELECTED: \n\n";
    foreach my $ces ( @{$self->{CEs}} ) {
	print LF " selected CE: $ces \n";
    }
    print LF "\n FOUND OK: \n\n";
    foreach my $index ( @ind ) {
	my $mace = $self->{reqs}->{$index}->{mace};
##	$self->{reqs}->{$index}->{stat1} = "[FAIL]" if ( $mace =~ 'lcgce' && $index == 3 );  ##  TEST
#	print LF " CE found running: $mace \n" if ( $self->{reqs}->{$index}->{stat1} eq "[OK]" );
	$stat1 = "[FAIL]" if ( $self->{reqs}->{$index}->{stat1} eq "[FAIL]" );
	if ( $self->{CEu}->{$mace} eq "[OK]" ) { $self->{CEu}->{$mace} = "$self->{reqs}->{$index}->{stat1}"; }
    }
    foreach my $ceu ( sort keys %{$self->{CEu}} ) {
	print LF " CE found $self->{CEu}->{$ceu}: $ceu \n"
    }
    close LF;
    $self->{reqs}->{$lind}->{stat1} = "$stat1";
    $self->{reqs}->{$lind}->{mawn} = "RUNNING";
    if ( $stat1 eq "[FAIL]" ) { $self->{reqs}->{$lind}->{olog} = "$lfile"; }
    else { $self->{reqs}->{$lind}->{olout} = "$lfile"; }

    my $rc = $self->SUPER::evaluateTest();
    return $rc if ($rc);
    $rc = 7 ;
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
#p	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
	my $retval = $self->{reqs}->{$index}->{stat1};
	if ( $retval != 0 &&  $retval < $rc ) {
	    $rc = $retval ;
	}
          
    }
    $rc=0 if ( $rc == 7 );   
    return $rc;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
