#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::CheckStorm;

use FindBin qw($Bin);
use lib $Bin;

use File::Basename;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

CheckStorm.pm - Check that globusrun authentication works on the hosts
# $Id: CheckStorm.pm,v 1.1.1.2 2005/02/08 16:09:43 mbarroso Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<CheckStorm.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;

    $self->SUPER::setContext();

    print "CheckStorm: setContext\n";

    $self->addOption('maxSubs=i',
                     'Number of jobs.',
                     '6');
    $self->addOption('maxStack=i',
                     'Number of simultaneous jobs in the stack.',
                     '2');
    $self->addOption('reqLapse=i',
                     'Delay between 2 submissions (in sec).',
                     '15');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '135');
    $self->addOption ("cksumSizeMb=i", 
		      "Size of the file for checksum test (Mb)", 
		      2000);


    # Additionnal Commands necessary to run this test.
    my @commands = (
#			  "edg-vo-env",
#			  "edg-gridftp-rm",
#			  "edg-gridftp-ls",
#			  "edg-gridftp-mkdir",
#			  "gdmp_host_subscribe",
#			  "gdmp_job_status",
#			  "globus-replica-catalog",
#			  "ldapsearch",
#			  "edg-replica-manager-replicateFile",
#			  "edg-replica-manager-copyAndRegisterFile",
#			  "edg-replica-manager-deleteFile",
#			  "edg-replica-manager-listReplicas"
		   );
    push @{$self->{commands}}, @commands;

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack::CheckStorm [options]

END
    ;

    $self->{DescText} = <<'END'

 The CheckStorm is a regular JobStorm where the jobs submitted for execution on 
the WNs are supplied with a big file thru 2-way sandbox transfers, 
and this includes both end checksums. The result is OK when the 
checksum for the returned file is identical to the original one and to 
the one computed on the WN.

 All standard JobStorm options are also valid for the CheckStorm itself, and
an additional --cksumSizeMb allows to provide the size of the file in MB.
Please otherwise refer to the JStorm test for description.

END
    ;

	$self->{type} = "checkStorm";
	$self->{brfinfo} = "SandBox Transfers + CheckSum";
#	 $self->{subDir} = "";
#	 $self->{location} = "";
#	 $self->{VO} = "";
#	 $self->{dataFL} = [];      ## List
#	 $self->{SPs} = {};         ## Hash
	 $self->{SEs} = [];         ## List
#	 $self->{SEpairs} = {};     ## Hash
	$self->{CEs} = [];         ## List
    $self->{bigFile} = "";

    return 0;

}

# Build any necessary setup for the storm.
sub setupEnvironment {
    my $self = shift;

    print "CheckStorm: setupEnvironment\n";

    return 0;
}

sub dgjob {
    my $cmdn = shift;
    my $root = "dg-job-";
    if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
	$root = "e".$root;
    }
    return "$root"."$cmdn";
}

# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

    print "CheckStorm: addArguments\n";

## Create the checksum file ...
    $self->{reqs}->{$index}->{bigo} = $self->{tempdir}."/BigOrig-$index.crap";
    my $checksum1 = $self->createBig($self->{reqs}->{$index}->{bigo});
    $checksum1 =~ s/ /-/;
    $checksum1 = "=$checksum1=";

    my $bid = `echo 'Arguments = "S$ENV{STACK_SERIE} I${index} $checksum1"; ' >> $jdlTemp`;
    $bid = `echo '#BigFileName = "mumbojombo"; ' >> $jdlTemp`;
    $bid = `echo 'InputSandbox = {"$self->{shFile}","$self->{reqs}->{$index}->{bigo}"}; ' >> $jdlTemp`;

    my $nCE = scalar(@{$self->{CEs}});
    my $rnd = int(rand($nCE));
    my $ce = ${$self->{CEs}}[$rnd];
    if ( $self->{opts}->{useCEList} ) {
#glue    my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
#norm    my $reqs = 'Requirements = other.CEId=="'.$ce.'"';
	my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
	print STDERR " Requirements: $reqs \n";
	$bid = `echo '$reqs; ' >> $jdlTemp`;
	my @m = split /:/, $ce;
	$self->{reqs}->{$index}->{mace} = $m[0];
    } else {
	$self->{reqs}->{$index}->{mace} = "free";
    }


    return 0;
}


# Create a BigFile for checksuming ...
sub createBig {
    my $self = shift;
    my $bigOrig = shift;

    my $size = $self->{opts}->{cksumSizeMb};  ## in Mb
##    $size = 2;  ## temp
##    my $time = $size*70000;
    open BGF, "> $bigOrig";
    my $bgf = "";
    for ( my $t=0; $t<$size; $t++ ) {
	my $r = rand($t);
	print BGF "$r";
	$bgf .= "$r";
    }
    print BGF "\n";
    $bgf .= "\n";
    my $cumul = 70000;
    for ( my $t=0; $t<$cumul; $t++ ) {
	print BGF "$bgf\n";
    }
    close BGF;
    #system("ls -l $bigOrig; cksum $bigOrig");
    my $checksum = `cksum $bigOrig | cut -d" " -f1-2`;
    chomp($checksum);

    return $checksum;

}


# Prepare everything for running the test itself ...
sub setupTest {
    my $self = shift;

    print "CheckStorm: setupTest\n";

    @{$self->{SEs}} = @{$self->{args}};

#    print " Defaults: @{$self->{optionDefaults}} \n";
#    print " Linkage:  @{$self->{optionLinkage}}  \n";
#    exit 0 if (1);

    my $rc = $self->SUPER::setupTest();
    return $rc if $rc;

    ## Possible only when jdlFile and shFile get created
    $self->buildCEList();
    if ( $self->{opts}->{useCEList} ) {
	if ( ! scalar(@{$self->{CEs}}) ) {
	    $self->setErrorMessage('Setup failed while building the CEList which returned empty');
	    return 1;
	}	
    }

    $self->{zombSlep} = ( $self->{zombSlep} > $self->{reqLapse} ) ? $self->{reqLapse} : $self->{zombSlep};
    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "\n";

    return $rc;
}


sub createJDL {
    my ($self, $file, $script) = @_;

    print "CheckStorm: createJDL\n";
    print " file script: $file $script \n";

    my ($sname, $spath, $suffix) = fileparse($script, []);

#    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable = "$sname";
##chk InputSandbox = {"$script"};
StdOutput = "result.out";
StdError = "result.err";
##chk OutputSandbox = {"result.out","result.err"};
OutputSandbox = {"result.out","result.err","mumbojombo"};
    Requirements = other.GlueCEPolicyMaxCPUTime<600000;
    Rank = other.GlueCEPolicyMaxCPUTime;
END
    ;
    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


sub createScript {
#   my $dir = shift;
#   my $file = "$dir/JobScript.sh";
    my ($self, $file) = @_;

    print "CheckStorm: createScript\n";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<'END'
#!/usr/bin/perl -w

my ($a, $b, $c) = @ARGV;
print STDERR "checkSum: runTest ARGS: $a $b $c\n";
print STDOUT "checkSum: runTest ARGS: $a $b $c\n";
print STDOUT "=checksum1= $c\n";
my $bigOrig = `ls -1 *BigOrig*`;
my $checksum2 = `cksum $bigOrig | cut -d" " -f1-2`;
    $checksum2 =~ s/ /-/;
    $checksum2 = "=$checksum2=";
print STDOUT "=checksum2= $c\n";
open BGO, "$bigOrig";
open MBJ, "> mumbojombo";
if (1) {
# Reverse
my @lines = reverse <BGO>;
foreach my $line (@lines) {
    print MBJ "$line";
}
} else {
# Direct
while (<BGO>) {
    print MBJ;
}
}
close BGO;
close MBJ;

my $service = `date; pwd; host $ENV{HOSTNAME}; host $ENV{HOST}; echo EDG_WL_JOBID: $ENV{EDG_WL_JOBID}; uname -a; ls -l`;
print STDOUT $service;

exit 0;

__DATA__

#!/bin/sh
set -xv
echo "Hello from $Bin "
echo "Arguments: " \$1 \$2 \$3
date
##sleep 2700
##date
pwd
echo "-- end of $self->{type}.sh -- "
##notyet /usr/afsws/bin/tokens
##printenv
host \$HOSTNAME
host \$HOST
uname -a
ls -l
END
    ;
    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
sub evaluateTest {
    my $self = shift;

    print "CheckStorm: evaluateTest\n";

    my $rc = $self->SUPER::evaluateTest();
    return $rc if ($rc);
    $rc = 7 ;
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
#p	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
	my $retval = $self->{reqs}->{$index}->{stat1};
	if ( $retval != 0 &&  $retval < $rc ) {
	    $rc = $retval ;
	}
          
    }

    $rc=0 if ( $rc == 7 );   
    return $rc;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
