#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::JobStorm;

use FindBin qw($Bin);
use lib $Bin;

use File::Basename;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

JobStorm.pm - Check that globusrun authentication works on the hosts
# $Id: JobStorm.pm,v 1.1.1.2 2005/02/08 16:09:43 mbarroso Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<JobStorm.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;

    $self->SUPER::setContext();

    print "JobStorm: setContext\n";

    $self->addOption('maxSubs=i',
                     'Number of jobs.',
                     '6');
    $self->addOption('maxStack=i',
                     'Number of simultaneous jobs in the stack.',
                     '2');
    $self->addOption('reqLapse=i',
                     'Delay between 2 submissions (in sec).',
                     '15');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '111');

    # Additionnal Commands necessary to run this test.
    my @commands = (
#			  "edg-vo-env",
#			  "edg-gridftp-rm",
#			  "edg-gridftp-ls",
#			  "edg-gridftp-mkdir",
#			  "gdmp_host_subscribe",
#			  "gdmp_job_status",
#			  "globus-replica-catalog",
#			  "ldapsearch",
#			  "edg-replica-manager-replicateFile",
#			  "edg-replica-manager-copyAndRegisterFile",
#			  "edg-replica-manager-deleteFile",
#			  "edg-replica-manager-listReplicas"
		   );
    push @{$self->{commands}}, @commands;

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack::JobStorm [options]

END
    ;

    $self->{DescText} = <<'END'

 This submits a simple job which simply echos the [OK] string.  It does
this from multiple jobs running on WNs, i.e. a job "storm".

 Based on BaseTest::EDGLifecycle::Stack::JobStorm (Gilberts test).

 The simplest basic functionalities for a job going thru a RB are tested.

 Exercises very simple Input and Output sandbox transfers, 
job submission and retrieval using the RB, 
and checks the contents of the output files.

 Could easily be one job only (--maxSubs=1), for Functionality check purposes.
Also, a --batchSleep option is available to provide the sleep
time (sec.) of the jobs while running on the WNs.

 It is the basic brick of all the Storms of the Stack family:
CStorm, KStorm, DStorm, (RStorm), CaStorm, UStorm, GfalStorm, ...

 Standard submission from the MainScript, with 10sec sleepers:
opt/edg/bin/MainScript --forcingTB=CertTB --TList=JStorm --verbose --forcingVO=dteam \
  --addOptionList="--keepTempDir --reqLapse=2 --maxStack=50 --maxSubs=100 --batchSleep=10 \
  --polling=120 --pollingPeriod=30 --singleSubmit --useCEList --serie=11002"

 Special case, the JStorm with multi RBs, where the jobs are sent over several
RBs in turn:
opt/edg/bin/MainScript --forcingTB=CertTB --TList=JStorm --verbose --forcingVO=dteam \
  --addOptionList="--keepTempDir --reqLapse=2 --maxStack=50 --maxSubs=30 --batchSleep=10 \
  --polling=120 --pollingPeriod=30 --singleSubmit --useCEList --serie=11007 \
  --multiRB='lxshare0234.cern.ch,lxshare0240.cern.ch,lxshare0239.cern.ch' "

END
    ;

	$self->{type} = "jobStorm";
#	 $self->{subDir} = "";
#	 $self->{location} = "";
#	 $self->{VO} = "";
#	 $self->{dataFL} = [];      ## List
#	 $self->{SPs} = {};         ## Hash
	 $self->{SEs} = [];         ## List
#	 $self->{SEpairs} = {};     ## Hash
	$self->{CEs} = [];         ## List

    return 0;

}

# Build any necessary setup for the storm.
sub setupEnvironment {
    my $self = shift;

    print "JobStorm: setupEnvironment\n";

    return 0;
}

sub dgjob {
    my $cmdn = shift;
    my $root = "dg-job-";
    if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
	$root = "e".$root;
    }
    return "$root"."$cmdn";
}

# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

#old    print "JobStorm: addArguments\n";
    $self->printLocation( (caller(0))[3] );

    my $bid = `echo 'Arguments = "S$ENV{STACK_SERIE} I${index} rien"; ' >> $jdlTemp`;

if (0) {
    my $nCE = scalar(@{$self->{CEs}});
    my $rnd = int(rand($nCE));
    my $ce = ${$self->{CEs}}[$rnd];
    if ( $self->{opts}->{useCEList} ) {
#glue    my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
#norm    my $reqs = 'Requirements = other.CEId=="'.$ce.'"';
	my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
	print STDERR " Requirements: $reqs \n";
	$bid = `echo '$reqs; ' >> $jdlTemp`;
	my @m = split /:/, $ce;
	$self->{reqs}->{$index}->{mace} = "CE:$m[0]";
    } else {
	$self->{reqs}->{$index}->{mace} = "free";
    }
} else {
    $self->chooseCE($index, $jdlTemp);
}
    
    return 0;
}

# Prepare everything for running the test itself ...
sub setupTest {
    my $self = shift;

    print "JobStorm: setupTest\n";

    @{$self->{SEs}} = @{$self->{args}};

#    print " Defaults: @{$self->{optionDefaults}} \n";
#    print " Linkage:  @{$self->{optionLinkage}}  \n";
#    exit 0 if (1);

    my $rc = $self->SUPER::setupTest();
    return $rc if $rc;

    ## Possible only when jdlFile and shFile get created
    $rc = $self->buildCEList();
    return $rc if $rc;

#     if ( $self->{opts}->{useCEList} ) {
# 	if ( ! scalar(@{$self->{CEs}}) ) {
# 	    $self->setErrorMessage('Setup failed while building the CEList which returned empty');
# 	    return 1;
# 	}	
#     }

    $self->{zombSlep} = ( $self->{zombSlep} > $self->{reqLapse} ) ? $self->{reqLapse} : $self->{zombSlep};
    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "\n";

    return $rc;
}


sub createJDL {
    my ($self, $file, $script) = @_;

    print "JobStorm: createJDL\n";
    print " file script: $file $script \n";

    my ($sname, $spath, $suffix) = fileparse($script, []);

#    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable = "$sname";
InputSandbox = {"$script"};
StdOutput = "result.out";
StdError = "result.err";
OutputSandbox = {"result.out","result.err"};
    Requirements = other.GlueCEPolicyMaxCPUTime<6000000;
    Rank = other.GlueCEPolicyMaxCPUTime;
END
    ;

    if ( $self->{opts}->{site} ) {
	my $site = $self->{opts}->{site};
	print JDL "Requirements = RegExp(\".*$site.*\",other.CEId);\n" if $site;
    }
    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


sub createScript {
#   my $dir = shift;
#   my $file = "$dir/JobScript.sh";
    my ($self, $file) = @_;

    print "JobStorm: createScript\n";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<"END"
#!/bin/sh
set -xv
echo "Hello from $Bin "
echo "Arguments: " \$1 \$2 \$3
date
pwd
echo "-- end of $self->{type}.sh -- "
ghostname=`hostname --long 2>&1`
gipname=`hostname --ip-address 2>&1`
echo \$ghostname "has address" \$gipname
#echo \$ghostname "has address empty"
host \$HOSTNAME
host \$HOST
echo EDG_WL_JOBID: \$EDG_WL_JOBID
uname -a
ls -l
END
    ;
    close SCRIPT;
    if ( $self->{opts}->{batchSleep} ) {
	my $res = `echo 'sleep $self->{opts}->{batchSleep}' >> $file `;
    }

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}


# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
sub evaluateTest {
    my $self = shift;

    print "JobStorm: evaluateTest\n";

    my $rc = $self->SUPER::evaluateTest();
    return $rc if ($rc);
    $rc = 7 ;
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
#p	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
	my $retval = $self->{reqs}->{$index}->{stat1};
	if ( $retval != 0 &&  $retval < $rc ) {
	    $rc = $retval ;
	}
          
    }
    $rc=0 if ( $rc == 7 );   
    return $rc;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
