#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::MatchMaking;

use strict;
use vars qw(@ISA);
use EDGUtils;

use File::Basename;

@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

MatchMaking.pm - Check the EDG matchmaking mechanism at RB level.
# $Id: MatchMaking.pm,v 1.1.1.2 2005/02/08 16:09:43 mbarroso Exp $

=head1 SYNOPSIS

 The matchmaking test is achieved in this way:
 For each triplet of SE machines,
a set of 3 different data files is created, registered through the ERM, 
and they are all replicated to only 2 of these SEs. 
On the remaining SE, only 2 out of these 3 files are replicated.

 In the JDL of the job which is built for the test, the 3 files are required 
(with the InputData clause), and they MUST be accessible through
the "file" protocol (DataAccessProtocol clause) .

 Then, the "edg-job-list-match" is used to create a list of matching CEs,
selecting only the fastest queue through the Rank clause,
and the job is actually sent for execution on each of the matching machines.

 Within the job, a printout of each data file is achieved to cross-check that 
the files are indeed visible through the "file" protocol.

 Obviously, out of 3 CEs, only 2 of them must match when they are close to the 
SEs where all 3 files were replicated. 

=head1 DESCRIPTION 

B<MatchMaking.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;
    $self->SUPER::setContext();

#    print "MatchMaking: setContext\n";
    $self->printLocation( (caller(0))[3] );

    # Add the resource option.
    if (0) {
    $self->addOption('mds=s',
                     'The top-level mds node.',
                     'gppinfo06.gridpp.rl.ac.uk');
    $self->addOption('mdsport=i',
                     'The top-level mds port number.',
                     '2135');
    $self->addOption('mdsname=s',
                     'The mds-vo-name for MDS.',
                     'local');
    $self->addOption('bdii=s',
                     'The default BDII node.',
                     'ccedgli02.in2p3.fr');
    $self->addOption('bdiiport=i',
                     'The BDII port number.',
                     '2170');
    $self->addOption('bdiiname=s',
                     'The mds-vo-name for BDII.',
                     'local');
}
#     $self->addOption('vo=s',
# 		     'VO to use in the matchmaking.',
#                      'wpsix');
    $self->addOption('reqLapse=i',
                     'Minimum delay between 2 submissions (in sec).',
                     '5');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '101');
    $self->addOption('simple',
		     'Matchmaking with a single SRM per SE.');
    $self->addOption('execute',
		     "Send the JDL for execution, in addition to the matchmaking step.\n\tImplies the oneStep option");
    $self->addOption('oneStep',
		     'Only one file configuration is tried for matchmaking.');
    $self->addOption('infoBDII',
		     'Force using standard (dynamic) configuration instead of the default static one.');
    $self->addOption('protocol:s',
		     'DataAccessProtocol used in the Matchmaking.',
		     "file");
    $self->addOption('selectNodes:s',
		     'Space separated list of CE nodes where the matching check will be run against.\n\tThese nodes must be part of the gang matching list, of course.');

    # The command needed to run this test.
    $self->{commands} = ["grid-proxy-info",
#			 "edg-vo-env",
			 "edg-gridftp-rm",
                         "edg-gridftp-ls",
			 "edg-gridftp-mkdir",
#p			 "gdmp_ping",
# 			 "gdmp_host_subscribe",
# 			 "gdmp_job_status",
# 			 "globus-replica-catalog",
# 			 "ldapsearch",
# 			 "edg-replica-manager-replicateFile",
# 			 "edg-replica-manager-copyAndRegisterFile",
# 			 "edg-replica-manager-deleteFile",
# 			 "edg-replica-manager-listReplicas"
 			 "edg-replica-manager"
			 ];

    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::MatchMaking [options] <SE node> [SE node...] 

END
    ;

    $self->{DescText} = <<'END'

 The matchmaking test is achieved in this way:

 For each possible triplet of SE machines,
a set of 3 different data files is created, registered through the ERM, 
and they are all replicated to only 2 of these SEs. 
On the remaining SE, only 2 out of these 3 files are replicated.

 In the JDL of each job which is built for the test, the 3 files are required 
(with the InputData clause), and they MUST be accessible through
the selected protocol (DataAccessProtocol clause) .

 Then, the "edg-job-list-match" is used to create a list of matching CEs,
selecting only the fastest queue through the Rank clause.

 When the --execute option is selected, which it must be to achieve a full test,
the job is actually sent for execution on each of the matching machines.
Within the job, a printout of each of the 3 data files is achieved to 
cross-check that the files are indeed visible through the required protocol.

 Obviously, out of 3 CEs, only 2 of them must match when they are close to the 
SEs where all 3 files were replicated. 

 There are currently 4 flavors of this test in the available in the MainScript
suite, one for each protocol type: file, gsiftp, rfio, gsidcap. They are selected
thru the --protocol option.
However, the file protocol is deprecated, and gsiftp is an alias for gridftp.

 All options described in the JStorm test itself are available.

 This test is rather time consuming because of the delay occuring when 
registering and replicating the file within the ERM tool. Due to this limitation,
one usually selects only a small subset of the SEs to be tested, 
as shown in the example below.

 A typical submission used in a regression test is:
opt/edg/bin/MainScript --forcingTB=CertTB --TList=MM --verbose \
  --addOptionList=\"--keepTempDir --execute --protocol=gsiftp\" \
  --forceMachineList=\"lxshare0236.cern.ch lxshare0291.cern.ch lxshare0278.cern.ch\"

END
    ;

	$self->{type} = "MatchMaking";
	$self->{reqM}   = {};    ## Hash
	$self->{res}    = {};    ## Hash
	$self->{steps}  = {};    ## Hash
	$self->{plargs} = ();    ## List

        $self->{stepI} = 0;
	$self->{SRM}  = {};    ## Hash of hashes
	$self->{GUIDs}  = {};    ## Hash

}


sub checkPrerequisites {
    my $self = shift;
    my $rc = $self->SUPER::checkPrerequisites();
    return $rc if $rc;

#    print "MatchMaking: checkPrerequisites\n";
    $self->printLocation( (caller(0))[3] );

       $rc = checkProxyExists();
    $self->setErrorMessage('user proxy does not exist') if $rc;

    return $rc;
}

sub runTest {
    my $self = shift;

#    print "MatchMaking: runTest\n";
#    $self->printLocation( (caller(0))[3] );
    $self->printStack();

    my $rc;
    my @SEs;
    my %SPs = {};
    my @tList;
    my @tListf;
    my @tListd;
    my @tprev = ();
    if (0) {
    } else {
	@SEs = @{$self->{args}};
	if (0) {
	    push @SEs, ($SEs[0], $SEs[1]);
	    print "\n Final full list of SEs: @SEs \n";
	} else {
	    @tList = arr(3, @SEs);
	    my $ptlist = join("\n", @tList);
	    print "\n Final full list of SE triplets: \n$ptlist \n";
	    foreach my $trip ( @tList ) {
		my @t = split /\s+/, $trip;
		push @tListf, "@t";
		if ( scalar(@tprev) == 0 ) {
		    push @tListd, " ";
		} else {
		    my %seen = ();
		    my @uniqu = sort grep { ! $seen{$_}++ } @t;
		    my @notseen = sort grep { ! $seen{$_} } @tprev;
		    push @tListd, "@notseen";
		}
		push @tListf, "$t[1] $t[2] $t[0]";
		push @tListd, " ";
		push @tListf, "$t[2] $t[0] $t[1]";
		push @tListd, " ";
		@tprev = @t;
	    }
	}
    }
    print "\n Final full list of SE triplets after rotation: \n";
    for my $nt ( 0 .. ($#tListf) ) {
	print " $nt  > $tListf[$nt] >> $tListd[$nt] \n";
    }

#   exit 0;

    # Keep a list of all SEs which fail
    my @failed;

    # Loop over each SE triplet doing a full test for each.
    if (0) {
    for my $n ( 0 .. ($#SEs-2) ) {
	my @SEtrip = ($SEs[$n], $SEs[$n+1], $SEs[$n+2]);
	print "\n Current SE triplet: @SEtrip \n\n";
	my $SEprev = ( $n == 0 ) ? " " : $SEs[$n-1];
        my $error = $self->testMatchMaking(\@SEtrip, $SEprev, $n, \%SPs);
        push @failed, @SEtrip if ($error);
    }
} else {
    for my $nt ( 0 .. ($#tListf) ) {
	my @SEtrip = split /\s+/, $tListf[$nt];
	print "\n Current SE triplet: @SEtrip \n\n";
	my @SEprev = ( $tListd[$nt] eq " " ) ? () : split /\s+/, $tListd[$nt];
        my $error = $self->testMatchMaking(\@SEtrip, \@SEprev, $nt, \%SPs);
#prev        push @failed, @SEtrip if ($error);
        push @failed, "$nt << $tListf[$nt]" if ($error);
    }
}

    # If there are any servers in the list, then the test has failed.
    if ($#failed>=0) {
#        my $names = join(",",@failed);
        my $names = join("\n",@failed);
        $self->setErrorMessage("MatchMaking test failed for: \n$names");
        return 1;
    }

    return 0;
}


## 
## The actual test is here.  It just takes a server name, extracts the
## supported VOs and runs checks. 
##

sub testMatchMaking {

    my ($self, $arrayref, $SEprevref, $iter, $hashref) = @_;

    $self->printLocation( (caller(0))[3] );

    my $SE2;  ## deprecated ??
    my @SEtrip = @$arrayref;
    my @SEprevL = @$SEprevref;
    my %SPs = %$hashref;
    my $rc = 0;

    print "\n Iteration: $iter ; Working this SE triplet: @SEtrip\n";
    $self->{iteration} = $iter;

#     print "\n Working this SE triplet: @SEtrip with these mount points: \n";
#     foreach my $se ( @SEtrip ) { 
# 	print "     FS: $se => $SPs{$se} \n";
#     }

    ## Step Index (used in XML display)
    my $stepI = $self->{stepI};
    my $clout;
    my $command;
    my $stat;

    my $subDir = "testGG";
#    my $edgRMopts = "--config=edg-replica-manager-CertTB.conf --insecure --vo=$self->{opts}->{VO}";
    my $config = " ";
    if ( $self->{opts}->{infoBDII} ) {
	$config = "--config=/opt/edg/etc/edg-replica-manager/edg-replica-manager.conf";
    } else {
	## Because we are using now the static InfoStub for ERM related stuff ...
	$config = "--config=$ENV{ERM_CONFIG}" if ( $ENV{ERM_CONFIG} );
    }
    ## Default options to be used for all ERM related commands ...
    my $edgRMopts = "$config --insecure --vo=$self->{opts}->{VO}";
    print " edgRMopts: $edgRMopts \n";
#    my $edgRMopts = "--vo=atlas --verbose";

#    if ( $SEprev eq " " ) {  
    if ( $iter == 0 ) {  
	# grodid-20643-0-rmstorm.txt
	my $upid = $$;
	    foreach my $loop ( 0..2 ) {
		my $file = "$ENV{USER}-$upid-$loop-nmm.txt";
		my $remGh = removeGhostFile($edgRMopts, $file);
	    }
    } 

##E    my $nn = int(rand(3));
    my $nn = 0;
    my $SE1 = $SEtrip[$nn];
    my $SEA = $SEtrip[($nn+1)%3];
    my $SEB = $SEtrip[($nn+2)%3];
    my $SRMaster;
    my $file;
    chomp(my $hostname = `hostname`);
    ## We have yet to find a solution to make this in a better way. Hardcoded for the time being.
    ## One can use $ENV{SE_HOSTS_} and or $ENV{CE_CLOSE_SE_ID_} + $siteTB
#    my %SECE = (
#		 "lxshare0235.cern.ch" => "lxshare0236.cern.ch",
#		 "lxshare0241.cern.ch" => "lxshare0236.cern.ch",
#		 "lxshare0290.cern.ch" => "lxshare0291.cern.ch",
#		 "lxshare0277.cern.ch" => "lxshare0278.cern.ch"
#	 );
    my @CElist = split /,/, $ENV{CEhostname_};
#p    print " CElist: @CElist \n";
    my @SElist = split /,/, $ENV{CE_CLOSE_SE_ID_};
    my %SECE = ();
    foreach my $ncs ( 0 .. $#CElist ) {
	$SECE{$CElist[$ncs]} = $SElist[$ncs];
    }
    print " SECElist:\n", map { "    CE: $_ => SE: $SECE{$_}\n" } sort { $SECE{$a} cmp $SECE{$b} } keys %SECE; 

##E    my %GUIDs = ();
    my %LFNs  = ();
    my %SRMs  = ();
    my @LFNl  = ();
##E    my $loopS = int(rand(3));
    my $loopS = 2;
##E    my $SRM  = {};  ## a hash of hashes
    my $tmpdir = "$self->{tempdir}";
    my @resRcLR = ();

    foreach my $loop (0..2) {  ## FOR2

	if ( $self->{opts}->{simple} ) {
	    ##  Special short
	    my $nm = $nn + $loop;
	       $SE1 = $SEtrip[$nm%3];
	       $SEA = $SEtrip[($nm+1)%3];
	       $SEB = $SEtrip[($nm+2)%3];
	    ##  Special short
	}

    ## Create a temporary file ...
#	my $value = int(rand(32768));
    $file = "$ENV{USER}-$$-$loop-nmm.txt";
#tmp    open TMP, ">/tmp/$file";

	if ( ! -e "$tmpdir/$file" ) {  ##E
	    open TMP, ">$tmpdir/$file";
	    print TMP "Good_Morning_to_all_Everybody_! ";
	    #tmp    print TMP "/tmp/$file\n";	
	    print TMP "$tmpdir/$file\n";	
	    close TMP;
	}

    my %PFNs  = ();
	my @SElistr2 = ($SEA, $SEB);

	if ( ! $self->{SRM}->{$SE1}->{$file} ) {  ##IF2

	    if ( ! $self->{GUIDs}->{$file} ) { ##IF22
##		@SElistr2 = ($SEA, $SEB);

    $command = "edg-replica-manager $edgRMopts copyAndRegisterFile file:$tmpdir/${file} -l lfn:${file} -d $SE1";
#NO    $command = "lcg-cr --verbose $edgRMopts -l lfn:${file} -d $SE1 file:$tmpdir/${file}";
    $command = "lcg-cr $edgRMopts -l lfn:${file} -d $SE1 file:$tmpdir/${file}";
    print "\n CMD: $command\n";

    ## Brand new for command routine design ...
    my ($stat, $resRcCARF) = $self->commandProc( $command,
						 "copyAndRegisterFile 1",
						 "$file",
						 ++$stepI);

    if ( $stat != 0 ) {
	print STDERR " File: ${file} on: $SE1 not correctly registered or created \n";
	return 1;
    } else {
	chomp($resRcCARF);
	if ( $resRcCARF !~ "guid:" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong GUID returned: $resRcCARF \n";
	    return 1;
	}
	$self->{reqM}->{$stepI}->{olout} = $resRcCARF;
    }

    $self->{GUIDs}->{$file} = $resRcCARF;
    $PFNs{"$SE1/$file"} = "created";
    $LFNs{$file} = "lfn:${file}";
    push @LFNl, "${file}";

    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    $command = "lcg-lr $edgRMopts lfn:${file}";

    my ($stat, @resRcLR) = $self->commandProc( $command,
					       "listReplicas 1",
					       "$file",
					       ++$stepI);

    foreach my $l ( @resRcLR ) {
        chomp $l;
        my @i = split /[:\/]/, $l;
#prev	if ( $i[0] ne "srm" ) {
	if ( $i[0] !~ /srm|sfn/ ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE1 File: $gsrm previously existed: $l ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    $SRMs{$l} = "existing";
	    $self->{SRM}->{$SE1}->{$file} = $l;
	    $SRMaster = $l;
#p	    print " SRM keys SE1: $SE1 $file value: $self->{SRM}->{$SE1}->{$file} \n";
	}
    }
    if ( $PFNs{"$SE1/$file"} ne "existing" ) {
	print STDERR " Step: $SE1 File: $file not correctly copied/registered ! \n";
	return 1;
    }

	    }  ##End of IF22
	    else {
		@SElistr2 = ($SE1, $SEA, $SEB);
##NO		$self->{SRM}->{$SE1}->{$file} = PASBON AFINIR ...
	    }

}   ## End IF2

#prev    foreach my $SE2 ( $SEA, $SEB ) {  ##FOR3
    foreach my $SE2 ( @SElistr2 ) {  ##FOR3

	## To limit arbitrarily the number of replicates ...
	next if $self->{opts}->{simple};  ##  Special short
	next if $loop == $loopS && $SE2 eq $SEB;  ## To create a hole

	if ( ! $self->{SRM}->{$SE2}->{$file} ) {  ##IF3

#prev    $command = "edg-replica-manager $edgRMopts replicateFile $self->{SRM}->{$SE1}->{$file} -d $SE2";
    $command = "edg-replica-manager $edgRMopts replicateFile $self->{GUIDs}->{$file} -d $SE2";
    $command = "lcg-rep --verbose $edgRMopts -d $SE2 $self->{GUIDs}->{$file}";
    print "\n CMD: $command\n";

    my ($stat, $resRcRF) = $self->commandProc( $command,
					       "replicateFile 2",
					       "$file",
					       ++$stepI);

    if ( $stat != 0 ) {
        print STDERR " File: ${file} on: $SE2 not correctly registered or created \n";
	return 1;
    } else {
	my $href = $self->parseTURLs($resRcRF);
	my @m = grep { /srm:|sfn:/ } @{$href->{turls}};
	$resRcRF = $m[0];
	chomp($resRcRF);
#prev	if ( $resRcRF !~ "srm:" ) {
	if ( $resRcRF !~ /srm:|sfn:/ ) {
	    print STDERR " File: ${file} on: $SE2 : Wrong SRM returned: $resRcRF \n";
	    return 1;
	} else {
	    $PFNs{"$SE2/$file"} = "created";
	    $SRMs{$resRcRF} = "existing";
	    $self->{SRM}->{$SE2}->{$file} = $resRcRF;
#p	    print " SRM keys SE2: $SE2 $file value: $self->{SRM}->{$SE2}->{$file} \n";
	}
    }

    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    $command = "lcg-lr $edgRMopts lfn:${file}";

    my ($stat, @resRcLR) = $self->commandProc( $command,
					       "listReplicas 2",
					       "$file",
					       ++$stepI);


    }  ## End of if SRM (IF3)

}  ## End of foreach my $SE2  (FOR3)

    my $limit = 3;
    $limit-- if ( $loop == $loopS );
    $limit = 1 if $self->{opts}->{simple};  ##  Special short

    print "\nPFN0: \n", map { "$_ => $PFNs{$_}\n" } sort keys %PFNs;
    print "-----------------------------------\n";

}  ## End of foreach my $loop  (FOR2)

    ## Non-first pass case ...
    if ( $self->{LFNs} ) {
	## Restoring some useful arrays ..
	%LFNs = %{$self->{LFNs}};
	@LFNl = @{$self->{LFNl}};

	## Removing some unwanted files ...
	my $n0 = 2;
	my $machSE = $SEB;
	if ( $self->{SRM}->{$machSE}->{$LFNl[$n0]} ) {
	    print " Removing: $self->{SRM}->{$machSE}->{$LFNl[$n0]} from: $machSE for: $LFNl[$n0] \n";
	    $command = `edg-replica-manager $edgRMopts unregisterFile $self->{GUIDs}->{$LFNl[$n0]} $self->{SRM}->{$machSE}->{$LFNl[$n0]}`;
	    $command = `lcg-uf $edgRMopts $self->{GUIDs}->{$LFNl[$n0]} $self->{SRM}->{$machSE}->{$LFNl[$n0]}`;
	    my $resRcURF = `$command 2>&1`;
	    print " resRcURF1: \n$resRcURF\n";
	    delete $self->{SRM}->{$machSE}->{$LFNl[$n0]};
	}	
#	if ( $SEprev ne " " ) {
	foreach my $SEprev ( @SEprevL ) {
	    foreach my $n0 ( 0..2 ) {
		my $machSE = $SEprev;
		if ( $self->{SRM}->{$machSE}->{$LFNl[$n0]} ) {
		    print " Removing: $self->{SRM}->{$machSE}->{$LFNl[$n0]} from: $machSE for: $LFNl[$n0] \n";
		    $command = `edg-replica-manager $edgRMopts unregisterFile $self->{GUIDs}->{$LFNl[$n0]} $self->{SRM}->{$machSE}->{$LFNl[$n0]}`;
		    $command = `lcg-uf $edgRMopts $self->{GUIDs}->{$LFNl[$n0]} $self->{SRM}->{$machSE}->{$LFNl[$n0]}`;
		    my $resRcURF = `$command 2>&1`;
		    print " resRcURF3: \n$resRcURF\n";
		    delete $self->{SRM}->{$machSE}->{$LFNl[$n0]};
		}	
	    }
	}
    ## First pass case ...
    } else {
	## Saving some useful arrays ..
	%{$self->{LFNs}} = %LFNs;
	@{$self->{LFNl}} = @LFNl;
    }

    print "\nGUIDs: \n", map { "$_ => $self->{GUIDs}->{$_}\n" } sort keys %{$self->{GUIDs}};
    print "\nLFNs: \n", map { "$_ => $LFNs{$_}\n" } sort keys %LFNs;
    print "\nLFNlist: @LFNl \n\n";

    my $loopM = 20;
    ##E May be always put this one to 0 ??
    $loopM    = 0 if $self->{opts}->{oneStep};
    my $try = 0;
##    my $n0 = int(rand(3));
    ## REMARK: here, from now on, $loopM is always 0
    ## So that $loopT is also always 0
    ## And the foreach loop is useless indeed
    ## And n0/n1/n2 are always 0/1/2 ==> to be simplified ...
    foreach my $loopT ( 0 .. $loopM ) {  ##  FOR4
	my $n0 = $loopT%3;
	my $n1 = ($n0+1)%3;
	my $n2 = ($n0+2)%3;


	## Section where the JDL is being matched ...

	## Adding required InputData clause to JDL file ...
	my $resCp = `cp -p $self->{jdlm} $self->{jdl}`;
	my $resE = `echo '    InputData = {"$LFNs{$LFNl[$n0]}", "$self->{GUIDs}->{$LFNl[$n1]}", "$LFNs{$LFNl[$n2]}"};' >> $self->{jdl}`;
	$resE =    `echo '    Arguments = "$LFNs{$LFNl[$n0]} $self->{GUIDs}->{$LFNl[$n1]} $LFNs{$LFNl[$n2]}";'    >> $self->{jdl}`;

	$self->{plargs} = ();  ##  To reuse it during the evaluation step after execution (if any) ...
	$self->{plargs}->{lfn}->{1} = "$LFNs{$LFNl[$n0]}";
	$self->{plargs}->{lfn}->{2} = "$self->{GUIDs}->{$LFNl[$n1]}";
	$self->{plargs}->{lfn}->{3} = "$LFNs{$LFNl[$n2]}";
	$self->{plargs}->{pfn}->{1} = "$tmpdir/$LFNl[$n0]";
	$self->{plargs}->{pfn}->{2} = "$tmpdir/$LFNl[$n1]";
	$self->{plargs}->{pfn}->{3} = "$tmpdir/$LFNl[$n2]";

	print " Try # $try ============================================================================================= \n";
	## Listing the current replicas ...
	foreach my $nl ( $n0, $n1, $n2) {  ## FOR5
	    my $file = $LFNl[$nl];
	    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
	    $command = "lcg-lr $edgRMopts lfn:${file}";

	    my ($stat, @resRcLR) = $self->commandProc( $command,
						       "listReplicas sum",
						       "$file",
						       ++$stepI);


	}  ##End of FOR5

	## Achieving the matching indeed ...
	$command = "edg-job-list-match $self->{jdl}";
	print "\n InputDatal = {$LFNs{$LFNl[$n0]}, $LFNs{$LFNl[$n1]}, $LFNs{$LFNl[$n2]}} \n";
	print " InputData  = {$LFNs{$LFNl[$n0]}, $self->{GUIDs}->{$LFNl[$n1]}, $LFNs{$LFNl[$n2]}} \n";

	my ($stat, $resEJLM) = $self->commandProc( $command,
						   "edg-job-list-match $iter",
						   "",
						   ++$stepI);

	## Retrieving the matching CEs ...
	my %seen = ();
	my @machL = grep { ! $seen{$_}++ } map { /(\S+):2119\/jobmanager-/ } split /\s+/, $resEJLM;
	print " matchingCEs: @machL\n";

	## Retrieving the matching CE queues ...
     	%seen = ();
	my @machS = grep { ! $seen{$_}++ } map { /(\S+:2119)\/jobmanager-/ } split /\s+/, $resEJLM;
	   @machS = grep { /\S+:2119\/jobmanager-/ } split /\s+/, $resEJLM;
	my %CEq;
	foreach my $ma ( @machS ) {
#p	    print " ma: $ma \n";
	    $ma =~ /(\S+:2119)\/(jobmanager-\S+)/;
#p	    print " ma12: $1 $2 $seen{$1} \n";
	    if ( $seen{$1} ) {
		my $mb = $1;
		my @m = split /:/, $mb;

		## To filter out unwanted nodes  ...
		if ( ! $self->{opts}->{selectNodes} ) {
		    $CEq{$mb} = $ma;
		} elsif ( $self->{opts}->{selectNodes} =~ /$m[0]/ ) {
		    $CEq{$mb} = $ma;
		}
	    }
	}

	my @CEl;
	foreach my $ma ( values %CEq ) {
	    push @CEl, $ma;
	}
	print " MatchingCEqueues: @CEl \n\n";
	@{$self->{CEq}} = @CEl;
#	@{$self->{CEq}} = split /\s+/, $resEJLM;

	## next may be more accurate ... but I think its useless :->
	last if ( scalar(@machL) == 0 );  ##  No more matching ...
##	next if ( scalar(@machL) == 0 );  ##  No more matching ...

	## $loopT always equal $loopM now (and both 0 indeed) ...
	last if ( $loopT == $loopM );     ## Don't remove any new SE after last iteration ...

	## Section where some files were removed in the previous version
	## May be not keeping this part in the new version

	$try++;  ##  useless here ...
    }  ##End of FOR4

    $self->{stepI} = $stepI; 
    return 0 unless $self->{opts}->{execute};  ##  Do not execute the JDL unless requested ...

    ## New design with the loop being achieved by SUPER::runTest itself (from Stack)

    %{$self->{reqs}} = ();  ## cleanup
    $self->{opts}->{reqLapse} = 5;
    my $nCE = scalar(@{$self->{CEq}});

    return 0 if ( ! $nCE );  ##  There was no machine in the matching list ...

    $self->{opts}->{maxSubs} = $nCE;
    $self->{opts}->{maxStack} = $nCE;

    ## Hummm ... no explanations ...
    $self->{maxStack} = $self->{opts}->{maxStack};
    $self->{maxSubs} = $self->{opts}->{maxSubs};
    $self->{reqLapse} = $self->{opts}->{reqLapse};

    my $resJob = $self->SUPER::runTest($iter+1);
    if ( $resJob ) { 
	print " MatchMaking::Job : Failed ! \n"; 
	$self->{globalresult} .= " [FAIL]" if $self->{globalresult};
    }

    ## Evaluate the results to check that there was indeed access to the matched files ...
    $resJob = $self->levaluateTest();  ##  local evaluation
    $rc = $resJob;

    return $rc;
}

sub cleanup {
    my $self = shift;
    my $upid = $$;
    my $edgRMopts = "--insecure --vo=$self->{opts}->{VO}";
    foreach my $loop ( 0..2 ) {
	my $file = "$ENV{USER}-$upid-$loop-nmm.txt";
	my $remGh = removeGhostFile($edgRMopts, $file);
#	unlink "/tmp/$file";
    }
    my $rc = $self->SUPER::cleanup();
    return $rc;
}


# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

#old    print "JobStorm: addArguments\n";
    $self->printLocation( (caller(0))[3] );

    my $bid;
##    $bid = `echo 'Arguments = "S$ENV{GGSERIE} I${index} rien"; ' >> $jdlTemp`;

#p    print STDERR " CElist: @{$self->{CEq}} \n";

#no    my $resCp = `cp -p $self->{jdlm} $self->{jdl}`;

#     my $nCE = scalar(@{$self->{CEq}});
#     my $rnd = int(rand($nCE));
#     my $ce = ${$self->{CEq}}[$rnd];
    my $ce = ${$self->{CEq}}[$index];

    if (1) {
#glue    my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
#norm    my $reqs = 'Requirements = other.CEId=="'.$ce.'"';
	my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
	print STDERR " Requirements: $reqs \n";
	$bid = `echo '$reqs; ' >> $jdlTemp`;
	my @m = split /:/, $ce;
#temp	$self->{reqs}->{$index}->{mace} = $m[0];
	$self->{reqs}->{$index}->{mace} = $m[0];
    } else {
#temp	$self->{reqs}->{$index}->{mace} = "free";
    }

    return 0;
}


# The setup creates a temporary directory to be used to hold the job's
# files (JDL and scripts).  It will also be used to hold the retrieved
# job output.  (The key in the context is 'tempdir'.)
sub setupTest {
    my $self = shift;

    $self->{debug} = 1;  ## auxiliary to new printLocation method.
    $self->printLocation( (caller(0))[3] );


    # Call the parent method to ensure that the temporary directory is
    # created.
    my $error = $self->SUPER::setupTest();
    if ($error) {
        $self->setErrorMessage('MatchMaking setup failed');
        return 1;
    }

    # Retrieve the temporary directory.
    my $tmpdir = $self->{tempdir};
#p  print " tmpdir: $tmpdir \n";

    $self->{opts}->{oneStep} = 1 if $self->{opts}->{execute};
    $self->{opts}->{VO} = $self->{opts}->{vo};

    STDOUT->autoflush(1);
    STDERR->autoflush(1);

## gg for XML creation and processing ...
#prevGOOD    if ( $ENV{MAINSCRIPT} ) {
    if ( $self->{opts}->{xml} ) {
        my $base = "$self->{tempdir}";
        $self->{type}     = "MatchMaking";
        $self->{xmlfile}  = "$base/".$self->{type}.".XML";
        $self->{xmlxist}  = "$self->{type}";
        my $html_dir = ( defined $ENV{HTML_DIR} ) ? "$ENV{HTML_DIR}" : "$ENV{HOME}/TSTG/HTML";
        $html_dir = "${html_dir}/$self->{type}";
        my $resmk = `mkdir -p $html_dir`;
        $self->{htmldir}  = "$html_dir";
        print " HTMLDIR: $self->{htmldir} \n";
    }

    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "---\n";

    return 0;
}


##
## Below here are utility functions for supporting the methods above.
## These should not need to be called by subclasses.
##


sub createJDL {
    my ($self, $dir, $script, $site) = @_;

    $self->printStack();

    my ($sname, $spath, $suffix) = fileparse($script, []);

#    my $file = "$dir/JobDescription.jdl";
    my $file = "$dir";
#    my $bin = dirname($ENV{MAINSCRIPT});

    my $protocol = "$self->{opts}->{protocol}";
    $protocol = "gsiftp" if ( $protocol eq "gridftp" );

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
    Executable    = "$sname";
    StdOutput     = "std.out";
    StdError      = "std.err";
    InputSandbox  = "$script";
    OutputSandbox = {"std.out","std.err", "BrokerInfo"};
    StdInput = "/dev/null";
    RetryCount = 3;
#    DataAccessProtocol = "gridftp";
#    DataAccessProtocol = "file";
#    DataAccessProtocol = "$self->{opts}->{protocol}";
    DataAccessProtocol = "$protocol";
    VirtualOrganisation = "$self->{opts}->{vo}";
##    Requirements = IsMember(other.RunTimeEnvironment, other.CEId=="lxshare0290.cern.ch:2119/jobmanager-pbs-short");
    Requirements = other.GlueCEPolicyMaxCPUTime<600000;
    Rank = other.GlueCEPolicyMaxCPUTime;
END
    ;

#     print JDL "Requirements = $self->{opts}->{requirements};\n"
#         if $self->{opts}->{requirements};
#     print JDL "Rank = $self->{opts}->{rank};\n"
#         if $self->{opts}->{rank};
    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {

        $self->{jdl} = $file;
	## Creating JDLmaster ...
        $self->{jdlm} = "${file}m";
	my $resCp = `cp -p $self->{jdl} $self->{jdlm}`;

        return $file;
    } else {
        return "";
    }
}

sub createScriptGeneric {
    my ($self, $file) = @_;

    my $protocol = "$self->{opts}->{protocol}";
    $protocol = "gsiftp" if ( $protocol eq "gridftp" );

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<"END0"
#!/bin/bash
echo Arg: \$1 \$2 \$3
P="$protocol"
echo " Selected Protocol: \$P"
END0
;
    print SCRIPT <<'END'
#GSF=`edg-brokerinfo getSelectedFile $1 file`
#echo GSF: $GSF
TFN=`echo $GSF | cut -d " " -f 2`
echo Printing local file $TFN:
cat $TFN
echo "=+-+-+-+-+-+-+-+-"
ghostname=`/bin/hostname --long 2>&1`
gipname=`hostname --ip-address 2>&1`
echo $ghostname "has address" $gipname
#echo $ghostname "has address empty"
/usr/bin/host $HOSTNAME
/usr/bin/host $HOST
/bin/echo EDG_WL_JOBID: $EDG_WL_JOBID
/bin/uname -a
/bin/echo "Hello From Sgript"
/bin/echo "Goodbye. [OK]"
GCE=`edg-brokerinfo getCE`
echo CE: $GCE
GSE=`edg-brokerinfo getSEs`
echo SEs: $GSE
CSE=`edg-brokerinfo getCloseSEs`
echo getCloseSEs: $CSE
#CS2=`echo $CSE | cut -d " " -f 2`
#CSE=`echo $CSE | cut -d " " -f 1`
    CSEL=$CSE;
CS2=`echo $CSE | awk '{print $2}'`
CSE=`echo $CSE | awk '{print $1}'`
if [ ! -z $CS2 ] ; then
#    echo getCloseSE1: $CSE
    echo getCloseSE1: $CS2
    SMP=`edg-brokerinfo getSEMountPoint $CS2`
    echo getSEMountP1: $SMP
    SPT=`edg-brokerinfo getSEProtocols $CS2`
    echo getSEProto1: $SPT
    for pt in $SPT; do
        if [ $pt == $P ]; then
            CSE=$CS2
        fi
    done
fi

    for CSEN in $CSEL; do
	echo getCloseSEN: $CSEN
	SMPN=`edg-brokerinfo getSEMountPoint $CSEN`   
	echo getSEMountPN: $SMPN
        SPTN=`edg-brokerinfo getSEProtocols $CSEN`
        echo getSEProtoN: $SPTN
    for pt in $SPTN; do
        if [ $pt == $P ]; then
	    if [ -z $CSELF ]
		then CSELF=$CSEN
		else CSELF="$CSELF $CSEN"
	    fi
		    
        fi
    done
        
	done

#echo getCloseSE2: $CSE
#SMP=`edg-brokerinfo getSEMountPoint $CSE`
#echo getSEMountPoint: $SMP
#SPT=`edg-brokerinfo getSEProtocols $CSE`
#echo getSEProtocols: $SPT
echo CSE list after filtering: $CSELF
unset CSE
WN=`hostname --long`
CE=`edg-brokerinfo getCE | cut -d":" -f 1`

cp .BrokerInfo BrokerInfo
echo "+-+-+-+-+-+-+-+-"

files="$1 $2 $3";
n=0
case $P in
    file)    cutop='cut -d/ -f4-';;
    gsiftp)  cutop='cut -d: -f2-';;
    rfio)    cutop='cut -d/ -f3- | sed -e"s;/;:/;"';;
#    rfio)    cutop='cut -d/ -f3- | sed -e''"'"s;/;:/;"'"';;
    gsidcap) cutop='cut -d/ -f2-';;
esac

m=0
    for CSE in $CSELF; do
    m=`expr $m + 1`;
for file in $files; do
    n=`expr $n + 1`;
#p    echo " ==>> >$n: edg-brokerinfo getLFN2SFN $file | grep $CSE | $cutop<";
###    echo " ==>> >$n: edg-brokerinfo getLFN2SFN $file | grep $CSE | eval $cutop<";
    Forig0=`edg-brokerinfo getLFN2SFN $file | grep $CSE`;
###    echo " ==>> Forig0: >$Forig0<";
    Forig=`edg-brokerinfo getLFN2SFN $file | grep $CSE | eval $cutop`;
###    echo " ==>> Forig1: >$Forig<";
    outF=$PWD/Final

    export SRM_PATH=/opt/d-cache-client/srm
    export DCACHE_IO_TUNNEL=libgsiTunnel.so

    case $P in
        file)    outF="/"$Forig;;
        gsiftp)  out=`globus-url-copy gsiftp:$Forig file://$outF 2>&1`;;
        rfio)    out=`rfcp $Forig $outF 2>&1`;;
##template: dccp -u -d 2 gsidcap://lxshare0247.cern.ch:22128/pnfs/cern.ch/data/dteam/generated/2004-04-12/filea6fed04d-8c4c-11d8-a66c-b2f2c98f41b8 fffff
        gsidcap) Forig=`echo $Forig | sed -e's/:8443/:22128/'`; out=`dccp -u -d 2 gsidcap:/$Forig $outF 2>&1`;;
#GOOD        gsidcap) out=`/opt/d-cache-client/srm/bin/srmcp -x509_user_proxy=$X509_USER_PROXY $Forig0 file:///$outF 2>&1`;;
    esac
    echo " OUTrc: " $out
    contents=`cat $outF`;
    echo CUR $n $WN $CE $CSE $P $m
    echo LFN $n $file $Forig
    echo LFC $n $contents
    echo "+-+-+-+-+-+-+-+-"
#    rm $outF
done
done

END
    ;
    close SCRIPT;

    return;
}

sub createScript {
    my ($self, $dir) = @_;
    print "MatchMaking: createScript\n";

    $self->printStack();

#p    print "MatchMaking: $dir\n";

#    my $file = "$dir/JobScript.sh";
    my $file = "$dir";

    if (0) {
    if ( $self->{opts}->{protocol} eq "file" ) {
	&createScriptFile($file);
    } elsif ( $self->{opts}->{protocol} eq "gridftp" ) {
	&createScriptGridftp($file);
    } elsif ( $self->{opts}->{protocol} eq "rfio" ) {
	&createScriptRfio($file);
    } else {
#p	print STDERR "Wrong protocol selected to create job script\n";
	print STDOUT "Wrong protocol selected to create job script\n";
	return "";
    }
} else {
    $self->createScriptGeneric($file);
}

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
	print "MatchMaking: Wrong $file\n";
        return "";
    }
}


## Should be called by the evaluateTest method of the Parent Class
## Dummy from now on
sub evaluateOutput {
    my ($self,$stdout,$stderr) = @_;
    return 0;
}

## Dummy from now on
sub evaluateTest {
    my ($self) = @_;
    print "MatchMaking: evaluateTest\n";
    $self->printStack();
    return 0;
}

## Local now ...
sub levaluateTest {
    my $self = shift;

#    $self->printLocation( (caller(0))[3] );
    $self->printStack();

    ## Looping over the request results prepared by the Stack runTest ...
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	print " JOBL\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
#p	print " JOBL\t$index :\n$self->{reqs}->{$index}->{olout} \n";
#p	print " JOBL\t$index :\n$self->{reqs}->{$index}->{stat1} \n";
#p	print " JOUT\t$index :\n".`cat $self->{reqs}->{$index}->{olout}`." \n" if $self->{reqs}->{$index}->{olout};
	next unless $self->{reqs}->{$index}->{olout};
	my $rc = $self->levaluateOutput("$self->{reqs}->{$index}->{olout}",
					"$self->{reqs}->{$index}->{olout}");
	if ($rc) {
	    print STDERR "\n Output check failed for Job $index.\n\n";
	    $self->{globalresult} .= " [FAIL]" if $self->{globalresult};
	}
    }

    return 0;
}

## Local now ...
sub levaluateOutput {
    my ($self,$stdout,$stderr) = @_;

    $self->printLocation( (caller(0))[3] );

    # Check that the std.out and std.err files exist.
    return 1 if (! (-e "$stdout"));
# #     return 1 if (! (-e "$stderr"));


    my $stepI = $self->{stepI};
# #     my $logsum = " --> Submit: \n\n$self->{job}->{submit}->{log}\n" 
# #                . " --> Status: \n$self->{job}->{status}->{log}" 
# # 	       . " --> Getout: \n\n$self->{job}->{getout}->{log}";
# #     $self->{reqM}->{$stepI}->{mess} = "$logsum";
    $stepI++;

    # Check that the standard error has a NON zero size.
# #     return 1 if ( -z "$stderr");

    # Check the standard output ... 
    my $value = 0;
    my $score = 0;
    $self->{res} = {};
    print "\n";
    open FILE, "<$stdout";
    while ( (my $l = <FILE>) !~ '\+-\+-' ) {	
	    print " LINB: ", $l;
    }
    while ( <FILE> ) {
	if ( /^\+-\+-/ ) { next; }
	elsif ( /^Linux/ ) {
	    print " LINE: ", $_;
	    my @m = split /\s+/, $_;
	    $self->{reqM}->{$stepI}->{stat2} = "Matches, was run on WN: $m[1]";
	}
	elsif ( /LFN/ ) {
	    print " LINE: ", $_;
	    my @m = split /\s+/, $_;
#old	    $self->{res}->{$m[1]}->{lfn} = "$m[2]";
	    $self->{res}->{lfn}->{$m[1]} = "$m[2]";
	}
	elsif ( /CUR/ ) {
	    print " LINE: ", $_;
	    my @m = split /\s+/, $_;
#good	    $self->{res}->{nodel}->{$m[1]} = "Match:$m[6] WN:$m[2] CE:$m[3] SE:$m[4] Prot:$m[5]";
	    $self->{res}->{match}->{$m[6]}->{nodel} = "WN:$m[2] CE:$m[3] SE:$m[4] Prot:$m[5]";
	    $self->{res}->{nodel}->{$m[1]} = $m[6];
	}
	elsif ( /LFC/ ) {
	    print " LINE: ", $_;
	    my @m = split /\s+/, $_;
	    if ( "$m[2]" !~ "^Good_" ) { next; }
	    if ( "$m[3]" !~ "^/tmp/" ) { next; }
#old	    $self->{res}->{$m[1]}->{pfn} = "$m[3]";
	    $self->{res}->{pfn}->{$m[1]} = "$m[3]";
	}
    }
    close FILE;
    print "\n";

    my $cumul = "";
    foreach my $arg ( sort keys %{$self->{res}->{lfn}} ) {
#old	print " NODES: $arg $self->{res}->{nodel}->{$arg} \n";
	my $nodel = $self->{res}->{nodel}->{$arg};
	print " ARG: $arg lfnres:$self->{res}->{lfn}->{$arg} lfnarg:$self->{plargs}->{lfn}->{1+($arg-1)%3} pfnres:$self->{res}->{pfn}->{$arg} pfnarg:$self->{plargs}->{pfn}->{1+($arg-1)%3} \n";
#	$cumul .= " NODES: $arg $self->{res}->{nodel}->{$arg} \n";
	$cumul .= " ARG: $arg lfnres:$self->{res}->{lfn}->{$arg} lfnarg:$self->{plargs}->{lfn}->{1+($arg-1)%3} pfnres:$self->{res}->{pfn}->{$arg} pfnarg:$self->{plargs}->{pfn}->{1+($arg-1)%3} \n";
	$value = 1 if ( $self->{res}->{lfn}->{$arg} ne $self->{plargs}->{lfn}->{1+($arg-1)%3} );
	$value = 1 if ( $self->{res}->{pfn}->{$arg} ne $self->{plargs}->{pfn}->{1+($arg-1)%3} );
	$self->{res}->{match}->{$nodel}->{score}++ if ( $self->{res}->{lfn}->{$arg} eq $self->{plargs}->{lfn}->{1+($arg-1)%3} );
	$self->{res}->{match}->{$nodel}->{score}++ if ( $self->{res}->{pfn}->{$arg} eq $self->{plargs}->{pfn}->{1+($arg-1)%3} );
	$cumul .= " NODES: $arg Match:$nodel $self->{res}->{match}->{$nodel}->{nodel} Val:$value Score:$self->{res}->{match}->{$nodel}->{score} \n";
	print " NODES: $arg Match:$nodel $self->{res}->{match}->{$nodel}->{nodel} Val:$value Score:$self->{res}->{match}->{$nodel}->{score} \n";
    }

    $value = 1;
    my @matchnode = ();
    foreach my $nodel ( sort keys %{$self->{res}->{match}} ) {
	my $score = $self->{res}->{match}->{$nodel}->{score};
	if ( $score == 6 ) { 
	    $value = 0;
	    my @m = split /\s+/, $self->{res}->{match}->{$nodel}->{nodel};
	    push @matchnode, $m[2];
	}
    }

#above	$stepI++;
#p    print " Current stepI: $stepI \n";
	$self->{reqM}->{$stepI}->{olog} = "$stdout";
	$self->{reqM}->{$stepI}->{stat1} = $value;
        $self->{reqM}->{$stepI}->{stat2} = "Wrong" if $value;
    if ( $value ) {
	$self->{reqM}->{$stepI}->{stat2} = "No matches found";
    } else {
	$self->{reqM}->{$stepI}->{stat2} = "Match found with @matchnode";
    }
#        $self->{reqM}->{$stepI}->{stat2} = "Matching" unless $value;
	$self->{step}->{$stepI}->{brfinfo} = "Cross-checking";
	$self->{step}->{$stepI}->{detinfo} = "Checking that contents of output file match with submitted filenames:\n"."$cumul";

    print "\n Result Iteration $self->{iteration}: $self->{reqM}->{$stepI}->{stat2} \n\n";

    $self->{stepI} = $stepI;

#     if ( $ENV{MAINSCRIPT} ) {
# 	my @resC = `cat $stdout`;
# 	print " stdout: $stdout \n @resC";
#     }

    return $value;
}


## Type with 1 column only
sub buildXMLFile {  
 
    my $self = shift;

#    print "Stack: buildXMLFile\n";
#    $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_rows;
#old    my $Title = "MatchMaking";
    my $Title = "$self->{type}";
    chomp(my $date = `date`);
#     my $brf_info = "Rather Simple Storm";
#     my $Jdl = `cat $self->{jdlFile}`;
#     my $Scr = `cat  $self->{shFile}`;
#     my $det_info = "Job Storm with a simple JDL script, and minimal sandbox transfers.\n\nThe JDL file is:\n\n".$Jdl."\n\nWhile the Script file is:\n\n".$Scr;
#    $det_info =~ s/\n/<br>/g;
    my %ret = ( 
	     '[OK]' => 0,
	     '[FAIL]' => 1,
	     '[WARNING]' => 2,
	     '[XFAIL]' => 3,
	     '[UNRESOLVED]' => 4,
	     '[UNTESTED]' => 5,
	     '[UNSUPPORTED]' => 6
	    );

    print STDERR " Called buildXMLFile1Col() to write $self->{xmlfile} \n";

#     my $cols = {};
#     my $order = 0;
# #    my $machType = 'mawn';
#     my $machType = 'mace';
#     foreach my $index ( sort { $a <=> $b } keys %{$self->{reqM}} ) {
#         $cols->{$self->{reqM}->{$index}->{$machType}}->{freq}++;
#         if ( $cols->{$self->{reqM}->{$index}->{$machType}}->{freq} == 1 ) { $cols->{$self->{reqM}->{$index}->{$machType}}->{order} = $order++; }
# 	my $mac = " ";
# 	if ( $self->{reqM}->{$index}->{mawn} ) { $mac = $self->{reqM}->{$index}->{mawn}; }
# 	else { $mac = "CE:".$self->{reqM}->{$index}->{mace}; }
# 	push @lof_rows, $mac;
#     }

#prev	 foreach my $col ( sort { $cols->{$a}->{order} <=> $cols->{$b}->{order} } keys %{$cols} ) {
#prev	     print " $col $cols->{$col}->{order} $cols->{$col}->{freq} \n";
#prev	     push @lof_cols, $col;
#prev	 }
    push @lof_cols, "Status";
    
    my $ncols = scalar(@lof_cols)-1;
    my $colM = "";
    my $rowM = "";
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqM}} ) {
	## directly coded as 0/1 ...
#	my $retv = $ret{$self->{reqM}->{$index}->{stat1}};
	my $retv = $self->{reqM}->{$index}->{stat1};
	my $subs = $self->{reqM}->{$index}->{stat2};
	my $logfn;
	my $logf;

# 	if ( $retv == 0 ) {
# 	    $logfn = $self->{reqM}->{$index}->{olout};
# 	    $logf = `cat $logfn if $logfn`;
# 	} elsif ( $retv == 5 ) {  ## UNTESTED
# ##edg2.0	    $logf = $self->{reqM}->{$index}->{mess};
# 	    $logfn = $self->{reqM}->{$index}->{olog};
# 	    $logf = `cat $logfn if $logfn`;
# 	} else {
# ##edg2.0            if ( $self->{reqM}->{$index}->{stat2} eq "MISSING_FILES" ) { $logf = $self->{reqM}->{$index}->{mess}; }
# ##edg2.0            else {
# 	    $logfn = $self->{reqM}->{$index}->{olog};
# 	    $logf = `cat $logfn if $logfn`;
# ##edg2.0	}
# 	}

	if ( $self->{reqM}->{$index}->{olog} ) { $logf = `cat $self->{reqM}->{$index}->{olog}`; }
	elsif ( $self->{reqM}->{$index}->{mess} ) { $logf = "$self->{reqM}->{$index}->{mess}"; }
	else { $logf = " "; }

	unlink $logfn if $logfn;
	my $coln = 0;
#	$coln   = $cols->{$self->{reqM}->{$index}->{$machType}}->{order};
#bad	$rowM = "";
	for my $c (0..$ncols) {
	    if ( $c == $coln ) {
		$logf =~ s/&/&amp;/g; # This must be changed first.
		$logf =~ s/\"/&quot;/g;
		$logf =~ s/</&lt;/g;
		$logf =~ s/>/&gt;/g;
#		$rowM .= makeX(makeX(makeX("$logf", "output").makeX("$retv", "retval").makeX("$subs", "substatus"), "step"), "col");
		$rowM .= makeX(makeX("$logf", "output").makeX("$retv", "retval").makeX("$subs", "substatus"), "step");
	    } else {
#		$rowM .= makeX(makeX(makeX(" ", "output").makeX(" ", "retval"), "step"), "col");
		$rowM .= makeX(makeX(" ", "output").makeX(" ", "retval"), "step");
	    }
	}
#	$colM .= makeX($rowM, "row");
    }

    ## To make each of the above a single step ...
    ## then merging it into a single col and single row for the time being ...
    $rowM .= makeX($rowM, "col");
    $colM .= makeX($rowM, "row");

    ## Building of the Step Info stack...
#my $stepM = makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
    my $stepM = "";
    foreach my $index ( sort { $a <=> $b } keys %{$self->{step}} ) {
#old	$stepM .= makeX(makeX("$self->{step}->{$index}->{brfinfo}", "brf_info").makeX("$self->{step}->{$index}->{detinfo}", "det_info"), "step");
	my $stepb = makeX("$self->{step}->{$index}->{brfinfo}", "brf_info");
	my $steps = makeX("_", "substatus_info");
	my $stepd = makeX("$self->{step}->{$index}->{detinfo}", "det_info");
	$stepM .= makeX("$stepb $steps $stepd", "step");
    }

my $body = $colM;

my $info = makeX($Title, "title");
$info .= makeX($date, "date");
@lof_rows = map { makeX($_, "row"); } @lof_rows;
$info .= makeX("@lof_rows", "rows_list");
@lof_cols = map { makeX($_, "col"); } @lof_cols;
$info .= makeX("@lof_cols", "cols_list");
$info .= $stepM;
$info = makeX($info, "info");

## The whole log is made of INFO+BODY
open XML, "> $self->{xmlfile}";
print XML makeX($info." \n".$body, "log");
close XML;
#p print STDOUT makeX($info." \n ".$body, "log");

    return 0;
}

sub commandProc {

    my ($self, $command, $ctitle,  $file, $stepI) = @_;

    my $resComm = `$command 2>&1`;
    my $stat = $?;
    print "\n $ctitle: $file -$stat- \n$resComm";

	$self->{reqM}->{$stepI}->{mess} = $resComm;
	$self->{reqM}->{$stepI}->{stat1} = ($stat == 0) ? 0 : 1;
	$self->{reqM}->{$stepI}->{stat2} = $stat;
	$self->{step}->{$stepI}->{brfinfo} = "$ctitle";
	$self->{step}->{$stepI}->{detinfo} = $command;

    $self->{stepI} = $stepI; 

    return ($stat, $resComm);

}

sub removeGhostFile {

#     my ($self, $file) = @_;
#     my $edgRMopts = $self->{edgRMopts};
    my ($edgRMopts, $file) = @_;

#    $self->printLocation( (caller(0))[3] );

##	    foreach my $loop ( 0..2 ) {
##		my $file = "$ENV{USER}-$upid-$loop-rmstorm.txt";

		my $command = "edg-replica-manager $edgRMopts listGUID lfn:${file}";
    $command = "lcg-lg $edgRMopts lfn:${file}";
		chomp(my $resRcLR = `$command 2>&1`);
		my $stat = $?;
		print "$stat\t                                   $resRcLR\n";
#		if ( $stat ) { next; }
    return 0 if $stat;

		$command = "edg-replica-manager $edgRMopts deleteFile --all-available --all-lfn $resRcLR";
    $command = "lcg-del --verbose $edgRMopts -a $resRcLR";
		my $resRcDF = `$command 2>&1`;
		$stat = $?;
		print "\n resRcDF: $file -$stat- \n$resRcDF\n";
    return $stat if $stat;

		$command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    $command = "lcg-lr $edgRMopts lfn:${file}";
		my $resRcLRafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLRafter: $file -$stat- \n$resRcLRafter";
#    return $stat if $stat;

		$command = "edg-replica-manager $edgRMopts listReplicas $resRcLR";
    $command = "lcg-lr $edgRMopts $resRcLR";
		my $resRcLGafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLGafter: $file -$stat- \n$resRcLGafter";
#    return $stat if $stat;
##	    }

    return 0;

}

## Notation A(n,p): arrangement de n objets p a p
## n est dans ce cas la taille de @data
## et l'appel: arr(p, @data)
sub arr {
      my $p = shift;
      my @data = @_;
      my @dout;
      if ( $p == 1 ) {
	  return @data;
      } else {
	  foreach my $i ( 0 .. $#data ) {
	      my @loc = @data;
	      my ($d) = splice(@loc, $i, 1);
#p	      print " Before: $p $i $d @loc \n";
	      my @inf = arr($p-1, @loc);
	      foreach ( @inf ) {
		  my @m = split;
		  unshift @m, $d;
		  @m = sort @m;
		  push @dout, "@m";
#		  push @dout, "$d $_";
	      }
	  }
	  my %seen = ();
	  my @uniqu = sort grep { ! $seen{$_}++ } @dout;
	  return @uniqu;
      }
  }

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
