#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::PileStorm;

use FindBin qw($Bin);
use lib $Bin;

use File::Basename;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

PileStorm.pm - Check that globusrun authentication works on the hosts
# $Id: PileStorm.pm,v 1.1.1.2 2005/02/08 16:09:43 mbarroso Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<PileStorm.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;

    $self->SUPER::setContext(2);

#p    print "PileStorm: setContext\n";
    $self->printLocation( (caller(0))[3] );

    $self->addOption('maxSubs=i',
                     'Number of jobs.',
                     '6');
    $self->addOption('maxStack=i',
                     'Number of simultaneous jobs in the stack.',
                     '2');
    $self->addOption('reqLapse=i',
                     'Delay between 2 submissions (in sec).',
                     '15');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '111');

    $self->addOption('CElReverse=i',
                     'To reverse the CE stack (use together with RB_val test).');

    # Additionnal Commands necessary to run this test.
    my @commands = (
#			  "edg-vo-env",
#			  "edg-gridftp-rm",
#			  "edg-gridftp-ls",
#			  "edg-gridftp-mkdir",
#			  "gdmp_host_subscribe",
#			  "gdmp_job_status",
#			  "globus-replica-catalog",
#			  "ldapsearch",
#			  "edg-replica-manager-replicateFile",
#			  "edg-replica-manager-copyAndRegisterFile",
#			  "edg-replica-manager-deleteFile",
#			  "edg-replica-manager-listReplicas"
		   );
    push @{$self->{commands}}, @commands;

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack::PileStorm [options]

END
    ;

    $self->{DescText} = <<'END'

 This PileStorm derives from the standard JobStorm in a simple way:
a stack of JDL files together with their relevant Script files was 
preliminary created (using the --NOXcute option of the JStorm family),
while the filenames are recorded on a service file.
 This allows to mix in a single storm several kinds of tests, in order
to submit the jobs in parallel (time savings).
 Then the PileStorm itself is launched and brokers the jobs on both stacks.

 This rather special storm is currently only used to for Unit testing 
of the RBs and CEs, and the simple jobs which are executed all return
an '[OK]' string in their output. This allows for simple cross-checking 
of the results: they are all members of the EDGLifecycle OO class.
 From the set of CEs and RBs to test, a matrix is built which allows for
running every of the basic tests on each of the CEs thru each of the RBs 
in turn, and this several times (MainScript general option --RBjFact). 
The latter feature allows to send these jobs to run on several different
WNs on the same CE: indeed, there is NO way to target a specific WN,
so it would not be possible otherwise to test them (quasi) systematically.
The RB, CE and WN names where the job ran
are all displayed on the result matrix, allowing for detailed examination
of the output log file (whenever a failure occured, for ex.).

 All of this is currently a rather delicate setup, and its use must be 
avoided outside the regression files of the MainScript framework.
Indeed, the number of jobs to be submitted depends on the number
of available RBs and CEs, and it must be an integer multiple of the latter.

 A typical submission command used in a regression test is:
opt/edg/bin/MainScript --forcingTB=CertTB --forcingVO=dteam --TList=RB_val \
  --verbose --RBjFact=3 --addOptionList="--keepTempDir"

END
    ;

	$self->{type} = "PileStorm";
	$self->{brfinfo} = "RB tests";
#	 $self->{subDir} = "";
#	 $self->{location} = "";
#	 $self->{VO} = "";
#	 $self->{dataFL} = [];      ## List
#	 $self->{SPs} = {};         ## Hash
	 $self->{SEs} = [];         ## List
#	 $self->{SEpairs} = {};     ## Hash
	$self->{CEs} = [];         ## List

    return 0;

}

# Build any necessary setup for the storm.
sub setupEnvironment {
    my $self = shift;

#p    print "PileStorm: setupEnvironment\n";
    $self->printLocation( (caller(0))[3] );

    return 1 unless $ENV{TSTG_PILE};
    return 1 unless ( -e $ENV{TSTG_PILE} );

##    $self->{pileCount} = 1;

    return 0;
}

sub dgjobUSELESS {
    my $cmdn = shift;
    my $root = "dg-job-";
    if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
	$root = "e".$root;
    }
    return "$root"."$cmdn";
}

# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

#old    print "PileStorm: addArguments\n";
    $self->printLocation( (caller(0))[3] );

    my $dum = "DUM";
    my $script = $self->createScript($dum);
    my $file   = $self->createJDL($index, $dum);
    $dum = `cat $file > $jdlTemp`;

    $self->{jdlFile} = $jdlTemp;
    $self->{shFile}  = $script;

##NOOoo    my $bid = `echo 'Arguments = "S$ENV{STACK_SERIE} I${index} rien"; ' >> $jdlTemp`;

# if (0) {
#     my $nCE = scalar(@{$self->{CEs}});
#     my $rnd = int(rand($nCE));
#     my $ce = ${$self->{CEs}}[$rnd];
#     if ( $self->{opts}->{useCEList} ) {
# #glue    my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
# #norm    my $reqs = 'Requirements = other.CEId=="'.$ce.'"';
# 	my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
# 	print STDERR " Requirements: $reqs \n";
# 	$bid = `echo '$reqs; ' >> $jdlTemp`;
# 	my @m = split /:/, $ce;
# 	$self->{reqs}->{$index}->{mace} = "CE:$m[0]";
#     } else {
# 	$self->{reqs}->{$index}->{mace} = "free";
#     }
# } else {

##good    if ( $index%4 == 0 ) { @{$self->{CEs}} = reverse(@{$self->{CEs}}); }
    if ( $self->{opts}->{CElReverse} ) { 
#p	print " addArguments Before: @{$self->{CEs}} \n";
	if ( $index%$self->{opts}->{CElReverse} == 0 ) { @{$self->{CEs}} = reverse(@{$self->{CEs}}); }
#p	print " addArguments Aftere: @{$self->{CEs}} \n";
    }

    $self->chooseCE($index, $jdlTemp);

# }
    
    return 0;
}

# Prepare everything for running the test itself ...
sub setupTest {
    my $self = shift;

#p    print "PileStorm: setupTest\n";
    $self->printLocation( (caller(0))[3] );

##notanymore    @{$self->{SEs}} = @{$self->{args}};

#    print " Defaults: @{$self->{optionDefaults}} \n";
#    print " Linkage:  @{$self->{optionLinkage}}  \n";
#    exit 0 if (1);

    $self->{pileCount} = 1;

    my $rc = $self->SUPER::setupTest();
    return $rc if $rc;

    my ($testN, $jdlFile, $shFile) = split /\s+/, `head -1 $ENV{TSTG_PILE}`;

    my $jdlTemp = "$self->{jdlFile}" . "-temp";
    my $res = `cat $jdlFile > $jdlTemp`;
    open JDL, ">>$jdlTemp";
    print JDL <<"END"
##VirtualOrganisation = "$self->{opts}->{vo}";
    Requirements = other.GlueCEPolicyMaxCPUTime<6000000;
    Rank = other.GlueCEPolicyMaxCPUTime;
END
    ;

    if ( $self->{opts}->{site} ) {
	my $site = $self->{opts}->{site};
	print JDL "Requirements = RegExp(\".*$site.*\",other.CEId);\n" if $site;
    }
    close JDL;

    $self->{jdlFile} = $jdlTemp;
    $self->{shFile}  = $shFile;

    ## Possible only when jdlFile and shFile get created
    $rc = $self->buildCEList();
    return $rc if $rc;

    $self->{jdlFile} = $jdlFile;
    $self->{shFile}  = $shFile;

    $self->{pileCount} = 1;

    $self->{zombSlep} = ( $self->{zombSlep} > $self->{reqLapse} ) ? $self->{reqLapse} : $self->{zombSlep};
    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "\n";

    return $rc;
}

sub createJDL {
    my ($self, $index, $dscript) = @_;

#    print "PileStorm: createJDL\n";
    $self->printLocation( (caller(0))[3] );

    my $pileCount = $self->{pileCount};
    print " pileCount: $pileCount \n";
#    my ($testN, $jdlFile, $shFile) = split /\s+/, `head -$self->{pileCount} $ENV{TSTG_PILE} | tail -1`;
    my ($testN, $jdlFile, $shFile) = split /\s+/, `head -$pileCount $ENV{TSTG_PILE} | tail -1`;

    $self->{pile}->{$self->{pileCount}}->{testName} = $testN;
    $self->{pile}->{$self->{pileCount}}->{jdlFile}  = $jdlFile;
    $self->{pile}->{$self->{pileCount}}->{shFile}   = $shFile;
    unless ( $index =~ "tmp" ) {
	$self->{pile}->{$testN}++;
	my $iLab = int(($self->{pile}->{$testN}-1)/$ENV{TSTG_RBJCOLS});
#	my $iLab = 0;  ##  temp test
	$self->{reqs}->{$index}->{testN} = "$testN"."$iLab";
    }
#GOOD    $self->{reqs}->{$index}->{testN} = "$testN" unless ( $index =~ "tmp" );

    $self->{pileCount}++;
    my $file = $jdlFile;
    my $script = $shFile;

#    print " file script testN: $file $script $testN $self->{reqs}->{$index}->{testN} \n";

    return "$file";

    my ($sname, $spath, $suffix) = fileparse($script, []);

#    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable = "$sname";
InputSandbox = {"$script"};
StdOutput = "result.out";
StdError = "result.err";
OutputSandbox = {"result.out","result.err"};
    Requirements = other.GlueCEPolicyMaxCPUTime<600000;
    Rank = other.GlueCEPolicyMaxCPUTime;
END
    ;
    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

sub createScript {
#   my $dir = shift;
#   my $file = "$dir/JobScript.sh";
    my ($self, $file) = @_;

#p    print "PileStorm: createScript\n";
    $self->printLocation( (caller(0))[3] );

    my $pileCount = $self->{pileCount};
    print " pileCount: $pileCount \n";
    my ($testN, $jdlFile, $shFile) = split /\s+/, `head -$pileCount $ENV{TSTG_PILE} | tail -1`;

    my $seen = `grep SEENSEEN $shFile`;
    unless ( $seen ) {
    my @m = `cat $shFile`;
    my $first = shift @m;
    open SCRIPT, ">$shFile";
    print SCRIPT <<"END"
#!/bin/sh
set -xv
date
pwd
host \$HOSTNAME
host \$HOST
echo EDG_WL_JOBID: \$EDG_WL_JOBID
uname -a
echo
ls -l
##echo SEENSEEN
END
    ;
    foreach ( @m ) {
	print SCRIPT;
    }
    close SCRIPT;
#my $service = `echo; date; pwd; host $ENV{HOSTNAME}; host $ENV{HOST}; echo EDG_WL_JOBID: $ENV{EDG_WL_JOBID}; uname -a; echo; ls -l`;
}

    return "$shFile";

    # Write out the job description.
    open SCRIPT, ">$file";
    print SCRIPT <<"END"
#!/bin/sh
set -xv
echo "Hello from $Bin "
echo "Arguments: " \$1 \$2 \$3
date
pwd
echo "-- end of $self->{type}.sh -- "
host \$HOSTNAME
host \$HOST
echo EDG_WL_JOBID: \$EDG_WL_JOBID
uname -a
ls -l
END
    ;
    close SCRIPT;
    if ( $self->{opts}->{batchSleep} ) {
	my $res = `echo 'sleep $self->{opts}->{batchSleep}' >> $file `;
    }

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
sub evaluateTest {
    my $self = shift;

#p    print "PileStorm: evaluateTest\n";
    $self->printLocation( (caller(0))[3] );

    my $rc = $self->SUPER::evaluateTest();
    return $rc if ($rc);
    $rc = 7 ;
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
#p	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
	my $retval = $self->{reqs}->{$index}->{stat1};
	if ( $retval != 0 &&  $retval < $rc ) {
	    $rc = $retval ;
	}
          
    }
    $rc=0 if ( $rc == 7 );   
    return $rc;

}

## ***************************************************************************************
sub buildXMLFile {
 
    my $self = shift;
    my ($moptc, $moptr) = @_;
    if ( ! $moptc ) {
	$moptc = "mace";
	$moptr = "mawn";
    } elsif ( ! $moptr ) {
	$moptr = "jind";
    }

    print "PileStorm: buildXMLFile\n";
#   $self->{debug} = 1;
    $self->printLocation( (caller(0))[3] );

    my @lof_cols;
    my @lof_rows;
    my $Title = "$self->{type}";
    chomp(my $date = `date`);
    my $brf_info = "Rather Simple Storm";
    $brf_info = $self->{brfinfo} if ( $self->{brfinfo} );
    my $Jdl = `cat $self->{jdlFile}`;
    my $Scr = `cat  $self->{shFile}`;
    #old    my $det_info = "Job Storm with a simple JDL script, and minimal sandbox transfers.\n\nThe JDL file is:\n\n".$Jdl."\n\nWhile the Script file is:\n\n".$Scr;
    my $det_info = "$brf_info.\n\nThe JDL file is:\n\n".$Jdl."\n\nWhile the Script file is:\n\n".$Scr;
    ##notgood?    $det_info =~ s/\n/<br>/g;
    my %ret = ( 
	       '[OK]' => 0,
	       '[FAIL]' => 1,
	       '[WARNING]' => 2,
	       '[XFAIL]' => 3,
	       '[UNRESOLVED]' => 4,
	       '[UNTESTED]' => 5,
	       '[UNSUPPORTED]' => 6
	      );

    print STDERR " Called buildXMLFile() to write $self->{xmlfile} \n";

    my $cols = {};
    my $order = 0;
    #prevGOOD    my $machType = 'mace';
    #    my $machType = 'mawn';
    my $machType = "$moptc";
#     foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
# 	if ( $machType eq "mawn" ) {
# 	    ## Some fixing ...
# 	    if ( ! $self->{reqs}->{$index}->{mawn} ) {
# 		$self->{reqs}->{$index}->{mawn} = "unknown";
# 	    }
# 	}
#         $cols->{$self->{reqs}->{$index}->{$machType}}->{freq}++;
#         if ( $cols->{$self->{reqs}->{$index}->{$machType}}->{freq} == 1 ) {
# 	    $cols->{$self->{reqs}->{$index}->{$machType}}->{order} = $order++;
# 	}

# 	#prevGOOD	push @lof_rows, "Job_".$index;
# 	#good	push @lof_rows, ( $self->{reqs}->{$index}->{mawn} || "Job_".$index );
# 	push @lof_rows, ( $self->{reqs}->{$index}->{$moptr} || "Job_".$index );
#     }
    my $prevTest = "";
    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	if ( $machType eq "mawn" ) {
	    ## Some fixing ...
	    if ( ! $self->{reqs}->{$index}->{mawn} ) {
		$self->{reqs}->{$index}->{mawn} = "unknown";
	    }
	}
        $cols->{$self->{reqs}->{$index}->{$machType}}->{freq}++;
        if ( $cols->{$self->{reqs}->{$index}->{$machType}}->{freq} == 1 ) {
	    $cols->{$self->{reqs}->{$index}->{$machType}}->{order} = $order++;
	}

	#prevGOOD	push @lof_rows, "Job_".$index;
	#good	push @lof_rows, ( $self->{reqs}->{$index}->{mawn} || "Job_".$index );
	#p print " PREV: $index =$self->{reqs}->{$index}->{$machType}= $self->{reqs}->{$index}->{testN} $prevTest \n";
	if ( $self->{reqs}->{$index}->{testN} ne $prevTest ) {
	    push @lof_rows, $self->{reqs}->{$index}->{testN};
	    $prevTest = $self->{reqs}->{$index}->{testN};
	}
#prev	push @lof_rows, ( $self->{reqs}->{$index}->{$moptr} || "Job_".$index );
    }

#p    print " lof_rows: @lof_rows \n";

    foreach my $col ( sort { $cols->{$a}->{order} <=> $cols->{$b}->{order} } keys %{$cols} ) {
	#    foreach my $col ( sort keys %{$cols} ) {
	#p	print " $col $cols->{$col}->{order} $cols->{$col}->{freq} \n";
	#	$col =~ s/\./ /g;
	push @lof_cols, "$col";
	#p	print "  $col \n";
    }

    my $ncols = scalar(@lof_cols)-1;
    my $colM = "";

    foreach my $row ( @lof_rows ) {
	my $rowM = "";

    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	#p print " iBuild: $index $self->{reqs}->{$index}->{testN} ?? $row \n";
	next unless ( $self->{reqs}->{$index}->{testN} eq $row );
	#p print " iBuild: $index $self->{reqs}->{$index}->{testN} eq $row \n";

	my $retv = $ret{$self->{reqs}->{$index}->{stat1}};
#	my $subs = $self->{reqM}->{$index}->{stat2};
	my $subs = $self->{reqs}->{$index}->{$moptr} || "Job_".$index;
#NO	my $subs = $self->{reqM}->{$index}->{mawn} || "Job_".$index;
	my $logfn;
	my $logf;
	if ( $retv == 0 ) {
	    $logfn = $self->{reqs}->{$index}->{olout};
	    $logf = `cat $logfn` if $logfn;
	} elsif ( $retv == 5 ) {
	    ##edg2.0	    $logf = $self->{reqs}->{$index}->{mess};
	    $logfn = $self->{reqs}->{$index}->{olog};
	    $logf = `cat $logfn` if $logfn;
	} else {
	    ##edg2.0	    if ( $self->{reqs}->{$index}->{stat2} eq "MISSING_FILES" ) { $logf = $self->{reqs}->{$index}->{mess}; }
	    ##edg2.0	    else {
	    $logfn = $self->{reqs}->{$index}->{olog};
	    $logf = `cat $logfn` if $logfn;
	    ##edg2.0	}
	}
	unlink $logfn if $logfn;
#	my $coln   = $cols->{$self->{reqs}->{$index}->{$machType}}->{order};
# 	my $rowM = "";
# 	for my $c (0..$ncols) {
# 	    if ( $c == $coln ) {
		$logf =~ s/&/&amp;/g; # This must be changed first.
		$logf =~ s/\"/&quot;/g;
		$logf =~ s/</&lt;/g;
		$logf =~ s/>/&gt;/g;
#		$rowM .= makeX(makeX(makeX("$logf", "output").makeX("$retv", "retval"), "step"), "col");
		$rowM .= makeX(makeX(makeX("$logf", "output").makeX("$retv", "retval").makeX("$subs", "substatus"), "step"), "col");
# 	    } else {
# 		$rowM .= makeX(makeX(makeX(" ", "output").makeX(" ", "retval"), "step"), "col");
# 	    }
# 	}
    }
	$colM .= makeX($rowM, "row");
}

    $colM =~ s/:lxshare/:h/g;

#   my $stepM = makeX(makeX("$brf_info", "brf_info").makeX("$det_info", "det_info"), "step");
    my $stepM = makeX(makeX("$brf_info", "brf_info").makeX("baratin", "substatus_info").makeX("$det_info", "det_info"), "step");
    my $body = $colM;

    my $info = makeX($Title, "title");
    $info .= makeX($date, "date");
    @lof_rows = map { makeX($_, "row"); } @lof_rows;
    $info .= makeX("@lof_rows", "rows_list");
    @lof_cols = map { makeX($_, "col"); } @lof_cols;
    my $linfo = makeX("@lof_cols", "cols_list");
    $linfo =~ s/:lxshare/:h/g;
    $info .= $linfo;
    $info .= $stepM;
    $info = makeX($info, "info");

    ## The whole log is made of INFO+BODY
    open XML, "> $self->{xmlfile}";
    print XML makeX($info." \n".$body, "log");
    close XML;
    #p print STDOUT makeX($info." \n ".$body, "log");

    return 0;
}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
