#!/usr/bin/perl -w

package BaseTest::EDGLifecycle::Stack::DataStorm;

use FindBin qw($Bin);
use lib $Bin;

use File::Basename;
use EDGUtils;

use strict;
use vars qw(@ISA);
@ISA=qw(BaseTest::EDGLifecycle::Stack);

=head1 NAME

DataStorm.pm - Check that globusrun authentication works on the hosts
# $Id: DataStorm.pm,v 1.1 2005/06/06 16:29:24 reale Exp $

=head1 SYNOPSIS

Check that globusrun authentication works on the hosts

=head1 DESCRIPTION 

B<DataStorm.pm> checks that the Globusrun authentication works
for all hosts of the list for the testing user. 
 This test requires that a valid GSI proxy is available for the latter.

=head1 AUTHOR

Gilbert Grosdidier (Gilbert.Grosdidier@cern.ch).  Please report bugs via the
European DataGrid bug reporting service (Bugzilla) at
http://marianne.in2p3.fr/bugzilla.  

=head1 LICENSE

Copyright (c) 2003 by Gilbert Grosdidier and Centre National de
la Recherche Scientifique (CNRS).  All rights reserved.

The software was distributed with and is covered by the European
DataGrid License.  A copy of this license can be found in the included
LICENSE file.  This license can also be obtained from
http://www.eu-datagrid.org/license.

=cut

sub setContext {
    my $self = shift;

    $self->SUPER::setContext();

#    print "DataStorm: setContext\n";
    $self->printLocation( (caller(0))[3] );

    $self->addOption('maxSubs=i',
                     'Number of jobs.',
                     '6');
    $self->addOption('maxStack=i',
                     'Number of simultaneous jobs in the stack.',
                     '2');
    $self->addOption('reqLapse=i',
                     'Delay between 2 submissions (in sec).',
                     '15');
    $self->addOption('serie=i',
                     'Serial number of the storm.',
                     '135');
#     $self->addOption ("cksumSize=i", 
# 		      "Size of the file for checksum test (Mb)", 
# 		      2000);
#     $self->addOption('circular',
#                      'To scan circularly the CEList built with useCEList');
    $self->addOption('SEtype=s',
		     'To select the SE type where the file is going to be first created and registered (Classic, Castor, dCache, All=NOselection).',
		     'All');
    $self->addOption('lcgcr',
                     'To use lcg-cr command instead of edg-rm for the cr operation when set.');
    $self->addOption('defdest',
                     'To use the default SE as the primary target of the CR (instead of the selected SE).');


    # Additionnal Commands necessary to run this test.
    my @commands = (
#			  "edg-vo-env",
#			  "edg-gridftp-rm",
#			  "edg-gridftp-ls",
#			  "edg-gridftp-mkdir",
#			  "gdmp_host_subscribe",
#			  "gdmp_job_status",
#			  "globus-replica-catalog",
#			  "ldapsearch",
#			  "edg-replica-manager-replicateFile",
#			  "edg-replica-manager-copyAndRegisterFile",
#			  "edg-replica-manager-deleteFile",
#			  "edg-replica-manager-listReplicas"
		   );
    push @{$self->{commands}}, @commands;

    # Define the help and description.
    $self->{HelpText} = <<'END'

Usage: edg-testbed-test BaseTest::EDGLifecycle::Stack::DataStorm [options]

END
    ;

    $self->{DescText} = <<'END'

 This is a special JobStorm where, inside of each job running on a WN:
- a (small) data file is created 
- then copied and registered on the CloseSE (thru RM),
- then replicated (thru RM) to the closest SE from the submitting UI,
- then the LFN of this file is returned within the OutputSandbox of the job,
- then one checks that the latter is indeed available on the closest SE.

 All available options are described in the JStorm test itself.

 Command used: 
opt/edg/bin/MainScript --forcingTB=CertTB --TList=DStorm \
  --verbose --forcingVO=dteam \
  --addOptionList="--reqLapse=1 --maxStack=50 --maxSubs=100 --singleSubmit \
  --pollingPeriod=60 --keepTempDir --circular --useCEList --serie=11003 \
  --selectNodes='lxshare0277.cern.ch lxshare0290.cern.ch lxshare0241.cern.ch lxshare0235.cern.ch' "

Details on the parameters:
 Delay between 2 jobs at submit time: 1 sec. (--reqLapse=1).
 No more than --maxStack=50 submitted to the RB at the same time.
 Total number of jobs --maxSubs=100.
 NO resubmission even when the job is failing at submit time (--singleSubmit).
 Monitoring of the status of the jobs every --pollingPeriod=60 sec.
 Default EDG retryCount: 3.
 Do not erase the temporary directory when the test is over (--keepTempDir).
 Submit to all available CEs in a circular loop (--circular --useCEList).
 Out of all available CEs, select only the following ones (--selectNodes).

To be implemented soon:
- --lcgcr option to use lcg-cr command instead edg-rm wherever available. DONE
- --Castor, --dCache or --Classic options to select relevant SE types only
 (if none selected, use all types randomly). DONE: --SEtype=Classic, ...

END
    ;

	$self->{type} = "DataStorm";
	$self->{brfinfo} = "ERM Data Transfers from WNs";
#	 $self->{subDir} = "";
#	 $self->{location} = "";
#	 $self->{VO} = "";
#	 $self->{dataFL} = [];      ## List
#	 $self->{SPs} = {};         ## Hash
	 $self->{SEs} = [];         ## List
#	 $self->{SEpairs} = {};     ## Hash
	$self->{CEs} = [];         ## List
#    $self->{bigFile} = "";

    return 0;

}

# Build any necessary setup for the storm.
sub setupEnvironment {
    my $self = shift;

#    print "DataStorm: setupEnvironment\n";
    $self->printLocation( (caller(0))[3] );

## /opt/edg/etc/edg-replica-manager/edg-replica-manager.conf:defaultCE=adc0015.cern.ch

    my @m;
    if ( -e "/opt/edg/var/etc/edg-replica-manager/edg-replica-manager.conf" ) {
	chomp(my $res = `grep defaultSE /opt/edg/var/etc/edg-replica-manager/edg-replica-manager.conf`);
	@m = split /=/, $res;
    } elsif ( -e "/opt/edg/etc/edg-replica-manager/edg-replica-manager.conf.lcfg" ) {
	chomp(my $res = `grep 'DEFAULT.SE' /opt/edg/etc/edg-replica-manager/edg-replica-manager.conf.lcfg`);
	@m = split /\|/, $res;
    }

## Look up for overall SE declared in the locall BDII (assumed to be the first one)...
    my ($bdii, @rest) = split/\s+/, $ENV{IIhostname__};
#    my ($bdii, @rest) = $ENV{IIhostname__};
    my @SEs = `ldapsearch -x -H ldap://$bdii:2170 -b mds-vo-name=local,o=grid | grep 'GlueSEUniqueID:' `;
    my $nSE = scalar(@SEs);
    my $rnd = int(rand($nSE));
    my @s = split/\s+/, $SEs[$rnd];
    print " bdiiSE: $bdii $s[1] \n";

    if ( "@SEs" =~ /$m[1]/ ) {
	$self->{targetSE} = "$m[1]";
    } else {
	$self->{targetSE} = "$s[1]"; ## || "AHAHAH";
    }
    print " TargetSE: =$self->{targetSE}= \n";
    if ( ! $self->{targetSE} ) {  ##  || "=$self->{targetSE}=" eq "==" ) {
	print STDERR " TargetSE: =$self->{targetSE}= \n";
	$self->setErrorMessage('setupEnvironment failed while setting the targetSE which returned empty');
	return 1;
    }	

    return 0;
}

sub dgjobUSELESS {
    my $cmdn = shift;
    my $root = "dg-job-";
    if ( defined $ENV{EDG_FLAVOR} && $ENV{EDG_FLAVOR} ne 'edg1' ) {
	$root = "e".$root;
    }
    return "$root"."$cmdn";
}

# Add any necessary arguments to the existing local JDL file before submission.
sub addArguments {
    my ($self, $jdlTemp, $index) = @_;

#    print "DataStorm: addArguments\n";
    $self->printLocation( (caller(0))[3] );

## Create the checksum file ...
#     $self->{reqs}->{$index}->{bigo} = $self->{tempdir}."/BigOrig-$index.crap";
#     my $checksum1 = $self->createBig($self->{reqs}->{$index}->{bigo});
#     $checksum1 =~ s/ /-/;
#     $checksum1 = "=$checksum1=";

    my $bid = `echo 'Arguments = "S$ENV{STACK_SERIE} I${index} $self->{targetSE} $self->{opts}->{SEtype}"; ' >> $jdlTemp`;
#     $bid = `echo '#BigFileName = "mumbojombo"; ' >> $jdlTemp`;
#     $bid = `echo 'InputSandbox = {"$self->{shFile}","$self->{reqs}->{$index}->{bigo}"}; ' >> $jdlTemp`;

if (0) {
    my $nCE = scalar(@{$self->{CEs}});
    my $rnd = int(rand($nCE));
    my $ce;
    if ( $self->{opts}->{circular} ) {
	$ce = ${$self->{CEs}}[($index%$nCE)];
    } else {
	$ce = ${$self->{CEs}}[$rnd];
    }
##    my $ce = "";

    if ( $self->{opts}->{useCEList} ) {
#glue    my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
#norm    my $reqs = 'Requirements = other.CEId=="'.$ce.'"';
	my $reqs = 'Requirements = other.GlueCEUniqueID=="'.$ce.'"';
	print STDERR " Requirements: $reqs \n";
	$bid = `echo '$reqs; ' >> $jdlTemp`;
	my @m = split /:/, $ce;
	$self->{reqs}->{$index}->{mace} = "CE:$m[0]";
    } else {
	$self->{reqs}->{$index}->{mace} = "free";
    }
} else {
    $self->chooseCE($index, $jdlTemp);
}
    
    return 0;
}

# Create a BigFile for checksuming ...
sub createBigOLD {
    my $self = shift;
    my $bigOrig = shift;

    my $size = $self->{opts}->{cksumSize};  ## in Mb
##    $size = 2;  ## temp
##    my $time = $size*70000;
    open BGF, "> $bigOrig";
    my $bgf = "";
    for ( my $t=0; $t<$size; $t++ ) {
	my $r = rand($t);
	print BGF "$r";
	$bgf .= "$r";
    }
    print BGF "\n";
    $bgf .= "\n";
    my $cumul = 70000;
    for ( my $t=0; $t<$cumul; $t++ ) {
	print BGF "$bgf\n";
    }
    close BGF;
    #system("ls -l $bigOrig; cksum $bigOrig");
    my $checksum = `cksum $bigOrig | cut -d" " -f1-2`;
    chomp($checksum);

    return $checksum;

}


# Prepare everything for running the test itself ...
sub setupTest {
    my $self = shift;

#    print "DataStorm: setupTest\n";
    $self->printLocation( (caller(0))[3] );

    @{$self->{SEs}} = @{$self->{args}};

#    print " Defaults: @{$self->{optionDefaults}} \n";
#    print " Linkage:  @{$self->{optionLinkage}}  \n";
#    exit 0 if (1);

    my $rc = $self->SUPER::setupTest();
    return $rc if $rc;

    ## Possible only when jdlFile and shFile get created
    $rc = $self->buildCEList();
    return $rc if $rc;

#     if ( $self->{opts}->{useCEList} ) {
# 	if ( ! scalar(@{$self->{CEs}}) ) {
# 	    $self->setErrorMessage('Setup failed while building the CEList which returned empty');
# 	    return 1;
# 	}	
#     }

    $self->{zombSlep} = ( $self->{zombSlep} > $self->{reqLapse} ) ? $self->{reqLapse} : $self->{zombSlep};
    print "\nOPTIONS: \n", map { "$_ => $self->{opts}->{$_}\n" } keys %{$self->{opts}};
    print "\n";

    return $rc;
}


sub createJDL {
    my ($self, $file, $script) = @_;

#    print "DataStorm: createJDL\n";
    $self->printLocation( (caller(0))[3] );
    print " file script: $file $script \n";

    my ($sname, $spath, $suffix) = fileparse($script, []);

#    my $file = "$dir/JobDescription.jdl";

    # Write out the job description.
    open JDL, ">$file";
    print JDL <<"END"
Executable = "$sname";
InputSandbox = {"$script"};
StdOutput = "result.out";
StdError = "result.err";
OutputSandbox = {"result.out","result.err", "BrokerInfo"};
VirtualOrganisation = "$self->{opts}->{vo}";
    Requirements = other.GlueCEPolicyMaxCPUTime<600000;
    Rank = other.GlueCEPolicyMaxCPUTime;
END
    ;

    print JDL "RetryCount = 3;\n";   ##  Special ?

    close JDL;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

sub createScript {
#   my $dir = shift;
#   my $file = "$dir/JobScript.sh";
    my ($self, $file) = @_;

#    print "DataStorm: createScript\n";
    $self->printLocation( (caller(0))[3] );

    # Write out the job description.
    open SCRIPT, ">$file";
#OLDGOOD    print SCRIPT <<'END'
    my $SCRIPT = <<'END'
#!/usr/bin/perl -w

my ($a, $b, $c, $d) = @ARGV;
print STDERR "DataStorm: runTest ARGS: $a $b $c $d\n";
print STDOUT "DataStorm: runTest ARGS: $a JobIndex: $b returnSE: $c SEtype: $d\n";
my $returnSE = $c;
my $SEtype   = $d;

my $res = `cp .BrokerInfo BrokerInfo 2>&1`;
my (@CSE) = `edg-brokerinfo getCloseSEs 2>&1`;
    chomp(my $theCE = `edg-brokerinfo getCE 2>&1 | cut -d":" -f 1`);
    chomp(my $theWN = `hostname --long 2>&1`);

@CSE = grep { ! /castorgrid/ } @CSE;
chomp(my $closeSE = $CSE[0]);

my %cseType = ();
$closeSE = "";
foreach my $mycse ( @CSE ) {
    chomp $mycse;
    chomp(my $SEfs = `edg-brokerinfo getSEFreeSpace  $mycse 2>&1`);
    chomp(my $SEmp = `edg-brokerinfo getSEMountPoint $mycse 2>&1`);
    ## flatfile search is useless here ...
    my @ldap = `ldapsearch -x -H ldap://$mycse:2135 -b "mds-vo-name=local,o=grid" | egrep 'castor|pnfs|flatfile' 2>&1`;

my $pnfsO = grep { /pnfs/ } @ldap;
my $castorO = grep { /castor/ } @ldap;
print " MYCSE: $mycse $pnfsO $castorO \n";

    $cseType{$mycse} = "NADA";
#     if ( $SEfs =~ /\d+/ ) {
#         if ( $SEmp =~ /flatfiles/ || $SEmp !~ /dummy/ ) { $cseType{$mycse} = "Classic"; }
#         else { $cseType{$mycse} = "Castor"; }
#     } else { $cseType{$mycse} = "dCache"; }

    if ( $SEmp =~ /flatfiles/ || $SEmp !~ /dummy/ ) { $cseType{$mycse} = "Classic"; }
    else {
        if ( $pnfsO ) { $cseType{$mycse} = "dCache"; }
        if ( $castorO ) { $cseType{$mycse} = "Castor"; }
    }

    print STDOUT " CSEType: $mycse >$cseType{$mycse}< \n";
    if ( $cseType{$mycse} eq "NADA" ) { next; }

###    if ( $closeSE eq "" && $cseType{$mycse} ne "NADA" ) { $closeSE = $mycse; }
##    if ( $closeSE eq "" && $cseType{$mycse} ne "dCache" ) { $closeSE = $mycse; }
    if ( $closeSE eq "" && $cseType{$mycse} eq "dCache" ) { $closeSE = $mycse; }
#    if ( $closeSE eq "" && $cseType{$mycse} ne "Classic" ) { $closeSE = $mycse; }

}

    if (0) {
	my %cseF = (
		    'lxshare0235.cern.ch' => 'lxshare0282.cern.ch',
		    'lxshare0241.cern.ch' => 'lxshare0302.cern.ch',
		    'lxshare0277.cern.ch' => 'lxshare0247.cern.ch',
		    'lxshare0290.cern.ch' => 'lxshare0302.cern.ch'
		    );
	$closeSE = $cseF{$theCE};
    }

    if (0) {
	my %cseF = (
		    'lxshare0235.cern.ch' => 'lxshare0282.cern.ch',
		    'lxshare0241.cern.ch' => 'lxshare0303.cern.ch',
		    'lxshare0277.cern.ch' => 'lxshare0247.cern.ch',
		    'lxshare0290.cern.ch' => 'lxshare0229.cern.ch'
		    );
	$closeSE = $cseF{$theCE};
    }

my %SEofType = reverse %cseType;

if ( $SEtype eq 'All' ) {  ##  GOOD
    my $nCE = scalar(@CSE);
    my $rnd = int(rand($nCE));
    $closeSE = $CSE[$rnd];
} else {
    if ( $SEofType{$SEtype} ) { $closeSE = $SEofType{$SEtype}; }
    elsif ( $SEofType{'Classic'} ) { $closeSE = $SEofType{'Classic'}; }
}

if (1) {
    print STDOUT " CSE: >$closeSE< @CSE ( SEtype: $SEtype TypeOf: $cseType{$closeSE} ) \n";
##TEMP
    if ( scalar(@CSE) >= 2 && $closeSE =~ /lxshare0247|lxshare0282|lxshare0302/ ) {
##REMOVED??   chomp($closeSE = $CSE[1]);
    }
    if ( $closeSE eq "" ) { 
        print STDERR " FAILed: NO closeSE \n";
        exit 1; 
    }
}

## Look for default SE
my $defSE = `edg-rm --vo dteam pi | grep 'default SE' `;
if ( $defSE ) {
    my @m = split /\s+/, $defSE;
    $defSE = $m[3];
}

if ( DEFSE ) {
    $closeSE = $defSE;
    print " Warning: because of --defdest, the closeSE is now: $closeSE \n";
}

chomp(my $myVO = `edg-brokerinfo getVirtualOrganization 2>&1`);
    my $config = "";  ## Still empty
if ( ! LCGCR ) {
    $edgRMopts = "$config --insecure --vo=$myVO";
} else {
    $edgRMopts = "$config -i --vo=$myVO";
    $edgRMopts = "$config --insecure --vo=$myVO";
}
    print STDOUT " DataStorm: edgRMopts: $edgRMopts WN: $theWN CE: $theCE closeSE: $closeSE returnSE: $c \n";

my ($cres, $file) = createNewData($edgRMopts, $closeSE, $returnSE, $a, $b, "");
print " JobLFN: lfn:$file \n" unless $cres;

## New to target all WNs ?
sleep 60;

my $service = `echo; date; pwd; host $ENV{HOSTNAME}; host $ENV{HOST}; echo EDG_WL_JOBID: $ENV{EDG_WL_JOBID}; uname -a; echo; ls -l`;
print STDOUT $service;

exit $cres;

sub createNewData {
    my ($edgRMopts, $SE1, $SE2, $serie, $index, $subDir) = @_;

#    $self->printLocation( (caller(0))[3] );

    ## Create another temporary data file ...
    my $file = "$ENV{USER}-$$-dstorm-$serie-$index.data";
    ## Some Pre-Cleaning
    my $rc = removeGhostFile($edgRMopts, $file);
    return $rc if $rc;

    # Retrieve the temporary directory.
#    my $tmpdir = $self->{tempdir};
    my $tmpdir = "/tmp";

    open TMP, ">$tmpdir/$file";
    print TMP "Good Morning to all Everybody ! This is a DataStorm. Job# $index \n";
    close TMP;

    print " JobLFN0: lfn:$file \n";

#     my @kSEpairs = keys %{$self->{SEpairs}};
#     my ($SE1, $SE2) = split /\s+/, $kSEpairs[int(rand(scalar(@kSEpairs)))];

#     my $SP1 = "$self->{SPs}->{$SE1}/$subDir";
#     my $SP2 = "$self->{SPs}->{$SE2}/$subDir";

    $rc = createReplicas ($edgRMopts, $SE1, $SE2, $subDir, $file);
    return $rc if $rc;

    return (0, "$file");
}

## Dynamic creation of Replicas job by job ...
sub createReplicas {

#     my ($self, $SE1, $SE2, $subDir, $file) = @_;
#     my $edgRMopts = $self->{edgRMopts};
    my ($edgRMopts, $SE1, $SE2, $subDir, $file) = @_;

    # Retrieve the temporary directory.
#    my $tmpdir = $self->{tempdir};
    my $tmpdir = "/tmp";

#     my $command = "edg-replica-manager-copyAndRegisterFile -e -l ${subDir}/${file} -s $self->{hostname}//tmp/${file} -d $SE1/$SP1/$file";
#     print " CMD: $command\n";
#     my $resRcCARF = `$command 2>&1`;
#     my $stat = $?;
#     print "\n resRcCARF1: $file -$stat- \n$resRcCARF";
#     if ( $stat != 0 ) {
# 	print STDERR " File: ${subDir}/${file} on: $SE1 not correctly registered or created \n";
# 	return 1;
#     }

my $command;
if ( ! LCGCR ) {
##EDG-RM case
    $command = "edg-replica-manager --verbose $edgRMopts copyAndRegisterFile file:$tmpdir/${file} -l lfn:${file} -d $SE1 ";
} else {
##LCG-CR case
    chomp(my $gguid = `uuidgen 2>&1`);
    print STDOUT " UUIDGEN: $gguid \n";
    $command = "lcg-cr --verbose $edgRMopts -l lfn:${file} -d $SE1 -g $gguid file:$tmpdir/${file}";
}

    print "\n CMD: $command\n";
    my $resRcCARF = `$command 2>&1`;
    my $stat = $?;
    print "\n resRcCARF1: $file -$stat- \n$resRcCARF";
    if ( $stat != 0 ) {
	print STDERR " File: ${file} on: $SE1 not correctly registered or created \n";
	return 1;
    } else {
	chomp($resRcCARF);
	if ( $resRcCARF !~ "guid:" ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong GUID returned: $resRcCARF \n";
	    return 1;
	}
#	$self->{reqs}->{$stepI}->{olout} = $resRcCARF;
    }

    my %PFNs = ();
    $PFNs{"$SE1/$file"} = "created";

#    my @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    if ( LCGCR ) { $command = "lcg-lr $edgRMopts lfn:${file}"; }
    my @resRcLR = `$command 2>&1`;
    print "\n resRcLR1: $file -$?- =$#resRcLR= \n@resRcLR";
    foreach my $l ( @resRcLR ) {
        chomp $l;
        my @i = split /[:\/]/, $l;
#prev	if ( $i[0] ne "srm" ) {
	if ( $i[0] !~ /srm|sfn/ ) {
	    print STDERR " File: ${file} on: $SE1 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	my $csrm = "$SE1/${file}";
	if ( $csrm ne $gsrm ) {
	    print STDERR " File: ${file} on: $SE1 : A ghost file still existed: $gsrm \n";
	    return 1;
	}

        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE1 File: $file did not previously existed: $gsrm ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    print " SRM keys SE1: $SE1 $file value: $l \n";
	}
    }
    if ( $PFNs{"$SE1/$file"} ne "existing" ) {
	print STDERR " Step: $SE1 File: $file not correctly copied/registered ! \n";
	return 1;
    }

if ( "$SE1" ne "$SE2" ) {
#    my $resRcRF = `edg-replica-manager-replicateFile -e -l ${subDir}/${file} -s $SE1 -d $SE2 2>&1`;
    $command = "edg-replica-manager --verbose $edgRMopts replicateFile lfn:${file} -d $SE2";
    if ( LCGCR ) { $command = "lcg-rep --verbose $edgRMopts -d $SE2 lfn:${file}"; }
    my $resRcRF = `$command 2>&1`;
    $stat = $?;
    print "\n resRcRF2: $file -$stat- \n$resRcRF";
    if ( $stat != 0 ) {
        print STDERR " File: ${file} on: $SE2 not correctly registered or created \n";
	return 1;
    }
    $PFNs{"$SE2/$file"} = "created";
}

#       @resRcLR = grep { /location/ } `edg-replica-manager-listReplicas -e -l ${subDir}/${file} 2>&1`;
    $command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
    if ( LCGCR ) { $command = "lcg-lr $edgRMopts lfn:${file}"; }
    @resRcLR = `$command 2>&1`;
    print "\n resRcLR2: $file -$?- =$#resRcLR= \n@resRcLR";
    foreach my $l ( @resRcLR ) {
        chomp $l;
        my @i = split /[:\/]/, $l;
#	if ( $i[0] ne "srm" ) {
	if ( $i[0] !~ /srm|sfn/ ) {
	    print STDERR " File: ${file} on: $SE2 : Wrong SRM returned: $l \n";
	    return 1;
	} 

	my $gsrm = "$i[3]/${file}";
	my $csrm = "$SE2/${file}";
	if ( $csrm ne $gsrm ) {
	    next;
# 	    print STDERR " File: ${file} on: $SE2 : A ghost file still existed: $gsrm \n";
# 	    return 1;
	}

        if ( ! exists $PFNs{$gsrm} || $PFNs{$gsrm} !~ /created|existing/ ) {
            print STDERR " Step: $SE2 File: $file did not previously existed: $gsrm ! \n";
	    return 1;
        } else {
	    $PFNs{$gsrm} = "existing";
	    print " SRM keys SE2: $SE2 $file value: $l \n";
	}
    }

    print "\nPFN0: \n", map { "$_ => $PFNs{$_}\n" } keys %PFNs;
    print "-----------------------------------\n";
    if ( $#resRcLR < 1 || scalar(keys(%PFNs)) != 2 ) {
	print STDERR " Replicates not correctly created on: $SE1 $SE2 \n";
	return 1;
    }

    return 0;

}

sub removeGhostFile {

#     my ($self, $file) = @_;
#     my $edgRMopts = $self->{edgRMopts};
    my ($edgRMopts, $file) = @_;

#    $self->printLocation( (caller(0))[3] );

##	    foreach my $loop ( 0..2 ) {
##		my $file = "$ENV{USER}-$upid-$loop-rmstorm.txt";

		my $command = "edg-replica-manager $edgRMopts listGUID lfn:${file}";
                if ( $self->{opts}->{lcgcr} ) { $command = "lcg-lg $edgRMopts lfn:${file}"; }
		chomp(my $resRcLR = `$command 2>&1`);
		my $stat = $?;
		print "$stat\t                                   $resRcLR\n";
#		if ( $stat ) { next; }
    return 0 if $stat;

		$command = "edg-replica-manager --verbose $edgRMopts deleteFile --all-available --all-lfn $resRcLR";
                if ( $self->{opts}->{lcgcr} ) { $command = "lcg-del --verbose $edgRMopts -a $resRcLR"; }
		my $resRcDF = `$command 2>&1`;
		$stat = $?;
		print "\n resRcDF: $file -$stat- \n$resRcDF\n";
##BAD?    return $stat if $stat;

		$command = "edg-replica-manager $edgRMopts listReplicas lfn:${file}";
                if ( $self->{opts}->{lcgcr} ) { $command = "lcg-lr $edgRMopts lfn:${file}"; }
		my $resRcLRafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLRafter: $file -$stat- \n$resRcLRafter";
#    return $stat if $stat;

		$command = "edg-replica-manager $edgRMopts listReplicas $resRcLR";
                if ( $self->{opts}->{lcgcr} ) { $command = "lcg-lr $edgRMopts $resRcLR"; }
		my $resRcLGafter = `$command 2>&1`;
		$stat = $?;
		print "\n resRcLGafter: $file -$stat- \n$resRcLGafter";
#    return $stat if $stat;
##	    }

    return 0;

}

__DATA__

#additional storage here

END
    ;

    if ( ! $self->{opts}->{lcgcr} ) { $self->{opts}->{lcgcr} = "0"; }
    $SCRIPT =~ s/LCGCR/$self->{opts}->{lcgcr}/g;

    if ( $self->{opts}->{defdest} ) { 
	my $dest = ' -d \$SE1 ';
	$SCRIPT =~ s/$dest/ /g;
    } else {
	$self->{opts}->{defdest} = "0";	
    }
    $SCRIPT =~ s/DEFSE/$self->{opts}->{defdest}/g;

    print SCRIPT $SCRIPT;

    close SCRIPT;

    # Ensure that the script is executable.
    chmod 0755, $file;

    # Return the file only if it now exists and has a non-zero size.
    if (-s $file) {
        return $file;
    } else {
        return "";
    }
}

# This will run the evaluateOutput method on all of the jobs.  The
# evaluateOutput method may be overridden to achieve a different
# evaluation. 
sub evaluateTest {
    my $self = shift;
    my %cseLab = (
		   'castor'    => 'Castor',
		   'pnfs'      => 'dCache',
		   'flatfiles' => 'Classic',
		   'storage' => 'Classic'
		  );

#    print "DataStorm: evaluateTest\n";
    $self->printLocation( (caller(0))[3] );

    my $config = "";  ## Still empty
    my $edgRMopts = "$config --insecure --vo=$self->{opts}->{vo}";

    ## Implement proper parsing of output files, and subsequent evaluation here.
#     foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
# 	print " JOBL\t$index :\n$self->{reqs}->{$index}->{olout} \n";
#     }

    foreach my $index ( sort { $a <=> $b } keys %{$self->{reqs}} ) {
	print " JOBL\t$index :\n$self->{reqs}->{$index}->{olout} \n";
#p	print " JOUT\t$index :\n".`cat $self->{reqs}->{$index}->{olout}`." \n" if $self->{reqs}->{$index}->{olout};
	next unless $self->{reqs}->{$index}->{olout};
	chomp(my $res = `grep JobLFN0 $self->{reqs}->{$index}->{olout}`);
	my @m = split /\s+/, $res;
	print " JLFN:\t$m[2] \n";
	my $file = $m[2];
	my $match = 0;
	my $command = "edg-replica-manager $edgRMopts listReplicas ${file}";
	if ( $self->{opts}->{lcgcr} ) { $command = "lcg-lr $edgRMopts ${file}"; }
	my @resRcLR = `$command 2>&1`;
	print "\n resRcLR0: $file -$?- =$#resRcLR= \n@resRcLR";
	if ( ! $? ) {
	foreach my $l ( @resRcLR ) {
	    chomp($l);
	    if ( $l =~ "$self->{targetSE}" ) {
		print " Matching: [OK] $l \n";
		$match++;
	    }
	}
	chomp($res = `grep 'SRM keys SE1:' $self->{reqs}->{$index}->{olout}`);
	my @m = split /\s+/, $res;
#	@m = split /\./, $m[4];
	$self->{reqs}->{$index}->{cse} = $m[4];
	$res =~ /.*\/(castor|pnfs|flatfiles|storage)\/.*/;
	$self->{reqs}->{$index}->{stat2} = $cseLab{$1};
    }
##test	if ( $index == 1 ) { $match = 10; }  ##  TEMP TEST
	if ( $match != 1 ) { print " Matching: [FAILED] \n"; }
#old	$self->{reqs}->{$index}->{match} = $match;
	if ( $self->{reqs}->{$index}->{stat1} eq '[OK]' ) {
	    if ( $match != 1 ) {
		$self->{reqs}->{$index}->{stat1} = '[FAIL]';
		$self->{reqs}->{$index}->{stat2} = 'MATCHING';
		$self->{reqs}->{$index}->{olog} = $self->{reqs}->{$index}->{olout};
	    }
	}
#p	print " SUMM\t$index :\n", map { "\t$_\t=>\t$self->{reqs}->{$index}->{$_} \n" } sort keys %{$self->{reqs}->{$index}};
    }

    my $rc = $self->SUPER::evaluateTest();
    return $rc if ($rc);

    return 0;

}

# This ensures that the module returns a true value when loaded. 
# If this isn't here, loading the module may incorrectly fail.
1;
