#!/bin/bash
######################################################################
#
# Authors:  Gonzalo Merino  -  merino@ifae.es
#           Miquel Barcelo  -  barcelo@ifae.es
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
#######################################################################
#
# Requires :
#       - The EDG_TESTROOT environment variable being defined
#       - getopt (installed by default on most systems)
#
##################################################################
#
# SCRIPT RCTools_func_main.sh
#
# USAGE: RCTools_func_main.sh [-h] -n
#        RCTools_func_main.sh [-h] -i [-s int]
#        RCTools_func_main.sh [-h] [-s int] [-l string] <RC config file>
#
# ARGUMENTS: 
#        -h/--help           Shows help.
#        -n/--numstep        Prints the number of selectable steps on the test.
#        -s/--step           Run only the specified step, not the entire test.
#        -i/--info           Prints brief info about the selected step using "-s".
#                            Prints detailed info about the entire test if no step selected.
#        -l/--logfile        Log test results to this file. By default is dumped to stdout.
#        -d/--debug          Selects verbosity (debug) level.
#        <RC config file>    Replica Catalog configuration file (rc.conf)
#        
# DESCRIPTION:  This script tests the edg_rc command line tools against a certain RC 
#               described in the config file given as input.    
#               This is the complete test cycle:
#
# step 0 - edg_rc_addLogicalFileName
# step 1 - edg_rc_addLogicalFileAttribute
# step 2 - edg_rc_getLogicalFileAttributes
# step 3 - edg_rc_deleteLogicalFileAttribute
# step 4 - edg_rc_addPhysicalFileName
# step 5 - edg_rc_getPhysicalFileNames
# step 6 - edg_rc_getLogicalFileName
# step 7 - edg_rc_deletePhysicalFileName
# step 8 - edg_rc_deleteLogicalFileName
#
# RETURN VALUE: 0 PASSED
#	 	1 FAIL
#		2 WARNING
#		3 XFAIL
#		4 UNRESOLVED
#		5 UNTESTED
#		6 UNSUPPORTED
#
##################################################################

# We should take a look for an /etc/edg-test.conf or 
# $EDG_LOCATION/etc/edg-test.conf for configuration file

source $EDG_TESTROOT/sbin/testRetCodes.sh 
source $EDG_TESTROOT/sbin/common.sh 
source $EDG_TESTROOT/sbin/message.sh
source $EDG_TESTROOT/sbin/runCommand.sh
source $EDG_TESTROOT/sbin/saveVars.sh
source $EDG_TESTROOT/sbin/loadVars.sh
source $EDG_TESTROOT/sbin/echoFunc.sh

if [ -z "$EDG_TESTROOT" ]
then
      echo "EDG_TESTROOT environment variable must be defined!"
      exit $RET_UNRESOLVED
fi    

message_help ()
{
    message 0 "Usage: RCTools_func_main.sh [-h] -n"
    message 0 "       RCTools_func_main.sh [-h] -i [-s int]"
    message 0 "       RCTools_func_main.sh [-h] [-s int] [-l string] <config file name> "
    message 0 " "
    message 0 "Options/Arguments:" 
    message 0 "       -h/--help           Shows help."
    message 0 "       -n/--numstep        Prints the number of selectable steps on the test."
    message 0 "       -s/--step           Run only the specified step, not the entire test."
    message 0 "       -i/--info           Prints brief info about the selected step using ""-s""."
    message 0 "                           Prints detailed info about the entire test if no step selected."
    message 0 "       -l/--logfile        Log test results to this file. (Default: stdout)"
    message 0 "       -d/--debug          Selects verbosity (debug) level. 0=none 1=brief 2=full. (Default: 0)"
    message 0 "       <config file name>  RC config file (mandatory)"
}

# Take a look to the bottom to see the real body of script

dotest ()
{
    if [ -z "$1" ] || [ -z "$2" ]
    then
	message 1 "$0: wrong number of arguments."
	exit $RET_UNRESOLVED
    fi

    local SELECTION=$1
    local RC_CONF_FILE=$2

    if [ -z "$3" ] ; then
        local TMP_LOG_FILE=.tmplog_$PPID
    else
        local TMP_LOG_FILE=$3.new
    fi

    local RETVALUE=$RET_PASSED

    case "$SELECTION" in
    0)  runCommand "edg_rc_addLogicalFileName -c $RC_CONF_FILE -C -l $LFN" $TMP_LOG_FILE
        lookResult $? $TMP_LOG_FILE ; RETVALUE=$?
	;;

    1)  runCommand "edg_rc_addLogicalFileAttribute -c $RC_CONF_FILE -C -l $LFN -n $LFN_ATT -v $LFN_ATT_VAL" $TMP_LOG_FILE
        lookResult $? $TMP_LOG_FILE ; RETVALUE=$?
	;;

    2)  runCommand "edg_rc_getLogicalFileAttributes -c $RC_CONF_FILE -C -l $LFN" $TMP_LOG_FILE
	lookResult $? $TMP_LOG_FILE ; RETVALUE=$?
	;;

    3)  runCommand "edg_rc_deleteLogicalFileAttribute -c $RC_CONF_FILE -C -l $LFN -n $LFN_ATT -v $LFN_ATT_VAL" $TMP_LOG_FILE
        lookResult $? $TMP_LOG_FILE ; RETVALUE=$?
	;;

    4)  ldapGetPFN_PATH $RC_CONF_FILE $TMP_LOG_FILE
	if [ $? -ne 0 ] ; then
	    RETVALUE=$RET_FAIL
       	else
	    echo "ldapsearch query to the RC returned PFN Path: $PFN_PATH" >> $TMP_LOG_FILE
	    echo "" >> $TMP_LOG_FILE

	    # Build the PFN by appending the LFN to the SE /flatfiles path
	    PFN=$PFN_PATH/$LFN

	    
	    runCommand "edg_rc_addPhysicalFileName -c $RC_CONF_FILE -C -l $LFN -p $PFN" $TMP_LOG_FILE.B
	    lookResult $? "$TMP_LOG_FILE.B" ; RETVALUE=$?
	    cat $TMP_LOG_FILE.B >> $TMP_LOG_FILE
	fi
	;;

    5)  if [ -z $PFN ] ;  then
	    RETVALUE=$RET_UNTESTED
            echo "Test not done because there is no PFN available" > $TMP_LOG_FILE
        else
	    runCommand "edg_rc_getPhysicalFileNames -c $RC_CONF_FILE -C -l $LFN" $TMP_LOG_FILE
            lookResult $? $TMP_LOG_FILE ; RETVALUE=$?
        fi
	;;

    6)  if [ -z $PFN ] ;  then
	    RETVALUE=$RET_UNTESTED
            echo "Test not done because there is no PFN available" > $TMP_LOG_FILE
        else
	    runCommand "edg_rc_getLogicalFileName -c $RC_CONF_FILE -C -p $PFN" $TMP_LOG_FILE
	    lookResult $? $TMP_LOG_FILE ; RETVALUE=$?
	fi
	;;

    7)  if [ -z $PFN ] ;  then
	    RETVALUE=$RET_UNTESTED
            echo "Test not done because there is no PFN available" > $TMP_LOG_FILE
        else
	    runCommand "edg_rc_deletePhysicalFileName -c $RC_CONF_FILE -C -l $LFN -p $PFN" $TMP_LOG_FILE
	    lookResult $? $TMP_LOG_FILE ; RETVALUE=$?
	fi
	;;

    8)  runCommand "edg_rc_deleteLogicalFileName -c $RC_CONF_FILE -C -l $LFN" $TMP_LOG_FILE
        lookResult $? $TMP_LOG_FILE ; RETVALUE=$?
	;;

    *)  return $RET_UNSUPPORTED
    esac

    if [ -z "$3" ] ; then 
        # If no logfile was specified, just send log of this step to stdout
	cat $TMP_LOG_FILE
    else
        # If we specified a logfile, append the results of every test there
	cat $TMP_LOG_FILE >> $3
    fi

    rm -f $TMP_LOG_FILE

    return $RETVALUE
}

helptest ()
{
    local SELECTION=$1
    case "$SELECTION" in
    0)  echo "addLFN";;
    1)  echo "addLFAttr";;
    2)  echo "getLFAttr";;
    3)  echo "delLFAttr";;
    4)  echo "addPFN";;
    5)  echo "getPFNs";;
    6)  echo "getLFN";;
    7)  echo "delPFN";;
    8)  echo "delLFN";;
    *)  
# Here comes a long (html) description of all the tests
	echo "<br>"
	echo "<b>Tests performed</b><br>"
	echo "This is a test of the edg_rc command line tools<br><br>"

	echo "<u>add LFN</u><br>"
	echo "edg_rc_addLogicalFileName -c $RC_CONF_FILE -C -l $LFN <br><br>"

	echo "<u>add LF Attribute</u><br>"
	echo "edg_rc_addLogicalFileAttribute -c $RC_CONF_FILE -C -l $LFN -n $LFN_ATT -v $LFN_ATT_VAL <br><br>"
	echo "<u>get LF Attributes</u><br>"
	echo "edg_rc_getLogicalFileAttributes -c $RC_CONF_FILE -C -l $LFN <br><br>"

	echo "<u>del LF Attribute</u><br>"
	echo "edg_rc_deleteLogicalFileAttribute -c $RC_CONF_FILE -C -l $LFN -n $LFN_ATT -v $LFN_ATT_VAL <br><br>"

	echo "<u>add PFN</u><br>"
	echo "edg_rc_addPhysicalFileName -c $RC_CONF_FILE -C -l $LFN -p $PFN <br><br>"

	echo "<u>get PFNs</u><br>"
	echo "edg_rc_getPhysicalFileNames -c $RC_CONF_FILE -C -l $LFN <br><br>"

	echo "<u>get LFN</u><br>"
	echo "edg_rc_getLogicalFileName -c $RC_CONF_FILE -C -p $PFN <br><br>"

	echo "<u>del PFN</u><br>"
	echo "edg_rc_deletePhysicalFileName -c $RC_CONF_FILE -C -l $LFN -p $PFN <br><br> "

	echo "<u>del LFN</u><br>"
	echo "edg_rc_deleteLogicalFileName -c $RC_CONF_FILE -C -l $LFN <br><br>"
	;;
    esac
}

lookResult ()
# args:
#  $1 - Return code of previous statement (mandatory)
#  $2 - File where to look for appearances of FAIL, WARNING or OK (optional)
#
{
    local RETVALUE=$RET_PASSED

    if [ $1 -gt 0 ] ; then 
        RETVALUE=$RET_FAIL 
    elif [ ! -z "$2" ] ; then
	local TMP_FILE=$2
	if [ `grep FAIL $TMP_FILE > /dev/null ; echo $?` -eq 0 ] ; then
	    RETVALUE=$RET_FAIL
	elif [ `grep WARNING $TMP_FILE > /dev/null ; echo $?` -eq 0 ] ; then
	    RETVALUE=$RET_WARNING
	fi
    fi

    return $RETVALUE
}

ldapGetPFN_PATH ()
#
# args:
#  $1 - RC config file (mandatory)
#  $2 - Log file where to dump stdout of ldapsearch etc (mandatory)
#
{
    if [ -z "$1" ] || [ -z "$2" ] ; then
	message 1 "$0:ldapGetPFN_PATH: Wrong nr of arguments"
	exit $RET_UNRESOLVED
    fi

    local RC_CONF_FILE=$1      
    local LOG_FILE=$2
    local TMP_FILE=/tmp/ldapQueryRC.$$.tmp
    
# Get the RC host, port and Logical Collection base name from the RC_CONF_FILE
    local TMP_VAR=`grep RC_LOGICAL_COLLECTION $RC_CONF_FILE`
    local RC_HOST=`echo $TMP_VAR | awk -F/ '{print $3}' | awk -F: '{print $1}'`
    local RC_PORT=`echo $TMP_VAR | awk -F/ '{print $3}' | awk -F: '{print $2}'`
    local RC_BASE="`echo $TMP_VAR | awk -F/ '{print $4}'`"

    runCommand "ldapsearch -x -h $RC_HOST -p $RC_PORT -b \"$RC_BASE\" '(objectclass=GlobusReplicaInfo)' uc path -P 2" $TMP_FILE

    if [ $? -ne 0 ] ; then
       message 1 "$0 ldapsearch failure"
       cat $TMP_FILE >> $LOG_FILE 
       return 1
    fi

    local SE_HOST=`grep uc: $TMP_FILE | head -1 | awk -F": " '{print $2}'`
    local SE_PATH=`grep path: $TMP_FILE | head -1 | awk -F": " '{print $2}'`

    rm -f $TMP_FILE

    # Use a global variable to return a string
    PFN_PATH=$SE_HOST$SE_PATH

    return 0
}

TEST_NUM=9

#Global variables
VERBOSE_LEVEL=0
TIMINGOUT=30 #Overrides the defaut value at runCommand.sh

LFN=lfn.test
LFN_ATT="size"
LFN_ATT_VAL=100

NAME="`/bin/basename $0`"

ARGS_SHORT="hins:l:d:"
ARGS_LONG="help,info,numstep,step:,logfile:,debug:"

TEMP=`getopt -o $ARGS_SHORT --long $ARGS_LONG -n $NAME -- "$@"`

if [ $? != 0 ] ; then 
  message_help
  exit $RET_UNRESOLVED
fi

eval set -- "$TEMP"

while true ; do
  case "$1" in
      -h|--help)      HELP=1                   ;;
      -i|--info)      NFO=1                    ;;
      -n|--numstep)   NSTEP=1                  ;;
      -s|--step)      SELSTEP=$2       ; shift ;;
      -l|--logfile)   LOGOUT=$2        ; shift ;;
      -d|--debug)     VERBOSE_LEVEL=$2 ; shift ;;
      --) shift ; break ;;
	*) echo "Internal error!" ; exit $RET_UNRESOLVED ;;
  esac
  shift
done

# Take action depending on the found flags
if [ ! -z "$HELP" ] ; then
    message_help
    exit $RET_PASSED
fi
  
if [ ! -z "$NSTEP" ] ; then 
  echo "$TEST_NUM"
  exit $RET_PASSED
fi

if [ ! -z "$NFO" ] ; then
   helptest "$SELSTEP"
   exit $RET_PASSED
fi     

# Get the mandatory argument: RC config file
if [ -z "$1" ] ; then
 message 0 "You must specify a RC config file!"
 message_help
 exit $RET_UNRESOLVED
else
 RC_CONF_FILE=$1
fi

rm -f $LOGOUT
 
RETVAL=$RET_PASSED
RETVALNEW=$RET_PASSED

if [ -z "$SELSTEP" ]
then
   k=0
   while [ $k -lt $TEST_NUM ] 
   do
       dotest $k $RC_CONF_FILE $LOGOUT
       RETVALNEW=$?
       if [ $RETVAL -eq $RET_PASSED ] ; then RETVAL=$RETVALNEW ; fi 

       TEST_LABEL="`helptest $k`"
       case "$RETVALNEW" in
        $RET_PASSED)  echo -n $TEST_LABEL ; echo_success ;;
        $RET_FAIL)    echo -n $TEST_LABEL ; echo_failure ;;
        $RET_WARNING) echo -n $TEST_LABEL ; echo_passed  ;;
        *)            echo $TEST_LABEL ;;       
       esac

       k=`expr $k + 1`
   done
else
    TMP_STATE_FILE=/tmp/$NAME.state.$UID

    if [ "$SELSTEP" -eq 0 ] ;  then
	# Erase previous state file in the 1st step
	rm -f $TMP_STATE_FILE
    else
	# Load state file, if there, for the 2nd to last step
	loadVars $TMP_STATE_FILE
    fi

    dotest $SELSTEP $RC_CONF_FILE $LOGOUT
    RETVAL=$?

    TEST_LABEL="`helptest $SELSTEP`"
    case "$RETVAL" in
	$RET_PASSED)  echo -n $TEST_LABEL ; echo_success ;;
	$RET_FAIL)    echo -n $TEST_LABEL ; echo_failure ;;
	$RET_WARNING) echo -n $TEST_LABEL ; echo_passed  ;;
	*)            echo $TEST_LABEL ;;       
    esac
  
#
   if [ ! -z  $PFN ] ;  then
    saveVars $TMP_STATE_FILE "PFN"
   fi
fi    

exit $RETVAL
#
