#!/bin/bash
######################################################################
#
# Authors: Miquel Barcelo  -  barcelo@ifae.es
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
#######################################################################
#
# Requires :
#       - The EDG_TESTROOT environment variable being defined
#       - getopt (installed by default on most systems)
#
##################################################################
#
# SCRIPT RM_func_main.sh
#
# USAGE: RM_func_main.sh [-h] -n
#        RM_func_main.sh [-h] -i [-s int]
#        RM_func_main.sh [-h] [-s int] [-d int] [-S int] [-g int] [-r string] [-c string] [-v string] [-l string] [-x] test_host_list reference_host_list 
#
# ARGUMENTS: 
#        -h/--help           Shows help.
#        -n/--numstep        Prints the number of selectable steps on the test.
#        -s/--step           Run only the specified step, not the entire test.
#        -i/--info           Prints brief info about the selected step using "-s".
#                            Prints detailed info about the entire test if no step selected.
#        -d/--debug          Selects verbosity (debug) level. 0=none 1=brief 2=full. (Default: 0)
#        -S/--filesize       Size in kB of the file used for replication. (Default: 256)
#        -g/--grisport       Port to use for GRIS. (Default: 2135)
#        -r/--remotedir      Sub-directory (on the SE) to put the file. (Default: edg_testing)
#        -c/--rcconf         Use specified rc.conf file.
#        -v/--vo             VO to be used. (Default: wpsix)
#        -l/--logfile        Log test results to this file. By default is dumped to stdout.
#        -x/--logxml         Use XML format when logging to a file.
#
# DESCRIPTION:  Simple ReplicaManager test. Creates a file on test_host, replicates it to reference_host
#               and finally delete it on both hosts an on Replica Catalog.  
#
# RETURN VALUE: 0 PASSED
#	 	1 FAIL
#		2 WARNING
#		3 XFAIL
#		4 UNRESOLVED
#		5 UNTESTED
#		6 UNSUPPORTED
#
##################################################################

# We should take a look for an /etc/edg-test.conf or 
# $EDG_LOCATION/etc/edg-test.conf for configuration file

if [ -z "$EDG_TESTROOT" ]
then
      echo "EDG_TESTROOT environment variable must be defined!"
      exit 4
fi    

source $EDG_TESTROOT/sbin/common.sh 
source $EDG_TESTROOT/sbin/message.sh
source $EDG_TESTROOT/sbin/runCommand.sh
source $EDG_TESTROOT/sbin/saveVars.sh
source $EDG_TESTROOT/sbin/loadVars.sh
source $EDG_TESTROOT/sbin/echoFunc.sh
source $EDG_TESTROOT/sbin/testRetCodes.sh

# Take a look to the bottom to see the real body of script

message_help ()
{
    message 0 "Usage: RM_func_main.sh [-h] -n"
    message 0 "       RM_func_main.sh [-h] -i [-s int]"
    message 0 "       RM_func_main.sh [-h] [-s int] [-d int] [-S int] [-g int] [-c string] [-r string] [-v string] [-l string] test_host_list reference_host_list "
    message 0 " "
    message 0 "Options/Arguments:" 
    message 0 "       -h/--help           Shows help."
    message 0 "       -n/--numstep        Prints the number of selectable steps on the test."
    message 0 "       -s/--step           Run only the specified step, not the entire test."
    message 0 "       -i/--info           Prints brief info about the selected step using ""-s""."
    message 0 "                           Prints detailed info about the entire test if no step selected."
    message 0 "       -d/--debug          Selects verbosity (debug) level. 0=none 1=brief 2=full. (Default: 0)"
    message 0 "       -S/--filesize       Size in kB of the file used for file replication. (Default: 256)"
    message 0 "       -g/--grisport       Port to use for GRIS. (Default: 2135)"
    message 0 "       -r/--remotedir      Sub-directory (on the SE) to put the file. (Default: edg_testing)"
    message 0 "       -c/--rcconf         Used specified rc.conf file"
    message 0 "       -v/--vo             VO to be used. (Default: wpsix)"
    message 0 "       -l/--logfile        Log test results to this file. By default is dumped to stdout."
    message 0 "       -x/--logxml         Use XML format when logging to a file."

}

dotest ()
{
    if [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ]
    then
	message 1 "$0: wrong number of arguments."
	exit 7
    fi

    local SELECTION=$1
    local HOST_A=$2
    local HOST_B=$3
    if [ -z "$4" ] ; then
    local TMP_LOG_FILE=.tmplog_$PPID
    else
        local TMP_LOG_FILE=$4.new
    fi

    local RETVALUE=$RET_PASSED

    case "$SELECTION" in
    0)	runCommand "grid-info-search -h $HOST_B -p $GRIS_PORT -b mds-vo-name=local,o=grid objectclass=StorageElement -x SEvo" $TMP_LOG_FILE
	if [ $? -ne 0 ] ; then RETVALUE=$RET_WARNING ; fi
        PATH_B_HOST=`grep $TEST_VO $TMP_LOG_FILE | awk -F: '{print $3}'`
	if [ -z $PATH_B_HOST ] ; then 
	    echo "ERROR: $HOST_B does not support $TEST_VO vo" >> $TMP_LOG_FILE
	    RETVALUE=$RET_UNSUPPORTED
	fi
	PATH_B_HOST=$PATH_B_HOST/$TEST_DIR

	runCommand "grid-info-search -h $HOST_A -p $GRIS_PORT -b mds-vo-name=local,o=grid objectclass=StorageElement -x SEvo" $TMP_LOG_FILE.B
	if [ $? -ne 0 ] ; then RETVALUE=$RET_FAIL ; fi
    	PATH_A_HOST=`grep $TEST_VO $TMP_LOG_FILE.B | awk -F: '{print $3}'`
	if [ -z $PATH_A_HOST ] ; then 
	    echo "ERROR: $HOST_A does not support $TEST_VO vo" >> $TMP_LOG_FILE.B
	    RETVALUE=$RET_UNSUPPORTED
	fi
	PATH_A_HOST=$PATH_A_HOST/$TEST_DIR

	# Use now the edg-gridftp- tools to make a directory in the remote SEs, if needed
	runCommand "edg-gridftp-exists gsiftp://$HOST_B$PATH_B_HOST" $TMP_LOG_FILE.C
	if [ $? -ne 0 ] ; then
	    # If the directory does not exist yet, make it
	    runCommand "edg-gridftp-mkdir gsiftp://$HOST_B$PATH_B_HOST" $TMP_LOG_FILE.D
	    if [ $? -ne 0 ] ; then RETVALUE=$RET_UNSUPPORTED ; fi
	    cat $TMP_LOG_FILE.D >> $TMP_LOG_FILE.C ; rm -f $TMP_LOG_FILE.D
	fi

	runCommand "edg-gridftp-exists gsiftp://$HOST_A$PATH_A_HOST" $TMP_LOG_FILE.E
	if [ $? -ne 0 ] ; then
	    # If the directory does not exist yet, make it
	    runCommand "edg-gridftp-mkdir gsiftp://$HOST_A$PATH_A_HOST" $TMP_LOG_FILE.F
	    if [ $? -ne 0 ] ; then RETVALUE=$RET_UNSUPPORTED ; fi
            cat $TMP_LOG_FILE.F >> $TMP_LOG_FILE.E ; rm -f $TMP_LOG_FILE.F
	fi

	cat $TMP_LOG_FILE.B >> $TMP_LOG_FILE ; rm -f $TMP_LOG_FILE.B
	cat $TMP_LOG_FILE.C >> $TMP_LOG_FILE ; rm -f $TMP_LOG_FILE.C
	cat $TMP_LOG_FILE.E >> $TMP_LOG_FILE ; rm -f $TMP_LOG_FILE.E
	;;
    1)  FILE_SUFX=`date +%y%m%d%H%M%S`
        TEST_FILE=dummy_file_RM.$FILE_SUFX
        runCommand "dd if=/dev/urandom of=/tmp/$TEST_FILE bs=1024 count=$FILE_SIZE" /dev/null
	runCommand "edg-replica-manager-copyAndRegisterFile -s $UI_HOST/tmp/$TEST_FILE -d $HOST_A -l $TEST_DIR$TEST_FILE $RC_CONF" $TMP_LOG_FILE
        if [ $? -ne 0 ] ; then RETVALUE=$RET_FAIL ; fi
        ;;
    2)  runCommand "edg-replica-manager-listReplicas -l $TEST_DIR$TEST_FILE $RC_CONF" $TMP_LOG_FILE
        if [ $? -ne 0 ] ; then 
	    echo "Last command failed <$?>" >> $TMP_LOG_FILE
	    RETVALUE=$RET_FAIL 
	elif [ `grep location $TMP_LOG_FILE | awk -F: '{print $2}' | grep $HOST_A &> /dev/null ; echo $?` -ne 0 ] ; then
	    # Check as well that the new copied file is listed in the edg-rm-listReplicas result, otherwise FAIL
	    echo "Host $HOST_A not found in the list of replicas" >> $TMP_LOG_FILE
	    RETVALUE=$RET_FAIL
 	else
	    # If the replica is there, compare the size and checksum of the replicated file wrt the local copy
	    checkSumTest $HOST_A $TEST_FILE $PATH_A_HOST /tmp $TMP_LOG_FILE
	    RETVALUE=$?
	fi
	;;
    3)	runCommand "edg-replica-manager-replicateFile -s $HOST_A -d $HOST_B -l $TEST_DIR$TEST_FILE $RC_CONF" $TMP_LOG_FILE
	if [ $? -ne 0 ] ; then RETVALUE=$RET_FAIL ; fi
	;;
    4)  runCommand "edg-replica-manager-listReplicas -l $TEST_DIR$TEST_FILE $RC_CONF" $TMP_LOG_FILE
        if [ $? -ne 0 ] ; then 
	    echo "Last command failed <$?>" >> $TMP_LOG_FILE
	    RETVALUE=$RET_FAIL 
	# Check as well that the new replicated file is listed in the edg-rm-listReplicas result, otherwise FAIL
	elif [ `grep location $TMP_LOG_FILE | awk -F: '{print $2}' | grep $HOST_A &> /dev/null ; echo $?` -ne 0 ] ; then
	    echo "Host $HOST_A not found in the list of replicas" >> $TMP_LOG_FILE
	    RETVALUE=$RET_FAIL 
	elif [ `grep location $TMP_LOG_FILE | awk -F: '{print $2}' | grep $HOST_B &> /dev/null ; echo $?` -ne 0 ] ; then
	    echo "Host $HOST_B not found in the list of replicas" >> $TMP_LOG_FILE
	    RETVALUE=$RET_FAIL 
	else
	    # If the 2nd replica is there, compare the size and checksum of the replicated file wrt the local copy
	    checkSumTest $HOST_B $TEST_FILE $PATH_B_HOST /tmp $TMP_LOG_FILE
	    RETVALUE=$?
	fi
	;;
    5) 	runCommand "edg-replica-manager-deleteFile -a -l $TEST_DIR$TEST_FILE $RC_CONF" $TMP_LOG_FILE
	if [ $? -ne 0 ] ; then RETVALUE=$RET_FAIL ; fi
	
	# Finally, erase the local copy of the test file
	rm -f /tmp/$TEST_FILE
	;;
    6)  runCommand "edg-replica-manager-listReplicas -l $TEST_DIR$TEST_FILE $RC_CONF" $TMP_LOG_FILE
	if [ `grep location $TMP_LOG_FILE &> /dev/null ; echo $?` -eq 0 ] ; then
	    # Check that the deleted file is NOT listed in the edg-rm-listReplicas result, otherwise FAIL
	    echo "List replicas found NOT empty!" >> $TMP_LOG_FILE
	    RETVALUE=$RET_FAIL
	else
	   runCommand "edg-gridftp-exists gsiftp://$HOST_A$PATH_A_HOST$TEST_DIR$TEST_FILE" $TMP_LOG_FILE.B
	   if [ $? -eq 0 ] ; then RETVALUE=$RET_FAIL ; fi
	   runCommand "edg-gridftp-exists gsiftp://$HOST_B$PATH_B_HOST$TEST_DIR$TEST_FILE" $TMP_LOG_FILE.C 
	   if [ $? -eq 0 ] ; then RETVALUE=$RET_FAIL ; fi

	   cat $TMP_LOG_FILE.B >> $TMP_LOG_FILE ; rm -f $TMP_LOG_FILE.B
	   cat $TMP_LOG_FILE.C >> $TMP_LOG_FILE ; rm -f $TMP_LOG_FILE.C	   
	fi
	;;
    *)  return $RET_UNTESTED
    esac
    if [ -z $4 ] ; then 
	cat $TMP_LOG_FILE
    else
	cat $TMP_LOG_FILE >> $4
    fi
    rm -f $TMP_LOG_FILE
    return $RETVALUE
}

helptest_brf ()
{
    local SELECTION=$1
    case "$SELECTION" in
    0)  echo "Preparation"
	;;
    1)  echo "Copy and Register"
	;;
    2)  echo "Check Replica"
	;;
    3)  echo "Replicate File"
	;;
    4)  echo "Check Replica"
	;;
    5)  echo "Delete Files"
	;;
    6)  echo "Check Deletion"
	;;
    *)  echo ""
	;;
    esac
}

helptest_det ()
{
    local SELECTION=$1
    case "$SELECTION" in
    0) 	echo "<i>grid-info-search -h HOST_B -p $GRIS_PORT -b o=grid objectclass=StorageElement -x SEvo</i><br>"
	echo "grid-info-search -h HOST_A -p $GRIS_PORT -b o=grid objectclass=StorageElement -x SEvo<br>"
	echo "edg-gridftp-mkdir gsiftp://$HOST_B$PATH_B_HOST<br>"
	echo "edg-gridftp-mkdir gsiftp://$HOST_A$PATH_A_HOST<br><br>"
	;;
    1) 	echo "edg-replica-manager-copyAndRegisterFile -s $UI_HOST/tmp/TEST_FILE -d HOST_A -l TEST_FILE $RC_CONF<br><br>"
	;;
    2)  echo "edg-replica-manager-listReplicas -l TEST_FILE  $RC_CONF<br><br>"
	;;
    3)  echo "edg-replica-manager-replicateFile -s HOST_A -d HOST_B -l TEST_FILE $RC_CONF<br><br>"
	;;
    4)  echo "edg-replica-manager-listReplicas -l TEST_FILE  $RC_CONF<br><br>"
	;;
    5)  echo "edg-replica-manager-deleteFile -a -l TEST_FILE $RC_CONF<br>"
	;;
    6)  echo "edg-replica-manager-listReplicas -l TEST_FILE  $RC_CONF<br>"
	echo "edg-gridftp-exists gsiftp://$HOST_A$PATH_A_HOST<br>"
	echo "edg-gridftp-exists gsiftp://$HOST_B$PATH_B_HOST<br><br>"
	;;
    *)  echo ""
	;;
    esac
}

helptest ()
{
    case "$1" in
    0|1|2|3|4|5|6) helptest_brf $1 ;;
    *) 	local cnt=0;
	while [ $cnt -lt $TEST_NUM ] ; do
	    echo "<br>"
	    helptest_brf $cnt
	    echo "<br><br>"
	    helptest_det $cnt
	    cnt=$(($cnt + 1))
	done
	;;
    esac
}

checkSumTest() {
# Arguments:
#  $1 - Remote Host where the replicated file lives
#  $2 - Test file name
#  $3 - File dir path on the remote host (without the file name)
#  $4 - File dir path locally (original file)
#  $5 - LogFile name where to append logmessages

local L_HOST=$1
local L_TEST_FILE=$2
local L_PATH_HOST=$3
local L_PATH_LOCAL=$4
local L_MAIN_LOG_FILE=$5

local L_RETVALUE=$RET_PASSED
local L_TMP_LOG_FILE1=/tmp/checkSumTest.$$.tmp1
local L_TMP_LOG_FILE2=/tmp/checkSumTest.$$.tmp2

runCommand "globus-job-run $L_HOST /usr/bin/cksum $L_PATH_HOST/$L_TEST_FILE" $L_TMP_LOG_FILE1
if [ $? -ne 0 ] ; then 
    echo "FAIL. Previous command failed"
    L_RETVALUE=$RET_FAIL 
else
    runCommand "/usr/bin/cksum $L_PATH_LOCAL/$L_TEST_FILE" $L_TMP_LOG_FILE2

    local L_REMOTE_CKSUM=`grep $L_TEST_FILE $L_TMP_LOG_FILE1 | tail -1 | awk '{print $1 $2}'`
    local L_LOCAL_CKSUM=`grep $L_TEST_FILE $L_TMP_LOG_FILE2 | tail -1 | awk '{print $1 $2}'`

    cat $L_TMP_LOG_FILE2 >> $L_TMP_LOG_FILE1
    rm -f $L_TMP_LOG_FILE2

    if [ $L_REMOTE_CKSUM -ne $L_LOCAL_CKSUM ] ;  then
	echo "FAIL. Files checksum or size appear to be different!" >> $L_TMP_LOG_FILE1
	echo "Remote file ChecksumSize $L_HOST$L_PATH_HOST/$L_TEST_FILE = <$L_REMOTE_CKSUM>" >> $L_TMP_LOG_FILE1
	echo "Local file ChecksumSize $L_PATH_LOCAL/$TEST_FILE = <$L_LOCAL_CKSUM>" >> $L_TMP_LOG_FILE1
	L_RETVALUE=$RET_FAIL
    fi
fi

cat $L_TMP_LOG_FILE1 >> $L_MAIN_LOG_FILE
rm -f $L_TMP_LOG_FILE1 

return $L_RETVALUE
}

echo_log ()
{
    if [ "" != "$LOGOUT" ] && [ "1" = "$XML_LOG" ] ; then
	$ECHO $1 >> $LOGOUT
    fi
}

#Verbosity level
VERBOSE_LEVEL=0
#logname
#Size of file used to test replication
FILE_SIZE=256
#Replica Catalogue flag: -n do not write to RC ; -C write to RC
RC_FLAG=-C
#tests number
TEST_NUM=7
#VO to test
TEST_VO=wpsix
#Host where test is running
UI_HOST=`hostname`
# default GRIS
GRIS_PORT=2135
#sub-directory to place the files
TEST_DIR=edg_testing/


  NAME="`/bin/basename $0`"

  ARGS_SHORT="hinxs:l:d:S:v:g:r:c:"
  ARGS_LONG="help,info,numstep,logxml,step:,logfile:,debug:,filesize:,vo:,grisport:,remotedir:,rcconf:"

  TEMP=`getopt -o $ARGS_SHORT --long $ARGS_LONG -n $NAME -- "$@"`

  if [ $? != 0 ] ; then 
    message_help
    exit $RET_UNRESOLVED
  fi
  eval set -- "$TEMP"

  while true ; do
    case "$1" in
	-h|--help)      HELP=1                   ;;
	-i|--info)      NFO=1                    ;;
	-n|--numstep)   NSTEP=1                  ;;
	-s|--step)      SELSTEP=$2       ; shift ;;
	-l|--logfile)   LOGOUT=$2        ; shift ;;
	-d|--debug)     VERBOSE_LEVEL=$2 ; shift ;;
	-S|--filesize)  FILE_SIZE=$2     ; shift ;;
	-v|--vo)        TEST_VO=$2       ; shift ;;
	-g|--grisport)  GRIS_PORT=$2     ; shift ;;
	-r|--remotedir) TEST_DIR=$2/     ; shift ;;
	-c|--rcconf)    RC_CONF="-c $2"  ; shift ;;
	-x|--logxml)    XML_LOG=1                ;;
	--) shift ; break ;;
	*) echo "Internal error!" ; exit $RET_UNRESOLVED ;;
    esac
    shift
  done

# parse the positional parameters

  HOST_A_LIST=$1
  HOST_B_LIST=$2

  if [ -n "$HELP" ] ; then
      message_help
      exit 0
  fi
  
  if [ -n "$NSTEP" ] ; then 
    echo "$TEST_NUM"
    exit 0
  fi

  if [ -n "$NFO" ] ; then
    helptest "$SELSTEP"
    exit 0
  fi     

  if [ -z "$HOST_A_LIST" ] || [ -z "$HOST_B_LIST" ]
  then
    message 0 "Must specify a target host and a reference host!"
    message_help
    exit $RET_UNRESOLVED
  fi

# The host to test and the reference host can't be the same !  
#  if [ "$HOST_A" = "$HOST_B" ] ; then 
#    echo "Target host and reference host can't be the same!"
#    exit $RET_UNRESOLVED
#  fi

  rm -f $LOGOUT
 
  echo_log "<log>"
  echo_log "<info>"
  echo_log "<title>"
  echo_log "RM Test"
  echo_log "</title>"
  echo_log "<date>"
  echo_log "`date`"
  echo_log "</date>"
  echo_log "<rows_list>"
  for i in $HOST_A_LIST ; do echo_log $i ; done
  echo_log "</rows_list>"
  echo_log "<cols_list>"
  for i in $HOST_B_LIST ; do echo_log $i ; done
  echo_log "</cols_list>"
  
  i=0
  while [ $i -lt $TEST_NUM ] ; do
      echo_log "<step>"
      echo_log "<brf_info>"
      echo_log "`helptest_brf $i`"
      echo_log "</brf_info>"
      echo_log "<det_info>"
      echo_log "`helptest_det $i`"
      echo_log "</det_info>"
      echo_log "</step>"
      i=$(($i + 1))
  done
  echo_log "</info>"

  RETVAL=0
  RETVALNEW=0

  if [ -z "$SELSTEP" ] ; then
    for HOST_A in $HOST_A_LIST ; do
	echo_log "<row>"
	for HOST_B in $HOST_B_LIST ; do
	    echo_log "<col>"
	    k=0
	    while [ $k -lt $TEST_NUM ] ; do
		echo_log "<step>" 
		echo_log "<output>" 
		if [ "$HOST_A" = "$HOST_B" ] ; then 
		    echo_log "Target host and reference host can't be the same!" 
		    RETVAL=$RET_UNRESOLVED
		    k=$TEST_NUM
		else
		    dotest $k $HOST_A $HOST_B $LOGOUT
		    RETVALNEW=$?
		    if [ $RETVAL -ne 1 ] ; then RETVAL=$RETVALNEW ; fi 
		    $ECHO -n "`helptest $k`"
		    case "$RETVALNEW" in
			0) echo_success ;;
			1) echo_failure ;;
			2) echo_passed  ;;
		    esac
		    k=`expr $k + 1`
		fi
		echo_log "</output>"
		echo_log "<retval>"
		echo_log "$RETVAL"
		echo_log "</retval>"
		echo_log "</step>"
	    done
	    echo_log "</col>"
	done
	echo_log "</row>"
    done
  else
    if [ "$HOST_A_LIST" = "$HOST_B_LIST" ] ; then 
	echo "Target host and reference host can't be the same!" >> $LOGOUT
	RETVAL=$RET_UNRESOLVED
	exit $RETVAL
    fi
    loadVars /tmp/$NAME.state_$UID
    dotest $SELSTEP $HOST_A_LIST $HOST_B_LIST $LOGOUT
    RETVAL=$?
    echo $RETVAL
    $ECHO -n "`helptest $SELSTEP`"
    case "$RETVAL" in
	0) echo_success ;;
	1) echo_failure ;;
	2) echo_passed  ;;
    esac
    saveVars /tmp/$NAME.state_$UID "PATH_A_HOST PATH_B_HOST FILE_SUFX TEST_FILE"
  fi    
  echo_log "</log>"


exit $RETVAL





