#!/bin/bash
######################################################################
#
# Authors: Miquel Barcelo  -  barcelo@ifae.es
#
# Copyright 2001-2002 EU DataGrid project (2002), WP6 TestBed
#                    http://www.edg.org/license.html
#
#######################################################################
#
# Requires :
#       - The EDG_TESTROOT environment variable being defined
#       - getopt (installed by default on most systems)
#
##################################################################
#
# SCRIPT 1D_sample_test.sh
#
# USAGE: 1D_sample_test.sh [-h] -n
#        1D_sample_test.sh [-h] -i [-s int]
#        1D_sample_test.sh [-h] [-s int] [-l string] test_host 
#
# ARGUMENTS: 
#        -h/--help           Shows help.
#        -n/--numstep        Prints the number of selectable steps on the test.
#        -s/--step           Run only the specified step, not the entire test.
#        -i/--info           Prints brief info about the selected step using "-s".
#                            Prints detailed info about the entire test if no step selected.
#        -l/--logfile        Log test results to this file. By default is dumped to stdout.
#
# DESCRIPTION:  Sample/dummy 1D (remote host) test.
#               
#
# RETURN VALUE: 0 PASSED
#	 	1 FAIL
#		2 WARNING
#		3 XFAIL
#		4 UNRESOLVED
#		5 UNTESTED
#		6 UNSUPPORTED
#
##################################################################

RET_PASSED=0
RET_FAIL=1
RET_WARNING=2
RET_XFAIL=3
RET_UNRESOLVED=4
RET_UNTESTED=5
RET_UNSUPPORTED=6

# We should take a look for an /etc/edg-test.conf or 
# $EDG_LOCATION/etc/edg-test.conf for configuration file

if [ -z "$EDG_TESTROOT" ]
then
      echo "EDG_TESTROOT environment variable must be defined!"
      exit $RET_UNRESOLVED
fi    

source $EDG_TESTROOT/sbin/common.sh 
source $EDG_TESTROOT/sbin/message.sh
source $EDG_TESTROOT/sbin/runCommand.sh
source $EDG_TESTROOT/sbin/saveVars.sh
source $EDG_TESTROOT/sbin/loadVars.sh

# Take a look to the bottom to see the real body of script

message_help ()
{
    message 0 "Usage: 1D_sample_test.sh [-h] -n"
    message 0 "       1D_sample_test.sh [-h] -i [-s int]"
    message 0 "       1D_sample_test.sh [-h] [-s int] [-l string] test_host "
    message 0 " "
    message 0 "Options/Arguments:" 
    message 0 "       -h/--help           Shows help."
    message 0 "       -n/--numstep        Prints the number of selectable steps on the test."
    message 0 "       -s/--step           Run only the specified step, not the entire test."
    message 0 "       -i/--info           Prints brief info about the selected step using ""-s""."
    message 0 "                           Prints detailed info about the entire test if no step selected."
    message 0 "       -l/--logfile        Log test results to this file. By default is dumped to stdout."
}

dotest ()
{
    if [ -z "$1" ] || [ -z "$2" ]
    then
	message 1 "$0: wrong number of arguments."
	exit $RET_UNRESOLVED
    fi

    local SELECTION=$1
    local HOST_A=$2
    if [ -z "$3" ] ; then
    local TMP_LOG_FILE=.tmplog_$PPID
    else
        local TMP_LOG_FILE=$3.new
    fi

    local RETVALUE=$RET_PASSED

    echo "Testing $HOST_A, step $SELECTION" > $TMP_LOG_FILE
    case "$SELECTION" in
    0)  RETVALUE=$RET_PASSED ;;
    1)  RETVALUE=$RET_WARNING ;;
    2)  RETVALUE=$RET_FAIL ;;
    3)  RETVALUE=$RET_XFAIL ;;
    4)  RETVALUE=$RET_UNRESOLVED ;;
    5)  RETVALUE=$RET_UNTESTED ;;
    6)  RETVALUE=$RET_UNSUPPORTED ;;
    *)  return $RET_UNTESTED
    esac

    if [ -z $3 ] ; then 
	cat $TMP_LOG_FILE
    else
	cat $TMP_LOG_FILE >> $3
    fi
    rm -f $TMP_LOG_FILE
    return $RETVALUE
}

helptest ()
{
    local SELECTION=$1
    case "$SELECTION" in
    0|1|2|3|4|5|6)  
#put here a brief description on each step 
	message 0 "step $SELECTION"
	;;
    *)  
#put here a long description of all test
	echo "<br>"
	echo "<b>Tests performed</b><br>"
	echo "This is a sample test, it actually does nothing<br><br>"
	;;
    esac
}

TEST_NUM=7

  NAME="`/bin/basename $0`"

  ARGS_SHORT="hins:l:"
  ARGS_LONG="help,info,numstep,step:,logfile:"

  TEMP=`getopt -o $ARGS_SHORT --long $ARGS_LONG -n $NAME -- "$@"`

  if [ $? != 0 ] ; then 
    message_help
    exit $RET_UNRESOLVED
  fi
  eval set -- "$TEMP"

  while true ; do
    case "$1" in
	-h|--help)      HELP=1                   ;;
	-i|--info)      NFO=1                    ;;
	-n|--numstep)   NSTEP=1                  ;;
	-s|--step)      SELSTEP=$2       ; shift ;;
	-l|--logfile)   LOGOUT=$2        ; shift ;;
	--) shift ; break ;;
	*) message 0 "Internal error!" ; exit $RET_UNRESOLVED ;;
    esac
    shift
  done

# parse the positional parameters

  HOST_A=$1

  if [ -n "$HELP" ] ; then
      message_help
      exit $RET_PASSED
  fi
  
  if [ -n "$NSTEP" ] ; then 
    echo "$TEST_NUM"
    exit $RET_PASSED
  fi

  if [ -n "$NFO" ] ; then
    helptest "$SELSTEP"
    exit $RET_PASSED
  fi     

  if [ -z "$HOST_A" ]
  then
    message 0 "Must specify a target host!"
    message_help
    exit $RET_UNRESOLVED
  fi

  rm -f $LOGOUT
 
  RETVAL=0
  RETVALNEW=0

  if [ -z "$SELSTEP" ]
  then
    k=0
    while [ $k -lt $TEST_NUM ] 
    do
        dotest $k $HOST_A $LOGOUT
        RETVALNEW=$?
        if [ $RETVAL -ne 1 ] ; then RETVAL=$RETVALNEW ; fi 
        k=`expr $k + 1`
    done
  else
        dotest $SELSTEP $HOST_A $LOGOUT
        RETVAL=$?
  fi    

exit $RETVAL





